@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)amiconf.h	3.3	95/09/03	*/
+/*	SCCS Id: @(#)amiconf.h	3.4	2000/01/12	*/
 /* Copyright (c) Kenneth Lorber, Bethesda, Maryland, 1990, 1991, 1992, 1993. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -30,13 +30,23 @@
 # define DCC30_BUG	/* A bitfield bug (from dog.c, others) in DICE 3.0. */
 #endif
 
+#ifndef __GNUC__
 typedef long off_t;
+#endif
 
 #define MICRO		/* must be defined to allow some inclusions */
 
+#define NOCWD_ASSUMPTIONS	/* Allow paths to be specified for HACKDIR,
+				   LEVELDIR, SAVEDIR, BONESDIR, DATADIR,
+				   SCOREDIR, LOCKDIR, and CONFIGDIR. */
+
 /* data librarian defs */
-#define DLBFILE		"NetHack:nhdat"		/* main library */
-#define DLBFILE2	"NetHack:nhsdat"	/* sound library */
+#ifndef NOCWD_ASSUMPTIONS
+# define DLBFILE	"NetHack:nhdat"		/* main library */
+# define DLBFILE2	"NetHack:nhsdat"	/* sound library */
+#else
+# define DLBFILE	"nhdat"			/* main library */
+# define DLBFILE2	"nhsdat"		/* sound library */
 #define FILENAME_CMP	stricmp			/* case insensitive */
 
 #ifndef __SASC_60
@@ -55,6 +65,26 @@ typedef long off_t;
 			 */
 #define RANDOM
 
+/* ### amidos.c ### */
+
+extern void  FDECL(nethack_exit, (int));
+
+/* ### amiwbench.c ### */
+
+extern void NDECL(ami_wbench_init);
+extern void NDECL(ami_wbench_args);
+extern int FDECL(ami_wbench_getsave, (int));
+extern void FDECL(ami_wbench_unlink, (char *));
+extern int FDECL(ami_wbench_iconsize, (char *));
+extern void FDECL(ami_wbench_iconwrite, (char *));
+extern int FDECL(ami_wbench_badopt, (const char *));
+extern void NDECL(ami_wbench_cleanup);
+extern void FDECL(getlind, (const char *,char *,const char *));
+
+/* ### winreq.c ### */
+
+extern void amii_setpens(int);
+
 extern void FDECL(exit, (int));
 extern void NDECL(CleanUp);
 extern void FDECL(Abort, (long));
@@ -66,7 +96,8 @@ extern void FDECL(ami_argset, (int *, char *[]));
 extern void FDECL(ami_mkargline, (int *, char **[]));
 extern void ami_wininit_data(void);
 
-extern boolean FromWBench;	/* how were we run? */
+#define FromWBench 0 /* A hint for compiler ... */
+/* extern boolean FromWBench;	/* how were we run? */
 extern int ami_argc;
 extern char **ami_argv;
 
@@ -81,7 +112,7 @@ extern char **ami_argv;
 #define remove(x)	unlink(x)
 
 /* DICE wants rewind() to return void.	We want it to return int. */
-#ifdef _DCC
+#if defined(_DCC) || defined(__GNUC__)
 # define rewind(f)	fseek(f, 0, 0)
 #endif
 
@@ -109,7 +140,7 @@ extern char *FDECL(gets, (char *));
 #define MAIL			/* Get mail at unexpected occasions */
 #define DEFAULT_ICON "NetHack:default.icon"	/* private icon */
 #define AMIFLUSH		/* toss typeahead (select flush in .cnf) */
-#define OPT_DISPMAP		/* enable fast_map option */
+/* #define OPT_DISPMAP		/* enable fast_map option */
 
 /* new window system options */
 			/* WRONG - AMIGA_INTUITION should go away */
