@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)dlb.h	3.3	97/07/29	*/
+/*	SCCS Id: @(#)dlb.h	3.4	1997/07/29	*/
 /* Copyright (c) Kenneth Lorber, Bethesda, Maryland, 1993. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -125,7 +125,7 @@ long FDECL(dlb_ftell, (DLB_P));
 #endif
 
 #define RDTMODE "r"
-#if (defined(MSDOS) || defined(WIN32)) && defined(DLB)
+#if (defined(MSDOS) || defined(WIN32) || defined(TOS)) && defined(DLB)
 #define WRTMODE "w+b"
 #else
 #define WRTMODE "w+"
