@@ -1,14 +1,10 @@
-/*	SCCS Id: @(#)mondata.h	3.2	95/07/29	*/
+/*	SCCS Id: @(#)mondata.h	3.4	2001/02/14	*/
 /* Copyright (c) 1989 Mike Threepoint				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #ifndef MONDATA_H
 #define MONDATA_H
 
-#ifndef ALIGN_H
-#include "align.h"
-#endif
-
 #define verysmall(ptr)		((ptr)->msize < MZ_SMALL)
 #define bigmonst(ptr)		((ptr)->msize >= MZ_LARGE)
 
@@ -37,20 +33,21 @@
 #define hides_under(ptr)	(((ptr)->mflags1 & M1_CONCEAL) != 0L)
 #define is_hider(ptr)		(((ptr)->mflags1 & M1_HIDE) != 0L)
 #define haseyes(ptr)		(((ptr)->mflags1 & M1_NOEYES) == 0L)
+#define eyecount(ptr)		(!haseyes(ptr) ? 0 : \
+				 ((ptr) == &mons[PM_CYCLOPS] || \
+				  (ptr) == &mons[PM_FLOATING_EYE]) ? 1 : 2)
 #define nohands(ptr)		(((ptr)->mflags1 & M1_NOHANDS) != 0L)
 #define nolimbs(ptr)		(((ptr)->mflags1 & M1_NOLIMBS) == M1_NOLIMBS)
+#define notake(ptr)		(((ptr)->mflags1 & M1_NOTAKE) != 0L)
 #define has_head(ptr)		(((ptr)->mflags1 & M1_NOHEAD) == 0L)
 #define is_whirly(ptr)		((ptr)->mlet == S_VORTEX || \
 				 (ptr) == &mons[PM_AIR_ELEMENTAL])
+#define is_silent(ptr)		((ptr)->msound == MS_SILENT)
 #define unsolid(ptr)		(((ptr)->mflags1 & M1_UNSOLID) != 0L)
 #define mindless(ptr)		(((ptr)->mflags1 & M1_MINDLESS) != 0L)
 #define humanoid(ptr)		(((ptr)->mflags1 & M1_HUMANOID) != 0L)
 #define is_animal(ptr)		(((ptr)->mflags1 & M1_ANIMAL) != 0L)
 #define slithy(ptr)		(((ptr)->mflags1 & M1_SLITHY) != 0L)
-#define is_meaty(ptr)		(!amorphous(ptr) && \
-				 (ptr)->mlet != S_BLOB && \
-				 (ptr)->mlet != S_JELLY && \
-				 (ptr)->mlet != S_FUNGUS)
 #define is_wooden(ptr)		((ptr) == &mons[PM_WOOD_GOLEM])
 #define thick_skinned(ptr)	(((ptr)->mflags1 & M1_THICK_HIDE) != 0L)
 #define lays_eggs(ptr)		(((ptr)->mflags1 & M1_OVIPAROUS) != 0L)
@@ -59,7 +56,8 @@
 #define can_teleport(ptr)	(((ptr)->mflags1 & M1_TPORT) != 0L)
 #define control_teleport(ptr)	(((ptr)->mflags1 & M1_TPORT_CNTRL) != 0L)
 #define telepathic(ptr)		((ptr) == &mons[PM_FLOATING_EYE] || \
-				 (ptr) == &mons[PM_MIND_FLAYER])
+				 (ptr) == &mons[PM_MIND_FLAYER] || \
+				 (ptr) == &mons[PM_MASTER_MIND_FLAYER])
 #define is_armed(ptr)		attacktype(ptr, AT_WEAP)
 #define acidic(ptr)		(((ptr)->mflags1 & M1_ACID) != 0L)
 #define poisonous(ptr)		(((ptr)->mflags1 & M1_POIS) != 0L)
@@ -71,11 +69,17 @@
 #define is_were(ptr)		(((ptr)->mflags2 & M2_WERE) != 0L)
 #define is_elf(ptr)		(((ptr)->mflags2 & M2_ELF) != 0L)
 #define is_dwarf(ptr)		(((ptr)->mflags2 & M2_DWARF) != 0L)
+#define is_gnome(ptr)		(((ptr)->mflags2 & M2_GNOME) != 0L)
+#define is_orc(ptr)		(((ptr)->mflags2 & M2_ORC) != 0L)
+#define is_human(ptr)		(((ptr)->mflags2 & M2_HUMAN) != 0L)
+#define your_race(ptr)		(((ptr)->mflags2 & urace.selfmask) != 0L)
+#define is_bat(ptr)		((ptr) == &mons[PM_BAT] || \
+				 (ptr) == &mons[PM_GIANT_BAT] || \
+				 (ptr) == &mons[PM_VAMPIRE_BAT])
+#define is_bird(ptr)		((ptr)->mlet == S_BAT && !is_bat(ptr))
 #define is_giant(ptr)		(((ptr)->mflags2 & M2_GIANT) != 0L)
 #define is_golem(ptr)		((ptr)->mlet == S_GOLEM)
 #define is_domestic(ptr)	(((ptr)->mflags2 & M2_DOMESTIC) != 0L)
-#define is_orc(ptr)		(((ptr)->mflags2 & M2_ORC) != 0L)
-#define is_human(ptr)		(((ptr)->mflags2 & M2_HUMAN) != 0L)
 #define is_demon(ptr)		(((ptr)->mflags2 & M2_DEMON) != 0L)
 #define is_mercenary(ptr)	(((ptr)->mflags2 & M2_MERC) != 0L)
 #define is_male(ptr)		(((ptr)->mflags2 & M2_MALE) != 0L)
@@ -84,10 +88,13 @@
 #define is_wanderer(ptr)	(((ptr)->mflags2 & M2_WANDER) != 0L)
 #define always_hostile(ptr)	(((ptr)->mflags2 & M2_HOSTILE) != 0L)
 #define always_peaceful(ptr)	(((ptr)->mflags2 & M2_PEACEFUL) != 0L)
+#define race_hostile(ptr)	(((ptr)->mflags2 & urace.hatemask) != 0L)
+#define race_peaceful(ptr)	(((ptr)->mflags2 & urace.lovemask) != 0L)
 #define extra_nasty(ptr)	(((ptr)->mflags2 & M2_NASTY) != 0L)
 #define strongmonst(ptr)	(((ptr)->mflags2 & M2_STRONG) != 0L)
 #define can_breathe(ptr)	attacktype(ptr, AT_BREA)
 #define cantwield(ptr)		(nohands(ptr) || verysmall(ptr))
+#define could_twoweap(ptr)	((ptr)->mattk[1].aatyp == AT_WEAP)
 #define cantweararm(ptr)	(breakarm(ptr) || sliparm(ptr))
 #define throws_rocks(ptr)	(((ptr)->mflags2 & M2_ROCKTHROW) != 0L)
 #define type_is_pname(ptr)	(((ptr)->mflags2 & M2_PNAME) != 0L)
@@ -105,33 +112,73 @@
 #define likes_objs(ptr)		(((ptr)->mflags2 & M2_COLLECT) != 0L || \
 				 is_armed(ptr))
 #define likes_magic(ptr)	(((ptr)->mflags2 & M2_MAGIC) != 0L)
+#define webmaker(ptr)		((ptr) == &mons[PM_CAVE_SPIDER] || \
+				 (ptr) == &mons[PM_GIANT_SPIDER])
+#define is_unicorn(ptr)		((ptr)->mlet == S_UNICORN && likes_gems(ptr))
 #define is_longworm(ptr)	(((ptr) == &mons[PM_BABY_LONG_WORM]) || \
 				 ((ptr) == &mons[PM_LONG_WORM]) || \
 				 ((ptr) == &mons[PM_LONG_WORM_TAIL]))
 #define is_covetous(ptr)	((ptr->mflags3 & M3_COVETOUS))
+#define infravision(ptr)	((ptr->mflags3 & M3_INFRAVISION))
+#define infravisible(ptr)	((ptr->mflags3 & M3_INFRAVISIBLE))
 #define is_mplayer(ptr)		(((ptr) >= &mons[PM_ARCHEOLOGIST]) && \
 				 ((ptr) <= &mons[PM_WIZARD]))
 #define is_rider(ptr)		((ptr) == &mons[PM_DEATH] || \
 				 (ptr) == &mons[PM_FAMINE] || \
 				 (ptr) == &mons[PM_PESTILENCE])
-#define is_placeholder(ptr)     ((ptr) == &mons[PM_ORC] || \
+#define is_placeholder(ptr)	((ptr) == &mons[PM_ORC] || \
 				 (ptr) == &mons[PM_GIANT] || \
+				 (ptr) == &mons[PM_ELF] || \
 				 (ptr) == &mons[PM_HUMAN])
+/* return TRUE if the monster tends to revive */
+#define is_reviver(ptr)		(is_rider(ptr) || (ptr)->mlet == S_TROLL)
+
 /* this returns the light's range, or 0 if none; if we add more light emitting
    monsters, we'll likely have to add a new light range field to mons[] */
 #define emits_light(ptr)	(((ptr)->mlet == S_LIGHT || \
+				  (ptr) == &mons[PM_FLAMING_SPHERE] || \
+				  (ptr) == &mons[PM_SHOCKING_SPHERE] || \
 				  (ptr) == &mons[PM_FIRE_VORTEX]) ? 1 : \
 				 ((ptr) == &mons[PM_FIRE_ELEMENTAL]) ? 1 : 0)
 /*	[note: the light ranges above were reduced to 1 for performance...] */
 #define likes_lava(ptr)		(ptr == &mons[PM_FIRE_ELEMENTAL] || \
 				 ptr == &mons[PM_SALAMANDER])
-#define pm_invisible(ptr) ((ptr)->mlet == S_STALKER || \
+#define pm_invisible(ptr) ((ptr) == &mons[PM_STALKER] || \
 			   (ptr) == &mons[PM_BLACK_LIGHT])
 
 /* could probably add more */
 #define likes_fire(ptr)		((ptr) == &mons[PM_FIRE_VORTEX] || \
+				  (ptr) == &mons[PM_FLAMING_SPHERE] || \
 				 likes_lava(ptr))
 
-#define nonliving(ptr)		(is_golem(ptr) || is_undead(ptr))
+#define touch_petrifies(ptr)	((ptr) == &mons[PM_COCKATRICE] || \
+				 (ptr) == &mons[PM_CHICKATRICE])
+
+#define is_mind_flayer(ptr)	((ptr) == &mons[PM_MIND_FLAYER] || \
+				 (ptr) == &mons[PM_MASTER_MIND_FLAYER])
+
+#define nonliving(ptr)		(is_golem(ptr) || is_undead(ptr) || \
+				 (ptr)->mlet == S_VORTEX)
+
+/* Used for conduct with corpses, tins, and digestion attacks */
+/* G_NOCORPSE monsters might still be swallowed as a purple worm */
+/* Maybe someday this could be in mflags... */
+#define vegan(ptr)		((ptr)->mlet == S_BLOB || \
+				 (ptr)->mlet == S_JELLY ||            \
+				 (ptr)->mlet == S_FUNGUS ||           \
+				 (ptr)->mlet == S_VORTEX ||           \
+				 (ptr)->mlet == S_LIGHT ||            \
+				((ptr)->mlet == S_ELEMENTAL &&        \
+				 (ptr) != &mons[PM_STALKER]) ||       \
+				((ptr)->mlet == S_GOLEM &&            \
+				 (ptr) != &mons[PM_FLESH_GOLEM] &&    \
+				 (ptr) != &mons[PM_LEATHER_GOLEM]) || \
+				 noncorporeal(ptr))
+#define vegetarian(ptr)		(vegan(ptr) || \
+				((ptr)->mlet == S_PUDDING &&         \
+				 (ptr) != &mons[PM_BLACK_PUDDING]))
+
+#define befriend_with_obj(ptr, obj) ((obj)->oclass == FOOD_CLASS && \
+				     is_domestic(ptr))
 
 #endif /* MONDATA_H */
