@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)pcconf.h	3.2	95/10/11	*/
+/*	SCCS Id: @(#)pcconf.h	3.4	1995/10/11	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -17,6 +17,7 @@
  *     _MSC_VER is defined automatically by Microsoft C.
  *     __BORLANDC__ is defined automatically by Borland C.
  *     __SC__ is defined automatically by Symantec C.
+ *	Note: 3.4.0 was not verified with Symantec C.
  */
 
 /*
@@ -28,50 +29,45 @@
  *  For pre-V7.0 Microsoft Compilers only, manually define OVERLAY here.
  */
 
-/*#define OVERLAY	/* Manual overlay definition (MSC 6.0ax only) */
-
+/*#define OVERLAY */	/* Manual overlay definition (MSC 6.0ax only) */
 
 # ifndef __GO32__
-#  define MFLOPPY	/* Support for floppy drives and ramdisks by dgk */
+#define MFLOPPY		/* Support for floppy drives and ramdisks by dgk */
 # endif
 
 # define SHELL		/* via exec of COMMAND.COM */
 
-/*
- * Music option
- */
-
 # ifdef __BORLANDC__
-#define PCMUSIC		/* enable very basic pc speaker music notes */
+#define PCMUSIC		/* Music option, enable very basic pc speaker music notes */
 # endif
 
 /*
  * Screen control options
  *
  * You may uncomment:
- *                     ANSI_DEFAULT
- *                or   TERMLIB
- *                or   ANSI_DEFAULT and TERMLIB
- *                or   NO_TERMS
+ *		       ANSI_DEFAULT
+ *		  or   TERMLIB
+ *		  or   ANSI_DEFAULT and TERMLIB
+ *		  or   NO_TERMS
  */
 
-/* # define TERMLIB        /* enable use of termcap file /etc/termcap */
+/* # define TERMLIB */	   /* enable use of termcap file /etc/termcap */
 			/* or ./termcap for MSDOS (SAC) */
 			/* compile and link in Fred Fish's termcap library, */
 			/* enclosed in TERMCAP.ARC, to use this */
 
-/* # define ANSI_DEFAULT    /* allows NetHack to run without a ./termcap */
+/* # define ANSI_DEFAULT */   /* allows NetHack to run without a ./termcap */
 
 # define NO_TERMS	/* Allows Nethack to run without ansi.sys by linking */
 			/* screen routines into the .exe     */
 
 # ifdef NO_TERMS	/* if NO_TERMS select one screen package below */
 #define SCREEN_BIOS		/* Use bios calls for all screen control */
-/* #define SCREEN_DJGPPFAST	/* Use djgpp fast screen routines       */
+/* #define SCREEN_DJGPPFAST */	/* Use djgpp fast screen routines	*/
 # endif
 
 
-/* # define PC9800	/* Allows NetHack to run on NEC PC-9800 machines */
+/* # define PC9800 */	/* Allows NetHack to run on NEC PC-9800 machines */
 			/* Yamamoto Keizo */
 
 
@@ -81,13 +77,21 @@
  * You may uncomment any/all of the options below.
  *
  */
-# if (defined(SCREEN_BIOS) || defined(SCREEN_DJGPPFAST)) && !defined(PC9800)
+# ifndef SUPPRESS_GRAPHICS
+#  if (defined(SCREEN_BIOS) || defined(SCREEN_DJGPPFAST)) && !defined(PC9800)
 #   ifdef USE_TILES
-#define SCREEN_VGA	/* Include VGA    graphics routines in the build */
+#define SCREEN_VGA	/* Include VGA	  graphics routines in the build */
 #   endif
+#  endif
+# else
+# undef NO_TERMS
+# undef SCREEN_BIOS
+# undef SCREEN_DJGPPFAST
+# undef SCREEN_VGA
+# undef TERMLIB
+# define ANSI_DEFAULT
 # endif
 
-
 # define RANDOM		/* have Berkeley random(3) */
 
 # define MAIL		/* Allows for fake mail daemon to deliver mail */
@@ -95,6 +99,10 @@
 			/* amiconf.h).	In the future this will be the */
 			/* hook for mail reader implementation.        */
 
+/*# define PC_LOCKING */	/* Allow confirmation before overwriting game  */
+			/* that is in progress or aborted when another */
+			/* game is started with the same player name.  */
+
 /* The following is needed for prototypes of certain functions */
 
 #if defined(_MSC_VER) || defined(__BORLANDC__) || defined(__SC__)
@@ -128,21 +136,23 @@
 
 #define TIMED_DELAY	/* enable the `timed_delay' run-time option */
 
-#ifdef PCMUSIC
+# ifdef PCMUSIC
 #define TIMED_DELAY	/* need it anyway */
-#endif
-
+# endif
+#define NOCWD_ASSUMPTIONS	/* Allow paths to be specified for HACKDIR,
+				   LEVELDIR, SAVEDIR, BONESDIR, DATADIR,
+				   SCOREDIR, LOCKDIR, and CONFIGDIR */
 
 #endif /* MSDOS configuration stuff */
 
-
 #define PATHLEN		64	/* maximum pathlength */
 #define FILENAME	80	/* maximum filename length (conservative) */
 #ifndef MICRO_H
 #include "micro.h"		/* contains necessary externs for [os_name].c */
 #endif
 
-/*
+
+/* ===================================================
  *  The remaining code shouldn't need modification.
  */
 
@@ -150,36 +160,52 @@
 #include "system.h"
 #endif
 
+#ifdef __GO32__
+#include <unistd.h> /* close(), etc. */
+/* setmode is in io.h but lock() in io.h interferes with lock[] in decl.h */
+extern int FDECL(setmode, (int,int));
+#include <pc.h> /* kbhit() */
+#define PC_LOCKING
+#endif
+
 # ifdef MSDOS
 #  ifndef EXEPATH
 #define EXEPATH		/* HACKDIR is .exe location if not explicitly defined */
 #  endif
 # endif
 
-# if defined(_MSC_VER) && (_MSC_VER >= 700)
-#  ifndef MOVERLAY
+# if defined(_MSC_VER) && defined(MSDOS)
+#  if (_MSC_VER >= 700) && !defined(FUNCTION_LEVEL_LINKING)
+#   ifndef MOVERLAY
 #define MOVERLAY	/* Microsoft's MOVE overlay system (MSC >= 7.0) */
+#   endif
 #  endif
+#define PC_LOCKING
 # endif
 
+/* Borland Stuff */
+# if defined(__BORLANDC__)
+#  if defined(__OVERLAY__) && !defined(VROOMM)
 /* __OVERLAY__ is automatically defined by Borland C if overlay option is on */
-# if defined(__BORLANDC__) && defined(__OVERLAY__)
-#  ifndef VROOMM
 #define VROOMM		/* Borland's VROOMM overlay system */
 #  endif
+#  if !defined(STKSIZ)
+#define STKSIZ	5*1024	/* Use a default of 5K stack for Borland C	*/
+			/* This macro is used in any file that contains */
+			/* a main() function.				*/
+#  endif
+#define PC_LOCKING
 # endif
 
-# if defined (__BORLANDC__) && !defined(STKSIZ)
-#define STKSIZ	5*1024	/* Use a default of 5K stack for Borland C	*/
-			/* This macro is used in any file that contains	*/
-			/* a main() function.				*/
-# endif
+#ifdef PC_LOCKING
+#define HLOCK "NHPERM"
+#endif
 
 #ifndef index
 # define index	strchr
 #endif
 #ifndef rindex
-# define rindex	strrchr
+# define rindex strrchr
 #endif
 
 #ifndef AMIGA
@@ -188,19 +214,19 @@
 
 #ifdef RANDOM
 /* Use the high quality random number routines. */
-# define Rand()	random()
+# define Rand() random()
 #else
-# define Rand()	rand()
+# define Rand() rand()
 #endif
 
 #ifndef TOS
-# define FCMASK	0660	/* file creation mask */
+# define FCMASK 0660	/* file creation mask */
 #endif
 
 #include <fcntl.h>
 
 #ifndef REDO
-# undef	Getchar
+# undef Getchar
 # define Getchar nhgetch
 #endif
 
@@ -213,13 +239,22 @@
 /* Sanity check, do not modify these blocks. */
 
 /* OVERLAY must be defined with MOVERLAY or VROOMM */
-#if defined(MOVERLAY) || defined(VROOMM)
+#if (defined(MOVERLAY) || defined(VROOMM))
 # ifndef OVERLAY
 #  define OVERLAY
 # endif
 #endif
 
-#if defined(OVERLAY) && !defined(MOVERLAY) && !defined(VROOMM)
+#if defined(FUNCTION_LEVEL_LINKING)
+#define OVERLAY
+#define OVL0
+#define OVL1
+#define OVL2
+#define OVL3
+#define OVLB
+#endif
+
+#if defined(OVERLAY) && !defined(MOVERLAY) && !defined(VROOMM) && !defined(FUNCTION_LEVEL_LINKING)
 #define USE_TRAMPOLI
 #endif
 
@@ -248,10 +283,10 @@
 #    if defined(_MSC_VER) || defined(__SC__)
 #    pragma message("           Forcing undef of SCREEN_DJGPPFAST")
 #    endif
-#undef SCREEN_DJGPPFAST	  /* Can't use djgpp fast with other compilers anyway */
+#undef SCREEN_DJGPPFAST   /* Can't use djgpp fast with other compilers anyway */
 #   endif
 #  else
-/* djgpp C compiler     */
+/* djgpp C compiler	*/
 #   if defined(SCREEN_BIOS)
 #undef SCREEN_BIOS
 #   endif
@@ -278,7 +313,7 @@
 #endif			/* End of sanity check block */
 
 #if defined(MSDOS) && defined(DLB)
-#define FILENAME_CMP  stricmp                 /* case insensitive */
+#define FILENAME_CMP  stricmp		      /* case insensitive */
 #endif
 
 #ifdef MSC7_WARN	/* define with cl /DMSC7_WARN	*/
