@@ -1,13 +1,13 @@
-/*	SCCS Id: @(#)rect.h	3.2	90/02/22	*/
+/*	SCCS Id: @(#)rect.h	3.4	1990/02/22	*/
 /* Copyright (c) 1990 by Jean-Christophe Collet			  */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #ifndef RECT_H
 #define RECT_H
 
-typedef struct {
-	xchar lx,ly;
-	xchar hx,hy;
+typedef struct nhrect {
+	xchar lx, ly;
+	xchar hx, hy;
 } NhRect;
 
 #endif /* RECT_H */
