@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)artifact.c	3.2	96/01/15	*/
+/*	SCCS Id: @(#)artifact.c 3.4	2002/02/21	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -23,37 +23,53 @@ extern boolean notonhead;	/* for long worms */
 STATIC_DCL int FDECL(spec_applies, (const struct artifact *,struct monst *));
 STATIC_DCL int FDECL(arti_invoke, (struct obj*));
 
-/* The amount added to normal damage which should guarantee that the
+/* The amount added to the victim's total hit points to insure that the
    victim will be killed even after damage bonus/penalty adjustments.
-   It needs to be big enough so that halving will still kill, but not
-   so big that doubling ends up overflowing 15 bits.  This value used
-   to be 1234, but it was possible for players to accumulate enough
-   hit points so that taking (uhp + 1234)/2 damage was survivable. */
-#define FATAL_DAMAGE 9999
+   Most such penalties are small, and 200 is plenty; the exception is
+   half physical damage.  3.3.1 and previous versions tried to use a very
+   large number to account for this case; now, we just compute the fatal
+   damage by adding it to 2 times the total hit points instead of 1 time.
+   Note: this will still break if they have more than about half the number
+   of hit points that will fit in a 15 bit integer. */
+#define FATAL_DAMAGE_MODIFIER 200
 
 #ifndef OVLB
 STATIC_DCL int spec_dbon_applies;
-
+STATIC_DCL xchar artidisco[NROFARTIFACTS];
 #else	/* OVLB */
 /* coordinate effects from spec_dbon() with messages in artifact_hit() */
 STATIC_OVL int spec_dbon_applies = 0;
 
 /* flags including which artifacts have already been created */
 static boolean artiexist[1+NROFARTIFACTS+1];
+/* and a discovery list for them (no dummy first entry here) */
+STATIC_OVL xchar artidisco[NROFARTIFACTS];
 
-static void NDECL(hack_artifacts);
-static boolean FDECL(attacks, (int,struct obj *));
+STATIC_DCL void NDECL(hack_artifacts);
+STATIC_DCL boolean FDECL(attacks, (int,struct obj *));
 
 /* handle some special cases; must be called after u_init() */
-static void
+STATIC_OVL void
 hack_artifacts()
 {
+	struct artifact *art;
+	int alignmnt = aligns[flags.initalign].value;
+
+	/* Fix up the alignments of "gift" artifacts */
+	for (art = artilist+1; art->otyp; art++)
+	    if (art->role == Role_switch && art->alignment != A_NONE)
+		art->alignment = alignmnt;
+
 	/* Excalibur can be used by any lawful character, not just knights */
-	if (!Role_is('K'))
-	    artilist[ART_EXCALIBUR].class = '\0';
-	/* Mitre of Holiness has same alignment as priest starts out with */
-	if (Role_is('P'))
-	    artilist[ART_MITRE_OF_HOLINESS].alignment = u.ualignbase[1];
+	if (!Role_if(PM_KNIGHT))
+	    artilist[ART_EXCALIBUR].role = NON_PM;
+
+	/* Fix up the quest artifact */
+	if (urole.questarti) {
+	    artilist[urole.questarti].alignment = alignmnt;
+	    artilist[urole.questarti].role = Role_switch;
+	}
+	return;
 }
 
 /* zero out the artifact existence list */
@@ -61,6 +77,7 @@ void
 init_artifacts()
 {
 	(void) memset((genericptr_t) artiexist, 0, sizeof artiexist);
+	(void) memset((genericptr_t) artidisco, 0, sizeof artidisco);
 	hack_artifacts();
 }
 
@@ -69,6 +86,7 @@ save_artifacts(fd)
 int fd;
 {
 	bwrite(fd, (genericptr_t) artiexist, sizeof artiexist);
+	bwrite(fd, (genericptr_t) artidisco, sizeof artidisco);
 }
 
 void
@@ -76,6 +94,7 @@ restore_artifacts(fd)
 int fd;
 {
 	mread(fd, (genericptr_t) artiexist, sizeof artiexist);
+	mread(fd, (genericptr_t) artidisco, sizeof artidisco);
 	hack_artifacts();	/* redo non-saved special cases */
 }
 
@@ -90,7 +109,9 @@ int artinum;
 /*
    Make an artifact.  If a specific alignment is specified, then an object of
    the appropriate alignment is created from scratch, or 0 is returned if
-   none is available.  If no alignment is given, then 'otmp' is converted
+   none is available.  (If at least one aligned artifact has already been
+   given, then unaligned ones also become eligible for this.)
+   If no alignment is given, then 'otmp' is converted
    into an artifact of matching type, or returned as-is if that's not possible.
    For the 2nd case, caller should use ``obj = mk_artifact(obj, A_NONE);''
    for the 1st, ``obj = mk_artifact((struct obj *)0, some_alignment);''.
@@ -100,30 +121,30 @@ mk_artifact(otmp, alignment)
 struct obj *otmp;	/* existing object; ignored if alignment specified */
 aligntyp alignment;	/* target alignment, or A_NONE */
 {
-	register const struct artifact *a;
-	register int n = 0, m;
-	register boolean by_align = (alignment != A_NONE);
-	register short o_typ = (by_align || !otmp) ? 0 : otmp->otyp;
+	const struct artifact *a;
+	int n, m;
+	boolean by_align = (alignment != A_NONE);
+	short o_typ = (by_align || !otmp) ? 0 : otmp->otyp;
 	boolean unique = !by_align && otmp && objects[o_typ].oc_unique;
+	short eligible[NROFARTIFACTS];
 
-	/* count eligible artifacts */
-	for (a = artilist+1,m = 1; a->otyp; a++,m++)
-	    if ((by_align ? a->alignment == alignment : a->otyp == o_typ) &&
+	/* gather eligible artifacts */
+	for (n = 0, a = artilist+1, m = 1; a->otyp; a++, m++)
+	    if ((!by_align ? a->otyp == o_typ :
+		    (a->alignment == alignment ||
+			(a->alignment == A_NONE && u.ugifts > 0))) &&
 		(!(a->spfx & SPFX_NOGEN) || unique) && !artiexist[m]) {
-		if (by_align && a->class == u.role)
+		if (by_align && a->race != NON_PM && race_hostile(&mons[a->race]))
+		    continue;	/* skip enemies' equipment */
+		else if (by_align && Role_if(a->role))
 		    goto make_artif;	/* 'a' points to the desired one */
 		else
-		    n++;
+		    eligible[n++] = m;
 	    }
 
 	if (n) {		/* found at least one candidate */
-	    /* select one, then find it again */
-	    if (n > 1) n = rnd(n);	/* [1..n] */
-	    for (a = artilist+1,m = 1; a->otyp; a++,m++)
-		if ((by_align ? a->alignment == alignment : a->otyp == o_typ)&&
-		    (!(a->spfx & SPFX_NOGEN) || unique) && !artiexist[m]) {
-		    if (!--n) break;	/* stop when chosen one reached */
-		}
+	    m = eligible[rn2(n)];	/* [0..n-1] */
+	    a = &artilist[m];
 
 	    /* make an appropriate object if necessary, then christen it */
 make_artif: if (by_align) otmp = mksobj((int)a->otyp, TRUE, FALSE);
@@ -257,7 +278,7 @@ register const char *name;
 	return FALSE;
 }
 
-static boolean
+STATIC_OVL boolean
 attacks(adtyp, otmp)
 register int adtyp;
 register struct obj *otmp;
@@ -308,24 +329,26 @@ long wp_mask;
 	register const struct artifact *oart = get_artifact(otmp);
 	uchar dtyp;
 	long spfx;
-	
+
 	if (!oart) return;
 
 	/* effects from the defn field */
 	dtyp = (wp_mask != W_ART) ? oart->defn.adtyp : oart->cary.adtyp;
 
 	if (dtyp == AD_FIRE)
-	    mask = &HFire_resistance;
+	    mask = &EFire_resistance;
 	else if (dtyp == AD_COLD)
-	    mask = &HCold_resistance;
+	    mask = &ECold_resistance;
 	else if (dtyp == AD_ELEC)
-	    mask = &HShock_resistance;
+	    mask = &EShock_resistance;
 	else if (dtyp == AD_MAGM)
-	    mask = &HAntimagic;
+	    mask = &EAntimagic;
 	else if (dtyp == AD_DISN)
-	    mask = &HDisint_resistance;
+	    mask = &EDisint_resistance;
+	else if (dtyp == AD_DRST)
+	    mask = &EPoison_resistance;
 
-	if(mask && wp_mask == W_ART && !on) {
+	if (mask && wp_mask == W_ART && !on) {
 	    /* find out if some other artifact also confers this intrinsic */
 	    /* if so, leave the mask alone */
 	    register struct obj* obj;
@@ -338,7 +361,7 @@ long wp_mask;
 		    }
 		}
 	}
-	if(mask) {
+	if (mask) {
 	    if (on) *mask |= wp_mask;
 	    else *mask &= ~wp_mask;
 	}
@@ -356,52 +379,76 @@ long wp_mask;
 	}
 
 	if (spfx & SPFX_SEARCH) {
-	    if(on) Searching |= wp_mask;
-	    else Searching &= ~wp_mask;
+	    if(on) ESearching |= wp_mask;
+	    else ESearching &= ~wp_mask;
 	}
 	if (spfx & SPFX_HALRES) {
 	    /* make_hallucinated must (re)set the mask itself to get
 	     * the display right */
-	    make_hallucinated((long)!on, TRUE, wp_mask);
+	    /* restoring needed because this is the only artifact intrinsic
+	     * that can print a message--need to guard against being printed
+	     * when restoring a game
+	     */
+	    make_hallucinated((long)!on, restoring ? FALSE : TRUE, wp_mask);
 	}
 	if (spfx & SPFX_ESP) {
-	    if(on) HTelepat |= wp_mask;
-	    else HTelepat &= ~wp_mask;
+	    if(on) ETelepat |= wp_mask;
+	    else ETelepat &= ~wp_mask;
 	    see_monsters();
 	}
 	if (spfx & SPFX_STLTH) {
-	    if (on) Stealth |= wp_mask;
-	    else Stealth &= ~wp_mask;
+	    if (on) EStealth |= wp_mask;
+	    else EStealth &= ~wp_mask;
 	}
 	if (spfx & SPFX_REGEN) {
-	    if (on) HRegeneration |= wp_mask;
-	    else HRegeneration &= ~wp_mask;
+	    if (on) ERegeneration |= wp_mask;
+	    else ERegeneration &= ~wp_mask;
 	}
 	if (spfx & SPFX_TCTRL) {
-	    if (on) HTeleport_control |= wp_mask;
-	    else HTeleport_control &= ~wp_mask;
+	    if (on) ETeleport_control |= wp_mask;
+	    else ETeleport_control &= ~wp_mask;
 	}
 	if (spfx & SPFX_WARN) {
-	    if (on) Warning |= wp_mask;
-	    else Warning &= ~wp_mask;
+	    if (spec_m2(otmp)) {
+	    	if (on) {
+			EWarn_of_mon |= wp_mask;
+			flags.warntype |= spec_m2(otmp);
+	    	} else {
+			EWarn_of_mon &= ~wp_mask;
+	    		flags.warntype &= ~spec_m2(otmp);
+		}
+		see_monsters();
+	    } else {
+		if (on) EWarning |= wp_mask;
+	    	else EWarning &= ~wp_mask;
+	    }
 	}
 	if (spfx & SPFX_EREGEN) {
-	    if (on) Energy_regeneration |= wp_mask;
-	    else Energy_regeneration &= ~wp_mask;
+	    if (on) EEnergy_regeneration |= wp_mask;
+	    else EEnergy_regeneration &= ~wp_mask;
 	}
 	if (spfx & SPFX_HSPDAM) {
-	    if (on) Half_spell_damage |= wp_mask;
-	    else Half_spell_damage &= ~wp_mask;
+	    if (on) EHalf_spell_damage |= wp_mask;
+	    else EHalf_spell_damage &= ~wp_mask;
 	}
 	if (spfx & SPFX_HPHDAM) {
-	    if (on) Half_physical_damage |= wp_mask;
-	    else Half_physical_damage &= ~wp_mask;
+	    if (on) EHalf_physical_damage |= wp_mask;
+	    else EHalf_physical_damage &= ~wp_mask;
+	}
+	if (spfx & SPFX_XRAY) {
+	    /* this assumes that no one else is using xray_range */
+	    if (on) u.xray_range = 3;
+	    else u.xray_range = -1;
+	}
+	if ((spfx & SPFX_REFLECT) && (wp_mask & W_WEP)) {
+	    if (on) EReflecting |= wp_mask;
+	    else EReflecting &= ~wp_mask;
 	}
 
 	if(wp_mask == W_ART && !on && oart->inv_prop) {
 	    /* might have to turn off invoked power too */
 	    if (oart->inv_prop <= LAST_PROP &&
-		(u.uprops[oart->inv_prop].p_flgs & W_ARTI))
+		(u.uprops[oart->inv_prop].extrinsic & W_ARTI))
 		(void) arti_invoke(otmp);
 	}
 }
@@ -427,14 +474,18 @@ touch_artifact(obj,mon)
     /* all quest artifacts are self-willed; it this ever changes, `badclass'
        will have to be extended to explicitly include quest artifacts */
     self_willed = ((oart->spfx & SPFX_INTEL) != 0);
-    if (yours || !(mon->data->mflags3 & M3_WANTSALL)) {
-	badclass = (oart->class && self_willed &&
-		    (!yours || oart->class != u.role));
-	badalign = (oart->spfx & SPFX_RESTR) &&
-	    ((oart->alignment !=
-	      (yours ? u.ualign.type : sgn(mon->data->maligntyp))) ||
-	     (yours && u.ualign.record < 0));
-    } else {	/* an M3_WANTSxxx monster */
+    if (yours) {
+	badclass = self_willed &&
+		   ((oart->role != NON_PM && !Role_if(oart->role)) ||
+		    (oart->race != NON_PM && !Race_if(oart->race)));
+	badalign = (oart->spfx & SPFX_RESTR) && oart->alignment != A_NONE &&
+		   (oart->alignment != u.ualign.type || u.ualign.record < 0);
+    } else if (!is_covetous(mon->data) && !is_mplayer(mon->data)) {
+	badclass = self_willed &&
+		   oart->role != NON_PM && oart != &artilist[ART_EXCALIBUR];
+	badalign = (oart->spfx & SPFX_RESTR) && oart->alignment != A_NONE &&
+		   (oart->alignment != sgn(mon->data->maligntyp));
+    } else {    /* an M3_WANTSxxx monster or a fake player */
 	/* special monsters trying to take the Amulet, invocation tools or
 	   quest item can touch anything except for `spec_applies' artifacts */
 	badclass = badalign = FALSE;
@@ -464,7 +515,7 @@ touch_artifact(obj,mon)
 
     /* can pick it up unless you're totally non-synch'd with the artifact */
     if (badclass && badalign && self_willed) {
-	if (yours) pline("%s evades your grasp!", The(xname(obj)));
+	if (yours) pline("%s your grasp!", Tobjnam(obj, "evade"));
 	return 0;
     }
 
@@ -474,7 +525,7 @@ touch_artifact(obj,mon)
 #endif /* OVLB */
 #ifdef OVL1
 
-/* decide whether an artifact's special attacks apply against `ptr' */
+/* decide whether an artifact's special attacks apply against mtmp */
 STATIC_OVL int
 spec_applies(weap, mtmp)
 register const struct artifact *weap;
@@ -490,27 +541,14 @@ struct monst *mtmp;
 	ptr = mtmp->data;
 
 	if (weap->spfx & SPFX_DMONS) {
-#if 0	/* (we don't have anything like "PlayerBane" or "HealerSmasher" :-) */
-	    if (weap->mtype == PM_PLAYERMON)
-		return yours;
-	    else if (player_mon() == &mons[(int)weap->mtype])
-		return 1;
-#endif
 	    return (ptr == &mons[(int)weap->mtype]);
 	} else if (weap->spfx & SPFX_DCLAS) {
 	    return (weap->mtype == (unsigned long)ptr->mlet);
 	} else if (weap->spfx & SPFX_DFLAG1) {
 	    return ((ptr->mflags1 & weap->mtype) != 0L);
 	} else if (weap->spfx & SPFX_DFLAG2) {
-	    if (yours) {
-		if ((weap->mtype & M2_HUMAN) != 0 &&
-		    (u.mtimedone ? is_human(uasmon) : human_role()))
-			return 1;
-		if ((weap->mtype & M2_ELF) != 0 &&
-		    (u.mtimedone ? is_elf(uasmon) : Role_is('E')))
-			return 1;
-	    }
-	    return ((ptr->mflags2 & weap->mtype) != 0L);
+	    return ((ptr->mflags2 & weap->mtype) ||
+		(yours && !Upolyd && (urace.selfmask & weap->mtype)));
 	} else if (weap->spfx & SPFX_DALIGN) {
 	    return yours ? (u.ualign.type != weap->alignment) :
 			   (ptr->maligntyp == A_NONE ||
@@ -531,16 +569,29 @@ struct monst *mtmp;
 		case AD_MAGM:
 		case AD_STUN:
 			return !(yours ? Antimagic : (rn2(100) < ptr->mr));
+		case AD_DRST:
+			return !(yours ? Poison_resistance : resists_poison(mtmp));
 		case AD_DRLI:
-			return !resists_drli(mtmp);
+			return !(yours ? Drain_resistance : resists_drli(mtmp));
 		case AD_STON:
-			return !resists_ston(mtmp);
+			return !(yours ? Stone_resistance : resists_ston(mtmp));
 		default:	impossible("Weird weapon special attack.");
 	    }
 	}
 	return(0);
 }
 
+/* return the M2 flags of monster that an artifact's special attacks apply against */
+long
+spec_m2(otmp)
+struct obj *otmp;
+{
+	register const struct artifact *artifact = get_artifact(otmp);
+	if (artifact)
+		return artifact->mtype;
+	return 0L;
+}
+
 /* special attack bonus */
 int
 spec_abon(otmp, mon)
@@ -549,8 +600,11 @@ struct monst *mon;
 {
 	register const struct artifact *weap = get_artifact(otmp);
 
-	if (weap && spec_applies(weap, mon))
-	    return weap->attk.damn ? rnd((int)weap->attk.damn) : 0;
+	/* no need for an extra check for `NO_ATTK' because this will
+	   always return 0 for any artifact which has that attribute */
+
+	if (weap && weap->attk.damn && spec_applies(weap, mon))
+	    return rnd((int)weap->attk.damn);
 	return 0;
 }
 
@@ -563,11 +617,73 @@ int tmp;
 {
 	register const struct artifact *weap = get_artifact(otmp);
 
-	if ((spec_dbon_applies = (weap && spec_applies(weap, mon))) != 0)
+	if (!weap || (weap->attk.adtyp == AD_PHYS && /* check for `NO_ATTK' */
+			weap->attk.damn == 0 && weap->attk.damd == 0))
+	    spec_dbon_applies = FALSE;
+	else
+	    spec_dbon_applies = spec_applies(weap, mon);
+
+	if (spec_dbon_applies)
 	    return weap->attk.damd ? rnd((int)weap->attk.damd) : max(tmp,1);
 	return 0;
 }
 
+/* add identified artifact to discoveries list */
+void
+discover_artifact(m)
+xchar m;
+{
+    int i;
+
+    /* look for this artifact in the discoveries list;
+       if we hit an empty slot then it's not present, so add it */
+    for (i = 0; i < NROFARTIFACTS; i++)
+	if (artidisco[i] == 0 || artidisco[i] == m) {
+	    artidisco[i] = m;
+	    return;
+	}
+    /* there is one slot per artifact, so we should never reach the
+       end without either finding the artifact or an empty slot... */
+    impossible("couldn't discover artifact (%d)", (int)m);
+}
+
+/* used to decide whether an artifact has been fully identified */
+boolean
+undiscovered_artifact(m)
+xchar m;
+{
+    int i;
+
+    /* look for this artifact in the discoveries list;
+       if we hit an empty slot then it's undiscovered */
+    for (i = 0; i < NROFARTIFACTS; i++)
+	if (artidisco[i] == m)
+	    return FALSE;
+	else if (artidisco[i] == 0)
+	    break;
+    return TRUE;
+}
+
+/* display a list of discovered artifacts; return their count */
+int
+disp_artifact_discoveries(tmpwin)
+winid tmpwin;		/* supplied by dodiscover() */
+{
+    int i, m, otyp;
+    char buf[BUFSZ];
+
+    for (i = 0; i < NROFARTIFACTS; i++) {
+	if (artidisco[i] == 0) break;	/* empty slot implies end of list */
+	if (i == 0) putstr(tmpwin, ATR_INVERSE, "Artifacts");
+	m = artidisco[i];
+	otyp = artilist[m].otyp;
+	Sprintf(buf, "  %s [%s %s]", artiname(m),
+		align_str(artilist[m].alignment), simple_typename(otyp));
+	putstr(tmpwin, 0, buf);
+    }
+    return i;
+}
+
 #endif /* OVL1 */
 
 #ifdef OVLB
@@ -592,9 +708,11 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 	boolean vis = (!youattack && magr && cansee(magr->mx, magr->my))
 		|| (!youdefend && cansee(mdef->mx, mdef->my));
 	boolean realizes_damage;
-
+	const char *wepdesc;
 	static const char you[] = "you";
-	const char *hittee = youdefend ? you : mon_nam(mdef);
+	char hittee[BUFSZ];
+
+	Strcpy(hittee, youdefend ? you : mon_nam(mdef));
 
 	/* The following takes care of most of the damage, but not all--
 	 * the exception being for level draining, which is specially
@@ -603,43 +721,61 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 	*dmgptr += spec_dbon(otmp, mdef, *dmgptr);
 
 	if (youattack && youdefend) {
-		impossible("attacking yourself with weapon?");
-		return FALSE;
-	} else if (!spec_dbon_applies) {
-		/* since damage bonus didn't apply, nothing more to do */
-		return FALSE;
+	    impossible("attacking yourself with weapon?");
+	    return FALSE;
 	}
 
 	realizes_damage = (youdefend || vis);
 
 	/* the four basic attacks: fire, cold, shock and missiles */
 	if (attacks(AD_FIRE, otmp)) {
-		if (realizes_damage) {
-			pline_The("fiery blade burns %s!", hittee);
-			return TRUE;
-		}
+	    if (realizes_damage)
+		pline_The("fiery blade %s %s%c",
+			!spec_dbon_applies ? "hits" :
+			(mdef->data == &mons[PM_WATER_ELEMENTAL]) ?
+			"vaporizes part of" : "burns",
+			hittee, !spec_dbon_applies ? '.' : '!');
+	    if (!rn2(4)) (void) destroy_mitem(mdef, POTION_CLASS, AD_FIRE);
+	    if (!rn2(4)) (void) destroy_mitem(mdef, SCROLL_CLASS, AD_FIRE);
+	    if (!rn2(7)) (void) destroy_mitem(mdef, SPBOOK_CLASS, AD_FIRE);
+	    if (youdefend && Slimed) burn_away_slime();
+	    return realizes_damage;
 	}
 	if (attacks(AD_COLD, otmp)) {
-		if (realizes_damage) {
-			pline_The("ice-cold blade freezes %s!", hittee);
-			return TRUE;
-		}
+	    if (realizes_damage)
+		pline_The("ice-cold blade %s %s%c",
+			!spec_dbon_applies ? "hits" : "freezes",
+			hittee, !spec_dbon_applies ? '.' : '!');
+	    if (!rn2(4)) (void) destroy_mitem(mdef, POTION_CLASS, AD_COLD);
+	    return realizes_damage;
 	}
 	if (attacks(AD_ELEC, otmp)) {
-		if (realizes_damage) {
-			if(youattack && otmp != uwep)
-			    pline("%s hits %s!", The(xname(otmp)), hittee);
-			pline("Lightning strikes %s!", hittee);
-			return TRUE;
-		}
+	    if (realizes_damage) {
+		if (youattack ? otmp != uwep : !spec_dbon_applies)
+		    pline("%s %s%c", Tobjnam(otmp, "hit"),
+			  hittee, !spec_dbon_applies ? '.' : '!');
+		if (spec_dbon_applies)
+		    pline("Lightning strikes %s!", hittee);
+	    }
+	    if (!rn2(5)) (void) destroy_mitem(mdef, RING_CLASS, AD_ELEC);
+	    if (!rn2(5)) (void) destroy_mitem(mdef, WAND_CLASS, AD_ELEC);
+	    return realizes_damage;
 	}
 	if (attacks(AD_MAGM, otmp)) {
-		if (realizes_damage) {
-			if(youattack && otmp != uwep)
-			    pline("%s hits %s!", The(xname(otmp)), hittee);
-			pline("A hail of magic missiles strikes %s!", hittee);
-			return TRUE;
-		}
+	    if (realizes_damage) {
+		if (youattack ? otmp != uwep : !spec_dbon_applies)
+		    pline("%s %s%c", Tobjnam(otmp, "hit"),
+			  hittee, !spec_dbon_applies ? '.' : '!');
+		if (spec_dbon_applies)
+		    pline("A hail of magic missiles strikes %s!", hittee);
+	    }
+	    return realizes_damage;
+	}
+
+	if (!spec_dbon_applies) {
+	    /* since damage bonus didn't apply, nothing more to do;  
+	       no further attacks have side-effects on inventory */
+	    return FALSE;
 	}
 
 	/*
@@ -692,8 +828,8 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 				else {
 					nomul(-3);
 					nomovemsg = "";
-					if ((magr == u.ustuck)
-						&& sticks(uasmon)) {
+					if (magr && magr == u.ustuck
+						&& sticks(youmonst.data)) {
 					    u.ustuck = (struct monst *)0;
 					    You("release %s!", mon_nam(magr));
 					}
@@ -702,24 +838,13 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 				if (rn2(2) && resist(mdef,SPBOOK_CLASS,0,0)) {
 				    MB_RESIST_ATTACK;
 				} else {
-				    if (mdef == u.ustuck) {
-					if (u.uswallow)
-					    expels(mdef,mdef->data,TRUE);
-					else {
-					    if (!sticks(uasmon)) {
-						u.ustuck = (struct monst *)0;
-						You("get released!");
-					    }
-					}
-				    }
-				    mdef->mflee = 1;
-				    mdef->mfleetim += 3;
+				    monflee(mdef, 3, FALSE, TRUE);
 				}
 			}
 		}
 		if (dieroll <= (scare_dieroll / 2)) {	/* purge */
 			struct obj *ospell;
-			struct permonst *old_uasmon = uasmon;
+			struct permonst *old_uasmon = youmonst.data;
 
 			attack_index = MB_INDEX_PURGE;
 			*dmgptr += rnd(4);		/* 6d4 */
@@ -732,7 +857,7 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 			cancel_monst(mdef, ospell, youattack, FALSE, FALSE);
 
 			if (youdefend) {
-				if (old_uasmon != uasmon)
+				if (old_uasmon != youmonst.data)
 					/* rehumanized, no more damage */
 					*dmgptr = 0;
 				if (Antimagic)
@@ -780,6 +905,8 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 				} else if (attack_index == MB_INDEX_PROBE) {
 				    if (!rn2(4 * otmp->spe)) {
 					pline_The("probe is insightful!");
+					if (!canspotmon(mdef))
+					    map_invisible(u.ux+u.dx,u.uy+u.dy);
 					/* pre-damage status */
 					probe_monster(mdef);
 				    }
@@ -813,10 +940,11 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 	/* reverse from AD&D. */
 	if (spec_ability(otmp, SPFX_BEHEAD)) {
 	    if (otmp->oartifact == ART_TSURUGI_OF_MURAMASA && dieroll == 1) {
+		wepdesc = "The razor-sharp blade";
 		/* not really beheading, but so close, why add another SPFX */
 		if (youattack && u.uswallow && mdef == u.ustuck) {
 		    You("slice %s wide open!", mon_nam(mdef));
-		    *dmgptr = mdef->mhp + FATAL_DAMAGE;
+		    *dmgptr = 2 * mdef->mhp + FATAL_DAMAGE_MODIFIER;
 		    return TRUE;
 		}
 		if (!youdefend) {
@@ -830,19 +958,18 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 						mon_nam(mdef));
 				else if (vis)
 					pline("%s cuts deeply into %s!",
-					      Monnam(magr), mon_nam(mdef));
+					      Monnam(magr), hittee);
 				*dmgptr *= 2;
 				return TRUE;
 			}
-			*dmgptr = mdef->mhp + FATAL_DAMAGE;
-			pline_The("razor-sharp blade cuts %s in half!",
-			      mon_nam(mdef));
+			*dmgptr = 2 * mdef->mhp + FATAL_DAMAGE_MODIFIER;
+			pline("%s cuts %s in half!", wepdesc, mon_nam(mdef));
 			otmp->dknown = TRUE;
 			return TRUE;
 		} else {
-			if (bigmonst(uasmon)) {
+			if (bigmonst(youmonst.data)) {
 				pline("%s cuts deeply into you!",
-					Monnam(magr));
+				      magr ? Monnam(magr) : wepdesc);
 				*dmgptr *= 2;
 				return TRUE;
 			}
@@ -852,8 +979,8 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 			 * value to the damage so that this reduction in
 			 * damage does not prevent death.
 			 */
-			*dmgptr = u.uhp + FATAL_DAMAGE;
-			pline_The("razor-sharp blade cuts you in half!");
+			*dmgptr = 2 * (Upolyd ? u.mh : u.uhp) + FATAL_DAMAGE_MODIFIER;
+			pline("%s cuts you in half!", wepdesc);
 			otmp->dknown = TRUE;
 			return TRUE;
 		}
@@ -866,6 +993,7 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 
 		if (youattack && u.uswallow && mdef == u.ustuck)
 			return FALSE;
+		wepdesc = artilist[ART_VORPAL_BLADE].name;
 		if (!youdefend) {
 			if (!has_head(mdef->data) || notonhead || u.uswallow) {
 				if (youattack)
@@ -878,32 +1006,32 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 				return ((boolean)(youattack || vis));
 			}
 			if (noncorporeal(mdef->data) || amorphous(mdef->data)) {
-				pline("%s slices through %s neck.",
-				      artilist[ART_VORPAL_BLADE].name,
-				      s_suffix(mon_nam(mdef)));
-				return ((boolean)(youattack || vis));
+				pline("%s slices through %s %s.", wepdesc,
+				      s_suffix(mon_nam(mdef)),
+				      mbodypart(mdef,NECK));
+				return TRUE;
 			}
-			*dmgptr = mdef->mhp + FATAL_DAMAGE;
+			*dmgptr = 2 * mdef->mhp + FATAL_DAMAGE_MODIFIER;
 			pline(behead_msg[rn2(SIZE(behead_msg))],
-			      artilist[ART_VORPAL_BLADE].name,
-			      mon_nam(mdef));
+			      wepdesc, mon_nam(mdef));
 			otmp->dknown = TRUE;
 			return TRUE;
 		} else {
-			if (!has_head(uasmon)) {
+			if (!has_head(youmonst.data)) {
 				pline("Somehow, %s misses you wildly.",
-					mon_nam(magr));
+				      magr ? mon_nam(magr) : wepdesc);
 				*dmgptr = 0;
 				return TRUE;
 			}
-			if (noncorporeal(uasmon) || amorphous(uasmon)) {
-				pline("%s slices through your neck.",
-				      artilist[ART_VORPAL_BLADE].name);
+			if (noncorporeal(youmonst.data) || amorphous(youmonst.data)) {
+				pline("%s slices through your %s.",
+				      wepdesc, body_part(NECK));
 				return TRUE;
 			}
-			*dmgptr = u.uhp + FATAL_DAMAGE;
+			*dmgptr = 2 * (Upolyd ? u.mh : u.uhp)
+				  + FATAL_DAMAGE_MODIFIER;
 			pline(behead_msg[rn2(SIZE(behead_msg))],
-			      artilist[ART_VORPAL_BLADE].name, "you");
+			      wepdesc, "you");
 			otmp->dknown = TRUE;
 			/* Should amulets fall off? */
 			return TRUE;
@@ -923,7 +1051,7 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 				      mon_nam(mdef));
 			}
 			if (mdef->m_lev == 0) {
-			    *dmgptr = mdef->mhp + FATAL_DAMAGE;
+			    *dmgptr = 2 * mdef->mhp + FATAL_DAMAGE_MODIFIER;
 			} else {
 			    int drain = rnd(8);
 			    *dmgptr += drain;
@@ -946,8 +1074,8 @@ int dieroll; /* needed for Magicbane and vorpal blades */
 			else
 				pline("%s drains your life!",
 				      The(distant_name(otmp, xname)));
-			losexp();
-			if (magr->mhp < magr->mhpmax) {
+			losexp("life drainage");
+			if (magr && magr->mhp < magr->mhpmax) {
 			    magr->mhp += (u.uhpmax - oldhpmax)/2;
 			    if (magr->mhp > magr->mhpmax) magr->mhp = magr->mhpmax;
 			}
@@ -981,7 +1109,7 @@ arti_invoke(obj)
 	if(obj->otyp == CRYSTAL_BALL)
 	    use_crystal_ball(obj);
 	else
-	    pline("Nothing happens.");
+	    pline(nothing_happens);
 	return 1;
     }
 
@@ -998,22 +1126,29 @@ arti_invoke(obj)
 
 	switch(oart->inv_prop) {
 	case TAMING: {
-	    struct obj *pseudo = mksobj(SPE_CHARM_MONSTER, FALSE, FALSE);
-	    pseudo->blessed = pseudo->cursed = 0;
-	    pseudo->quan = 20L;			/* do not let useup get it */
-	    (void) seffects(pseudo);
-	    obfree(pseudo, (struct obj *)0);	/* now, get rid of it */
+	    struct obj pseudo;
+
+	    pseudo = zeroobj;	/* neither cursed nor blessed */
+	    pseudo.otyp = SCR_TAMING;
+	    (void) seffects(&pseudo);
 	    break;
 	  }
 	case HEALING: {
 	    int healamt = (u.uhpmax + 1 - u.uhp) / 2;
-	    if(healamt || Sick || (Blinded > 1))
+	    long creamed = (long)u.ucreamed;
+
+	    if (Upolyd) healamt = (u.mhmax + 1 - u.mh) / 2;
+	    if (healamt || Sick || Blinded > creamed)
 		You_feel("better.");
 	    else
 		goto nothing_special;
-	    if(healamt) u.uhp += healamt;
+	    if (healamt > 0) {
+		if (Upolyd) u.mh += healamt;
+		else u.uhp += healamt;
+	    }
 	    if(Sick) make_sick(0L,(char *)0,FALSE,SICK_ALL);
-	    if(Blinded > 1) make_blinded(0L,FALSE);
+	    if(Slimed) Slimed = 0L;
+	    if (Blinded > creamed) make_blinded(creamed, FALSE);
 	    flags.botl = 1;
 	    break;
 	  }
@@ -1044,8 +1179,10 @@ arti_invoke(obj)
 		obj->age = 0;
 		return 0;
 	    }
-	    b_effect = obj->blessed && (u.role == oart->class || !oart->class);
+	    b_effect = obj->blessed &&
+		(Role_switch == oart->role || !oart->role);
 	    recharge(otmp, b_effect ? 1 : obj->cursed ? -1 : 0);
+	    update_inventory();
 	    break;
 	  }
 	case LEV_TELE:
@@ -1107,14 +1244,36 @@ arti_invoke(obj)
 	    }
 	    break;
 	  }
+	case ENLIGHTENING:
+	    enlightenment(0);
+	    break;
+	case CREATE_AMMO: {
+	    struct obj *otmp = mksobj(ARROW, TRUE, FALSE);
+
+	    if (!otmp) goto nothing_special;
+	    otmp->blessed = obj->blessed;
+	    otmp->cursed = obj->cursed;
+	    otmp->bknown = obj->bknown;
+	    if (obj->blessed) {
+		if (otmp->spe < 0) otmp->spe = 0;
+		otmp->quan += rnd(10);
+	    } else if (obj->cursed) {
+		if (otmp->spe > 0) otmp->spe = 0;
+	    } else
+		otmp->quan += rnd(5);
+	    otmp->owt = weight(otmp);
+	    otmp = hold_another_object(otmp, "Suddenly %s out.",
+				       aobjnam(otmp, "fall"), (const char *)0);
+	    break;
+	  }
 	}
     } else {
-	long cprop = (u.uprops[oart->inv_prop].p_flgs ^= W_ARTI);
+	long cprop = (u.uprops[oart->inv_prop].extrinsic ^= W_ARTI);
 	boolean on = (cprop & W_ARTI) != 0; /* true if invoked prop just set */
 
 	if(on && obj->age > monstermoves) {
 	    /* the artifact is tired :-) */
-	    u.uprops[oart->inv_prop].p_flgs ^= W_ARTI;
+	    u.uprops[oart->inv_prop].extrinsic ^= W_ARTI;
 	    You_feel("that %s is ignoring you.", the(xname(obj)));
 	    return 1;
 	} else if(!on) {
@@ -1136,8 +1295,10 @@ nothing_special:
 	    else You_feel("the tension decrease around you.");
 	    break;
 	case LEVITATION:
-	    if(on) float_up();
-	    else (void) float_down(I_SPECIAL|W_ARTI|TIMEOUT);
+	    if(on) {
+		float_up();
+		spoteffects(FALSE);
+	    } else (void) float_down(I_SPECIAL|TIMEOUT, W_ARTI);
 	    break;
 	case INVIS:
 	    if (!See_invisible && !Blind) {
@@ -1156,6 +1317,59 @@ nothing_special:
     return 1;
 }
 
+
+/* WAC return TRUE if artifact is always lit */
+boolean
+artifact_light(obj)
+    struct obj *obj;
+{
+    return (get_artifact(obj) && obj->oartifact == ART_SUNSWORD);
+}
+
+/* KMH -- Talking artifacts are finally implemented */
+void arti_speak(obj)
+    struct obj *obj;
+{
+	register const struct artifact *oart = get_artifact(obj);
+	const char *line;
+	char buf[BUFSZ];
+
+
+	/* Is this a speaking artifact? */
+	if (!oart || !(oart->spfx & SPFX_SPEAK))
+		return;
+
+	line = getrumor(bcsign(obj), buf, TRUE);
+	if (!*line)
+		line = "NetHack rumors file closed for renovation.";
+	pline("%s:", Tobjnam(obj, "whisper"));
+	verbalize("%s", line);
+	return;
+}
+
+boolean
+artifact_has_invprop(otmp, inv_prop)
+struct obj *otmp;
+uchar inv_prop;
+{
+	const struct artifact *arti = get_artifact(otmp);
+
+	return((boolean)(arti && (arti->inv_prop == inv_prop)));
+}
+
+/* Return the price sold to the hero of a given artifact or unique item */
+long
+arti_cost(otmp)
+struct obj *otmp;
+{
+	if (!otmp->oartifact)
+	    return ((long)objects[otmp->otyp].oc_cost);
+	else if (artilist[(int) otmp->oartifact].cost)
+	    return (artilist[(int) otmp->oartifact].cost);
+	else
+	    return (100L * (long)objects[otmp->otyp].oc_cost);
+}
+
 #endif /* OVLB */
 
 /*artifact.c*/
