@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)attrib.c	3.2	96/03/28	*/
+/*	SCCS Id: @(#)attrib.c	3.4	2000/05/17	*/
 /*	Copyright 1988, 1989, 1990, 1992, M. Stephenson		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -7,7 +7,7 @@
 #include "hack.h"
 #include "artifact.h"
 
-/* #define DEBUG	/* uncomment for debugging info */
+/* #define DEBUG */	/* uncomment for debugging info */
 
 #ifdef OVLB
 
@@ -17,145 +17,88 @@ const char	*plusattr[] = {
 	"strong", "smart", "wise", "agile", "tough", "charismatic"
 },
 		*minusattr[] = {
-	"weak", "stupid", "foolish", "clumsy", "vulnerable", "ugly"
+	"weak", "stupid", "foolish", "clumsy", "fragile", "repulsive"
 };
 
-	/* maximum and minimum values for the attributes */
-struct attribs	attrmax = {
-	{ 118, 18, 18, 18, 18, 18 }
-},
-		attrmin = {
-	{ 3, 3, 3, 3, 3, 3 }
-};
 
 static
 const struct innate {
 	schar	ulevel;
 	long	*ability;
 	const char *gainstr, *losestr;
-}	a_abil[] = { {	 1, &(Stealth), "", "" },
-		     {   1, &(Fast), "", "" },
-		     {  10, &(Searching), "perceptive", "" },
+}	arc_abil[] = { {	 1, &(HStealth), "", "" },
+		     {   1, &(HFast), "", "" },
+		     {  10, &(HSearching), "perceptive", "" },
 		     {	 0, 0, 0, 0 } },
 
-	b_abil[] = { {	 1, &(HPoison_resistance), "", "" },
-		     {   7, &(Fast), "quick", "slow" },
-		     {  15, &(Stealth), "stealthy", "" },
+	bar_abil[] = { {	 1, &(HPoison_resistance), "", "" },
+		     {   7, &(HFast), "quick", "slow" },
+		     {  15, &(HStealth), "stealthy", "" },
 		     {	 0, 0, 0, 0 } },
 
-	c_abil[] = { {	 7, &(Fast), "quick", "slow" },
-		     {	15, &(Warning), "sensitive", "" },
+	cav_abil[] = { {	 7, &(HFast), "quick", "slow" },
+		     {	15, &(HWarning), "sensitive", "" },
 		     {	 0, 0, 0, 0 } },
 
-	e_abil[] = { {   1, &(Fast), "", "" },
-		     {	 1, &(HSee_invisible), "", "" },
-		     {	 1, &(Searching), "", "" },
-		     {	 1, &(HSleep_resistance), "", "" },
+	hea_abil[] = { {	 1, &(HPoison_resistance), "", "" },
+		     {	15, &(HWarning), "sensitive", "" },
 		     {	 0, 0, 0, 0 } },
 
-	h_abil[] = { {	 1, &(HPoison_resistance), "", "" },
-		     {	15, &(Warning), "sensitive", "" },
+	kni_abil[] = { {	 7, &(HFast), "quick", "slow" },
 		     {	 0, 0, 0, 0 } },
 
-	k_abil[] = { {	 7, &(Fast), "quick", "slow" },
-		     {	 0, 0, 0, 0 } },
+	mon_abil[] = { {   1, &(HFast), "", "" },
+		     {   1, &(HSleep_resistance), "", "" },
+		     {   1, &(HSee_invisible), "", "" },
+		     {   3, &(HPoison_resistance), "healthy", "" },
+		     {   5, &(HStealth), "stealthy", "" },
+		     {   7, &(HWarning), "sensitive", "" },
+		     {   9, &(HSearching), "perceptive", "unaware" },
+		     {  11, &(HFire_resistance), "cool", "warmer" },
+		     {  13, &(HCold_resistance), "warm", "cooler" },
+		     {  15, &(HShock_resistance), "insulated", "conductive" },
+		     {  17, &(HTeleport_control), "controlled","uncontrolled" },
+		     {   0, 0, 0, 0 } },
 
-	p_abil[] = { {	15, &(Warning), "sensitive", "" },
+	pri_abil[] = { {	15, &(HWarning), "sensitive", "" },
 		     {  20, &(HFire_resistance), "cool", "warmer" },
 		     {	 0, 0, 0, 0 } },
 
-	r_abil[] = { {	 1, &(Stealth), "", ""  },
-		     {  10, &(Searching), "perceptive", "" },
+	ran_abil[] = { {   1, &(HSearching), "", "" },
+		     {	 7, &(HStealth), "stealthy", "" },
+		     {	15, &(HSee_invisible), "", "" },
 		     {	 0, 0, 0, 0 } },
 
-	s_abil[] = { {	 1, &(Fast), "", "" },
-		     {  15, &(Stealth), "stealthy", "" },
+	rog_abil[] = { {	 1, &(HStealth), "", ""  },
+		     {  10, &(HSearching), "perceptive", "" },
 		     {	 0, 0, 0, 0 } },
 
-	t_abil[] = { {	10, &(Searching), "perceptive", "" },
+	sam_abil[] = { {	 1, &(HFast), "", "" },
+		     {  15, &(HStealth), "stealthy", "" },
+		     {	 0, 0, 0, 0 } },
+
+	tou_abil[] = { {	10, &(HSearching), "perceptive", "" },
 		     {	20, &(HPoison_resistance), "hardy", "" },
 		     {	 0, 0, 0, 0 } },
 
-	v_abil[] = { {	 1, &(HCold_resistance), "", "" },
-		     {	 1, &(Stealth), "", "" },
-		     {   7, &(Fast), "quick", "slow" },
+	val_abil[] = { {	 1, &(HCold_resistance), "", "" },
+		     {	 1, &(HStealth), "", "" },
+		     {   7, &(HFast), "quick", "slow" },
 		     {	 0, 0, 0, 0 } },
 
-	w_abil[] = { {	15, &(Warning), "sensitive", "" },
+	wiz_abil[] = { {	15, &(HWarning), "sensitive", "" },
 		     {  17, &(HTeleport_control), "controlled","uncontrolled" },
+		     {	 0, 0, 0, 0 } },
+
+	/* Intrinsics conferred by race */
+	elf_abil[] = { {	4, &(HSleep_resistance), "awake", "tired" },
+		     {	 0, 0, 0, 0 } },
+
+	orc_abil[] = { {	1, &(HPoison_resistance), "", "" },
 		     {	 0, 0, 0, 0 } };
 
-static
-const struct clattr {
-	struct	attribs	base, cldist;
-	align	align;
-	schar	shp, hd, xlev, ndx;
-/* According to AD&D, HD for some classes (ex. Wizard) should be smaller
- * (4-sided for wizards).  But this is not AD&D, and using the AD&D
- * rule here produces an unplayable character.  Thus I have used a minimum
- * of an 10-sided hit die for everything.  Another AD&D change: wizards get
- * a minimum strength of 6 since without one you can't teleport or cast
- * spells. --KAA
- */
-	const struct	innate *abil;
-}	a_attr = { {{  7, 10, 10,  7,  7,  7 }},  /* Archeologist */
-		   {{ 20, 20, 20, 10, 20, 10 }},
-		   { A_LAWFUL,  10 },  13, 10, 14,  2, a_abil },
-
-	b_attr = { {{ 16,  7,  7, 15, 16,  6 }},  /* Barbarian */
-		   {{ 30,  6,  7, 20, 30,  7 }},
-		   { A_NEUTRAL, 10 },  16, 12, 10,  3, b_abil },
-
-	c_attr = { {{ 10,  7,  7,  7,  8,  6 }},  /* Caveman (fighter) */
-		   {{ 30,  6,  7, 20, 30,  7 }},
-		   { A_LAWFUL,   0 },  16, 10, 10,  3, c_abil },
-
-	e_attr = { {{ 13, 13, 13,  9, 13,  7 }},  /* Elf (ranger) */
-		   {{ 30, 10, 10, 20, 20, 10 }},
-		   { A_CHAOTIC, 10 },  15, 10, 11,  2, e_abil },
-
-	h_attr = { {{  7,  7, 13,  7, 11, 16 }},  /* Healer (druid) */
-		   {{ 15, 20, 20, 15, 25, 10 }},
-		   { A_NEUTRAL, 10 },  13, 10, 20,  2, h_abil },
-
-	k_attr = { {{ 13,  7, 14,  8, 10, 17 }},  /* Knight (paladin) */
-		   {{ 20, 15, 15, 10, 20, 10 }},
-		   { A_LAWFUL,  10 },  16, 10, 10,  3, k_abil },
-
-	p_attr = { {{  7,  7, 10,  7,  7,  7 }},  /* Priest (cleric) */
-		   {{ 15, 10, 30, 15, 20, 10 }},
-		   { A_NEUTRAL,  0 },  14, 10, 10,  2, p_abil },
-
-	r_attr = { {{  7,  7,  7, 10,  7,  6 }},  /* Rogue (thief) */
-		   {{ 20, 10, 10, 30, 20, 10 }},
-		   { A_CHAOTIC, 10 },  12, 10, 11,  2, r_abil },
-
-	s_attr = { {{ 10,  8,  7, 10, 17,  6 }},  /* Samurai (fighter/thief) */
-		   {{ 30, 10, 10, 30, 14, 10 }},
-		   { A_LAWFUL,  10 },  15, 10, 11,  2, s_abil },
-
-#ifdef TOURIST
-	t_attr = { {{  7, 10,  6,  7,  7, 10 }},  /* Tourist */
-		   {{ 15, 10, 10, 15, 30, 20 }},
-		   { A_NEUTRAL,  0 },  10, 10, 14,  1, t_abil },
-#endif
-
-	v_attr = { {{ 10,  7,  7,  7, 10,  7 }},  /* Valkyrie (fighter) */
-		   {{ 30,  6,  7, 20, 30,  7 }},
-		   { A_NEUTRAL,  0 },  16, 10, 10,  3, v_abil },
-
-	w_attr = { {{  7, 10,  7,  7,  7,  7 }},  /* Wizard (magic-user) */
-		   {{ 10, 30, 10, 20, 20, 10 }},
-		   { A_NEUTRAL,  0 },  12, 10, 12,  1, w_abil },
-
-	X_attr = { {{  3,  3,  3,  3,  3,  3 }},
-		   {{ 20, 15, 15, 15, 20, 15 }},
-		   { A_NEUTRAL,  0 },  12, 10, 14,  1,  0 };
-
 static long next_check = 600L;	/* arbitrary first setting */
-static NEARDATA const struct clattr *NDECL(clx);
-static void NDECL(init_align);
-static void NDECL(exerper);
+STATIC_DCL void NDECL(exerper);
 
 /* adjust an attribute; return TRUE if change is made, FALSE otherwise */
 boolean
@@ -163,7 +106,7 @@ adjattrib(ndx, incr, msgflg)
 	int	ndx, incr;
 	int	msgflg;	    /* positive => no message, zero => message, and */
 {			    /* negative => conditional (msg if change made) */
-	if (!incr) return FALSE;
+	if (Fixed_abil || !incr) return FALSE;
 
 	if ((ndx == A_INT || ndx == A_WIS)
 				&& uarmh && uarmh->otyp == DUNCE_CAP) {
@@ -212,7 +155,7 @@ adjattrib(ndx, incr, msgflg)
 		  (incr > 1 || incr < -1) ? "very ": "",
 		  (incr > 0) ? plusattr[ndx] : minusattr[ndx]);
 	flags.botl = 1;
-	if (moves > 0 && (ndx == A_STR || ndx == A_CON))
+	if (moves > 1 && (ndx == A_STR || ndx == A_CON))
 		(void)encumber_msg();
 	return TRUE;
 }
@@ -227,7 +170,7 @@ gainstr(otmp, incr)
 	if(incr) num = incr;
 	else {
 	    if(ABASE(A_STR) < 18) num = (rn2(4) ? 1 : rnd(6) );
-	    else if (ABASE(A_STR) < 103) num = rnd(10);
+	    else if (ABASE(A_STR) < STR18(85)) num = rnd(10);
 	}
 	(void) adjattrib(A_STR, (otmp && otmp->cursed) ? -num : num, TRUE);
 }
@@ -239,10 +182,15 @@ losestr(num)	/* may kill you; cause may be poison or monster like 'a' */
 	int ustr = ABASE(A_STR) - num;
 
 	while(ustr < 3) {
-		ustr++;
-		num--;
+	    ++ustr;
+	    --num;
+	    if (Upolyd) {
+		u.mh -= 6;
+		u.mhmax -= 6;
+	    } else {
 		u.uhp -= 6;
 		u.uhpmax -= 6;
+	    }
 	}
 	(void) adjattrib(A_STR, -num, TRUE);
 }
@@ -323,7 +271,7 @@ boolean	inc_or_dec;
 	if (i == A_INT || i == A_CHA) return;	/* can't exercise these */
 
 	/* no physical exercise while polymorphed; the body's temporary */
-	if (u.umonnum >= LOW_PM && i != A_WIS) return;
+	if (Upolyd && i != A_WIS) return;
 
 	if(abs(AEXE(i)) < AVAL) {
 		/*
@@ -355,7 +303,7 @@ boolean	inc_or_dec;
 #define FAINTED		5
 #define STARVED		6
 
-static void
+STATIC_OVL void
 exerper()
 {
 	if(!(moves % 10)) {
@@ -397,12 +345,16 @@ exerper()
 		pline("exerper: Status checks");
 #endif
 		if ((HClairvoyant & (INTRINSIC|TIMEOUT)) &&
-			!(HClairvoyant & I_BLOCKED))	exercise(A_WIS, TRUE);
+			!BClairvoyant)                      exercise(A_WIS, TRUE);
 		if (HRegeneration)			exercise(A_STR, TRUE);
 
-		if(Sick || Vomiting)			exercise(A_CON, FALSE);
+		if(Sick || Vomiting)     exercise(A_CON, FALSE);
 		if(Confusion || Hallucination)		exercise(A_WIS, FALSE);
-		if(Wounded_legs || Fumbling || HStun)	exercise(A_DEX, FALSE);
+		if((Wounded_legs 
+#ifdef STEED
+		    && !u.usteed
+#endif
+			    ) || Fumbling || HStun)	exercise(A_DEX, FALSE);
 	}
 }
 
@@ -495,76 +447,25 @@ reset_attribute_clock()
 	if (moves > 600L) next_check = moves + rn1(50,800);
 }
 
-static const struct	clattr *
-clx()
-{
-	register const struct	clattr	*attr;
-
-	switch (u.role) {
-	    case 'A':	attr = &a_attr;
-			break;
-	    case 'B':	attr = &b_attr;
-			break;
-	    case 'C':	attr = &c_attr;
-			break;
-	    case 'E':	attr = &e_attr;
-			break;
-	    case 'H':	attr = &h_attr;
-			break;
-	    case 'K':	attr = &k_attr;
-			break;
-	    case 'P':	attr = &p_attr;
-			break;
-	    case 'R':	attr = &r_attr;
-			break;
-	    case 'S':	attr = &s_attr;
-			break;
-#ifdef TOURIST
-	    case 'T':	attr = &t_attr;
-			break;
-#endif
-	    case 'V':	attr = &v_attr;
-			break;
-	    case 'W':	attr = &w_attr;
-			break;
-	    default:	/* unknown type */
-			attr = &X_attr;
-			break;
-	}
-	return(attr);
-}
-
-static void
-init_align()	/* called from newhp if u.ulevel is 0 */
-{
-	register const struct	clattr	*attr = clx();
-
-	u.ualign = attr->align;
-	/* there should be priests of every stripe */
-	if (Role_is('P'))
-	     u.ualign.type = (rn2(2)) ? attr->align.type : (rn2(2)) ? 1 : -1;
-	else u.ualign.type = attr->align.type;
-}
 
 void
 init_attr(np)
 	register int	np;
 {
 	register int	i, x, tryct;
-	register const struct	clattr	*attr = clx();
+
 
 	for(i = 0; i < A_MAX; i++) {
-
-	    ABASE(i) = AMAX(i) = attr->base.a[i];
+	    ABASE(i) = AMAX(i) = urole.attrbase[i];
 	    ATEMP(i) = ATIME(i) = 0;
-	    np -= attr->base.a[i];
+	    np -= urole.attrbase[i];
 	}
 
 	tryct = 0;
 	while(np > 0 && tryct < 100) {
 
 	    x = rn2(100);
-	    for (i = 0; (i < A_MAX) && ((x -= attr->cldist.a[i]) > 0); i++) ;
+	    for (i = 0; (i < A_MAX) && ((x -= urole.attrdist[i]) > 0); i++) ;
 	    if(i >= A_MAX) continue; /* impossible */
 
 	    if(ABASE(i) >= ATTRMAX(i)) {
@@ -582,7 +483,7 @@ init_attr(np)
 	while(np < 0 && tryct < 100) {		/* for redistribution */
 
 	    x = rn2(100);
-	    for (i = 0; (i < A_MAX) && ((x -= attr->cldist.a[i]) > 0); i++) ;
+	    for (i = 0; (i < A_MAX) && ((x -= urole.attrdist[i]) > 0); i++) ;
 	    if(i >= A_MAX) continue; /* impossible */
 
 	    if(ABASE(i) <= ATTRMIN(i)) {
@@ -620,17 +521,48 @@ void
 adjabil(oldlevel,newlevel)
 int oldlevel, newlevel;
 {
-	register const struct clattr	*attr = clx();
-#ifdef GCC_WARN
-	/* this is the "right" definition */
-	register const struct innate	*abil = attr->abil;
-#else
-	/* this one satisfies more compilers */
-	register struct innate	*abil = (struct innate *)attr->abil;
+	register const struct innate *abil, *rabil;
+	long mask = FROMEXPER;
+
+
+	switch (Role_switch) {
+	case PM_ARCHEOLOGIST:   abil = arc_abil;	break;
+	case PM_BARBARIAN:      abil = bar_abil;	break;
+	case PM_CAVEMAN:        abil = cav_abil;	break;
+	case PM_HEALER:         abil = hea_abil;	break;
+	case PM_KNIGHT:         abil = kni_abil;	break;
+	case PM_MONK:           abil = mon_abil;	break;
+	case PM_PRIEST:         abil = pri_abil;	break;
+	case PM_RANGER:         abil = ran_abil;	break;
+	case PM_ROGUE:          abil = rog_abil;	break;
+	case PM_SAMURAI:        abil = sam_abil;	break;
+#ifdef TOURIST
+	case PM_TOURIST:        abil = tou_abil;	break;
 #endif
+	case PM_VALKYRIE:       abil = val_abil;	break;
+	case PM_WIZARD:         abil = wiz_abil;	break;
+	default:                abil = 0;		break;
+	}
+
+	switch (Race_switch) {
+	case PM_ELF:            rabil = elf_abil;	break;
+	case PM_ORC:            rabil = orc_abil;	break;
+	case PM_HUMAN:
+	case PM_DWARF:
+	case PM_GNOME:
+	default:                rabil = 0;		break;
+	}
+
+	while (abil || rabil) {
+	    /* Have we finished with the intrinsics list? */
+	    if (!abil || !abil->ability) {
+	    	/* Try the race intrinsics */
+	    	if (!rabil || !rabil->ability) break;
+	    	abil = rabil;
+	    	rabil = 0;
+	    	mask = FROMRACE;
+	    }
 
-	if(abil) {
-	    for(; abil->ability; abil++) {
 		if(oldlevel < abil->ulevel && newlevel >= abil->ulevel) {
 			/* Abilities gained at level 1 can never be lost
 			 * via level loss, only via means that remove _any_
@@ -639,56 +571,71 @@ int oldlevel, newlevel;
 			 * FROMOUTSIDE to avoid such gains.
 			 */
 			if (abil->ulevel == 1)
-				*(abil->ability) |= (FROMEXPER|FROMOUTSIDE);
+				*(abil->ability) |= (mask|FROMOUTSIDE);
 			else
-				*(abil->ability) |= FROMEXPER;
-			if(!(*(abil->ability) & FROMOUTSIDE)) {
+				*(abil->ability) |= mask;
+			if(!(*(abil->ability) & INTRINSIC & ~mask)) {
 			    if(*(abil->gainstr))
 				You_feel("%s!", abil->gainstr);
 			}
 		} else if (oldlevel >= abil->ulevel && newlevel < abil->ulevel) {
-			*(abil->ability) &= ~FROMEXPER;
-			if((*(abil->ability) & INTRINSIC)) {
+			*(abil->ability) &= ~mask;
+			if(!(*(abil->ability) & INTRINSIC)) {
 			    if(*(abil->losestr))
 				You_feel("%s!", abil->losestr);
 			    else if(*(abil->gainstr))
 				You_feel("less %s!", abil->gainstr);
 			}
 		}
-	    }
+	    abil++;
+	}
+
+	if (oldlevel > 0) {
+	    if (newlevel > oldlevel)
+		add_weapon_skill(newlevel - oldlevel);
+	    else
+		lose_weapon_skill(oldlevel - newlevel);
 	}
 }
 
+
 int
 newhp()
 {
-	register const struct clattr	*attr = clx();
 	int	hp, conplus;
 
-	if(u.ulevel == 0) {
 
-		hp = attr->shp;
-		init_align();	/* initialize alignment stuff */
+	if (u.ulevel == 0) {
+	    /* Initialize hit points */
+	    hp = urole.hpadv.infix + urace.hpadv.infix;
+	    if (urole.hpadv.inrnd > 0) hp += rnd(urole.hpadv.inrnd);
+	    if (urace.hpadv.inrnd > 0) hp += rnd(urace.hpadv.inrnd);
+
+	    /* Initialize alignment stuff */
+	    u.ualign.type = aligns[flags.initalign].value;
+	    u.ualign.record = urole.initrecord;
+
 		return hp;
 	} else {
-
-	    if(u.ulevel < attr->xlev)
-		hp = rnd(attr->hd);
-	    else
-		hp = attr->ndx;
+	    if (u.ulevel < urole.xlev) {
+	    	hp = urole.hpadv.lofix + urace.hpadv.lofix;
+	    	if (urole.hpadv.lornd > 0) hp += rnd(urole.hpadv.lornd);
+	    	if (urace.hpadv.lornd > 0) hp += rnd(urace.hpadv.lornd);
+	    } else {
+	    	hp = urole.hpadv.hifix + urace.hpadv.hifix;
+	    	if (urole.hpadv.hirnd > 0) hp += rnd(urole.hpadv.hirnd);
+	    	if (urace.hpadv.hirnd > 0) hp += rnd(urace.hpadv.hirnd);
+	    }
 	}
 
-	switch(ACURR(A_CON)) {
-		case	3:	conplus = -2; break;
-		case	4:
-		case	5:
-		case	6:	conplus = -1; break;
-		case	15:
-		case	16:	conplus = 1; break;
-		case	17:	conplus = 2; break;
-		case	18:	conplus = 3; break;
-		default:	conplus = 0;
-	}
+	if (ACURR(A_CON) <= 3) conplus = -2;
+	else if (ACURR(A_CON) <= 6) conplus = -1;
+	else if (ACURR(A_CON) <= 14) conplus = 0;
+	else if (ACURR(A_CON) <= 16) conplus = 1;
+	else if (ACURR(A_CON) == 17) conplus = 2;
+	else if (ACURR(A_CON) == 18) conplus = 3;
+	else conplus = 4;
+	
 	hp += conplus;
 	return((hp <= 0) ? 1 : hp);
 }
@@ -710,7 +657,7 @@ int x;
 		else return((schar)((tmp >= 125) ? 125 : (tmp <= 3) ? 3 : tmp));
 #endif
 	} else if (x == A_CHA) {
-		if (tmp < 18 && (u.usym == S_NYMPH ||
+		if (tmp < 18 && (youmonst.data->mlet == S_NYMPH ||
 		    u.umonnum==PM_SUCCUBUS || u.umonnum == PM_INCUBUS))
 		    return 18;
 	} else if (x == A_INT || x == A_WIS) {
