@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)display.c	3.3	97/01/24	*/
+/*	SCCS Id: @(#)display.c	3.4	2000/07/27	*/
 /* Copyright (c) Dean Luick, with acknowledgements to Kevin Darcy */
 /* and Dave Cohrs, 1990.					  */
 /* NetHack may be freely redistributed.  See license for details. */
@@ -120,9 +120,12 @@
 
 STATIC_DCL void FDECL(display_monster,(XCHAR_P,XCHAR_P,struct monst *,int,XCHAR_P));
 STATIC_DCL int FDECL(swallow_to_glyph, (int, int));
+STATIC_DCL void FDECL(display_warning,(struct monst *));
 
 STATIC_DCL int FDECL(check_pos, (int, int, int));
+#ifdef WA_VERBOSE
 STATIC_DCL boolean FDECL(more_than_one, (int, int, int, int, int));
+#endif
 STATIC_DCL int FDECL(set_twall, (int,int, int,int, int,int, int,int));
 STATIC_DCL int FDECL(set_wall, (int, int, int));
 STATIC_DCL int FDECL(set_corn, (int,int, int,int, int,int, int,int));
@@ -155,6 +158,36 @@ vobj_at(x,y)
 }
 #endif	/* else vobj_at() is defined in display.h */
 
+/*
+ * magic_map_background()
+ *
+ * This function is similar to map_background (see below) except we pay
+ * attention to and correct unexplored, lit ROOM and CORR spots.
+ */
+void
+magic_map_background(x, y, show)
+    xchar x,y;
+    int  show;
+{
+    int glyph = back_to_glyph(x,y);	/* assumes hero can see x,y */
+    struct rm *lev = &levl[x][y];
+
+    /*
+     * Correct for out of sight lit corridors and rooms that the hero
+     * doesn't remember as lit.
+     */
+    if (!cansee(x,y) && !lev->waslit) {
+	/* Floor spaces are dark if unlit.  Corridors are dark if unlit. */
+	if (lev->typ == ROOM && glyph == cmap_to_glyph(S_room))
+	    glyph = cmap_to_glyph(S_stone);
+	else if (lev->typ == CORR && glyph == cmap_to_glyph(S_litcorr))
+	    glyph = cmap_to_glyph(S_corr);
+    }
+    if (level.flags.hero_memory)
+	lev->glyph = glyph;
+    if (show) show_glyph(x,y, glyph);
+}
+
 /*
  * The routines map_background(), map_object(), and map_trap() could just
  * as easily be:
@@ -304,6 +337,9 @@ void map_location(x,y,show)
     _map_location(x,y,show);
 }
 
+#define DETECTED 	2
+#define PHYSICALLY_SEEN 1
+#define is_worm_tail(mon)	((mon) && ((x != (mon)->mx)  || (y != (mon)->my)))
 
 /*
  * display_monster()
@@ -317,16 +353,16 @@ void map_location(x,y,show)
  *
  */
 STATIC_OVL void
-display_monster(x, y, mon, in_sight, worm_tail)
+display_monster(x, y, mon, sightflags, worm_tail)
     register xchar x, y;	/* display position */
     register struct monst *mon;	/* monster to display */
-    int in_sight;		/* TRUE if the monster is physically seen */
+    int sightflags;		/* 1 if the monster is physically seen */
+    				/* 2 if detected using Detect_monsters */
     register xchar worm_tail;	/* mon is actually a worm tail */
 {
     register boolean mon_mimic = (mon->m_ap_type != M_AP_NOTHING);
     register int sensed = mon_mimic &&
 	(Protection_from_shape_changers || sensemon(mon));
-
     /*
      * We must do the mimic check first.  If the mimic is mimicing something,
      * and the location is in sight, we have to change the hero's memory
@@ -334,7 +370,7 @@ display_monster(x, y, mon, in_sight, worm_tail)
      * the mimic was mimicing.
      */
 
-    if (mon_mimic && in_sight) {
+    if (mon_mimic && sightflags) {
 	switch (mon->m_ap_type) {
 	    default:
 		impossible("display_monster:  bad m_ap_type value [ = %d ]",
@@ -371,7 +407,7 @@ display_monster(x, y, mon, in_sight, worm_tail)
 	    }
 
 	    case M_AP_MONSTER:
-		show_glyph(x,y, monnum_to_glyph(what_mon(mon->mappearance)));
+		show_glyph(x,y, monnum_to_glyph(what_mon((int)mon->mappearance)));
 		break;
 	}
 	
@@ -381,7 +417,10 @@ display_monster(x, y, mon, in_sight, worm_tail)
     if (!mon_mimic || sensed) {
 	int num;
 
-	if (Detect_monsters) {
+	/* [ALI] Only use detected glyphs when monster wouldn't be
+	 * visible by any other means.
+	 */
+	if (sightflags == DETECTED) {
 	    if (worm_tail)
 		num = detected_monnum_to_glyph(what_mon(PM_LONG_WORM_TAIL));
 	    else
@@ -401,6 +440,35 @@ display_monster(x, y, mon, in_sight, worm_tail)
     }
 }
 
+/*
+ * display_warning()
+ *
+ * This is also *not* a map_XXXX() function!  Monster warnings float
+ * above everything just like monsters do, but only if the monster
+ * is not showing.
+ *
+ * Do not call for worm tails.
+ */
+STATIC_OVL void
+display_warning(mon)
+    register struct monst *mon;
+{
+    int x = mon->mx, y = mon->my;
+    int wl = (int) (mon->m_lev / 4);
+    int glyph;
+
+    if (mon_warning(mon)) {
+        if (wl > WARNCOUNT - 1) wl = WARNCOUNT - 1;
+        glyph = warning_to_glyph(wl);
+    } else if (MATCH_WARN_OF_MON(mon)) {
+	glyph = mon_to_glyph(mon);
+    } else {
+    	impossible("display_warning did not match warning type?");
+        return;
+    }
+    show_glyph(x, y, glyph);
+}
+
 /*
  * feel_location()
  *
@@ -528,7 +596,9 @@ feel_location(x, y)
     }
     /* draw monster on top if we can sense it */
     if ((x != u.ux || y != u.uy) && (mon = m_at(x,y)) && sensemon(mon))
-	display_monster(x,y,mon,1,((x != mon->mx)  || (y != mon->my)));
+	display_monster(x, y, mon,
+		(tp_sensemon(mon) || MATCH_WARN_OF_MON(mon)) ? PHYSICALLY_SEEN : DETECTED,
+		is_worm_tail(mon));
 }
 
 /*
@@ -592,15 +662,27 @@ newsym(x,y)
 	}
 	else {
 	    mon = m_at(x,y);
-	    worm_tail = mon && ((x != mon->mx)  || (y != mon->my));
-	    if (mon &&
-		 ((see_it = (worm_tail
-			? (!mon->minvis || See_invisible)
-			: (mon_visible(mon)) || sensemon(mon))))) {
+	    worm_tail = is_worm_tail(mon);
+	    see_it = mon && (worm_tail
+		? (!mon->minvis || See_invisible)
+		: (mon_visible(mon)) || tp_sensemon(mon) || MATCH_WARN_OF_MON(mon));
+	    if (mon && (see_it || (!worm_tail && Detect_monsters))) {
+		if (mon->mtrapped) {
+		    struct trap *trap = t_at(x, y);
+		    int tt = trap ? trap->ttyp : NO_TRAP;
+
+		    /* if monster is in a physical trap, you see the trap too */
+		    if (tt == BEAR_TRAP || tt == PIT ||
+			tt == SPIKED_PIT ||tt == WEB) {
+			trap->tseen = TRUE;
+		    }
+		}
 		_map_location(x,y,0);	/* map under the monster */
 		/* also gets rid of any invisibility glyph */
-		display_monster(x,y,mon,see_it,worm_tail);
+		display_monster(x, y, mon, see_it ? PHYSICALLY_SEEN : DETECTED, worm_tail);
 	    }
+	    else if (mon && mon_warning(mon) && !is_worm_tail(mon))
+	        display_warning(mon);
 	    else if (glyph_is_invisible(levl[x][y].glyph))
 		map_invisible(x, y);
 	    else
@@ -615,14 +697,20 @@ newsym(x,y)
 
 	    if (canseeself()) display_self();
 	}
-	else if ((mon = m_at(x,y)) &&
-		(sensemon(mon) ||
-		    (see_with_infrared(mon) && mon_visible(mon))) &&
-		!((x != mon->mx) || (y != mon->my))) {
+	else if ((mon = m_at(x,y))
+		&& ((see_it = (tp_sensemon(mon) || MATCH_WARN_OF_MON(mon)
+		    		|| (see_with_infrared(mon) && mon_visible(mon))))
+		    || Detect_monsters)
+		&& !is_worm_tail(mon)) {
 	    /* Monsters are printed every time. */
 	    /* This also gets rid of any invisibility glyph */
-	    display_monster(x,y,mon,0,0);
+	    display_monster(x, y, mon, see_it ? 0 : DETECTED, 0);
 	}
+	else if ((mon = m_at(x,y)) && mon_warning(mon) &&
+		 !is_worm_tail(mon)) {
+	        display_warning(mon);
+	}		
+
 	/*
 	 * If the location is remembered as being both dark (waslit is false)
 	 * and lit (glyph is a lit room or lit corridor) then it was either:
@@ -659,6 +747,7 @@ show_mem:
     }
 }
 
+#undef is_worm_tail
 
 /*
  * shieldeff()
@@ -672,6 +761,7 @@ shieldeff(x,y)
 {
     register int i;
 
+    if (!flags.sparkle) return;
     if (cansee(x,y)) {	/* Don't see anything if can't see the location */
 	for (i = 0; i < SHIELD_COUNT; i++) {
 	    show_glyph(x, y, cmap_to_glyph(shield_static[i]));
@@ -697,6 +787,8 @@ shieldeff(x,y)
  *	(DISP_CHANGE, glyph)	change glyph
  *	(DISP_END,    0)	close & clean up (second argument doesn't
  *				matter)
+ *	(DISP_FREEMEM, 0)	only used to prevent memory leak during
+ *				exit)
  *	(x, y)			display the glyph at the location
  *
  * DISP_BEAM  - Display the given glyph at each location, but do not erase
@@ -705,61 +797,94 @@ shieldeff(x,y)
  *		previous location's glyph.
  * DISP_ALWAYS- Like DISP_FLASH, but vision is not taken into account.
  */
+
+static struct tmp_glyph {
+    coord saved[COLNO];	/* previously updated positions */
+    int sidx;		/* index of next unused slot in saved[] */
+    int style;		/* either DISP_BEAM or DISP_FLASH or DISP_ALWAYS */
+    int glyph;		/* glyph to use when printing */
+    struct tmp_glyph *prev;
+} tgfirst;
+
 void
 tmp_at(x, y)
     int x, y;
 {
-    static coord saved[COLNO];	/* prev positions, only for DISP_BEAM */
-    static int sidx = 0;	/* index of saved previous positions */
-    static int sx = -1, sy;	/* previous position, only for DISP_FLASH */
-    static int status;		/* either DISP_BEAM or DISP_FLASH */
-    static int glyph;		/* glyph to use when printing */
+    static struct tmp_glyph *tglyph = (struct tmp_glyph *)0;
+    struct tmp_glyph *tmp;
 
     switch (x) {
 	case DISP_BEAM:
 	case DISP_FLASH:
 	case DISP_ALWAYS:
-	    status = x;
-	    glyph  = y;
+	    if (!tglyph)
+		tmp = &tgfirst;
+	    else	/* nested effect; we need dynamic memory */
+		tmp = (struct tmp_glyph *)alloc(sizeof (struct tmp_glyph));
+	    tmp->prev = tglyph;
+	    tglyph = tmp;
+	    tglyph->sidx = 0;
+	    tglyph->style = x;
+	    tglyph->glyph = y;
 	    flush_screen(0);	/* flush buffered glyphs */
-	    break;
+	    return;
 
+	case DISP_FREEMEM:  /* in case game ends with tmp_at() in progress */
+	    while (tglyph) {
+		tmp = tglyph->prev;
+		if (tglyph != &tgfirst) free((genericptr_t)tglyph);
+		tglyph = tmp;
+	    }
+	    return;
+
+	default:
+	    break;
+    }
+
+    if (!tglyph) panic("tmp_at: tglyph not initialized");
+
+    switch (x) {
 	case DISP_CHANGE:
-	    glyph = y;
+	    tglyph->glyph = y;
 	    break;
 
 	case DISP_END:
-	    if (status == DISP_BEAM) {
+	    if (tglyph->style == DISP_BEAM) {
 		register int i;
 
 		/* Erase (reset) from source to end */
-		for (i = 0; i < sidx; i++)
-		    newsym(saved[i].x,saved[i].y);
-		sidx = 0;
-		
-	    } else if (sx >= 0) { /* DISP_FLASH/ALWAYS (called at least once) */
-		newsym(sx,sy);	/* reset the location */
-		sx = -1;	/* reset sx to an illegal pos for next time */
+		for (i = 0; i < tglyph->sidx; i++)
+		    newsym(tglyph->saved[i].x, tglyph->saved[i].y);
+	    } else {		/* DISP_FLASH or DISP_ALWAYS */
+		if (tglyph->sidx)	/* been called at least once */
+		    newsym(tglyph->saved[0].x, tglyph->saved[0].y);
 	    }
+	 /* tglyph->sidx = 0; -- about to be freed, so not necessary */
+	    tmp = tglyph->prev;
+	    if (tglyph != &tgfirst) free((genericptr_t)tglyph);
+	    tglyph = tmp;
 	    break;
 
 	default:	/* do it */
-	    if (!cansee(x,y) && status != DISP_ALWAYS) break;
-
-	    if (status == DISP_BEAM) {
-		saved[sidx  ].x = x;	/* save pos for later erasing */
-		saved[sidx++].y = y;
+	    if (tglyph->style == DISP_BEAM) {
+		if (!cansee(x,y)) break;
+		/* save pos for later erasing */
+		tglyph->saved[tglyph->sidx].x = x;
+		tglyph->saved[tglyph->sidx].y = y;
+		tglyph->sidx += 1;
+	    } else {	/* DISP_FLASH/ALWAYS */
+		if (tglyph->sidx) { /* not first call, so reset previous pos */
+		    newsym(tglyph->saved[0].x, tglyph->saved[0].y);
+		    tglyph->sidx = 0;	/* display is presently up to date */
+		}
+		if (!cansee(x,y) && tglyph->style != DISP_ALWAYS) break;
+		tglyph->saved[0].x = x;
+		tglyph->saved[0].y = y;
+		tglyph->sidx = 1;
 	    }
 
-	    else {	/* DISP_FLASH/ALWAYS */
-		if (sx >= 0)		/* not first call */
-		    newsym(sx,sy);	/* update the old position */
-		sx = x;		/* save previous pos for next call */
-		sy = y;
-	    }
-
-	    show_glyph(x,y,glyph);	/* show it */
-	    flush_screen(0);		/* make sure it shows up */
+	    show_glyph(x, y, tglyph->glyph);	/* show it */
+	    flush_screen(0);			/* make sure it shows up */
 	    break;
     } /* end case */
 }
@@ -922,6 +1047,7 @@ see_monsters()
 {
     register struct monst *mon;
     for (mon = fmon; mon; mon = mon->nmon) {
+	if (DEADMONSTER(mon)) continue;
 	newsym(mon->mx,mon->my);
 	if (mon->wormno) see_wsegs(mon);
     }
@@ -937,7 +1063,7 @@ set_mimic_blocking()
 {
     register struct monst *mon;
     for (mon = fmon; mon; mon = mon->nmon)
-	if(mon->minvis &&
+	if(!DEADMONSTER(mon) && mon->minvis &&
 	   ((mon->m_ap_type == M_AP_FURNITURE &&
 	      (mon->mappearance == S_vcdoor || mon->mappearance == S_hcdoor))||
 	    (mon->m_ap_type == M_AP_OBJECT && mon->mappearance == BOULDER))) {
@@ -1063,7 +1189,7 @@ show_glyph(x,y,glyph)
     /*
      * Check for bad positions and glyphs.
      */
-    if (x <= 0 || x >= COLNO || y < 0 || y >= ROWNO) {
+    if (!isok(x, y)) {
 	const char *text;
 	int  offset;
 
@@ -1074,10 +1200,15 @@ show_glyph(x,y,glyph)
 	 *  This assumes an ordering of the offsets.  See display.h for
 	 *  the definition.
 	 */
-	if (glyph >= GLYPH_SWALLOW_OFF) {		/* swallow border */
+
+	if (glyph >= GLYPH_WARNING_OFF) {	/* a warning */
+	    text = "warning";		offset = glyph - GLYPH_WARNING_OFF;
+	} else if (glyph >= GLYPH_SWALLOW_OFF) {	/* swallow border */
 	    text = "swallow border";	offset = glyph - GLYPH_SWALLOW_OFF;
-	}else if (glyph >= GLYPH_ZAP_OFF) {		/* zap beam */
+	} else if (glyph >= GLYPH_ZAP_OFF) {		/* zap beam */
 	    text = "zap beam";		offset = glyph - GLYPH_ZAP_OFF;
+	} else if (glyph >= GLYPH_EXPLODE_OFF) {	/* explosion */
+	    text = "explosion";		offset = glyph - GLYPH_EXPLODE_OFF;
 	} else if (glyph >= GLYPH_CMAP_OFF) {		/* cmap */
 	    text = "cmap_index";	offset = glyph - GLYPH_CMAP_OFF;
 	} else if (glyph >= GLYPH_OBJ_OFF) {		/* object */
@@ -1437,18 +1568,20 @@ check_pos(x, y, which)
 
 /* Return TRUE if more than one is non-zero. */
 /*ARGSUSED*/
+#ifdef WA_VERBOSE
 STATIC_OVL boolean
 more_than_one(x, y, a, b, c)
     int x, y, a, b, c;
 {
     if ((a && (b|c)) || (b && (a|c)) || (c && (a|b))) {
-#ifdef WA_VERBOSE
 	error4(x,y,a,b,c,0);
-#endif
 	return TRUE;
     }
     return FALSE;
 }
+#else
+#define more_than_one(x, y, a, b, c) (((a) && ((b)|(c))) || ((b) && ((a)|(c))) || ((c) && ((a)|(b))))
+#endif
 
 /* Return the wall mode for a T wall. */
 STATIC_OVL int
