@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)do.c	3.3	1999/11/29	*/
+/*	SCCS Id: @(#)do.c	3.4	2001/11/29	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -48,11 +48,15 @@ static NEARDATA const char drop_types[] =
 int
 dodrop()
 {
+#ifndef GOLDOBJ
 	int result, i = (invent || u.ugold) ? 0 : (SIZE(drop_types) - 1);
+#else
+	int result, i = (invent) ? 0 : (SIZE(drop_types) - 1);
+#endif
 
-	if (*u.ushops) sellobj_state(TRUE);
+	if (*u.ushops) sellobj_state(SELL_DELIBERATE);
 	result = drop(getobj(&drop_types[i], "drop"));
-	if (*u.ushops) sellobj_state(FALSE);
+	if (*u.ushops) sellobj_state(SELL_NORMAL);
 	reset_occupations();
 
 	return result;
@@ -102,7 +106,7 @@ boolean pushing;
 			boolean moat = (ltyp != WATER) &&
 			    !Is_medusa_level(&u.uz) && !Is_waterlevel(&u.uz);
 
-			pline("There is a large splash as %s %s the %s.",
+			There("is a large splash as %s %s the %s.",
 			      the(xname(otmp)), fills_up? "fills":"falls into",
 			      lava ? "lava" : ltyp==POOL ? "pool" :
 			      moat ? "moat" : "water");
@@ -162,14 +166,16 @@ const char *verb;
 		if (((mtmp = m_at(x, y)) && mtmp->mtrapped) ||
 			(u.utrap && u.ux == x && u.uy == y)) {
 		    if (*verb)
-			pline_The("boulder %ss into the pit%s.", verb,
+			pline_The("boulder %s into the pit%s.",
+				vtense((const char *)0, verb),
 				(mtmp) ? "" : " with you");
 		    if (mtmp) {
 			if (!passes_walls(mtmp->data) &&
 				!throws_rocks(mtmp->data)) {
-			    if (hmon(mtmp, obj, TRUE))
+			    if (hmon(mtmp, obj, TRUE) && !is_whirly(mtmp->data))
 				return FALSE;	/* still alive */
-			} else mtmp->mtrapped = 0;
+			}
+			mtmp->mtrapped = 0;
 		    } else {
 			if (!Passes_walls && !throws_rocks(youmonst.data)) {
 			    losehp(rnd(15), "squished under a boulder",
@@ -197,6 +203,8 @@ const char *verb;
 		bury_objs(x, y);
 		newsym(x,y);
 		return TRUE;
+	} else if (is_lava(x, y)) {
+		return fire_damage(obj, FALSE, FALSE, x, y);
 	} else if (is_pool(x, y)) {
 		water_damage(obj, FALSE, FALSE);
 	}
@@ -210,20 +218,20 @@ void
 doaltarobj(obj)  /* obj is an object dropped on an altar */
 	register struct obj *obj;
 {
-	if (Blind) return;
+	if (Blind || obj->oclass == GOLD_CLASS)
+		return;
 
 	/* KMH, conduct */
 	u.uconduct.gnostic++;
 
 	if (obj->blessed || obj->cursed) {
-		pline("There is %s flash as %s hit%s the altar.",
+		There("is %s flash as %s %s the altar.",
 			an(hcolor(obj->blessed ? amber : Black)),
-			doname(obj),
-			(obj->quan == 1L) ? "s" : "");
+			doname(obj), otense(obj, "hit"));
 		if (!Hallucination) obj->bknown = 1;
 	} else {
-		pline("%s land%s on the altar.", Doname2(obj),
-			(obj->quan == 1L) ? "s" : "");
+		pline("%s %s on the altar.", Doname2(obj),
+			otense(obj, "land"));
 		obj->bknown = 1;
 	}
 }
@@ -284,7 +292,7 @@ giveback:
 			(obj->spe<0) ? "weak" : "strong");
 		break;
 	    case RIN_GAIN_CONSTITUTION:
-		pline("The water flow seems %ser now.",
+		pline_The("water flow seems %ser now.",
 			(obj->spe<0) ? "less" : "great");
 		break;
 	    case RIN_INCREASE_ACCURACY:	/* KMH */
@@ -302,8 +310,8 @@ giveback:
 		    if (otmp != uball && otmp != uchain &&
 			    !obj_resists(otmp, 1, 99)) {
 			if (!Blind) {
-			    pline("Suddenly, %s vanishes from the sink!",
-							doname(otmp));
+			    pline("Suddenly, %s %s from the sink!",
+				  doname(otmp), otense(otmp, "vanish"));
 			    ideed = TRUE;
 			}
 			delobj(otmp);
@@ -417,7 +425,7 @@ register const char *word;
 	}
 	if (obj->otyp == LEASH && obj->leashmon != 0) {
 		if (*word)
-			pline ("The leash is tied around your %s.",
+			pline_The("leash is tied around your %s.",
 					body_part(HAND));
 		return(FALSE);
 	}
@@ -446,7 +454,6 @@ register struct obj *obj;
 			return(0);
 		}
 		setuwep((struct obj *)0);
-		if(uwep) return 0; /* lifesaved and rewielded */
 	}
 	if(obj == uquiver) {
 		setuqwep((struct obj *)0);
@@ -458,10 +465,14 @@ register struct obj *obj;
 	if (u.uswallow) {
 		/* barrier between you and the floor */
 		if(flags.verbose)
-			You("drop %s into %s %s.", doname(obj),
-				s_suffix(mon_nam(u.ustuck)),
-				is_animal(u.ustuck->data) ?
-				"stomach" : "interior");
+		{
+			char buf[BUFSZ];
+
+			/* doname can call s_suffix, reusing its buffer */
+			Strcpy(buf, s_suffix(mon_nam(u.ustuck)));
+			You("drop %s into %s %s.", doname(obj), buf,
+				mbodypart(u.ustuck, STOMACH));
+		}
 	} else {
 #ifdef SINKS
 	    if((obj->oclass == RING_CLASS || obj->otyp == MEAT_RING) &&
@@ -491,8 +502,17 @@ void
 dropx(obj)
 register struct obj *obj;
 {
-	/* Money is usually not in our inventory */
+#ifndef GOLDOBJ
 	if (obj->oclass != GOLD_CLASS || obj == invent) freeinv(obj);
+#else
+        /* Ensure update when we drop gold objects */
+        if (obj->oclass == GOLD_CLASS) flags.botl = 1;
+	/* Money is usually not in our inventory */
+	/*if (obj->oclass != GOLD_CLASS || obj == invent)*/ 
+        /* !!!! make sure we don't drop "created" gold not in inventory any more,*/
+        /* or this will crash !!!! */
+        freeinv(obj);
+#endif
 	if (!u.uswallow && ship_object(obj, u.ux, u.uy, FALSE)) return;
 	dropy(obj);
 }
@@ -501,27 +521,59 @@ void
 dropy(obj)
 register struct obj *obj;
 {
+	if (obj == uwep) setuwep((struct obj *)0);
+	if (obj == uquiver) setuqwep((struct obj *)0);
+	if (obj == uswapwep) setuswapwep((struct obj *)0);
+
 	if (!u.uswallow && flooreffects(obj,u.ux,u.uy,"drop")) return;
-	/* KMH -- Fixed crysknives have only 10% chance of reverting */
-	if (obj->otyp == CRYSKNIFE && (!obj->oerodeproof || !rn2(10))) {
-		obj->otyp = WORM_TOOTH;
-		obj->oerodeproof = 0;
-	}
 	/* uswallow check done by GAN 01/29/87 */
 	if(u.uswallow) {
-		if (obj != uball) {		/* mon doesn't pick up ball */
-		    mpickobj(u.ustuck,obj);
+	    boolean could_petrify;
+	    if (obj != uball) {		/* mon doesn't pick up ball */
+		could_petrify = obj->otyp == CORPSE &&
+		    touch_petrifies(&mons[obj->corpsenm]);
+		(void) mpickobj(u.ustuck,obj);
+		if (could_petrify && is_animal(u.ustuck->data)) {
+		    minstapetrify(u.ustuck, TRUE);
+		    /* Don't leave a cockatrice corpse available in a statue */
+		    if (!u.uswallow) delobj(obj);
 		}
+	    }
 	} else  {
-		place_object(obj, u.ux, u.uy);
-		if (obj == uball)
-		    drop_ball(u.ux,u.uy);
-		else
-		    sellobj(obj, u.ux, u.uy);
-		stackobj(obj);
-		if(Blind && Levitation)
-		    map_object(obj, 0);
-		newsym(u.ux,u.uy);	/* remap location under self */
+	    place_object(obj, u.ux, u.uy);
+	    if (obj == uball)
+		drop_ball(u.ux,u.uy);
+	    else
+		sellobj(obj, u.ux, u.uy);
+	    stackobj(obj);
+	    if(Blind && Levitation)
+		map_object(obj, 0);
+	    newsym(u.ux,u.uy);	/* remap location under self */
+	}
+}
+
+/* things that must change when not held; recurse into containers.
+   Called for both player and monsters */
+void
+obj_no_longer_held(obj)
+struct obj *obj;
+{
+	if (!obj) {
+	    return;
+	} else if ((Is_container(obj) || obj->otyp == STATUE) && obj->cobj) {
+	    struct obj *contents;
+	    for(contents=obj->cobj; contents; contents=contents->nobj)
+		obj_no_longer_held(contents);
+	}
+	switch(obj->otyp) {
+	case CRYSKNIFE:
+	    /* KMH -- Fixed crysknives have only 10% chance of reverting */
+	    /* only changes when not held by player or monster */
+	    if (!obj->oerodeproof || !rn2(10)) {
+		obj->otyp = WORM_TOOTH;
+		obj->oerodeproof = 0;
+	    }
+	    break;
 	}
 }
 
@@ -532,11 +584,11 @@ doddrop()
 	int result = 0;
 
 	add_valid_menu_class(0); /* clear any classes already there */
-	if (*u.ushops) sellobj_state(TRUE);
+	if (*u.ushops) sellobj_state(SELL_DELIBERATE);
 	if (flags.menu_style != MENU_TRADITIONAL ||
-		(result = ggetobj("drop", drop, 0, FALSE)) < -1)
+		(result = ggetobj("drop", drop, 0, FALSE, (unsigned *)0)) < -1)
 	    result = menu_drop(result);
-	if (*u.ushops) sellobj_state(FALSE);
+	if (*u.ushops) sellobj_state(SELL_NORMAL);
 	reset_occupations();
 
 	return result;
@@ -549,11 +601,15 @@ int retry;
 {
     int n, i, n_dropped = 0;
     long cnt;
-    struct obj *otmp, *otmp2, *u_gold = 0;
+    struct obj *otmp, *otmp2;
+#ifndef GOLDOBJ
+    struct obj *u_gold = 0;
+#endif
     menu_item *pick_list;
     boolean all_categories = TRUE;
     boolean drop_everything = FALSE;
 
+#ifndef GOLDOBJ
     if (u.ugold) {
 	/* Hack: gold is not in the inventory, so make a gold object
 	   and put it at the head of the inventory list. */
@@ -563,14 +619,15 @@ int retry;
 	u_gold->nobj = invent;
 	invent = u_gold;
     }
-
+#endif
     if (retry) {
 	all_categories = (retry == -2);
     } else if (flags.menu_style == MENU_FULL) {
 	all_categories = FALSE;
 	n = query_category("Drop what type of items?",
 			invent,
-			UNPAID_TYPES | ALL_TYPES | CHOOSE_ALL,
+			UNPAID_TYPES | ALL_TYPES | CHOOSE_ALL |
+			BUC_BLESSED | BUC_CURSED | BUC_UNCURSED | BUC_UNKNOWN,
 			&pick_list, PICK_ANY);
 	if (!n) goto drop_done;
 	for (i = 0; i < n; i++) {
@@ -583,10 +640,15 @@ int retry;
 	}
 	free((genericptr_t) pick_list);
     } else if (flags.menu_style == MENU_COMBINATION) {
+	unsigned ggoresults = 0;
 	all_categories = FALSE;
 	/* Gather valid classes via traditional NetHack method */
-	i = ggetobj("drop", drop, 0, TRUE);
+	i = ggetobj("drop", drop, 0, TRUE, &ggoresults);
 	if (i == -2) all_categories = TRUE;
+	if (ggoresults & ALL_FINISHED) {
+		n_dropped = i;
+		goto drop_done;
+	}
     }
 
     if (drop_everything) {
@@ -605,11 +667,12 @@ int retry;
 		cnt = pick_list[i].count;
 		if (cnt < otmp->quan && !welded(otmp) &&
 			(!otmp->cursed || otmp->otyp != LOADSTONE)) {
-		    otmp2 = splitobj(otmp, cnt);
-		    /* assume other worn items aren't mergable */
-		    if (otmp == uwep) setuwep(otmp2);
-			if (otmp == uquiver) setuqwep(otmp2);
-			if (otmp == uswapwep) setuswapwep(otmp2);
+#ifndef GOLDOBJ
+		    if (otmp->oclass == GOLD_CLASS)
+			(void) splitobj(otmp, otmp->quan - cnt);
+		    else
+#endif
+		    otmp = splitobj(otmp, cnt);
 		}
 		n_dropped += drop(otmp);
 	    }
@@ -618,12 +681,15 @@ int retry;
     }
 
  drop_done:
+#ifndef GOLDOBJ
     if (u_gold && invent && invent->oclass == GOLD_CLASS) {
 	/* didn't drop [all of] it */
 	u_gold = invent;
 	invent = u_gold->nobj;
 	dealloc_obj(u_gold);
+	update_inventory();
     }
+#endif
     return n_dropped;
 }
 
@@ -641,11 +707,33 @@ dodown()
 		    (u.ux == sstairs.sx && u.uy == sstairs.sy && !sstairs.up)),
 		ladder_down = (u.ux == xdnladder && u.uy == ydnladder);
 
+#ifdef STEED
+	if (u.usteed && !u.usteed->mcanmove) {
+		pline("%s won't move!", Monnam(u.usteed));
+		return(0);
+	} else if (u.usteed && u.usteed->meating) {
+		pline("%s is still eating.", Monnam(u.usteed));
+		return(0);
+	} else
+#endif
 	if (Levitation) {
 	    if ((HLevitation & I_SPECIAL) || (ELevitation & W_ARTI)) {
 		/* end controlled levitation */
-			if (float_down(I_SPECIAL|TIMEOUT, W_ARTI))
-			    return (1);   /* came down, so moved */
+		if (ELevitation & W_ARTI) {
+		    struct obj *obj;
+
+		    for(obj = invent; obj; obj = obj->nobj) {
+			if (obj->oartifact &&
+					artifact_has_invprop(obj,LEVITATION)) {
+			    if (obj->age < monstermoves)
+				obj->age = monstermoves + rnz(100);
+			    else
+				obj->age += rnz(100);
+			}
+		    }
+		}
+		if (float_down(I_SPECIAL|TIMEOUT, W_ARTI))
+		    return (1);   /* came down, so moved */
 	    }
 	    floating_above(stairs_down ? "stairs" : ladder_down ?
 			   "ladder" : surface(u.ux, u.uy));
@@ -660,7 +748,9 @@ dodown()
 		}
 	}
 	if(u.ustuck) {
-		You("are being held, and cannot go down.");
+		You("are %s, and cannot go down.",
+			!u.uswallow ? "being held" : is_animal(u.ustuck->data) ?
+			"swallowed" : "engulfed");
 		return(1);
 	}
 	if (on_level(&valley_level, &u.uz) && !u.uevent.gehennom_entered) {
@@ -702,8 +792,19 @@ doup()
 		You_cant("go up here.");
 		return(0);
 	}
+#ifdef STEED
+	if (u.usteed && !u.usteed->mcanmove) {
+		pline("%s won't move!", Monnam(u.usteed));
+		return(0);
+	} else if (u.usteed && u.usteed->meating) {
+		pline("%s is still eating.", Monnam(u.usteed));
+		return(0);
+	} else
+#endif
 	if(u.ustuck) {
-		You("are being held, and cannot go up.");
+		You("are %s, and cannot go up.",
+			!u.uswallow ? "being held" : is_animal(u.ustuck->data) ?
+			"swallowed" : "engulfed");
 		return(1);
 	}
 	if(near_capacity() > SLT_ENCUMBER) {
@@ -735,7 +836,7 @@ currentlevel_rewrite()
 	register int fd;
 
 	/* since level change might be a bit slow, flush any buffered screen
-	 *  output (like "you fall through a trapdoor") */
+	 *  output (like "you fall through a trap door") */
 	mark_synch();
 
 	fd = create_levelfile(ledger_no(&u.uz));
@@ -873,7 +974,7 @@ boolean at_stairs, falling, portal;
 	fd = currentlevel_rewrite();
 	if (fd < 0) return;
 
-	if (falling) /* assuming this is only trapdoor or hole */
+	if (falling) /* assuming this is only trap door or hole */
 	    impact_drop((struct obj *)0, u.ux, u.uy, newlevel->dlevel);
 
 	check_special_room(TRUE);		/* probably was a trap door */
@@ -1018,6 +1119,12 @@ boolean at_stairs, falling, portal;
 			}
 		    }
 		    losehp(rnd(3), "falling downstairs", KILLED_BY);
+#ifdef STEED
+		    if (u.usteed) {
+			dismount_steed(DISMOUNT_FELL);
+			if (Punished) unplacebc();
+		    }
+#endif
 		    selftouch("Falling, you");
 		} else if (u.dz && at_ladder)
 		    You("climb down the ladder.");
@@ -1124,22 +1231,27 @@ boolean at_stairs, falling, portal;
 	    static const char *fam_msgs[4] = {
 		"You have a sense of deja vu.",
 		"You feel like you've been here before.",
-		"This place looks familiar...",
+		"This place %s familiar...",
 		0	/* no message */
 	    };
 	    static const char *halu_fam_msgs[4] = {
-		"Whoa!  Everything looks different.",
+		"Whoa!  Everything %s different.",
 		"You are surrounded by twisty little passages, all alike.",
-		"Gee, this looks like uncle Conan's place...",
+		"Gee, this %s like uncle Conan's place...",
 		0	/* no message */
 	    };
 	    const char *mesg;
+	    char buf[BUFSZ];
 	    int which = rn2(4);
 
 	    if (Hallucination)
 		mesg = halu_fam_msgs[which];
 	    else
 		mesg = fam_msgs[which];
+	    if (mesg && index(mesg, '%')) {
+		Sprintf(buf, mesg, !Blind ? "looks" : "seems");
+		mesg = buf;
+	    }
 	    if (mesg) pline(mesg);
 	}
 
@@ -1155,7 +1267,7 @@ boolean at_stairs, falling, portal;
 
 	/* the message from your quest leader */
 	if (!In_quest(&u.uz0) && at_dgn_entrance("The Quest") &&
-		!(u.uevent.qexpelled || u.uevent.qcompleted || leaderless())) {
+		!(u.uevent.qexpelled || u.uevent.qcompleted || quest_status.leader_is_dead)) {
 
 		if (u.uevent.qcalled) {
 			com_pager(Role_if(PM_ROGUE) ? 4 : 3);
@@ -1170,7 +1282,7 @@ boolean at_stairs, falling, portal;
 		You("penetrated a high security area!");
 		pline("An alarm sounds!");
 		for(mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-		    if (mtmp->msleeping) mtmp->msleeping = 0;
+		    if (!DEADMONSTER(mtmp) && mtmp->msleeping) mtmp->msleeping = 0;
 	}
 
 	if (on_level(&u.uz, &astral_level))
@@ -1183,7 +1295,7 @@ boolean at_stairs, falling, portal;
 	save_currentstate();
 #endif
 
-	pickup(1);
+	(void) pickup(1);
 }
 
 STATIC_OVL void
@@ -1196,7 +1308,7 @@ final_level()
 
 	/* reset monster hostility relative to player */
 	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-	    reset_hostility(mtmp);
+	    if (!DEADMONSTER(mtmp)) reset_hostility(mtmp);
 
 	/* create some player-monsters */
 	create_mplayers(rn1(4, 3), TRUE);
@@ -1235,7 +1347,8 @@ final_level()
 					d((int)mtmp->m_lev,10) + 30 + rnd(30);
 		    if ((otmp = select_hwep(mtmp)) == 0) {
 			otmp = mksobj(SILVER_SABER, FALSE, FALSE);
-			mpickobj(mtmp, otmp);
+			if (mpickobj(mtmp, otmp))
+			    panic("merged weapon?");
 		    }
 		    bless(otmp);
 		    if (otmp->spe < 4) otmp->spe += rnd(4);
@@ -1320,12 +1433,22 @@ struct obj *corpse;
     boolean is_uwep, chewed;
     xchar where;
     char *cname, cname_buf[BUFSZ];
-
+    struct obj *container = (struct obj *)0;
+    int container_where = 0;
+    
     where = corpse->where;
     is_uwep = corpse == uwep;
     cname = eos(strcpy(cname_buf, "bite-covered "));
     Strcpy(cname, corpse_xname(corpse, TRUE));
     mcarry = (where == OBJ_MINVENT) ? corpse->ocarry : 0;
+
+    if (where == OBJ_CONTAINED) {
+    	struct monst *mtmp2 = (struct monst *)0;
+	container = corpse->ocontainer;
+    	mtmp2 = get_container_location(container, &container_where, (int *)0);
+	/* container_where is the outermost container's location even if nested */
+	if (container_where == OBJ_MINVENT && mtmp2) mcarry = mtmp2;
+    }
     mtmp = revive(corpse);	/* corpse is gone if successful */
 
     if (mtmp) {
@@ -1355,7 +1478,27 @@ struct obj *corpse;
 			      Adjmonnam(mtmp, "bite-covered") : Monnam(mtmp));
 		}
 		break;
-
+	   case OBJ_CONTAINED:
+	   	if (container_where == OBJ_MINVENT && cansee(mtmp->mx, mtmp->my) &&
+		    mcarry && canseemon(mcarry) && container) {
+		        char sackname[BUFSZ];
+		        Sprintf(sackname, "%s %s", s_suffix(mon_nam(mcarry)),
+				xname(container)); 
+	   		pline("%s writhes out of %s!", Amonnam(mtmp), sackname);
+	   	} else if (container_where == OBJ_INVENT && container) {
+		        char sackname[BUFSZ];
+		        Strcpy(sackname, an(xname(container)));
+	   		pline("%s %s out of %s in your pack!",
+	   			Blind ? Something : Amonnam(mtmp),
+				locomotion(mtmp->data,"writhes"),
+	   			sackname);
+	   	} else if (container_where == OBJ_FLOOR && container &&
+		            cansee(mtmp->mx, mtmp->my)) {
+		        char sackname[BUFSZ];
+		        Strcpy(sackname, an(xname(container)));
+			pline("%s escapes from %s!", Amonnam(mtmp), sackname);
+		}
+		break;
 	    default:
 		/* we should be able to handle the other cases... */
 		impossible("revive_corpse: lost corpse @ %d", where);
