@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)do_name.c	3.3	1999/10/10	*/
+/*	SCCS Id: @(#)do_name.c	3.4	2002/01/17	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -77,6 +77,7 @@ const char *goal;
 	    break;
 	}
 	if(c == 0) {
+	    if (!isok(tx, ty)) continue;
 	    /* a mouse click event, just assign and return */
 	    cx = tx;
 	    cy = ty;
@@ -154,7 +155,7 @@ const char *goal;
 			    }	/* column */
 			}	/* row */
 		    }		/* pass */
-		    pline("Can't find dungeon feature '%c'", c);
+		    pline("Can't find dungeon feature '%c'.", c);
 		    msg_given = TRUE;
 		    goto nxtc;
 		} else {
@@ -267,10 +268,7 @@ do_mname()
 		return(0);
 	}
 	/* special case similar to the one in lookat() */
-	if (mtmp->data != &mons[PM_HIGH_PRIEST])
-	    Strcpy(buf, x_monnam(mtmp, 0, (char *)0, 1));
-	else
-	    Sprintf(buf, "the high priest%s", mtmp->female ? "ess" : "");
+	(void) distant_monnam(mtmp, ARTICLE_THE, buf);
 	Sprintf(qbuf, "What do you want to call %s?", buf);
 	getlin(qbuf,buf);
 	if(!*buf || *buf == '\033') return(0);
@@ -297,7 +295,8 @@ register struct obj *obj;
 	const char *aname;
 	short objtyp;
 
-	Sprintf(qbuf, "What do you want to name %s?", doname(obj));
+	Sprintf(qbuf, "What do you want to name %s %s?",
+		is_plural(obj) ? "these" : "this", xname(obj));
 	getlin(qbuf, buf);
 	if(!*buf || *buf == '\033')	return;
 	/* strip leading and trailing spaces; unnames item if all spaces */
@@ -322,8 +321,6 @@ register struct obj *obj;
 		You("engrave: \"%s\".",buf);
 	}
 	obj = oname(obj, buf);
-	if (obj->where == OBJ_INVENT)
-		update_inventory();
 }
 
 /*
@@ -424,6 +421,8 @@ const char *name;
 			      (genericptr_t)obj->oextra, lth, name);
 	}
 	if (lth) artifact_exists(obj, name, TRUE);
+	if (obj->oartifact && obj == uswapwep) untwoweapon();
+	if (carried(obj)) update_inventory();
 	return obj;
 }
 
@@ -538,8 +537,10 @@ rndghostname()
  * x_monnam is the generic monster-naming function.
  *		  seen	      unseen	   detected		  named
  * mon_nam:	the newt	it	the invisible orc	Fido
+ * noit_mon_nam:the newt (as if detected) the invisible orc	Fido
  * l_monnam:	newt		it	invisible orc		dog called fido
  * Monnam:	The newt	It	The invisible orc	Fido
+ * noit_Monnam: The newt (as if detected) The invisible orc	Fido
  * Adjmonnam:	The poor newt	It	The poor invisible orc	The poor Fido
  * Amonnam:	A newt		It	An invisible orc	Fido
  * a_monnam:	a newt		it	an invisible orc	Fido
@@ -547,20 +548,26 @@ rndghostname()
  * y_monnam:	your newt     your xan	your invisible orc	Fido
  */
 
+/* Bug: if the monster is a priest or shopkeeper, not every one of these
+ * options works, since those are special cases.
+ */
 char *
-x_monnam(mtmp, article, adjective, called)
+x_monnam(mtmp, article, adjective, suppress, called)
 register struct monst *mtmp;
-/* Articles:
- * 0: "the" in front of everything except names and "it"
- * 1: "the" in front of everything except "it"; looks bad for names unless you
- *    are also using an adjective.
- * 2: "a" in front of everything except "it".
- * 3: no article at all.
- * 4: no article; "it" and invisible suppressed.
- * 5: "your" instead of an article; include "invisible" even when unseen
+int article;
+/* ARTICLE_NONE, ARTICLE_THE, ARTICLE_A: obvious
+ * ARTICLE_YOUR: "your" on pets, "the" on everything else
+ *
+ * If the monster would be referred to as "it" or if the monster has a name
+ * _and_ there is no adjective, "invisible", "saddled", etc., override this
+ * and always use no article.
  */
-int article, called;
 const char *adjective;
+int suppress;
+/* SUPPRESS_IT, SUPPRESS_INVISIBLE, SUPPRESS_HALLUCINATION, SUPPRESS_SADDLE.
+ * EXACT_NAME: combination of all the above
+ */
+boolean called;
 {
 #ifdef LINT	/* static char buf[BUFSZ]; */
 	char buf[BUFSZ];
@@ -568,102 +575,165 @@ const char *adjective;
 	static char buf[BUFSZ];
 #endif
 	struct permonst *mdat = mtmp->data;
-	char *name = 0;
-	int force_the = 0, trunam = (article == 4), use_your = (article == 5);
+	boolean do_hallu, do_invis, do_it, do_saddle;
+	boolean name_at_start, has_adjectives;
+	char *bp;
 
-	buf[0] = '\0';
-	if (use_your && !mtmp->mtame)
-	    use_your = 0,  article = 0;
+	if (program_state.gameover)
+	    suppress |= SUPPRESS_HALLUCINATION;
+	if (article == ARTICLE_YOUR && !mtmp->mtame)
+	    article = ARTICLE_THE;
+
+	do_hallu = Hallucination && !(suppress & SUPPRESS_HALLUCINATION);
+	do_invis = mtmp->minvis && !(suppress & SUPPRESS_INVISIBLE);
+	do_it = !canspotmon(mtmp) && 
+	    article != ARTICLE_YOUR &&
+	    !program_state.gameover &&
+#ifdef STEED
+	    mtmp != u.usteed &&
+#endif
+	    !(u.uswallow && mtmp == u.ustuck) &&
+	    !(suppress & SUPPRESS_IT);
+	do_saddle = !(suppress & SUPPRESS_SADDLE);
+
+	buf[0] = 0;
+
+	/* unseen monsters, etc.  Use "it" */
+	if (do_it) {
+	    Strcpy(buf, "it");
+	    return buf;
+	}
+
+	/* priests and minions: don't even use this function */
 	if (mtmp->ispriest || mtmp->isminion) {
 	    char priestnambuf[BUFSZ];
+	    char *name;
 	    long save_prop = EHalluc_resistance;
+	    unsigned save_invis = mtmp->minvis;
 
 	    /* when true name is wanted, explicitly block Hallucination */
-	    if (trunam) EHalluc_resistance = 1L;
+	    if (!do_hallu) EHalluc_resistance = 1L;
+	    if (!do_invis) mtmp->minvis = 0;
 	    name = priestname(mtmp, priestnambuf);
 	    EHalluc_resistance = save_prop;
-	    if ((trunam || article == 3) && !strncmp(name, "the ", 4))
+	    mtmp->minvis = save_invis;
+	    if (article == ARTICLE_NONE && !strncmp(name, "the ", 4))
 		name += 4;
 	    return strcpy(buf, name);
 	}
-	if (!trunam && !use_your && !canspotmon(mtmp) &&
-#ifdef STEED
-		(mtmp != u.usteed) &&
-#endif
-		!(u.uswallow && mtmp == u.ustuck) && !killer) {
-	    if (!mon_visible(mtmp) || (!cansee(bhitpos.x,bhitpos.y) &&
-!see_with_infrared(mtmp))) {
-		Strcpy(buf, "it");
-		return  buf;
-	    }
-	}
-	if (trunam || !Hallucination) {
-	    if (mtmp->isshk) {
-		Strcpy(buf, shkname(mtmp));
-		if ((mdat == &mons[PM_SHOPKEEPER] && !mtmp->minvis) || trunam)
-		    return buf;
-		/* For normal shopkeepers, just 'Asidonhopo'.
-		 * For unusual ones, 'Asidonhopo the invisible shopkeeper'
-		 * or 'Asidonhopo the blue dragon'.
-		 */
-		Strcat(buf, " ");
-	    } else if (mtmp->mnamelth) {
-		name = NAME(mtmp);
-		if (mdat == &mons[PM_GHOST]) {
-		    called = 0;
-		    Sprintf(buf, "%s ghost", s_suffix(name));
-		    goto bot_nam;
-		}
-	    }
-	}
 
-	if (!trunam) {
-	    force_the = (!Hallucination &&
-			 (mdat == &mons[PM_WIZARD_OF_YENDOR] || mtmp->isshk));
-	    if (force_the ||
-		    ((article == 1 || ((!name || called) && article == 0)) &&
-			(Hallucination || !type_is_pname(mdat))))
-		Strcat(buf, "the ");
-	    else if (use_your && !name)
-		Strcat(buf, "your ");
-	    if (adjective)
+	/* Shopkeepers: use shopkeeper name.  For normal shopkeepers, just
+	 * "Asidonhopo"; for unusual ones, "Asidonhopo the invisible
+	 * shopkeeper" or "Asidonhopo the blue dragon".  If hallucinating,
+	 * none of this applies.
+	 */
+	if (mtmp->isshk && !do_hallu) {
+	    if (adjective && article == ARTICLE_THE) {
+		/* pathological case: "the angry Asidonhopo the blue dragon"
+		   sounds silly */
+		Strcpy(buf, "the ");
 		Strcat(strcat(buf, adjective), " ");
-	    if (mtmp->minvis && !Blind)
+		Strcat(buf, shkname(mtmp));
+		return buf;
+	    }
+	    Strcat(buf, shkname(mtmp));
+	    if (mdat == &mons[PM_SHOPKEEPER] && !do_invis)
+		return buf;
+	    Strcat(buf, " the ");
+	    if (do_invis)
 		Strcat(buf, "invisible ");
+	    Strcat(buf, mdat->mname);
+	    return buf;
+	}
+
+	/* Put the adjectives in the buffer */
+	if (adjective)
+	    Strcat(strcat(buf, adjective), " ");
+	if (do_invis)
+	    Strcat(buf, "invisible ");
 #ifdef STEED
-	    if ((mtmp->misc_worn_check & W_SADDLE) &&
-			!Blind && (!name || called) && mtmp != u.usteed)
-		Strcat(buf, "saddled ");
+	if (do_saddle && (mtmp->misc_worn_check & W_SADDLE) &&
+	    !Blind && !Hallucination)
+	    Strcat(buf, "saddled ");
 #endif
-	}
+	if (buf[0] != 0)
+	    has_adjectives = TRUE;
+	else
+	    has_adjectives = FALSE;
 
-	if (name && !called) {
-	    Strcat(buf, name);
-	    goto bot_nam;
-	}
-
-	if (Hallucination && !trunam) {
+	/* Put the actual monster name or type into the buffer now */
+	/* Be sure to remember whether the buffer starts with a name */
+	if (do_hallu) {
 	    Strcat(buf, rndmonnam());
+	    name_at_start = FALSE;
+	} else if (mtmp->mnamelth) {
+	    char *name = NAME(mtmp);
+
+	    if (mdat == &mons[PM_GHOST]) {
+		Sprintf(eos(buf), "%s ghost", s_suffix(name));
+		name_at_start = TRUE;
+	    } else if (called) {
+		Sprintf(eos(buf), "%s called %s", mdat->mname, name);
+		name_at_start = (boolean)type_is_pname(mdat);
+	    } else if (is_mplayer(mdat) && (bp = strstri(name, " the ")) != 0) {
+		/* <name> the <adjective> <invisible> <saddled> <rank> */
+		char pbuf[BUFSZ];
+
+		Strcpy(pbuf, name);
+		pbuf[bp - name + 5] = '\0'; /* adjectives right after " the " */
+		if (has_adjectives)
+		    Strcat(pbuf, buf);
+		Strcat(pbuf, bp + 5);	/* append the rest of the name */
+		Strcpy(buf, pbuf);
+		article = ARTICLE_NONE;
+		name_at_start = TRUE;
+	    } else {
+		Strcat(buf, name);
+		name_at_start = TRUE;
+	    }
 	} else if (is_mplayer(mdat) && !In_endgame(&u.uz)) {
 	    char pbuf[BUFSZ];
 	    Strcpy(pbuf, rank_of((int)mtmp->m_lev,
 				 monsndx(mdat),
 				 (boolean)mtmp->female));
 	    Strcat(buf, lcase(pbuf));
+	    name_at_start = FALSE;
 	} else {
 	    Strcat(buf, mdat->mname);
+	    name_at_start = (boolean)type_is_pname(mdat);
 	}
 
-	if (name) {	/* if we reach here, `name' implies `called' */
-	    Strcat(buf, " called ");
-	    Strcat(buf, NAME(mtmp));
+	if (name_at_start && !has_adjectives) {
+	    if (mdat == &mons[PM_WIZARD_OF_YENDOR])
+		article = ARTICLE_THE;
+	    else
+		article = ARTICLE_NONE;
+	} else if (mons[monsndx(mdat)].geno & G_UNIQ &&
+		   article == ARTICLE_A) {
+	    article = ARTICLE_THE;
+	}
+
+	{
+	    char buf2[BUFSZ];
+
+	    switch(article) {
+		case ARTICLE_YOUR:
+		    Strcpy(buf2, "your ");
+		    Strcat(buf2, buf);
+		    Strcpy(buf, buf2);
+		    return buf;
+		case ARTICLE_THE:
+		    Strcpy(buf2, "the ");
+		    Strcat(buf2, buf);
+		    Strcpy(buf, buf2);
+		    return buf;
+		case ARTICLE_A:
+		    return(an(buf));
+		case ARTICLE_NONE:
+		default:
+		    return buf;
+	    }
 	}
- bot_nam:
-	if (article == 2 && !force_the && (!name || called) &&
-		(Hallucination || !type_is_pname(mdat)))
-	    return an(buf);
-	else
-	    return buf;
 }
 
 #endif /* OVL0 */
@@ -673,7 +743,8 @@ char *
 l_monnam(mtmp)
 register struct monst *mtmp;
 {
-	return(x_monnam(mtmp, 3, (char *)0, 1));
+	return(x_monnam(mtmp, ARTICLE_NONE, (char *)0, 
+		mtmp->mnamelth ? SUPPRESS_SADDLE : 0, TRUE));
 }
 
 #endif /* OVLB */
@@ -683,7 +754,21 @@ char *
 mon_nam(mtmp)
 register struct monst *mtmp;
 {
-	return(x_monnam(mtmp, 0, (char *)0, 0));
+	return(x_monnam(mtmp, ARTICLE_THE, (char *)0,
+		mtmp->mnamelth ? SUPPRESS_SADDLE : 0, FALSE));
+}
+
+/* print the name as if mon_nam() was called, but assume that the player
+ * can always see the monster--used for probing and for monsters aggravating
+ * the player with a cursed potion of invisibility
+ */
+char *
+noit_mon_nam(mtmp)
+register struct monst *mtmp;
+{
+	return(x_monnam(mtmp, ARTICLE_THE, (char *)0,
+		mtmp->mnamelth ? (SUPPRESS_SADDLE|SUPPRESS_IT) :
+		    SUPPRESS_IT, FALSE));
 }
 
 char *
@@ -696,18 +781,22 @@ register struct monst *mtmp;
 	return(bp);
 }
 
+char *
+noit_Monnam(mtmp)
+register struct monst *mtmp;
+{
+	register char *bp = noit_mon_nam(mtmp);
+
+	*bp = highc(*bp);
+	return(bp);
+}
+
 /* monster's own name */
 char *
 m_monnam(mtmp)
 struct monst *mtmp;
 {
-	char *result;
-	unsigned save_invis = mtmp->minvis;
-
-	mtmp->minvis = 0;  /* affects priestname() as well as x_monnam() */
-	result = x_monnam(mtmp, 4, (char *)0, 0);
-	mtmp->minvis = save_invis;
-	return result;
+	return x_monnam(mtmp, ARTICLE_NONE, (char *)0, EXACT_NAME, FALSE);
 }
 
 /* pet name: "your little dog" */
@@ -715,7 +804,8 @@ char *
 y_monnam(mtmp)
 struct monst *mtmp;
 {
-	return x_monnam(mtmp, 5, (char *)0, 0);
+	return x_monnam(mtmp, ARTICLE_YOUR, (char *)0, 
+		mtmp->mnamelth ? SUPPRESS_SADDLE : 0, FALSE);
 }
 
 #endif /* OVL0 */
@@ -726,7 +816,8 @@ Adjmonnam(mtmp, adj)
 register struct monst *mtmp;
 register const char *adj;
 {
-	register char *bp = x_monnam(mtmp,1,adj,0);
+	register char *bp = x_monnam(mtmp, ARTICLE_THE, adj,
+		mtmp->mnamelth ? SUPPRESS_SADDLE : 0, FALSE);
 
 	*bp = highc(*bp);
 	return(bp);
@@ -736,7 +827,8 @@ char *
 a_monnam(mtmp)
 register struct monst *mtmp;
 {
-	return x_monnam(mtmp, 2, (char *)0, 0);
+	return x_monnam(mtmp, ARTICLE_A, (char *)0,
+		mtmp->mnamelth ? SUPPRESS_SADDLE : 0, FALSE);
 }
 
 char *
@@ -749,6 +841,27 @@ register struct monst *mtmp;
 	return(bp);
 }
 
+/* used for monster ID by the '/', ';', and 'C' commands to block remote
+   identification of the endgame altars via their attending priests */
+char *
+distant_monnam(mon, article, outbuf)
+struct monst *mon;
+int article;	/* only ARTICLE_NONE and ARTICLE_THE are handled here */
+char *outbuf;
+{
+    /* high priest(ess)'s identity is concealed on the Astral Plane,
+       unless you're adjacent (overridden for hallucination which does
+       its own obfuscation) */
+    if (mon->data == &mons[PM_HIGH_PRIEST] && !Hallucination &&
+	    Is_astralevel(&u.uz) && distu(mon->mx, mon->my) > 2) {
+	Strcpy(outbuf, article == ARTICLE_THE ? "the " : "");
+	Strcat(outbuf, mon->female ? "high priestess" : "high priest");
+    } else {
+	Strcpy(outbuf, x_monnam(mon, article, (char *)0, 0, TRUE));
+    }
+    return outbuf;
+}
+
 static const char *bogusmons[] = {
 	"jumbo shrimp", "giant pigmy", "gnu", "killer penguin",
 	"giant cockroach", "giant slug", "maggot", "pterodactyl",
@@ -756,7 +869,7 @@ static const char *bogusmons[] = {
 	"efreeti", "marid", "rot grub", "bookworm", "master lichen",
 	"shadow", "hologram", "jester", "attorney", "sleazoid",
 	"killer tomato", "amazon", "robot", "battlemech",
-	"rhinovirus", "harpy", "lion-dog", "rat-ant",
+	"rhinovirus", "harpy", "lion-dog", "rat-ant", "Y2K bug",
 						/* misc. */
 	"grue", "Christmas-tree monster", "luck sucker", "paskald",
 	"brogmoid", "dornbeast",		/* Quendor (Zork, &c.) */
@@ -803,8 +916,14 @@ static const char *bogusmons[] = {
 	"mother-in-law"				/* common pest */
 };
 
+
+/* Return a random monster name, for hallucination.
+ * KNOWN BUG: May be a proper name (Godzilla, Barney), may not
+ * (the Terminator, a Dalek).  There's no elegant way to deal
+ * with this without radically modifying the calling functions.
+ */
 const char *
-rndmonnam()	/* Random name of monster type, if hallucinating */
+rndmonnam()
 {
 	int name;
 
@@ -817,38 +936,13 @@ rndmonnam()	/* Random name of monster type, if hallucinating */
 	return mons[name].mname;
 }
 
-const char *pronoun_pairs[][2] = {
-	{"him", "her"}, {"Him", "Her"}, {"his", "her"}, {"His", "Her"},
-	{"he", "she"}, {"He", "She"},
-	{0, 0}
-};
-
-char *
-self_pronoun(str, pronoun)
-const char *str;
-const char *pronoun;
-{
-	static NEARDATA char buf[BUFSZ];
-	register int i;
-
-	for(i=0; pronoun_pairs[i][0]; i++) {
-		if(!strncmp(pronoun, pronoun_pairs[i][0], 3)) {
-			Sprintf(buf, str, pronoun_pairs[i][flags.female]);
-			return buf;
-		}
-	}
-	impossible("never heard of pronoun %s?", pronoun);
-	Sprintf(buf, str, pronoun_pairs[i][0]);
-	return buf;
-}
-
 #ifdef REINCARNATION
 const char *
 roguename() /* Name of a Rogue player */
 {
 	char *i, *opts;
 
-	if ((opts = getenv("ROGUEOPTS")) != 0) {
+	if ((opts = nh_getenv("ROGUEOPTS")) != 0) {
 		for (i = opts; *i; i++)
 			if (!strncmp("name=",i,5)) {
 				char *j;
@@ -902,14 +996,16 @@ static const char *coynames[] = {
 	"Nemesis Riduclii","Canis latrans"
 };
 	
-char *coyotename(buf)
+char *coyotename(mtmp, buf)
+struct monst *mtmp;
 char *buf;
 {
-	if (buf)
-		Sprintf(buf,
-			"coyote - %s",
-			coynames[rn2(SIZE(coynames)-1)]);
-	return buf;
+    if (mtmp && buf) {
+	Sprintf(buf, "%s - %s",
+	    x_monnam(mtmp, ARTICLE_NONE, (char *)0, 0, TRUE),
+	    mtmp->mcan ? coynames[SIZE(coynames)-1] : coynames[rn2(SIZE(coynames)-1)]);
+    }
+    return buf;
 }
 #endif /* OVL2 */
 
