@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)dog.c	3.2	96/02/11	*/
+/*	SCCS Id: @(#)dog.c	3.4	2002/03/09	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -7,7 +7,7 @@
 
 #ifdef OVLB
 
-static int NDECL(pet_type);
+STATIC_DCL int NDECL(pet_type);
 
 void
 initedog(mtmp)
@@ -15,6 +15,7 @@ register struct monst *mtmp;
 {
 	mtmp->mtame = is_domestic(mtmp->data) ? 10 : 5;
 	mtmp->mpeaceful = 1;
+	mtmp->mavenge = 0;
 	set_malign(mtmp); /* recalc alignment now that it's tamed */
 	mtmp->mleashed = 0;
 	mtmp->meating = 0;
@@ -22,41 +23,33 @@ register struct monst *mtmp;
 	EDOG(mtmp)->dropdist = 10000;
 	EDOG(mtmp)->apport = 10;
 	EDOG(mtmp)->whistletime = 0;
-	EDOG(mtmp)->hungrytime = 1000 + moves;
+	EDOG(mtmp)->hungrytime = 1000 + monstermoves;
 	EDOG(mtmp)->ogoal.x = -1;	/* force error if used before set */
 	EDOG(mtmp)->ogoal.y = -1;
+	EDOG(mtmp)->abuse = 0;
+	EDOG(mtmp)->revivals = 0;
+	EDOG(mtmp)->mhpmax_penalty = 0;
+	EDOG(mtmp)->killed_by_u = 0;
 }
 
-static int
+STATIC_OVL int
 pet_type()
 {
-	register int pettype;
-
-	switch (u.role) {
-		/* some character classes have restricted ideas of pets */
-		case 'C':
-		case 'S':
-			pettype = PM_LITTLE_DOG;
-			break;
-		case 'W':
-			pettype = PM_KITTEN;
-			break;
-		/* otherwise, see if the player has a preference */
-		default:
-			if (preferred_pet == 'c')
-				pettype = PM_KITTEN;
-			else if (preferred_pet == 'd')
-				pettype = PM_LITTLE_DOG;
-			else	pettype = rn2(2) ? PM_KITTEN : PM_LITTLE_DOG;
-			break;
-	}
-	return pettype;
+	if (urole.petnum != NON_PM)
+	    return (urole.petnum);
+	else if (preferred_pet == 'c')
+	    return (PM_KITTEN);
+	else if (preferred_pet == 'd')
+	    return (PM_LITTLE_DOG);
+	else
+	    return (rn2(2) ? PM_KITTEN : PM_LITTLE_DOG);
 }
 
-void
-make_familiar(otmp,x,y)
+struct monst *
+make_familiar(otmp,x,y,quietly)
 register struct obj *otmp;
 xchar x, y;
+boolean quietly;
 {
 	struct permonst *pm;
 	struct monst *mtmp = 0;
@@ -70,24 +63,24 @@ xchar x, y;
 	    } else {
 		pm = rndmonst();
 		if (!pm) {
-		    pline("There seems to be nothing available for a familiar.");
-		    break;
+		  if (!quietly)
+		    There("seems to be nothing available for a familiar.");
+		  break;
 		}
 	    }
 
-	    pm->pxlth += sizeof (struct edog);
-	    mtmp = makemon(pm, x, y, NO_MM_FLAGS);
-	    pm->pxlth -= sizeof (struct edog);
+	    mtmp = makemon(pm, x, y, MM_EDOG);
 	    if (otmp && !mtmp) { /* monster was genocided or square occupied */
-		pline_The("figurine writhes and then shatters into pieces!");
+	 	if (!quietly)
+		   pline_The("figurine writhes and then shatters into pieces!");
 		break;
 	    }
 	} while (!mtmp && --trycnt > 0);
 
-	if (!mtmp) return;
+	if (!mtmp) return (struct monst *)0;
 
 	initedog(mtmp);
-	mtmp->msleep = 0;
+	mtmp->msleeping = 0;
 	if (otmp) { /* figurine; resulting monster might not become a pet */
 	    chance = rn2(10);	/* 0==tame, 1==peaceful, 2==hostile */
 	    if (chance > 2) chance = otmp->blessed ? 0 : !otmp->cursed ? 1 : 2;
@@ -95,10 +88,15 @@ xchar x, y;
 	    if (chance > 0) {
 		mtmp->mtame = 0;	/* not tame after all */
 		if (chance == 2) { /* hostile (cursed figurine) */
-		    You("get a bad feeling about this.");
+		    if (!quietly)
+		       You("get a bad feeling about this.");
 		    mtmp->mpeaceful = 0;
+		    set_malign(mtmp);
 		}
 	    }
+	    /* if figurine has been named, give same name to the monster */
+	    if (otmp->onamelth)
+		mtmp = christen_monst(mtmp, ONAME(otmp));
 	}
 	set_malign(mtmp); /* more alignment changes */
 	newsym(mtmp->mx, mtmp->my);
@@ -108,35 +106,56 @@ xchar x, y;
 		mtmp->weapon_check = NEED_HTH_WEAPON;
 		(void) mon_wield_item(mtmp);
 	}
+	return mtmp;
 }
 
 struct monst *
 makedog()
 {
 	register struct monst *mtmp;
+#ifdef STEED
+	register struct obj *otmp;
+#endif
 	const char *petname;
 	int   pettype;
 	static int petname_used = 0;
 
+	if (preferred_pet == 'n') return((struct monst *) 0);
+
 	pettype = pet_type();
 	if (pettype == PM_LITTLE_DOG)
 		petname = dogname;
+	else if (pettype == PM_PONY)
+		petname = horsename;
 	else
 		petname = catname;
 
 	/* default pet names */
-	if (!*petname) {
-	    if(Role_is('C')) petname = "Slasher";   /* The Warrior */
-	    if(Role_is('S')) petname = "Hachi";     /* Shibuya Station */
-	    if(Role_is('B')) petname = "Idefix";    /* Obelix */
+	if (!*petname && pettype == PM_LITTLE_DOG) {
+	    /* All of these names were for dogs. */
+	    if(Role_if(PM_CAVEMAN)) petname = "Slasher";   /* The Warrior */
+	    if(Role_if(PM_SAMURAI)) petname = "Hachi";     /* Shibuya Station */
+	    if(Role_if(PM_BARBARIAN)) petname = "Idefix";  /* Obelix */
+	    if(Role_if(PM_RANGER)) petname = "Sirius";     /* Orion's dog */
 	}
 
-	mons[pettype].pxlth = sizeof(struct edog);
-	mtmp = makemon(&mons[pettype], u.ux, u.uy, NO_MM_FLAGS);
-	mons[pettype].pxlth = 0;
+	mtmp = makemon(&mons[pettype], u.ux, u.uy, MM_EDOG);
 
 	if(!mtmp) return((struct monst *) 0); /* pets were genocided */
 
+#ifdef STEED
+	/* Horses already wear a saddle */
+	if (pettype == PM_PONY && !!(otmp = mksobj(SADDLE, TRUE, FALSE))) {
+	    if (mpickobj(mtmp, otmp))
+		panic("merged saddle?");
+	    mtmp->misc_worn_check |= W_SADDLE;
+	    otmp->dknown = otmp->bknown = otmp->rknown = 1;
+	    otmp->owornmask = W_SADDLE;
+	    otmp->leashmon = mtmp->m_id;
+	    update_mon_intrinsics(mtmp, otmp, TRUE);
+	}
+#endif
+
 	if (!petname_used++ && *petname)
 		mtmp = christen_monst(mtmp, petname);
 
@@ -144,6 +163,19 @@ makedog()
 	return(mtmp);
 }
 
+/* record `last move time' for all monsters prior to level save so that
+   mon_arrive() can catch up for lost time when they're restored later */
+void
+update_mlstmv()
+{
+	struct monst *mon;
+
+	/* monst->mlstmv used to be updated every time `monst' actually moved,
+	   but that is no longer the case so we just do a blanket assignment */
+	for (mon = fmon; mon; mon = mon->nmon)
+	    if (!DEADMONSTER(mon)) mon->mlstmv = monstermoves;
+}
+
 void
 losedogs()
 {
@@ -186,10 +218,11 @@ boolean with_you;
 	/* baby long worms have no tail so don't use is_longworm() */
 	if ((mtmp->data == &mons[PM_LONG_WORM]) &&
 #ifdef DCC30_BUG
-	    (mtmp->wormno = get_wormno(), mtmp->wormno != 0)) {
+	    (mtmp->wormno = get_wormno(), mtmp->wormno != 0))
 #else
-	    (mtmp->wormno = get_wormno()) != 0) {
+	    (mtmp->wormno = get_wormno()) != 0)
 #endif
+	{
 	    initworm(mtmp, num_segs);
 	    /* tail segs are not yet initialized or displayed */
 	} else mtmp->wormno = 0;
@@ -198,8 +231,30 @@ boolean with_you;
 	   _after_ the current level has been fully set up; see dochug() */
 	mtmp->mstrategy |= STRAT_ARRIVE;
 
+	/* make sure mnexto(rloc_to(set_apparxy())) doesn't use stale data */
+	mtmp->mux = u.ux,  mtmp->muy = u.uy;
+	xyloc	= mtmp->mtrack[0].x;
+	xyflags = mtmp->mtrack[0].y;
+	xlocale = mtmp->mtrack[1].x;
+	ylocale = mtmp->mtrack[1].y;
+	mtmp->mtrack[0].x = mtmp->mtrack[0].y = 0;
+	mtmp->mtrack[1].x = mtmp->mtrack[1].y = 0;
+
+#ifdef STEED
+	if (mtmp == u.usteed)
+	    return;	/* don't place steed on the map */
+#endif
 	if (with_you) {
-	    mnexto(mtmp);
+	    /* When a monster accompanies you, sometimes it will arrive
+	       at your intended destination and you'll end up next to
+	       that spot.  This code doesn't control the final outcome;
+	       goto_level(do.c) decides who ends up at your target spot
+	       when there is a monster there too. */
+	    if (!MON_AT(u.ux, u.uy) &&
+		    !rn2(mtmp->mtame ? 10 : mtmp->mpeaceful ? 5 : 2))
+		rloc_to(mtmp, u.ux, u.uy);
+	    else
+		mnexto(mtmp);
 	    return;
 	}
 	/*
@@ -208,14 +263,6 @@ boolean with_you;
 	 * specify its final destination.
 	 */
 
-	xyloc	= mtmp->mtrack[0].x;
-	xyflags = mtmp->mtrack[0].y;
-	xlocale = mtmp->mtrack[1].x;
-	ylocale = mtmp->mtrack[1].y;
-	mtmp->mtrack[0].x = mtmp->mtrack[0].y = 0;
-	mtmp->mtrack[1].x = mtmp->mtrack[1].y = 0;
-	mtmp->mux = u.ux,  mtmp->muy = u.uy;	/* not really req'd */
-
 	if (mtmp->mlstmv < monstermoves - 1L) {
 	    /* heal monster for time spent in limbo */
 	    long nmv = monstermoves - 1L - mtmp->mlstmv;
@@ -246,14 +293,24 @@ boolean with_you;
 	 case MIGR_SSTAIRS:	xlocale = sstairs.sx,  ylocale = sstairs.sy;
 		break;
 	 case MIGR_PORTAL:
+		if (In_endgame(&u.uz)) {
+		    /* there is no arrival portal for endgame levels */
+		    /* BUG[?]: for simplicity, this code relies on the fact
+		       that we know that the current endgame levels always
+		       build upwards and never have any exclusion subregion
+		       inside their TELEPORT_REGION settings. */
+		    xlocale = rn1(updest.hx - updest.lx + 1, updest.lx);
+		    ylocale = rn1(updest.hy - updest.ly + 1, updest.ly);
+		    break;
+		}
+		/* find the arrival portal */
 		for (t = ftrap; t; t = t->ntrap)
 		    if (t->ttyp == MAGIC_PORTAL) break;
 		if (t) {
 		    xlocale = t->tx,  ylocale = t->ty;
 		    break;
 		} else {
-		    if (!In_endgame(&u.uz))
-			impossible("mon_arrive: no corresponding portal?");
+		    impossible("mon_arrive: no corresponding portal?");
 		} /*FALLTHRU*/
 	 default:
 	 case MIGR_RANDOM:	xlocale = ylocale = 0;
@@ -345,6 +402,24 @@ long nmv;		/* number of moves */
 	    else if (mtmp->mtame > rn2(wilder)) mtmp->mtame = 0;  /* untame */
 	    else mtmp->mtame = mtmp->mpeaceful = 0;		/* hostile! */
 	}
+	/* check to see if it would have died as a pet; if so, go wild instead
+	 * of dying the next time we call dog_move()
+	 */
+	if (mtmp->mtame && !mtmp->isminion &&
+			(carnivorous(mtmp->data) || herbivorous(mtmp->data))) {
+	    struct edog *edog = EDOG(mtmp);
+
+	    if ((monstermoves > edog->hungrytime + 500 && mtmp->mhp < 3) ||
+		    (monstermoves > edog->hungrytime + 750))
+		mtmp->mtame = mtmp->mpeaceful = 0;
+	}
+
+	if (!mtmp->mtame && mtmp->mleashed) {
+	    /* leashed monsters should always be with hero, consequently
+	       never losing any time to be accounted for later */
+	    impossible("catching up for leashed monster?");
+	    m_unleash(mtmp, FALSE);
+	}
 
 	/* recover lost hit points */
 	if (!regenerates(mtmp->data)) imv /= 20;
@@ -368,13 +443,19 @@ boolean pets_only;	/* true for ascension or final escape */
 
 	for (mtmp = fmon; mtmp; mtmp = mtmp2) {
 	    mtmp2 = mtmp->nmon;
+	    if (DEADMONSTER(mtmp)) continue;
 	    if (pets_only && !mtmp->mtame) continue;
 	    if (((monnear(mtmp, u.ux, u.uy) && levl_follower(mtmp)) ||
+#ifdef STEED
+			(mtmp == u.usteed) ||
+#endif
 		/* the wiz will level t-port from anywhere to chase
 		   the amulet; if you don't have it, will chase you
 		   only if in range. -3. */
 			(u.uhave.amulet && mtmp->iswiz))
-			&& !mtmp->msleep && mtmp->mcanmove) {
+		&& !mtmp->msleeping && mtmp->mcanmove
+		/* monster won't follow if it hasn't noticed you yet */
+		&& !(mtmp->mstrategy & STRAT_WAITFORU)) {
 		stay_behind = FALSE;
 		if (mtmp->mtame && mtmp->meating) {
 			if (canseemon(mtmp))
@@ -385,14 +466,21 @@ boolean pets_only;	/* true for ascension or final escape */
 			    pline("%s seems very disoriented for a moment.",
 				Monnam(mtmp));
 			stay_behind = TRUE;
+		} else if (mtmp->mtame && mtmp->mtrapped) {
+			if (canseemon(mtmp))
+			    pline("%s is still trapped.", Monnam(mtmp));
+			stay_behind = TRUE;
 		}
+#ifdef STEED
+		if (mtmp == u.usteed) stay_behind = FALSE;
+#endif
 		if (stay_behind) {
 			if (mtmp->mleashed) {
 				pline("%s leash suddenly comes loose.",
 					humanoid(mtmp->data)
 					    ? (mtmp->female ? "Her" : "His")
 					    : "Its");
-				m_unleash(mtmp);
+				m_unleash(mtmp, FALSE);
 			}
 			continue;
 		}
@@ -418,6 +506,7 @@ boolean pets_only;	/* true for ascension or final escape */
 		newsym(mtmp->mx,mtmp->my);
 		mtmp->mx = mtmp->my = 0; /* avoid mnexto()/MON_AT() problem */
 		mtmp->wormno = num_segs;
+		mtmp->mlstmv = monstermoves;
 		mtmp->nmon = mydogs;
 		mydogs = mtmp;
 	    } else if (mtmp->iswiz) {
@@ -430,7 +519,7 @@ boolean pets_only;	/* true for ascension or final escape */
 		/* this can happen if your quest leader ejects you from the
 		   "home" level while a leashed pet isn't next to you */
 		pline("%s leash goes slack.", s_suffix(Monnam(mtmp)));
-		m_unleash(mtmp);
+		m_unleash(mtmp, FALSE);
 	    }
 	}
 }
@@ -468,14 +557,13 @@ migrate_to_level(mtmp, tolev, xyloc, cc)
 	    obj->no_charge = 0;
 	}
 
+	if (mtmp->mleashed) {
+		mtmp->mtame--;
+		m_unleash(mtmp, TRUE);
+	}
 	relmon(mtmp);
 	mtmp->nmon = migrating_mons;
 	migrating_mons = mtmp;
-	if (mtmp->mleashed)  {
-		m_unleash(mtmp);
-		mtmp->mtame--;
-		pline_The("leash comes off!");
-	}
 	newsym(mtmp->mx,mtmp->my);
 
 	new_lev.dnum = ledger_to_dnum((xchar)tolev);
@@ -515,33 +603,41 @@ register struct obj *obj;
 	switch(obj->oclass) {
 	case FOOD_CLASS:
 	    if (obj->otyp == CORPSE &&
-		((obj->corpsenm == PM_COCKATRICE && !resists_ston(mon))
+		((touch_petrifies(&mons[obj->corpsenm]) && !resists_ston(mon))
 		 || is_rider(fptr)))
 		    return TABU;
 
+	    /* Ghouls only eat old corpses... yum! */
+	    if (mon->data == &mons[PM_GHOUL])
+	    	return (obj->otyp == CORPSE && obj->age+50 <= monstermoves) ?
+	       			DOGFOOD : TABU;
+
 	    if (!carni && !herbi)
 		    return (obj->cursed ? UNDEF : APPORT);
 
 	    switch (obj->otyp) {
 		case TRIPE_RATION:
+		case MEATBALL:
+		case MEAT_RING:
+		case MEAT_STICK:
+		case HUGE_CHUNK_OF_MEAT:
 		    return (carni ? DOGFOOD : MANFOOD);
 		case EGG:
-		    if (obj->corpsenm == PM_COCKATRICE && !resists_ston(mon))
+		    if (touch_petrifies(&mons[obj->corpsenm]) && !resists_ston(mon))
 			return POISON;
 		    return (carni ? CADAVER : MANFOOD);
 		case CORPSE:
 		   if ((peek_at_iced_corpse_age(obj)+50 <= monstermoves
 					    && obj->corpsenm != PM_LIZARD
+					    && obj->corpsenm != PM_LICHEN
 					    && mon->data->mlet != S_FUNGUS) ||
 			(acidic(&mons[obj->corpsenm]) && !resists_acid(mon)) ||
 			(poisonous(&mons[obj->corpsenm]) &&
 						!resists_poison(mon)))
 			return POISON;
-		    else if (fptr->mlet == S_FUNGUS)
+		    else if (vegan(fptr))
 			return (herbi ? CADAVER : MANFOOD);
-		    else if (is_meaty(fptr))
-		        return (carni ? CADAVER : MANFOOD);
-		    else return (carni ? ACCFOOD : MANFOOD);
+		    else return (carni ? CADAVER : MANFOOD);
 		case CLOVE_OF_GARLIC:
 		    return (is_undead(mon->data) ? TABU :
 			    (herbi ? ACCFOOD : MANFOOD));
@@ -559,15 +655,19 @@ register struct obj *obj;
 			    (herbi ? ACCFOOD : MANFOOD));
 	    }
 	default:
+	    if (obj->otyp == AMULET_OF_STRANGULATION ||
+	    		obj->otyp == RIN_SLOW_DIGESTION)
+	    	return (TABU);
 	    if (hates_silver(mon->data) &&
 		objects[obj->otyp].oc_material == SILVER)
 		return(TABU);
 	    if (mon->data == &mons[PM_GELATINOUS_CUBE] && is_organic(obj))
 		return(ACCFOOD);
-	    if (metallivorous(mon->data) && is_metallic(obj))
-		/* Ferrous based metals are preferred. */
-		return(objects[obj->otyp].oc_material == IRON ? DOGFOOD :
-		       ACCFOOD);
+	    if (metallivorous(mon->data) && is_metallic(obj) && (is_rustprone(obj) || mon->data != &mons[PM_RUST_MONSTER])) {
+		/* Non-rustproofed ferrous based metals are preferred. */
+		return((is_rustprone(obj) && !obj->oerodeproof) ? DOGFOOD :
+			ACCFOOD);
+	    }
 	    if(!obj->cursed && obj->oclass != BALL_CLASS &&
 						obj->oclass != CHAIN_CLASS)
 		return(APPORT);
@@ -607,7 +707,7 @@ register struct obj *obj;
 	if (mtmp == u.ustuck) {
 	    if (u.uswallow)
 		expels(mtmp, mtmp->data, TRUE);
-	    else if (!(Upolyd && sticks(uasmon)))
+	    else if (!(Upolyd && sticks(youmonst.data)))
 		unstuck(mtmp);
 	}
 
@@ -617,7 +717,7 @@ register struct obj *obj;
 
 	    if (mtmp->mcanmove && !mtmp->mconf && !mtmp->meating &&
 		((tasty = dogfood(mtmp, obj)) == DOGFOOD ||
-		 (tasty <= ACCFOOD && EDOG(mtmp)->hungrytime <= moves))) {
+		 (tasty <= ACCFOOD && EDOG(mtmp)->hungrytime <= monstermoves))) {
 		/* pet will "catch" and eat this thrown food */
 		if (canseemon(mtmp)) {
 		    boolean big_corpse = (obj->otyp == CORPSE &&
@@ -627,7 +727,7 @@ register struct obj *obj;
 			  Monnam(mtmp), the(xname(obj)),
 			  !big_corpse ? "." : ", or vice versa!");
 		} else if (cansee(mtmp->mx,mtmp->my))
-		    pline("%s stops.", The(xname(obj)));
+		    pline("%s.", Tobjnam(obj, "stop"));
 		/* dog_eat expects a floor object */
 		place_object(obj, mtmp->mx, mtmp->my);
 		(void) dog_eat(mtmp, obj, mtmp->mx, mtmp->my, FALSE);
@@ -643,9 +743,12 @@ register struct obj *obj;
 	    /* monsters with conflicting structures cannot be tamed */
 	    mtmp->isshk || mtmp->isgd || mtmp->ispriest || mtmp->isminion ||
 	    is_covetous(mtmp->data) || is_human(mtmp->data) ||
-	    (is_demon(mtmp->data) && !is_demon(uasmon)) ||
+	    (is_demon(mtmp->data) && !is_demon(youmonst.data)) ||
 	    (obj && dogfood(mtmp, obj) >= MANFOOD)) return (struct monst *)0;
 
+	if (mtmp->m_id == quest_status.leader_m_id)
+	    return((struct monst *)0);
+
 	/* make a new monster which has the pet extension */
 	mtmp2 = newmonst(sizeof(struct edog) + mtmp->mnamelth);
 	*mtmp2 = *mtmp;
@@ -672,6 +775,61 @@ register struct obj *obj;
 	return(mtmp2);
 }
 
+/*
+ * Called during pet revival or pet life-saving.
+ * If you killed the pet, it revives wild.
+ * If you abused the pet a lot while alive, it revives wild.
+ * If you abused the pet at all while alive, it revives untame.
+ * If the pet wasn't abused and was very tame, it might revive tame.
+ */
+void
+wary_dog(mtmp, quietly)
+struct monst *mtmp;
+boolean quietly;
+{
+    int has_edog;
+
+    if (!mtmp->mtame) return;
+    has_edog = !mtmp->isminion;
+    if (has_edog &&
+		((EDOG(mtmp)->killed_by_u == 1) || (EDOG(mtmp)->abuse > 2))) {
+	mtmp->mpeaceful = mtmp->mtame = 0;
+	if (EDOG(mtmp)->abuse >= 0 && EDOG(mtmp)->abuse < 10)
+		if (!rn2(EDOG(mtmp)->abuse + 1)) mtmp->mpeaceful = 1;
+	if(!quietly && cansee(mtmp->mx, mtmp->my)) {
+	    if (haseyes(youmonst.data)) {
+		if (haseyes(mtmp->data))
+			pline("%s %s to look you in the %s.",
+				Monnam(mtmp),
+				mtmp->mpeaceful ? "seems unable" :
+					    "refuses",
+				body_part(EYE));
+		else 
+			pline("%s avoids your gaze.",
+				Monnam(mtmp));
+	    }
+	}
+    } else {
+    	/* chance it goes wild anyway - Pet Semetary */
+    	if (!rn2(mtmp->mtame)) {
+    		mtmp->mpeaceful = mtmp->mtame = 0;
+    	}
+    }
+    if (!mtmp->mtame) {
+	newsym(mtmp->mx, mtmp->my);
+	/* a life-saved monster might be leashed;
+	   don't leave it that way if it's no longer tame */
+	if (mtmp->mleashed) m_unleash(mtmp, TRUE);
+    }
+
+    /* if its still a pet, start a clean pet-slate now */
+    if (has_edog && mtmp->mtame) {
+	EDOG(mtmp)->revivals++;
+    	EDOG(mtmp)->killed_by_u = 0;
+    	EDOG(mtmp)->abuse = 0;
+    }
+}
+
 void
 abuse_dog(mtmp)
 struct monst *mtmp;
@@ -681,10 +839,20 @@ struct monst *mtmp;
 	if (Aggravate_monster || Conflict) mtmp->mtame /=2;
 	else mtmp->mtame--;
 
-	if (mtmp->mtame && rn2(mtmp->mtame)) yelp(mtmp);
-	else growl(mtmp);	/* give them a moment's worry */
+	if (mtmp->mtame && !mtmp->isminion)
+	    EDOG(mtmp)->abuse++;
 
-	if (!mtmp->mtame) newsym(mtmp->mx, mtmp->my);
+	if (!mtmp->mtame && mtmp->mleashed)
+	    m_unleash(mtmp, TRUE);
+
+	/* don't make a sound if pet is in the middle of leaving the level */
+	/* newsym isn't necessary in this case either */
+	if (mtmp->mx != 0) {
+	    if (mtmp->mtame && rn2(mtmp->mtame)) yelp(mtmp);
+	    else growl(mtmp);	/* give them a moment's worry */
+	
+	    if (!mtmp->mtame) newsym(mtmp->mx, mtmp->my);
+	}
 }
 
 #endif /* OVLB */
