@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)dogmove.c	3.3	97/05/25	*/
+/*	SCCS Id: @(#)dogmove.c	3.4	2002/03/09	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -16,6 +16,8 @@ STATIC_DCL int FDECL(dog_invent,(struct monst *,struct edog *,int));
 STATIC_DCL int FDECL(dog_goal,(struct monst *,struct edog *,int,int,int));
 
 STATIC_DCL struct obj *FDECL(DROPPABLES, (struct monst *));
+STATIC_DCL boolean FDECL(can_reach_food,(struct monst *,XCHAR_P,XCHAR_P,XCHAR_P,
+    XCHAR_P));
 
 STATIC_OVL struct obj *
 DROPPABLES(mon)
@@ -30,7 +32,8 @@ register struct monst *mon;
 	if (!tunnels(mon->data) || !needspick(mon->data))
 		item1 = TRUE;
 	for(obj = mon->minvent; obj; obj = obj->nobj) {
-		if (!item1 && is_pick(obj)) {
+		if (!item1 && is_pick(obj) && (obj->otyp != DWARVISH_MATTOCK
+						|| !which_armor(mon, W_ARMS))) {
 			item1 = TRUE;
 			continue;
 		}
@@ -138,6 +141,11 @@ boolean devour;
 	}
 	edog->hungrytime += nutrit;
 	mtmp->mconf = 0;
+	if (edog->mhpmax_penalty) {
+	    /* no longer starving */
+	    mtmp->mhpmax += edog->mhpmax_penalty;
+	    edog->mhpmax_penalty = 0;
+	}
 	if (mtmp->mflee && mtmp->mfleetim > 1) mtmp->mfleetim /= 2;
 	if (mtmp->mtame < 20) mtmp->mtame++;
 	if (x != mtmp->mx || y != mtmp->my) {	/* moved & ate on same turn */
@@ -150,7 +158,7 @@ boolean devour;
 	} else
 	/* hack: observe the action if either new or old location is in view */
 	if (cansee(x, y) || cansee(mtmp->mx, mtmp->my))
-	    pline("%s %s %s.", Monnam(mtmp),
+	    pline("%s %s %s.", noit_Monnam(mtmp),
 		  devour ? "devours" : "eats",
 		  (obj->oclass == FOOD_CLASS) ?
 			singular(obj, doname) : doname(obj));
@@ -176,9 +184,10 @@ boolean devour;
 	    delobj(obj);
 	} else if (obj == uchain)
 	    unpunish();
-	else if (obj->quan > 1L && obj->oclass == FOOD_CLASS)
+	else if (obj->quan > 1L && obj->oclass == FOOD_CLASS) {
 	    obj->quan--;
-	else
+	    obj->owt = weight(obj);
+	} else
 	    delobj(obj);
 
 	if (poly) {
@@ -192,7 +201,7 @@ boolean devour;
 #ifdef STEED
 	    mtmp->misc_worn_check = mw;
 #endif
-	    if (newcham(mtmp, (struct permonst *)0) &&
+	    if (newcham(mtmp, (struct permonst *)0, FALSE) &&
 			cansee(mtmp->mx, mtmp->my)) {
 		uchar save_mnamelth = mtmp->mnamelth;
 		mtmp->mnamelth = 0;
@@ -221,9 +230,12 @@ register struct edog *edog;
 	    if (!carnivorous(mtmp->data) && !herbivorous(mtmp->data)) {
 		edog->hungrytime = monstermoves + 500;
 		/* but not too high; it might polymorph */
-	    } else if (!mtmp->mconf) {
+	    } else if (!edog->mhpmax_penalty) {
+		/* starving pets are limited in healing */
+		int newmhpmax = mtmp->mhpmax / 3;
 		mtmp->mconf = 1;
-		mtmp->mhpmax /= 3;
+		edog->mhpmax_penalty = mtmp->mhpmax - newmhpmax;
+		mtmp->mhpmax = newmhpmax;
 		if (mtmp->mhp > mtmp->mhpmax)
 		    mtmp->mhp = mtmp->mhpmax;
 		if (mtmp->mhp < 1) goto dog_died;
@@ -233,6 +245,7 @@ register struct edog *edog;
 		    beg(mtmp);
 		else
 		    You_feel("worried about %s.", y_monnam(mtmp));
+		stop_occupation();
 	    } else if (monstermoves > edog->hungrytime + 750 || mtmp->mhp < 1) {
 	    dog_died:
 		if (mtmp->mleashed)
@@ -270,7 +283,11 @@ int udist;
 	/* if we are carrying sth then we drop it (perhaps near @) */
 	/* Note: if apport == 1 then our behaviour is independent of udist */
 	/* Use udist+1 so steed won't cause divide by zero */
+#ifndef GOLDOBJ
 	if(DROPPABLES(mtmp) || mtmp->mgold) {
+#else
+	if(DROPPABLES(mtmp)) {
+#endif
 	    if (!rn2(udist+1) || !rn2(edog->apport))
 		if(rn2(10) < edog->apport){
 		    relobj(mtmp, (int)mtmp->minvis, TRUE);
@@ -287,20 +304,25 @@ int udist;
 		if (dogfood(mtmp, obj) <= CADAVER)
 		    return dog_eat(mtmp, obj, omx, omy, FALSE);
 
-		if(can_carry(mtmp, obj) && !obj->cursed)
-		    if(rn2(20) < edog->apport+3)
+		if(can_carry(mtmp, obj) && !obj->cursed &&
+			!is_pool(mtmp->mx,mtmp->my)) {
+		    if(rn2(20) < edog->apport+3) {
 			if (rn2(udist) || !rn2(edog->apport)) {
 			    if (cansee(omx, omy) && flags.verbose)
 				pline("%s picks up %s.", Monnam(mtmp),
 				    distant_name(obj, doname));
 			    obj_extract_self(obj);
 			    newsym(omx,omy);
-			    mpickobj(mtmp,obj);
-			    if (attacktype(mtmp->data, AT_WEAP)) {
+			    (void) mpickobj(mtmp,obj);
+			    if (attacktype(mtmp->data, AT_WEAP) &&
+					mtmp->weapon_check == NEED_WEAPON) {
 				mtmp->weapon_check = NEED_HTH_WEAPON;
 				(void) mon_wield_item(mtmp);
 			    }
+			    m_dowear(mtmp, FALSE);
 			}
+		    }
+		}
 	    }
 	}
 	return 0;
@@ -361,7 +383,8 @@ int after, udist, whappr;
 			continue;
 		    if (cursed_object_at(nx, ny))
 			continue;
-		    if (otyp < MANFOOD) {
+		    if (otyp < MANFOOD &&
+			    can_reach_food(mtmp, mtmp->mx, mtmp->my, nx, ny)) {
 			if (otyp < gtyp || DDIST(nx,ny) < DDIST(gx,gy)) {
 			    gx = nx;
 			    gy = ny;
@@ -485,10 +508,14 @@ register int after;	/* this is extra fast monster movement */
 
 	udist = distu(omx,omy);
 #ifdef STEED
-	/* Let steeds eat */
-	if (mtmp == u.usteed)
-		udist = 1;
-	else
+	/* Let steeds eat and maybe throw rider during Conflict */
+	if (mtmp == u.usteed) {
+	    if (Conflict && !resist(mtmp, RING_CLASS, 0, 0)) {
+		dismount_steed(DISMOUNT_THROWN);
+		return (1);
+	    }
+	    udist = 1;
+	} else
 #endif
 	/* maybe we tamed him while being swallowed --jgm */
 	if (!udist) return(0);
@@ -538,12 +565,19 @@ register int after;	/* this is extra fast monster movement */
 
 	    }
 	}
+	if (!Conflict && !mtmp->mconf &&
+	    mtmp == u.ustuck && !sticks(youmonst.data)) {
+	    unstuck(mtmp);	/* swallowed case handled above */
+	    You("get released!");
+	}
 	if (!nohands(mtmp->data) && !verysmall(mtmp->data)) {
 		allowflags |= OPENDOOR;
 		if (m_carrying(mtmp, SKELETON_KEY)) allowflags |= BUSTDOOR;
 	}
 	if (is_giant(mtmp->data)) allowflags |= BUSTDOOR;
-	if (tunnels(mtmp->data) && !needspick(mtmp->data))
+	if (tunnels(mtmp->data) && (!needspick(mtmp->data) ||
+					m_carrying(mtmp, PICK_AXE) ||
+					m_carrying(mtmp, DWARVISH_MATTOCK)))
 		allowflags |= ALLOW_DIG;
 	cnt = mfndpos(mtmp, poss, info, allowflags);
 
@@ -677,17 +711,27 @@ register int after;	/* this is extra fast monster movement */
 	}
 newdogpos:
 	if (nix != omx || niy != omy) {
+		struct obj *mw_tmp;
+
 		if (info[chi] & ALLOW_U) {
 			if (mtmp->mleashed) { /* play it safe */
 				pline("%s breaks loose of %s leash!",
-				      Monnam(mtmp), his[pronoun_gender(mtmp)]);
-				m_unleash(mtmp);
+				      Monnam(mtmp), mhis(mtmp));
+				m_unleash(mtmp, FALSE);
 			}
 			(void) mattacku(mtmp);
 			return(0);
 		}
 		if (!m_in_out_region(mtmp, nix, niy))
 		    return 1;
+		if(IS_ROCK(levl[nix][niy].typ) && may_dig(nix,niy) &&
+		    mtmp->weapon_check != NO_WEAPON_WANTED &&
+		    tunnels(mtmp->data) && needspick(mtmp->data) &&
+			(!(mw_tmp = MON_WEP(mtmp)) || !is_pick(mw_tmp))) {
+		    mtmp->weapon_check = NEED_PICK_AXE;
+		    if (mon_wield_item(mtmp))
+			return 0;
+		}
 		/* insert a worm_move() if worms ever begin to eat things */
 		remove_monster(omx, omy);
 		place_monster(mtmp, nix, niy);
@@ -737,6 +781,50 @@ dognext:
 	return(1);
 }
 
+/* Hack to prevent a dog from being endlessly stuck near a piece of food that
+ * it can't reach, such as caught in a teleport scroll niche.  It recursively
+ * checks to see if the squares inbetween are good.  The checking could be a
+ * little smarter; a full check would probably be useful in m_move() too.
+ * Since the maximum food distance is 5, this should never be more than 5 calls
+ * deep.
+ */
+STATIC_OVL  boolean
+can_reach_food(mon, mx, my, fx, fy)
+struct monst *mon;
+xchar mx, my, fx, fy;
+{
+    int i, j;
+    int dist;
+
+    if (mx == fx && my == fy) return TRUE;
+    if (!isok(mx, my)) return FALSE; /* should not happen */
+    
+    dist = dist2(mx, my, fx, fy);
+    for(i=mx-1; i<=mx+1; i++) {
+	for(j=my-1; j<=my+1; j++) {
+	    if (!isok(i, j))
+		continue;
+	    if (dist2(i, j, fx, fy) >= dist)
+		continue;
+	    if (IS_ROCK(levl[i][j].typ) && !passes_walls(mon->data) &&
+				    (!may_dig(i,j) || !tunnels(mon->data)))
+		continue;
+	    if (IS_DOOR(levl[i][j].typ) &&
+				(levl[i][j].doormask & (D_CLOSED | D_LOCKED)))
+		continue;
+	    if (is_pool(i, j) && !is_swimmer(mon->data))
+		continue;
+	    if (is_lava(i, j) && !likes_lava(mon->data))
+		continue;
+	    if (sobj_at(BOULDER,i,j) && !throws_rocks(mon->data))
+		continue;
+	    if (can_reach_food(mon, i, j, fx, fy))
+		return TRUE;
+	}
+    }
+    return FALSE;
+}
+
 #endif /* OVL0 */
 #ifdef OVLB
 
