@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)eat.c	3.3	1999/08/18	*/
+/*	SCCS Id: @(#)eat.c	3.4	2002/01/02	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -35,11 +35,12 @@ STATIC_DCL void FDECL(start_eating, (struct obj *));
 STATIC_DCL void FDECL(fprefx, (struct obj *));
 STATIC_DCL void FDECL(fpostfx, (struct obj *));
 STATIC_DCL int NDECL(bite);
+STATIC_DCL int FDECL(edibility_prompts, (struct obj *));
 
 STATIC_DCL int FDECL(rottenfood, (struct obj *));
 STATIC_DCL void NDECL(eatspecial);
 STATIC_DCL void FDECL(eataccessory, (struct obj *));
-STATIC_DCL const char * FDECL(foodword, (struct obj *));
+STATIC_DCL const char *FDECL(foodword, (struct obj *));
 
 char msgbuf[BUFSZ];
 
@@ -103,11 +104,12 @@ register struct obj *obj;
 	/* above also prevents the Amulet from being eaten, so we must never
 	   allow fake amulets to be eaten either [which is already the case] */
 
-	if (metallivorous(youmonst.data) && is_metallic(obj))
+	if (metallivorous(youmonst.data) && is_metallic(obj) &&
+	    (youmonst.data != &mons[PM_RUST_MONSTER] || is_rustprone(obj)))
 		return TRUE;
 	if (u.umonnum == PM_GELATINOUS_CUBE && is_organic(obj) &&
 		/* [g.cubes can eat containers and retain all contents
-		    as engulged items, but poly'd player can't do that] */
+		    as engulfed items, but poly'd player can't do that] */
 	    !Has_contents(obj))
 		return TRUE;
 
@@ -139,6 +141,7 @@ static const struct { const char *txt; int nut; } tintxts[] = {
 	{"boiled",       50},
 	{"dried",        55},
 	{"szechuan",     70},
+#define FRENCH_FRIED_TIN 11
 	{"french fried", 40},
 	{"sauteed",      95},
 	{"broiled",      80},
@@ -155,11 +158,15 @@ static NEARDATA struct {
 static NEARDATA struct {
 	struct	obj *piece;	/* the thing being eaten, or last thing that
 				 * was partially eaten, unless that thing was
-				 * a tin, which uses the tin structure above */
+				 * a tin, which uses the tin structure above,
+				 * in which case this should be 0 */
+	/* doeat() initializes these when piece is valid */
 	int	usedtime,	/* turns spent eating */
 		reqtime;	/* turns required to eat */
 	int	nmod;		/* coded nutrition per turn */
 	Bitfield(canchoke,1);	/* was satiated at beginning */
+
+	/* start_eating() initializes these */
 	Bitfield(fullwarn,1);	/* have warned about being full */
 	Bitfield(eating,1);	/* victual currently being eaten */
 	Bitfield(doreset,1);	/* stop eating at end of turn */
@@ -219,7 +226,7 @@ choke(food)	/* To a full belly all food is bad. (It.) */
 {
 	/* only happens if you were satiated */
 	if (u.uhs != SATIATED) {
-		if (food->otyp != AMULET_OF_STRANGULATION)
+		if (!food || food->otyp != AMULET_OF_STRANGULATION)
 			return;
 	} else if (Role_if(PM_KNIGHT) && u.ualign.type == A_LAWFUL) {
 			adjalign(-1);		/* gluttony is unchivalrous */
@@ -230,12 +237,13 @@ choke(food)	/* To a full belly all food is bad. (It.) */
 
 	if (Breathless || (!Strangled && !rn2(20))) {
 		/* choking by eating AoS doesn't involve stuffing yourself */
-		if (food->otyp == AMULET_OF_STRANGULATION) {
+		if (food && food->otyp == AMULET_OF_STRANGULATION) {
 			You("choke, but recover your composure.");
 			return;
 		}
 		You("stuff yourself and then vomit voluminously.");
 		morehungry(1000);	/* you just got *very* sick! */
+		nomovemsg = 0;
 		vomit();
 	} else {
 		killer_format = KILLED_BY_AN;
@@ -259,19 +267,18 @@ choke(food)	/* To a full belly all food is bad. (It.) */
 	}
 }
 
+/* modify object wt. depending on time spent consuming it */
 STATIC_OVL void
-recalc_wt()	/* modify object wt. depending on time spent consuming it */
+recalc_wt()
 {
-	register struct obj *piece = victual.piece;
+	struct obj *piece = victual.piece;
 
 #ifdef DEBUG
 	debugpline("Old weight = %d", piece->owt);
 	debugpline("Used time = %d, Req'd time = %d",
 		victual.usedtime, victual.reqtime);
 #endif
-	/* weight(piece) = weight of full item */
-	if(victual.usedtime)
-	    piece->owt = eaten_stat(weight(piece), piece);
+	piece->owt = weight(piece);
 #ifdef DEBUG
 	debugpline("New weight = %d", piece->owt);
 #endif
@@ -298,9 +305,9 @@ register struct obj *otmp;
 {
 	if (otmp->quan > 1L) {
 	    if(!carried(otmp))
-		(void) splitobj(otmp, 1L);
+		(void) splitobj(otmp, otmp->quan - 1L);
 	    else
-		otmp = splitobj(otmp, otmp->quan - 1L);
+		otmp = splitobj(otmp, 1L);
 #ifdef DEBUG
 	    debugpline("split object,");
 #endif
@@ -314,7 +321,6 @@ register struct obj *otmp;
 		/* create a dummy duplicate to put on bill */
 		verbalize("You bit it, you bought it!");
 		bill_dummy_object(otmp);
-		otmp->no_charge = 1;	/* you now own this */
 	    }
 	    otmp->oeaten = (otmp->otyp == CORPSE ?
 				mons[otmp->corpsenm].cnutrit :
@@ -323,10 +329,15 @@ register struct obj *otmp;
 
 	if (carried(otmp)) {
 	    freeinv(otmp);
-	    if (inv_cnt() >= 52 && !merge_choice(invent, otmp))
+	    if (inv_cnt() >= 52) {
+		sellobj_state(SELL_DONTSELL);
 		dropy(otmp);
-	    else
-		otmp = addinv(otmp); /* unlikely but a merge is possible */
+		sellobj_state(SELL_NORMAL);
+	    } else {
+		otmp->oxlth++;		/* hack to prevent merge */
+		otmp = addinv(otmp);
+		otmp->oxlth--;
+	    }
 	}
 	return(otmp);
 }
@@ -435,14 +446,13 @@ register int pm;
 	if (touch_petrifies(&mons[pm]) || pm == PM_MEDUSA) {
 	    if (!Stone_resistance &&
 		!(poly_when_stoned(youmonst.data) && polymon(PM_STONE_GOLEM))) {
-		char kbuf[BUFSZ];
-
-		Sprintf(kbuf, "tasting %s meat", mons[pm].mname);
+		Sprintf(killer_buf, "tasting %s meat", mons[pm].mname);
 		killer_format = KILLED_BY;
-		killer = kbuf;
+		killer = killer_buf;
 		You("turn to stone.");
 		done(STONING);
-		victual.eating = FALSE;
+		if (victual.piece)
+		    victual.eating = FALSE;
 		return; /* lifesaved */
 	    }
 	}
@@ -475,17 +485,21 @@ register int pm;
 		    /* It so happens that since we know these monsters */
 		    /* cannot appear in tins, victual.piece will always */
 		    /* be what we want, which is not generally true. */
-		    (void) revive_corpse(victual.piece);
+		    if (revive_corpse(victual.piece))
+			victual.piece = (struct obj *)0;
 		    return;
 		}
 	    case PM_GREEN_SLIME:
-	    	if (!Unchanging && youmonst.data != &mons[PM_FIRE_VORTEX] &&
-	    			youmonst.data != &mons[PM_FIRE_ELEMENTAL] &&
-	    			youmonst.data != &mons[PM_GREEN_SLIME]) {
-	    	    You("don't feel very well.");
-	    	    Slimed = 10L;
-	    	}
-	    	/* Fall through */
+		if (!Slimed && !Unchanging &&
+			youmonst.data != &mons[PM_FIRE_VORTEX] &&
+			youmonst.data != &mons[PM_FIRE_ELEMENTAL] &&
+			youmonst.data != &mons[PM_SALAMANDER] &&
+			youmonst.data != &mons[PM_GREEN_SLIME]) {
+		    You("don't feel very well.");
+		    Slimed = 10L;
+		    flags.botl = 1;
+		}
+		/* Fall through */
 	    default:
 		if (acidic(&mons[pm]) && Stoned)
 		    fix_petrification();
@@ -497,6 +511,7 @@ void
 fix_petrification()
 {
 	Stoned = 0;
+	delayed_killer = 0;
 	if (Hallucination)
 	    pline("What a pity - you just ruined a future piece of %sart!",
 		  ACURR(A_CHA) > 15 ? "fine " : "");
@@ -703,7 +718,8 @@ register struct permonst *ptr;
 		debugpline("Trying to give poison resistance");
 #endif
 		if(!(HPoison_resistance & FROMOUTSIDE)) {
-			You_feel("healthy.");
+			You_feel(Poison_resistance ?
+				 "especially healthy." : "healthy.");
 			HPoison_resistance |= FROMOUTSIDE;
 		}
 		break;
@@ -805,8 +821,11 @@ register int pm;
 		tmp += 20;
 		if (youmonst.data->mlet != S_MIMIC) {
 		    char buf[BUFSZ];
-
 		    You_cant("resist the temptation to mimic a pile of gold.");
+#ifdef STEED
+                    /* A pile of gold can't ride. */
+		    if (u.usteed) dismount_steed(DISMOUNT_FELL);
+#endif
 		    nomul(-tmp);
 		    Sprintf(buf, "You now prefer mimicking %s again.",
 			    an(Upolyd ? youmonst.data->mname : urace.noun));
@@ -841,7 +860,7 @@ register int pm;
 	 /* case PM_SANDESTIN: */
 		if (!Unchanging) {
 		    You_feel("a change coming over you.");
-		    polyself();
+		    polyself(FALSE);
 		}
 		break;
 	    case PM_MIND_FLAYER:
@@ -918,6 +937,17 @@ register int pm;
 	return;
 }
 
+void
+violated_vegetarian()
+{
+    u.uconduct.unvegetarian++;
+    if (Role_if(PM_MONK)) {
+	You_feel("guilty.");
+	adjalign(-1);
+    }
+    return;
+}
+
 STATIC_PTR
 int
 opentin()		/* called during each move whilst opening a tin */
@@ -978,23 +1008,27 @@ opentin()		/* called during each move whilst opening a tin */
 			mons[tin.tin->corpsenm].mname);
 
 	    /* KMH, conduct */
-	    u.uconduct.flesh++;
-	    if (is_meaty(&mons[tin.tin->corpsenm])) {
-	    	u.uconduct.meat++;
-	    	if (Role_if(PM_MONK)) {
-	    	    pline("You feel guilty.");
-	    	    adjalign(-1);
-	    	}
-	    }
+	    u.uconduct.food++;
+	    if (!vegan(&mons[tin.tin->corpsenm]))
+		u.uconduct.unvegan++;
+	    if (!vegetarian(&mons[tin.tin->corpsenm]))
+		violated_vegetarian();
 
 	    tin.tin->dknown = tin.tin->known = TRUE;
 	    cprefx(tin.tin->corpsenm); cpostfx(tin.tin->corpsenm);
 
+	    if(((!carried(tin.tin) && costly_spot(tin.tin->ox, tin.tin->oy) &&
+		 !tin.tin->no_charge)
+		|| tin.tin->unpaid)) {
+		verbalize("You open it, you bought it!");
+		bill_dummy_object(tin.tin);
+	    }
+
 	    /* check for vomiting added by GAN 01/16/87 */
 	    if(tintxts[r].nut < 0) make_vomiting((long)rn1(15,10), FALSE);
 	    else lesshungry(tintxts[r].nut);
 
-	    if(r == 0) {			/* Deep Fried */
+	    if(r == 0 || r == FRENCH_FRIED_TIN) {
 	        /* Assume !Glib, because you can't open tins when Glib. */
 		incr_itimeout(&Glib, rnd(15));
 		pline("Eating deep fried food made your %s very slippery.",
@@ -1002,8 +1036,8 @@ opentin()		/* called during each move whilst opening a tin */
 	    }
 	} else {
 	    if (tin.tin->cursed)
-		pline("It contains some decaying %s substance.",
-			hcolor(green));
+		pline("It contains some decaying%s%s substance.",
+			Blind ? "" : " ", Blind ? "" : hcolor(green));
 	    else
 		pline("It contains spinach.");
 
@@ -1014,13 +1048,22 @@ opentin()		/* called during each move whilst opening a tin */
 		    You("discard the open tin.");
 		goto use_me;
 	    }
+
+	    tin.tin->dknown = tin.tin->known = TRUE;
+	    if(((!carried(tin.tin) && costly_spot(tin.tin->ox, tin.tin->oy) &&
+		 !tin.tin->no_charge)
+		|| tin.tin->unpaid)) {
+		verbalize("You open it, you bought it!");
+		bill_dummy_object(tin.tin);
+	    }
+
 	    if (!tin.tin->cursed)
 		pline("This makes you feel like %s!",
 		      Hallucination ? "Swee'pea" : "Popeye");
 	    lesshungry(600);
 	    gainstr(tin.tin, 0);
+	    u.uconduct.food++;
 	}
-	tin.tin->dknown = tin.tin->known = TRUE;
 use_me:
 	if (carried(tin.tin)) useup(tin.tin);
 	else useupf(tin.tin, 1L);
@@ -1072,11 +1115,7 @@ no_opener:
 			pline_The("tin slips from your %s.",
 			      makeplural(body_part(FINGER)));
 			if(otmp->quan > 1L) {
-				register struct obj *obj;
-				obj = splitobj(otmp, 1L);
-				if (otmp == uwep) setuwep(obj);
-				if (otmp == uswapwep) setuswapwep(obj);
-				if (otmp == uquiver) setuqwep(obj);
+			    otmp = splitobj(otmp, 1L);
 			}
 			if (carried(otmp)) dropx(otmp);
 			else stackobj(otmp);
@@ -1111,6 +1150,7 @@ struct obj *obj;
 	} else if(!rn2(4) && !Blind) {
 		pline("Everything suddenly goes dark.");
 		make_blinded((long)d(2,10),FALSE);
+		if (!Blind) Your(vision_clears);
 	} else if(!rn2(3)) {
 		const char *what, *where;
 		if (!Blind)
@@ -1119,7 +1159,11 @@ struct obj *obj;
 			 Is_waterlevel(&u.uz))
 		    what = "you lose control of",  where = "yourself";
 		else
-		    what = "you slap against the",  where = surface(u.ux,u.uy);
+		    what = "you slap against the", where =
+#ifdef STEED
+			   (u.usteed) ? "saddle" :
+#endif
+			   surface(u.ux,u.uy);
 		pline_The("world spins and %s %s.", what, where);
 		flags.soundok = 0;
 		nomul(-rnd(10));
@@ -1137,6 +1181,13 @@ eatcorpse(otmp)		/* called when a corpse is selected as food */
 	int tp = 0, mnum = otmp->corpsenm;
 	long rotted = 0L;
 	boolean uniq = !!(mons[mnum].geno & G_UNIQ);
+	int retcode = 0;
+	boolean stoneable = (touch_petrifies(&mons[mnum]) && !Stone_resistance &&
+				!poly_when_stoned(youmonst.data));
+
+	/* KMH, conduct */
+	if (!vegan(&mons[mnum])) u.uconduct.unvegan++;
+	if (!vegetarian(&mons[mnum])) violated_vegetarian();
 
 	if (mnum != PM_LIZARD && mnum != PM_LICHEN) {
 		long age = peek_at_iced_corpse_age(otmp);
@@ -1146,14 +1197,10 @@ eatcorpse(otmp)		/* called when a corpse is selected as food */
 		else if (otmp->blessed) rotted -= 2L;
 	}
 
-	/* KMH, conduct */
-	u.uconduct.flesh++;
-	if (is_meaty(&mons[mnum])) u.uconduct.meat++;
-
-	if (mnum != PM_ACID_BLOB && rotted > 5L) {
+	if (mnum != PM_ACID_BLOB && !stoneable && rotted > 5L) {
 		pline("Ulch - that %s was tainted!",
 		      mons[mnum].mlet == S_FUNGUS ? "fungoid vegetation" :
-		      is_meaty(&mons[mnum]) ? "meat" : "protoplasm");
+		      !vegetarian(&mons[mnum]) ? "meat" : "protoplasm");
 		if (Sick_resistance) {
 			pline("It doesn't seem at all sickening, though...");
 		} else {
@@ -1174,10 +1221,10 @@ eatcorpse(otmp)		/* called when a corpse is selected as food */
 		}
 		if (carried(otmp)) useup(otmp);
 		else useupf(otmp, 1L);
-		return(1);
+		return(2);
 	} else if (acidic(&mons[mnum]) && !Acid_resistance) {
 		tp++;
-		You("have a very bad case of stomach acid.");
+		You("have a very bad case of stomach acid."); /* not body_part() */
 		losehp(rnd(15), "acidic corpse", KILLED_BY_AN);
 	} else if (poisonous(&mons[mnum]) && rn2(5)) {
 		tp++;
@@ -1190,32 +1237,41 @@ eatcorpse(otmp)		/* called when a corpse is selected as food */
 	} else if ((rotted > 5L || (rotted > 3L && rn2(5)))
 					&& !Sick_resistance) {
 		tp++;
-		You("feel%s sick.", (Sick) ? " very" : "");
+		You_feel("%ssick.", (Sick) ? "very " : "");
 		losehp(rnd(8), "cadaver", KILLED_BY_AN);
 	}
+
+	/* delay is weight dependent */
+	victual.reqtime = 3 + (mons[mnum].cwt >> 6);
+
 	if (!tp && mnum != PM_LIZARD && mnum != PM_LICHEN &&
 			(otmp->orotten || !rn2(7))) {
 	    if (rottenfood(otmp)) {
 		otmp->orotten = TRUE;
 		(void)touchfood(otmp);
-		return(1);
+		retcode = 1;
 	    }
-	    otmp->oeaten >>= 2;
+
+	    if (!mons[otmp->corpsenm].cnutrit) {
+		/* no nutrution: rots away, no message if you passed out */
+		if (!retcode) pline_The("corpse rots away completely.");
+		if (carried(otmp)) useup(otmp);
+		else useupf(otmp, 1L);
+		retcode = 2;
+	    }
+		    
+	    if (!retcode) consume_oeaten(otmp, 2);	/* oeaten >>= 2 */
 	} else {
 	    pline("%s%s %s!",
 		  !uniq ? "This " : !type_is_pname(&mons[mnum]) ? "The " : "",
 		  food_xname(otmp, FALSE),
-		  (carnivorous(youmonst.data) && !herbivorous(youmonst.data)) ?
-			"is delicious" : "tastes terrible");
-	}
-	if (Role_if(PM_MONK) && is_meaty(&mons[mnum])) {
-	    pline("You feel guilty.");
-	    adjalign(-1);
+		  (vegan(&mons[mnum]) ?
+		   (!carnivorous(youmonst.data) && herbivorous(youmonst.data)) :
+		   (carnivorous(youmonst.data) && !herbivorous(youmonst.data)))
+		  ? "is delicious" : "tastes terrible");
 	}
 
-	/* delay is weight dependent */
-	victual.reqtime = 3 + (mons[mnum].cwt >> 6);
-	return(0);
+	return(retcode);
 }
 
 STATIC_OVL void
@@ -1234,13 +1290,15 @@ start_eating(otmp)		/* called as you start to eat */
 
 	if (otmp->otyp == CORPSE) {
 	    cprefx(victual.piece->corpsenm);
-	    if (!victual.piece && victual.eating) do_reset_eat();
-	    if (victual.eating == FALSE) return; /* died and lifesaved */
+	    if (!victual.piece || !victual.eating) {
+		/* rider revived, or died and lifesaved */
+		return;
+	    }
 	}
 
 	if (bite()) return;
 
-	if(++victual.usedtime >= victual.reqtime) {
+	if (++victual.usedtime >= victual.reqtime) {
 	    /* print "finish eating" message if they just resumed -dlc */
 	    done_eating(victual.reqtime > 1 ? TRUE : FALSE);
 	    return;
@@ -1255,29 +1313,36 @@ STATIC_OVL void
 fprefx(otmp)		/* called on "first bite" of (non-corpse) food */
 struct obj *otmp;
 {
-	/* KMH, conduct */
-	if (objects[otmp->otyp].oc_material == FLESH) u.uconduct.flesh++;
-
 	switch(otmp->otyp) {
 	    case FOOD_RATION:
 		if(u.uhunger <= 200)
-		    if (Hallucination) pline("Oh wow, like, superior, man!");
-		    else	       pline("That food really hit the spot!");
-		else if(u.uhunger <= 700) pline("That satiated your stomach!");
+		    pline(Hallucination ? "Oh wow, like, superior, man!" :
+			  "That food really hit the spot!");
+		else if(u.uhunger <= 700) pline("That satiated your %s!",
+						body_part(STOMACH));
 		break;
 	    case TRIPE_RATION:
-		u.uconduct.meat++;
 		if (carnivorous(youmonst.data) && !humanoid(youmonst.data))
 		    pline("That tripe ration was surprisingly good!");
+		else if (maybe_polyd(is_orc(youmonst.data), Race_if(PM_ORC)))
+		    pline(Hallucination ? "Tastes great! Less filling!" :
+			  "Mmm, tripe... not bad!");
 		else {
 		    pline("Yak - dog food!");
 		    more_experienced(1,0);
-		    flags.botl = 1;
-		}
-		if (rn2(2) && (!carnivorous(youmonst.data) || humanoid(youmonst.data))) {
+		    newexplevel();
+		    /* not cannibalism, but we use similar criteria
+		       for deciding whether to be sickened by this meal */
+		    if (rn2(2) && !CANNIBAL_ALLOWED())
 			make_vomiting((long)rn1(victual.reqtime, 14), FALSE);
 		}
 		break;
+	    case MEATBALL:
+	    case MEAT_STICK:
+	    case HUGE_CHUNK_OF_MEAT:
+	    case MEAT_RING:
+		goto give_feedback;
+	     /* break; */
 	    case CLOVE_OF_GARLIC:
 		if (is_undead(youmonst.data)) {
 			make_vomiting((long)rn1(victual.reqtime, 5), FALSE);
@@ -1285,8 +1350,6 @@ struct obj *otmp;
 		}
 		/* Fall through otherwise */
 	    default:
-		if (otmp->otyp == EGG && is_meaty(&mons[otmp->corpsenm]))
-			u.uconduct.meat++;
 		if (otmp->otyp==SLIME_MOLD && !otmp->cursed
 			&& otmp->spe == current_fruit)
 		    pline("My, that was a %s %s!",
@@ -1316,6 +1379,7 @@ struct obj *otmp;
 		    pline("Ugh.  Rotten egg.");	/* perhaps others like it */
 		    make_vomiting(Vomiting+d(10,4), TRUE);
 		} else
+ give_feedback:
 		    pline("This %s is %s", singular(otmp, xname),
 		      otmp->cursed ? (Hallucination ? "grody!" : "terrible!") :
 		      (otmp->otyp == CRAM_RATION
@@ -1325,6 +1389,33 @@ struct obj *otmp;
 		      Hallucination ? "gnarly!" : "delicious!");
 		break;
 	}
+
+	/* KMH, conduct */
+	switch (objects[otmp->otyp].oc_material) {
+	  case WAX: /* let's assume bees' wax */
+	    u.uconduct.unvegan++;
+	    break;
+
+	  case FLESH:
+	    if (otmp->otyp == EGG) {
+		u.uconduct.unvegan++;
+		break;
+	    }
+	  case LEATHER:
+	  case BONE:
+	  case DRAGON_HIDE:
+	    u.uconduct.unvegan++;
+	    violated_vegetarian();
+	    break;
+
+	  default:
+	    if (otmp->otyp == PANCAKE ||
+			otmp->otyp == FORTUNE_COOKIE || /* eggs */
+		otmp->otyp == CREAM_PIE || otmp->otyp == CANDY_BAR || /* milk */
+			otmp->otyp == LUMP_OF_ROYAL_JELLY)
+		u.uconduct.unvegan++;
+	    break;
+	}
 }
 
 STATIC_OVL void
@@ -1332,11 +1423,11 @@ eataccessory(otmp)
 struct obj *otmp;
 {
 	int typ = otmp->otyp;
-	int oldprop;
+	long oldprop;
 
 	/* Note: rings are not so common that this is unbalancing. */
 	/* (How often do you even _find_ 3 rings of polymorph in a game?) */
-	oldprop = !!(u.uprops[objects[typ].oc_oprop].intrinsic);
+	oldprop = u.uprops[objects[typ].oc_oprop].intrinsic;
 	if (otmp == uleft || otmp == uright) {
 	    Ring_gone(otmp);
 	    if (u.uhp <= 0) return; /* died from sink fall */
@@ -1358,14 +1449,14 @@ struct obj *otmp;
 		    set_mimic_blocking();
 		    see_monsters();
 		    if (Invis && !oldprop && !ESee_invisible &&
-		    		!perceives(youmonst.data) && !Blind) {
+				!perceives(youmonst.data) && !Blind) {
 			newsym(u.ux,u.uy);
 			pline("Suddenly you can see yourself.");
 			makeknown(typ);
 		    }
 		    break;
 		  case RIN_INVISIBILITY:
-			if (!oldprop && !EInvis && !BInvis &&
+		    if (!oldprop && !EInvis && !BInvis &&
 					!See_invisible && !Blind) {
 			newsym(u.ux,u.uy);
 			Your("body takes on a %s transparency...",
@@ -1377,6 +1468,8 @@ struct obj *otmp;
 		    rescham();
 		    break;
 		  case RIN_LEVITATION:
+		    /* undo the `.intrinsic |= FROMOUTSIDE' done above */
+		    u.uprops[LEVITATION].intrinsic = oldprop;
 		    if (!Levitation) {
 			float_up();
 			incr_itimeout(&HLevitation, d(10,20));
@@ -1425,7 +1518,7 @@ struct obj *otmp;
 		choke(otmp);
 		break;
 	    case AMULET_OF_RESTFUL_SLEEP: /* another bad idea! */
-		HSleeping = rnd(100);
+		HSleeping = FROMOUTSIDE | rnd(100);
 		break;
 		case RIN_SUSTAIN_ABILITY:
 	    case AMULET_OF_LIFE_SAVING:
@@ -1446,6 +1539,11 @@ eatspecial() /* called after eating non-food */
 	victual.piece = (struct obj *)0;
 	victual.eating = 0;
 	if (otmp->oclass == GOLD_CLASS) {
+#ifdef GOLDOBJ
+		if (carried(otmp))
+		    useupall(otmp);
+		else
+#endif
 		dealloc_obj(otmp);
 		return;
 	}
@@ -1512,7 +1610,7 @@ register struct obj *otmp;
 		    you_unwere(TRUE);
 		break;
 	    case CARROT:
-		make_blinded(0L,TRUE);
+		make_blinded((long)u.ucreamed,TRUE);
 		break;
 	    case FORTUNE_COOKIE:
 		outrumor(bcsign(otmp), BY_COOKIE);
@@ -1546,12 +1644,10 @@ register struct obj *otmp;
 		if (touch_petrifies(&mons[otmp->corpsenm])) {
 		    if (!Stone_resistance &&
 			!(poly_when_stoned(youmonst.data) && polymon(PM_STONE_GOLEM))) {
-			static char kbuf[BUFSIZ];
-
 			if (!Stoned) Stoned = 5;
 			killer_format = KILLED_BY_AN;
-			Sprintf(kbuf, "%s egg", mons[otmp->corpsenm].mname);
-			killer = kbuf;
+			Sprintf(killer_buf, "%s egg", mons[otmp->corpsenm].mname);
+			delayed_killer = killer_buf;
 		    }
 		}
 		break;
@@ -1565,12 +1661,147 @@ register struct obj *otmp;
 	return;
 }
 
+/*
+ * return 0 if the food was not dangerous.
+ * return 1 if the food was dangerous and you chose to stop.
+ * return 2 if the food was dangerous and you chose to eat it anyway.
+ */
+STATIC_OVL int
+edibility_prompts(otmp)
+struct obj *otmp;
+{
+	/* blessed food detection granted you a one-use
+	   ability to detect food that is unfit for consumption
+	   or dangerous and avoid it. */
+
+	char buf[BUFSZ], foodsmell[BUFSZ];
+	char *eat_it_anyway = "Eat it anyway?";
+	boolean cadaver = (otmp->otyp == CORPSE);
+	boolean stoneorslime = FALSE;
+	int material = objects[otmp->otyp].oc_material;
+	long rotted = 0L;
+	int mnum;
+
+#ifdef GCC_WARN
+	mnum = 0;
+#endif
+
+	Strcpy(foodsmell, Tobjnam(otmp, "smell"));
+	if (cadaver || otmp->otyp == EGG || otmp->otyp == TIN) {
+		mnum = otmp->corpsenm;
+		/* These checks must match those in eatcorpse() */
+	  	stoneorslime = (touch_petrifies(&mons[mnum]) &&
+			    !Stone_resistance && !poly_when_stoned(youmonst.data));
+
+		if (mnum == PM_GREEN_SLIME)
+		    stoneorslime = (!Unchanging &&
+			youmonst.data != &mons[PM_FIRE_VORTEX] &&
+			youmonst.data != &mons[PM_FIRE_ELEMENTAL] &&
+			youmonst.data != &mons[PM_SALAMANDER] &&
+			youmonst.data != &mons[PM_GREEN_SLIME]);
+
+		if (cadaver && mnum != PM_LIZARD && mnum != PM_LICHEN) {
+			long age = peek_at_iced_corpse_age(otmp);
+			/* worst case rather than random
+			   in this calculation to force prompt */
+			rotted = (monstermoves - age)/(10L + 0 /* was rn2(20) */);
+			if (otmp->cursed) rotted += 2L;
+			else if (otmp->blessed) rotted -= 2L;
+		}
+	}
+
+	/*
+	 * These problems with food should be checked in
+	 * order from most detrimental to least detrimental.
+	 */
+
+	if (cadaver && mnum != PM_ACID_BLOB && rotted > 5L && !Sick_resistance) {
+		/* Tainted meat */
+		Sprintf(buf, "%s like it could be tainted! %s",
+		     foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (stoneorslime) {
+		Sprintf(buf, "%s like it could be something very dangerous! %s",
+		     foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (otmp->orotten || (cadaver && rotted > 3L)) {
+		/* Rotten */
+		Sprintf(buf, "%s like it could be rotten! %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (cadaver && poisonous(&mons[mnum]) && !Poison_resistance) {
+		/* poisonous */
+		Sprintf(buf, "%s like it might be poisonous! %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (cadaver && !vegetarian(&mons[mnum]) &&
+	    !u.uconduct.unvegetarian && Role_if(PM_MONK)) {
+		Sprintf(buf, "%s unhealthy. %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (cadaver && acidic(&mons[mnum]) && !Acid_resistance) {
+		Sprintf(buf, "%s rather acidic. %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (Upolyd &&
+	    (u.umonnum == PM_RUST_MONSTER && is_metallic(otmp) && otmp->oerodeproof)) {
+		Sprintf(buf, "%s disgusting to you right now. %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+
+	/*
+	 * Breaks conduct, but otherwise safe.
+	 */
+	 
+	if (!u.uconduct.unvegan &&
+	    ((material == LEATHER || material == BONE ||
+	      material == DRAGON_HIDE || material == WAX) ||
+	     (cadaver && !vegan(&mons[mnum])))) {
+		Sprintf(buf, "%s foul and unfamiliar to you. %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	if (!u.uconduct.unvegetarian &&
+	    ((material == LEATHER || material == BONE || material == DRAGON_HIDE) ||
+	     (cadaver && !vegetarian(&mons[mnum])))) {
+		Sprintf(buf, "%s unfamiliar to you. %s",
+			foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+
+	if (cadaver && mnum != PM_ACID_BLOB && rotted > 5L && Sick_resistance) {
+		/* Tainted meat with Sick_resistance */
+		Sprintf(buf, "%s like it could be tainted! %s",
+		     foodsmell, eat_it_anyway);
+		if (yn_function(buf,ynchars,'n')=='n') return 1;
+		else return 2;
+	}
+	return 0;
+}
+
 int
 doeat()		/* generic "eat" command funtion (see cmd.c) */
 {
 	register struct obj *otmp;
 	int basenutrit;			/* nutrition of full item */
-
+	boolean dont_start = FALSE;
+	
 	if (Strangled) {
 		pline("If you can't breathe air, how can you consume solids?");
 		return 0;
@@ -1578,6 +1809,16 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	if (!(otmp = floorfood("eat", 0))) return 0;
 	if (check_capacity((char *)0)) return 0;
 
+	if (u.uedibility) {
+		int res = edibility_prompts(otmp);
+		if (res) {
+		    Your("%s stops tingling and your sense of smell returns to normal.",
+			body_part(NOSE));
+		    u.uedibility = 0;
+		    if (res == 1) return 0;
+		}
+	}
+
 	/* We have to make non-foods take 1 move to eat, unless we want to
 	 * do ridiculous amounts of coding to deal with partly eaten plate
 	 * mails, players who polymorph back to human in the middle of their
@@ -1599,16 +1840,16 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	    u.umonnum == PM_RUST_MONSTER && otmp->oerodeproof) {
 	    	otmp->rknown = TRUE;
 		if (otmp->quan > 1L) {
-			if(!carried(otmp))
-				(void) splitobj(otmp, 1L);
-			else
-				otmp = splitobj(otmp, otmp->quan - 1L);
+		    if(!carried(otmp))
+			(void) splitobj(otmp, otmp->quan - 1L);
+		    else
+			otmp = splitobj(otmp, 1L);
 		}
 		pline("Ulch - That %s was rustproofed!", xname(otmp));
 		/* The regurgitated object's rustproofing is gone now */
 		otmp->oerodeproof = 0;
 		make_stunned(HStun + rn2(10), TRUE);
-		pline("You spit %s out onto the %s.", the(xname(otmp)),
+		You("spit %s out onto the %s.", the(xname(otmp)),
 			surface(u.ux, u.uy));
 		if (carried(otmp)) {
 			freeinv(otmp);
@@ -1621,10 +1862,13 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	if (otmp->otyp == RIN_SLOW_DIGESTION) {
 		pline("This ring is undigestable!");
 		(void) rottenfood(otmp);
-		docall(otmp);
+		if (otmp->dknown && !objects[otmp->otyp].oc_name_known
+				&& !objects[otmp->otyp].oc_uname)
+			docall(otmp);
 		return (1);
 	}
 	if (otmp->oclass != FOOD_CLASS) {
+	    int material;
 	    victual.reqtime = 1;
 	    victual.piece = otmp;
 		/* Don't split it, we don't need to if it's 1 move */
@@ -1642,6 +1886,14 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	    victual.nmod = basenutrit;
 	    victual.eating = TRUE; /* needed for lesshungry() */
 
+	    material = objects[otmp->otyp].oc_material;
+	    if (material == LEATHER || material == BONE || material == DRAGON_HIDE) {
+	 		u.uconduct.unvegan++;
+	    		violated_vegetarian();
+	    } else if (material == WAX)
+			u.uconduct.unvegan++;
+	    u.uconduct.food++;
+	    
 	    if (otmp->cursed)
 		(void) rottenfood(otmp);
 
@@ -1656,13 +1908,11 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 		pline("This %s is delicious!",
 		      otmp->oclass == GOLD_CLASS ? foodword(otmp) :
 		      singular(otmp, xname));
+
 	    eatspecial();
 	    return 1;
 	}
 
-	/* KMH, conduct */
-	u.uconduct.food++;
-
 	if(otmp == victual.piece) {
 	/* If they weren't able to choke, they don't suddenly become able to
 	 * choke just because they were interrupted.  On the other hand, if
@@ -1670,10 +1920,7 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	 * they shouldn't be able to choke now.
 	 */
 	    if (u.uhs != SATIATED) victual.canchoke = FALSE;
-	    if(!carried(victual.piece)) {
-		if(victual.piece->quan > 1L)
-			(void) splitobj(victual.piece, 1L);
-	    }
+	    victual.piece = touchfood(otmp);
 	    You("resume your meal.");
 	    start_eating(victual.piece);
 	    return(1);
@@ -1681,11 +1928,15 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 
 	/* nothing in progress - so try to find something. */
 	/* tins are a special case */
+	/* tins must also check conduct separately in case they're discarded */
 	if(otmp->otyp == TIN) {
 	    start_tin(otmp);
 	    return(1);
 	}
 
+	/* KMH, conduct */
+	u.uconduct.food++;
+
 	victual.piece = otmp = touchfood(otmp);
 	victual.usedtime = 0;
 
@@ -1695,8 +1946,15 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	 * then adjusted in accordance with the amount of food left.
 	 */
 	if(otmp->otyp == CORPSE) {
-	    if(eatcorpse(otmp)) return(1);
-	    /* else eatcorpse sets up reqtime and oeaten */
+	    int tmp = eatcorpse(otmp);
+	    if (tmp == 2) {
+		/* used up */
+		victual.piece = (struct obj *)0;
+		return(1);
+	    } else if (tmp)
+		dont_start = TRUE;
+	    /* if not used up, eatcorpse sets up reqtime and may modify
+	     * oeaten */
 	} else {
 	    victual.reqtime = objects[otmp->otyp].oc_delay;
 	    if (otmp->otyp != FORTUNE_COOKIE &&
@@ -1706,9 +1964,9 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 
 		if (rottenfood(otmp)) {
 		    otmp->orotten = TRUE;
-		    return(1);
+		    dont_start = TRUE;
 		}
-		otmp->oeaten >>= 1;
+		consume_oeaten(otmp, 1);	/* oeaten >>= 1 */
 	    } else fprefx(otmp);
 	}
 
@@ -1731,16 +1989,16 @@ doeat()		/* generic "eat" command funtion (see cmd.c) */
 	 * TODO: add in a "remainder" value to be given at the end of the
 	 *	 meal.
 	 */
-	if(victual.reqtime == 0)
+	if (victual.reqtime == 0 || otmp->oeaten == 0)
 	    /* possible if most has been eaten before */
 	    victual.nmod = 0;
-	else if ((int)otmp->oeaten > victual.reqtime)
+	else if ((int)otmp->oeaten >= victual.reqtime)
 	    victual.nmod = -((int)otmp->oeaten / victual.reqtime);
 	else
 	    victual.nmod = victual.reqtime % otmp->oeaten;
 	victual.canchoke = (u.uhs == SATIATED);
 
-	start_eating(otmp);
+	if (!dont_start) start_eating(otmp);
 	return(1);
 }
 
@@ -1761,10 +2019,10 @@ bite()
 	force_save_hs = TRUE;
 	if(victual.nmod < 0) {
 		lesshungry(-victual.nmod);
-		victual.piece->oeaten -= -victual.nmod;
+		consume_oeaten(victual.piece, victual.nmod); /* -= -nmod */
 	} else if(victual.nmod > 0 && (victual.usedtime % victual.nmod)) {
 		lesshungry(1);
-		victual.piece->oeaten--;
+		consume_oeaten(victual.piece, -1);		  /* -= 1 */
 	}
 	force_save_hs = FALSE;
 	recalc_wt();
@@ -1831,18 +2089,21 @@ void
 lesshungry(num)	/* called after eating (and after drinking fruit juice) */
 register int num;
 {
+	/* See comments in newuhs() for discussion on force_save_hs */
+	boolean iseating = (occupation == eatfood) || force_save_hs;
 #ifdef DEBUG
 	debugpline("lesshungry(%d)", num);
 #endif
 	u.uhunger += num;
 	if(u.uhunger >= 2000) {
-	    if (!victual.eating || victual.canchoke)
-		if (victual.eating) {
-			choke(victual.piece);
-			reset_eat();
+	    if (!iseating || victual.canchoke) {
+		if (iseating) {
+		    choke(victual.piece);
+		    reset_eat();
 		} else
-			choke(tin.tin);	/* may be null */
+		    choke(occupation == opentin ? tin.tin : (struct obj *)0);
 		/* no reset_eat() */
+	    }
 	} else {
 	    /* Have lesshungry() report when you're nearly full so all eating
 	     * warns when you're about to choke.
@@ -1996,9 +2257,9 @@ boolean incr;
 		switch(newhs){
 		case HUNGRY:
 			if (Hallucination) {
-			    pline((!incr) ?
-				"You now have a lesser case of the munchies." :
-				"You are getting the munchies.");
+			    You((!incr) ?
+				"now have a lesser case of the munchies." :
+				"are getting the munchies.");
 			} else
 			    You((!incr) ? "only feel hungry now." :
 				  (u.uhunger < 145) ? "feel hungry." :
@@ -2013,10 +2274,11 @@ boolean incr;
 				  "You still have the munchies." :
       "The munchies are interfering with your motor capabilities.");
 			else if (incr &&
-				(Role_if(PM_WIZARD) || Race_if(PM_ELF) || Role_if(PM_VALKYRIE)))
+				(Role_if(PM_WIZARD) || Race_if(PM_ELF) ||
+				 Role_if(PM_VALKYRIE)))
 			    pline("%s needs food, badly!",
-			    		(Role_if(PM_WIZARD) || Role_if(PM_VALKYRIE)) ?
-			    		urole.name.m : "Elf");
+				  (Role_if(PM_WIZARD) || Role_if(PM_VALKYRIE)) ?
+				  urole.name.m : "Elf");
 			else
 			    You((!incr) ? "feel weak now." :
 				  (u.uhunger < 45) ? "feel weak." :
@@ -2055,6 +2317,16 @@ floorfood(verb,corpsecheck)	/* get food from floor or pack */
 	char c;
 	boolean feeding = (!strcmp(verb, "eat"));
 
+	/* if we can't touch floor objects then use invent food only */
+	if (!can_reach_floor() ||
+#ifdef STEED
+		(feeding && u.usteed) || /* can't eat off floor while riding */
+#endif
+		((is_pool(u.ux, u.uy) || is_lava(u.ux, u.uy)) &&
+		    (Wwalking || is_clinger(youmonst.data) ||
+			(Flying && !Breathless))))
+	    goto skipfloor;
+
 	if (feeding && metallivorous(youmonst.data)) {
 	    struct obj *gold;
 	    struct trap *ttmp = t_at(u.ux, u.uy);
@@ -2075,10 +2347,7 @@ floorfood(verb,corpsecheck)	/* get food from floor or pack */
 		}
 	    }
 
-	    if (
-#ifdef STEED
-	    	!u.usteed && 
-#endif
+	    if (youmonst.data != &mons[PM_RUST_MONSTER] &&
 		(gold = g_at(u.ux, u.uy)) != 0) {
 		if (gold->quan == 1L)
 		    Sprintf(qbuf, "There is 1 gold piece here; eat it?");
@@ -2095,19 +2364,13 @@ floorfood(verb,corpsecheck)	/* get food from floor or pack */
 	}
 
 	/* Is there some food (probably a heavy corpse) here on the ground? */
-	if (
-#ifdef STEED
-	    !u.usteed && 
-#endif
-	    !(Levitation && !Is_airlevel(&u.uz)  && !Is_waterlevel(&u.uz))
-	    && !u.uswallow) {
-	    for(otmp = level.objects[u.ux][u.uy]; otmp; otmp = otmp->nexthere) {
+	for (otmp = level.objects[u.ux][u.uy]; otmp; otmp = otmp->nexthere) {
 		if(corpsecheck ?
 		(otmp->otyp==CORPSE && (corpsecheck == 1 || tinnable(otmp))) :
 		    feeding ? (otmp->oclass != GOLD_CLASS && is_edible(otmp)) :
 						otmp->oclass==FOOD_CLASS) {
 			Sprintf(qbuf, "There %s %s here; %s %s?",
-				(otmp->quan == 1L) ? "is" : "are",
+				otense(otmp, "are"),
 				doname(otmp), verb,
 				(otmp->quan == 1L) ? "it" : "one");
 			if((c = yn_function(qbuf,ynqchars,'n')) == 'y')
@@ -2115,8 +2378,9 @@ floorfood(verb,corpsecheck)	/* get food from floor or pack */
 			else if(c == 'q')
 				return((struct obj *) 0);
 		}
-	    }
 	}
+
+ skipfloor:
 	/* We cannot use ALL_CLASSES since that causes getobj() to skip its
 	 * "ugly checks" and we need to check for inedible items.
 	 */
@@ -2149,11 +2413,85 @@ register struct obj *obj;
 	uneaten_amt = (long)obj->oeaten;
 	full_amount = (obj->otyp == CORPSE) ? (long)mons[obj->corpsenm].cnutrit
 					: (long)objects[obj->otyp].oc_nutrition;
+	if (uneaten_amt > full_amount) {
+	    impossible(
+	  "partly eaten food (%ld) more nutritious than untouched food (%ld)",
+		       uneaten_amt, full_amount);
+	    uneaten_amt = full_amount;
+	}
 
 	base = (int)(full_amount ? (long)base * uneaten_amt / full_amount : 0L);
 	return (base < 1) ? 1 : base;
 }
 
+/* reduce obj's oeaten field, making sure it never hits or passes 0 */
+void
+consume_oeaten(obj, amt)
+struct obj *obj;
+int amt;
+{
+    /*
+     * This is a hack to try to squelch several long standing mystery
+     * food bugs.  A better solution would be to rewrite the entire
+     * victual handling mechanism from scratch using a less complex
+     * model.  Alternatively, this routine could call done_eating()
+     * or food_disappears() but its callers would need revisions to
+     * cope with victual.piece unexpectedly going away.
+     *
+     * Multi-turn eating operates by setting the food's oeaten field
+     * to its full nutritional value and then running a counter which
+     * independently keeps track of whether there is any food left.
+     * The oeaten field can reach exactly zero on the last turn, and
+     * the object isn't removed from inventory until the next turn
+     * when the "you finish eating" message gets delivered, so the
+     * food would be restored to the status of untouched during that
+     * interval.  This resulted in unexpected encumbrance messages
+     * at the end of a meal (if near enough to a threshold) and would
+     * yield full food if there was an interruption on the critical
+     * turn.  Also, there have been reports over the years of food
+     * becoming massively heavy or producing unlimited satiation;
+     * this would occur if reducing oeaten via subtraction attempted
+     * to drop it below 0 since its unsigned type would produce a
+     * huge positive value instead.  So far, no one has figured out
+     * _why_ that inappropriate subtraction might sometimes happen.
+     */
+
+    if (amt > 0) {
+	/* bit shift to divide the remaining amount of food */
+	obj->oeaten >>= amt;
+    } else {
+	/* simple decrement; value is negative so we actually add it */
+	if ((int) obj->oeaten > -amt)
+	    obj->oeaten += amt;
+	else
+	    obj->oeaten = 0;
+    }
+
+    if (obj->oeaten == 0) {
+	if (obj == victual.piece)	/* always true unless wishing... */
+	    victual.reqtime = victual.usedtime;	/* no bites left */
+	obj->oeaten = 1;	/* smallest possible positive value */
+    }
+}
+
 #endif /* OVLB */
+#ifdef OVL1
+
+/* called when eatfood occupation has been interrupted,
+   or in the case of theft, is about to be interrupted */
+boolean
+maybe_finished_meal(stopping)
+boolean stopping;
+{
+	/* in case consume_oeaten() has decided that the food is all gone */
+	if (occupation == eatfood && victual.usedtime >= victual.reqtime) {
+	    if (stopping) occupation = 0;	/* for do_reset_eat */
+	    (void) eatfood(); /* calls done_eating() to use up victual.piece */
+	    return TRUE;
+	}
+	return FALSE;
+}
+
+#endif /* OVL1 */
 
 /*eat.c*/
