@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)engrave.c	3.3	1999/08/16	*/
+/*	SCCS Id: @(#)engrave.c	3.4	2001/11/04	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -43,7 +43,7 @@ char *outbuf;
 
 	/* a random engraving may come from the "rumors" file,
 	   or from the list above */
-	if (!rn2(4) || !(rumor = getrumor(0, outbuf)) || !*rumor)
+	if (!rn2(4) || !(rumor = getrumor(0, outbuf, TRUE)) || !*rumor)
 	    Strcpy(outbuf, random_mesg[rn2(SIZE(random_mesg))]);
 
 	wipeout_text(outbuf, (int)(strlen(outbuf) / 4), 0);
@@ -153,7 +153,7 @@ register int x, y;
 	if ((x == u.ux) && (y == u.uy) && u.uswallow &&
 		is_animal(u.ustuck->data))
 	    return "maw";
-	else if (IS_AIR(lev->typ))
+	else if (IS_AIR(lev->typ) && Is_airlevel(&u.uz))
 	    return "air";
 	else if (is_pool(x,y))
 	    return "water";
@@ -165,6 +165,10 @@ register int x, y;
 	    return "bridge";
 	else if(IS_ALTAR(levl[x][y].typ))
 	    return "altar";
+	else if(IS_GRAVE(levl[x][y].typ))
+	    return "headstone";
+	else if(IS_FOUNTAIN(levl[x][y].typ))
+	    return "fountain";
 	else if ((IS_ROOM(lev->typ) && !Is_earthlevel(&u.uz)) ||
 		 IS_WALL(lev->typ) || IS_DOOR(lev->typ) || lev->typ == SDOOR)
 	    return "floor";
@@ -216,8 +220,10 @@ xchar x, y;
 }
 
 #ifdef ELBERETH
-/* decide whether a particular string is engraved at a specified location;
-   a case-insensitive substring match used */
+/* Decide whether a particular string is engraved at a specified
+ * location; a case-insensitive substring match used.
+ * Ignore headstones, in case the player names herself "Elbereth".
+ */
 int
 sengr_at(s, x, y)
 	const char *s;
@@ -225,7 +231,8 @@ sengr_at(s, x, y)
 {
 	register struct engr *ep = engr_at(x,y);
 
-	return (ep && ep->engr_time <= moves && strstri(ep->engr_txt, s) != 0);
+	return (ep && ep->engr_type != HEADSTONE &&
+		ep->engr_time <= moves && strstri(ep->engr_txt, s) != 0);
 }
 #endif /* ELBERETH */
 
@@ -249,9 +256,10 @@ register xchar x,y,cnt;
 {
 	register struct engr *ep = engr_at(x,y);
 
-	if(ep){
+	/* Headstones are indelible */
+	if(ep && ep->engr_type != HEADSTONE){
 	    if(ep->engr_type != BURN || is_ice(x,y)) {
-		if(ep->engr_type != DUST && ep->engr_type != BLOOD) {
+		if(ep->engr_type != DUST && ep->engr_type != ENGR_BLOOD) {
 			cnt = rn2(1 + 50/(cnt+1)) ? 0 : 1;
 		}
 		wipeout_text(ep->engr_txt, (int)cnt, 0);
@@ -271,7 +279,11 @@ register int x,y;
 {
 	register struct engr *ep = engr_at(x,y);
 	register int	sensed = 0;
-
+	char buf[BUFSZ];
+	
+	/* Sensing an engraving does not require sight,
+	 * nor does it necessarily imply comprehension (literacy).
+	 */
 	if(ep && ep->engr_txt[0]) {
 	    switch(ep->engr_type) {
 	    case DUST:
@@ -282,6 +294,7 @@ register int x,y;
 		}
 		break;
 	    case ENGRAVE:
+	    case HEADSTONE:
 		if (!Blind || can_reach_floor()) {
 			sensed = 1;
 			pline("%s is engraved here on the %s.",
@@ -304,7 +317,7 @@ register int x,y;
 				surface(x,y));
 		}
 		break;
-	    case BLOOD:
+	    case ENGR_BLOOD:
 		/* "It's a message!  Scrawled in blood!"
 		 * "What's it say?"
 		 * "It says... `See you next Wednesday.'" -- Thriller
@@ -320,8 +333,16 @@ register int x,y;
 		sensed = 1;
 	    }
 	    if (sensed) {
+	    	char *et;
+	    	unsigned maxelen = BUFSZ - sizeof("You feel the words: \"\". ");
+	    	if (strlen(ep->engr_txt) > maxelen) {
+	    		(void) strncpy(buf,  ep->engr_txt, (int)maxelen);
+			buf[maxelen] = '\0';
+			et = buf;
+		} else
+			et = ep->engr_txt;
 		You("%s: \"%s\".",
-		      (Blind) ? "feel the words" : "read",  ep->engr_txt);
+		      (Blind) ? "feel the words" : "read",  et);
 		if(flags.run > 1) nomul(0);
 	    }
 	}
@@ -348,9 +369,10 @@ register xchar e_type;
 	ep->engr_y = y;
 	ep->engr_txt = (char *)(ep + 1);
 	Strcpy(ep->engr_txt, s);
-	if(strcmp(s, "Elbereth")) exercise(A_WIS, TRUE);
+	/* engraving Elbereth shows wisdom */
+	if(!strcmp(s, "Elbereth")) exercise(A_WIS, TRUE);
 	ep->engr_time = e_time;
-	ep->engr_type = e_type > 0 ? e_type : rnd(N_ENGRAVE);
+	ep->engr_type = e_type > 0 ? e_type : rnd(N_ENGRAVE-1);
 	ep->engr_lth = strlen(s) + 1;
 }
 
@@ -429,16 +451,14 @@ doengrave()
 	char post_engr_text[BUFSZ]; /* Text displayed after engraving prompt */
 	const char *everb;	/* Present tense of engraving type */
 	const char *eloc;	/* Where the engraving is (ie dust/floor/...) */
-	register char *sp;	/* Place holder for space count of engr text */
-	register int len;	/* # of nonspace chars of new engraving text */
-	register int maxelen;	/* Max allowable length of new engraving text */
-	register int spct;	/* # of spaces in new engraving text */
-	register struct engr *oep = engr_at(u.ux,u.uy);
+	char *sp;		/* Place holder for space count of engr text */
+	int len;		/* # of nonspace chars of new engraving text */
+	int maxelen;		/* Max allowable length of engraving text */
+	struct engr *oep = engr_at(u.ux,u.uy);
 				/* The current engraving */
-	register struct obj *otmp; /* Object selected with which to engrave */
+	struct obj *otmp;	/* Object selected with which to engrave */
 	char *writer;
 
-
 	multi = 0;		/* moves consumed */
 	nomovemsg = (char *)0;	/* occupation end message */
 
@@ -446,6 +466,8 @@ doengrave()
 	ebuf[0] = (char)0;
 	post_engr_text[0] = (char)0;
 	maxelen = BUFSZ - 1;
+	if (is_demon(youmonst.data) || youmonst.data->mlet == S_VAMPIRE)
+	    type = ENGR_BLOOD;
 
 	/* Can the adventurer engrave at all? */
 
@@ -507,6 +529,15 @@ doengrave()
 		altar_wrath(u.ux, u.uy);
 		return(0);
 	}
+	if (IS_GRAVE(levl[u.ux][u.uy].typ)) {
+	    if (!levl[u.ux][u.uy].disturbed) {
+		You("disturb the undead!");
+		levl[u.ux][u.uy].disturbed = 1;
+		(void) makemon(&mons[PM_GHOUL], u.ux, u.uy, NO_MM_FLAGS);
+		exercise(A_WIS, FALSE);
+		return(1);
+	    }
+	}
 
 	/* SPFX for items */
 
@@ -647,7 +678,7 @@ doengrave()
 			    break;
 		    case WAN_CANCELLATION:
 		    case WAN_MAKE_INVISIBLE:
-			if(oep) {
+			if (oep && oep->engr_type != HEADSTONE) {
 			    if (!Blind)
 				pline_The("engraving on the %s vanishes!",
 					surface(u.ux,u.uy));
@@ -655,7 +686,7 @@ doengrave()
 			}
 			break;
 		    case WAN_TELEPORTATION:
-			if (oep) {
+			if (oep && oep->engr_type != HEADSTONE) {
 			    if (!Blind)
 				pline_The("engraving on the %s vanishes!",
 					surface(u.ux,u.uy));
@@ -713,7 +744,7 @@ doengrave()
 			break;
 
 		    /* type = MARK wands */
-		    /* type = BLOOD wands */
+		    /* type = ENGR_BLOOD wands */
 		    }
 		} else /* end if zappable */
 		    if (!can_reach_floor()) {
@@ -723,11 +754,12 @@ doengrave()
 		break;
 
 	    case WEAPON_CLASS:
-		if(is_blade(otmp))
+		if (is_blade(otmp)) {
 		    if ((int)otmp->spe > -3)
 			type = ENGRAVE;
 		    else
 			Your("%s too dull for engraving.", aobjnam(otmp,"are"));
+		}
 		break;
 
 	    case TOOL_CLASS:
@@ -748,20 +780,21 @@ doengrave()
 			ptext = FALSE;
 			if (oep)
 			    if ((oep->engr_type == DUST ) ||
-				(oep->engr_type == BLOOD) ||
+				(oep->engr_type == ENGR_BLOOD) ||
 				(oep->engr_type == MARK )) {
 				if (!Blind)
 				    You("wipe out the message here.");
 				else
-				    Your("%s gets %s.", xname(otmp),
-					  is_ice(u.ux,u.uy) ?
-					  "frosty" : "dusty");
+				    Your("%s %s %s.", xname(otmp),
+					 otense(otmp, "get"),
+					 is_ice(u.ux,u.uy) ?
+					 "frosty" : "dusty");
 				dengr = TRUE;
 			    } else
 				Your("%s can't wipe out this engraving.",
 				     xname(otmp));
 			else
-			    Your("%s gets %s.", xname(otmp),
+			    Your("%s %s %s.", xname(otmp), otense(otmp, "get"),
 				  is_ice(u.ux,u.uy) ? "frosty" : "dusty");
 			break;
 		    default:
@@ -781,6 +814,18 @@ doengrave()
 		break;
 	}
 
+	if (IS_GRAVE(levl[u.ux][u.uy].typ)) {
+	    if (type == ENGRAVE || type == 0)
+		type = HEADSTONE;
+	    else {
+		/* ensures the "cannot wipe out" case */
+		type = DUST;
+		dengr = FALSE;
+		teleengr = FALSE;
+		buf[0] = (char)0;
+	    }
+	}
+
 	/* End of implement setup */
 
 	/* Identify stylus */
@@ -809,8 +854,9 @@ doengrave()
 	if (zapwand && (otmp->spe < 0)) {
 	    pline("%s %sturns to dust.",
 		  The(xname(otmp)), Blind ? "" : "glows violently, then ");
- You("are not going to get anywhere trying to write in the %s with your dust.",
-		is_ice(u.ux,u.uy) ? "frost" : "dust");
+	    if (!IS_GRAVE(levl[u.ux][u.uy].typ))
+		You("are not going to get anywhere trying to write in the %s with your dust.",
+		    is_ice(u.ux,u.uy) ? "frost" : "dust");
 	    useup(otmp);
 	    ptext = FALSE;
 	}
@@ -830,24 +876,27 @@ doengrave()
 
 	    /* Give player the choice to add to engraving. */
 
-	    if ( (type == oep->engr_type) && (!Blind ||
+	    if (type == HEADSTONE) {
+		/* no choice, only append */
+		c = 'y';
+	    } else if ( (type == oep->engr_type) && (!Blind ||
 		 (oep->engr_type == BURN) || (oep->engr_type == ENGRAVE)) ) {
 		c = yn_function("Do you want to add to the current engraving?",
 				ynqchars, 'y');
 		if (c == 'q') {
-		    pline("Never mind.");
+		    pline(Never_mind);
 		    return(0);
 		}
 	    }
 
 	    if (c == 'n' || Blind) {
 
-		if( (oep->engr_type == DUST) || (oep->engr_type == BLOOD) ||
+		if( (oep->engr_type == DUST) || (oep->engr_type == ENGR_BLOOD) ||
 		    (oep->engr_type == MARK) ) {
 		    if (!Blind) {
 			You("wipe out the message that was %s here.",
 			    ((oep->engr_type == DUST)  ? "written in the dust" :
-			    ((oep->engr_type == BLOOD) ? "scrawled in blood"   :
+			    ((oep->engr_type == ENGR_BLOOD) ? "scrawled in blood"   :
 							 "written")));
 			del_engr(oep);
 			oep = (struct engr *)0;
@@ -855,7 +904,7 @@ doengrave()
 		   /* Don't delete engr until after we *know* we're engraving */
 			eow = TRUE;
 		} else
-		    if ( (type == DUST) || (type == MARK) || (type == BLOOD) ) {
+		    if ( (type == DUST) || (type == MARK) || (type == ENGR_BLOOD) ) {
 			You(
 			 "cannot wipe out the message that is %s the %s here.",
 			 oep->engr_type == BURN ?
@@ -868,7 +917,7 @@ doengrave()
 				You("will overwrite the current message.");
 			    eow = TRUE;
 			}
-	}
+	    }
 	}
 
 	eloc = surface(u.ux,u.uy);
@@ -896,7 +945,7 @@ doengrave()
 		everb = (oep && !eow ? "add to the graffiti on" :
 				       "scribble on");
 		break;
-	    case BLOOD:
+	    case ENGR_BLOOD:
 		everb = (oep && !eow ? "add to the scrawl on" :
 				       "scrawl on");
 		break;
@@ -913,31 +962,36 @@ doengrave()
 	Sprintf(qbuf,"What do you want to %s the %s here?", everb, eloc);
 	getlin(qbuf, ebuf);
 
-	/* Mix up engraving if surface or state of mind is unsound.  */
-	/* Original kludge by stewr 870708.  modified by njm 910722. */
-	for (sp = ebuf; *sp; sp++)
-	    if ( ((type == DUST || type == BLOOD) && !rn2(25)) ||
-		 (Blind   && !rn2(9)) || (Confusion     && !rn2(12)) ||
-		 (Stunned && !rn2(4)) || (Hallucination && !rn2(1)) )
-		 *sp = '!' + rn2(93); /* ASCII-code only */
-
 	/* Count the actual # of chars engraved not including spaces */
 	len = strlen(ebuf);
+	for (sp = ebuf; *sp; sp++) if (isspace(*sp)) len -= 1;
 
-	for (sp = ebuf, spct = 0; *sp; sp++) if (isspace(*sp)) spct++;
-
-	if ( (len == spct) || index(ebuf, '\033') ) {
+	if (len == 0 || index(ebuf, '\033')) {
 	    if (zapwand) {
 		if (!Blind)
-		    pline("%s glows, then fades.", The(xname(otmp)));
+		    pline("%s, then %s.",
+			  Tobjnam(otmp, "glow"), otense(otmp, "fade"));
 		return(1);
 	    } else {
-		pline("Never mind.");
+		pline(Never_mind);
 		return(0);
 	    }
 	}
 
-	len -= spct;
+	/* A single `x' is the traditional signature of an illiterate person */
+	if (len != 1 || (!index(ebuf, 'x') && !index(ebuf, 'X')))
+	    u.uconduct.literate++;
+
+	/* Mix up engraving if surface or state of mind is unsound.
+	   Note: this won't add or remove any spaces. */
+	for (sp = ebuf; *sp; sp++) {
+	    if (isspace(*sp)) continue;
+	    if (((type == DUST || type == ENGR_BLOOD) && !rn2(25)) ||
+		    (Blind && !rn2(11)) || (Confusion && !rn2(7)) ||
+		    (Stunned && !rn2(4)) || (Hallucination && !rn2(2)))
+		*sp = ' ' + rnd(96 - 2);	/* ASCII '!' thru '~'
+						   (excludes ' ' and DEL) */
+	}
 
 	/* Previous engraving is overwritten */
 	if (eow) {
@@ -970,12 +1024,19 @@ doengrave()
 			 *	 "ere", then "th".
 			 */
 		    Your("%s dull.", aobjnam(otmp, "get"));
+		    if (otmp->unpaid) {
+			struct monst *shkp = shop_keeper(*u.ushops);
+			if (shkp) {
+			    You("damage it, you pay for it!");
+			    bill_dummy_object(otmp);
+			}
+		    }
 		    if (len > maxelen) {
 			multi = -maxelen;
 			otmp->spe = -3;
-		    } else
-			if (len > 1) otmp->spe -= len >> 1;
-			else otmp->spe -= 1; /* Prevent infinite engraving */
+		    } else if (len > 1)
+			otmp->spe -= len >> 1;
+		    else otmp->spe -= 1; /* Prevent infinite engraving */
 		} else
 		    if ( (otmp->oclass == RING_CLASS) ||
 			 (otmp->oclass == GEM_CLASS) )
@@ -1004,7 +1065,7 @@ doengrave()
 		}
 		if (multi) nomovemsg = "You finish defacing the dungeon.";
 		break;
-	    case BLOOD:
+	    case ENGR_BLOOD:
 		multi = -(len/10);
 		if (multi) nomovemsg = "You finish scrawling.";
 		break;
@@ -1033,10 +1094,9 @@ doengrave()
 	if (doblind && !resists_blnd(&youmonst)) {
 	    You("are blinded by the flash!");
 	    make_blinded((long)rnd(50),FALSE);
+	    if (!Blind) Your(vision_clears);
 	}
 
-	u.uconduct.literate++;
-
 	return(1);
 }
 
@@ -1127,6 +1187,62 @@ struct engr *ep;
 	ep->engr_y = ty;
 }
 
+
+/* Epitaphs for random headstones */
+static const char *epitaphs[] = {
+	"Rest in peace",
+	"R.I.P.",
+	"Rest In Pieces",
+	"Note -- there are NO valuable items in this grave",
+	"1994-1995. The Longest-Lived Hacker Ever",
+	"The Grave of the Unknown Hacker",
+	"We weren't sure who this was, but we buried him here anyway",
+	"Sparky -- he was a very good dog",
+	"Beware of Electric Third Rail",
+	"Made in Taiwan",
+	"Og friend. Og good dude. Og died. Og now food",
+	"Beetlejuice Beetlejuice Beetlejuice",
+	"Look out below!",
+	"Please don't dig me up. I'm perfectly happy down here. -- Resident",
+	"Postman, please note forwarding address: Gehennom, Asmodeus's Fortress, fifth lemure on the left",
+	"Mary had a little lamb/Its fleece was white as snow/When Mary was in trouble/The lamb was first to go",
+	"Be careful, or this could happen to you!",
+	"Soon you'll join this fellow in hell! -- the Wizard of Yendor",
+	"Caution! This grave contains toxic waste",
+	"Sum quod eris",
+	"Here lies an Atheist, all dressed up and no place to go",
+	"Here lies Ezekiel, age 102.  The good die young.",
+	"Here lies my wife: Here let her lie! Now she's at rest and so am I.",
+	"Here lies Johnny Yeast. Pardon me for not rising.",
+	"He always lied while on the earth and now he's lying in it",
+	"I made an ash of myself",
+	"Soon ripe. Soon rotten. Soon gone. But not forgotten.",
+	"Here lies the body of Jonathan Blake. Stepped on the gas instead of the brake.",
+	"Go away!"
+};
+
+/* Create a headstone at the given location.
+ * The caller is responsible for newsym(x, y).
+ */
+void
+make_grave(x, y, str)
+int x, y;
+const char *str;
+{
+	/* Can we put a grave here? */
+	if ((levl[x][y].typ != ROOM && levl[x][y].typ != GRAVE) || t_at(x,y)) return;
+
+	/* Make the grave */
+	levl[x][y].typ = GRAVE;
+
+	/* Engrave the headstone */
+	if (!str) str = epitaphs[rn2(SIZE(epitaphs))];
+	del_engr_at(x, y);
+	make_engr_at(x, y, str, 0L, HEADSTONE);
+	return;
+}
+
+
 #endif /* OVLB */
 
 /*engrave.c*/
