@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)exper.c	3.3	97/01/29	*/
+/*	SCCS Id: @(#)exper.c	3.4	2002/01/15	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -39,7 +39,7 @@ int
 experience(mtmp, nk)	/* return # of exp points for mtmp after nk killed */
 	register struct	monst *mtmp;
 	register int	nk;
-#if defined(applec)
+#if defined(macintosh) && (defined(__SC__) || defined(__MRC__))
 # pragma unused(nk)
 #endif
 {
@@ -110,34 +110,49 @@ more_experienced(exp, rexp)
 }
 
 void
-losexp()		/* hit by drain life attack */
+losexp(drainer)		/* e.g., hit by drain life attack */
+const char *drainer;	/* cause of death, if drain should be fatal */
 {
 	register int num;
 
 	if (resists_drli(&youmonst)) return;
 
-	if(u.ulevel > 1) {
+	if (u.ulevel > 1) {
 		pline("%s level %d.", Goodbye(), u.ulevel--);
 		/* remove intrinsic abilities */
 		adjabil(u.ulevel + 1, u.ulevel);
 		reset_rndmonst(NON_PM);	/* new monster selection */
-	} else
-		u.uhp = -1;
+	} else {
+		if (drainer) {
+			killer_format = KILLED_BY;
+			killer = drainer;
+			done(DIED);
+		}
+		/* no drainer or lifesaved */
+		u.uexp = 0;
+	}
 	num = newhp();
-	u.uhp -= num;
 	u.uhpmax -= num;
+	if (u.uhpmax < 1) u.uhpmax = 1;
+	u.uhp -= num;
+	if (u.uhp < 1) u.uhp = 1;
+	else if (u.uhp > u.uhpmax) u.uhp = u.uhpmax;
+
 	if (u.ulevel < urole.xlev)
-	    num = rn1(u.ulevel/2 + urole.enadv.lornd + urace.enadv.lornd,
+	    num = rn1((int)ACURR(A_WIS)/2 + urole.enadv.lornd + urace.enadv.lornd,
 	    		urole.enadv.lofix + urace.enadv.lofix);
 	else
-	    num = rn1(u.ulevel/2 + urole.enadv.hirnd + urace.enadv.hirnd,
+	    num = rn1((int)ACURR(A_WIS)/2 + urole.enadv.hirnd + urace.enadv.hirnd,
 	    		urole.enadv.hifix + urace.enadv.hifix);
 	num = enermod(num);		/* M. Stephenson */
-	u.uen -= num;
-	if (u.uen < 0)		u.uen = 0;
 	u.uenmax -= num;
-	if (u.uenmax < 0)	u.uenmax = 0;
-	u.uexp = newuexp(u.ulevel) - 1;
+	if (u.uenmax < 0) u.uenmax = 0;
+	u.uen -= num;
+	if (u.uen < 0) u.uen = 0;
+	else if (u.uen > u.uenmax) u.uen = u.uenmax;
+
+	if (u.uexp > 0)
+		u.uexp = newuexp(u.ulevel) - 1;
 	flags.botl = 1;
 }
 
@@ -164,12 +179,17 @@ boolean incr;	/* true iff via incremental experience growth */
 	num = newhp();
 	u.uhpmax += num;
 	u.uhp += num;
+	if (Upolyd) {
+	    num = rnd(8);
+	    u.mhmax += num;
+	    u.mh += num;
+	}
 	if (u.ulevel < urole.xlev)
 	    num = rn1((int)ACURR(A_WIS)/2 + urole.enadv.lornd + urace.enadv.lornd,
-	    		urole.enadv.lofix + urace.enadv.lofix);
+			urole.enadv.lofix + urace.enadv.lofix);
 	else
 	    num = rn1((int)ACURR(A_WIS)/2 + urole.enadv.hirnd + urace.enadv.hirnd,
-	    		urole.enadv.hifix + urace.enadv.hifix);
+			urole.enadv.hifix + urace.enadv.hifix);
 	num = enermod(num);	/* M. Stephenson */
 	u.uenmax += num;
 	u.uen += num;
