@@ -1,10 +1,14 @@
-/*	SCCS Id: @(#)files.c	3.3	97/01/26	*/
+/*	SCCS Id: @(#)files.c	3.4	2002/02/23	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
 #include "dlb.h"
 
+#ifdef TTY_GRAPHICS
+#include "wintty.h" /* more() */
+#endif
+
 #include <ctype.h>
 
 #if !defined(MAC) && !defined(O_WRONLY) && !defined(AZTEC_C)
@@ -29,6 +33,11 @@ extern int errno;
 # define O_BINARY 0
 #endif
 
+#ifdef PREFIXES_IN_USE
+#define FQN_NUMBUF 2
+static char fqn_filename_buffer[FQN_NUMBUF][FQN_MAX_FILENAME];
+#endif
+
 #if !defined(MFLOPPY) && !defined(VMS) && !defined(WIN32)
 char bones[] = "bonesnn.xxx";
 char lock[PL_NSIZ+14] = "1lock"; /* long enough for uid+name+.99 */
@@ -43,7 +52,7 @@ char lock[PL_NSIZ+17] = "1lock"; /* long enough for _uid+name+.99;1 */
 # endif
 # if defined(WIN32)
 char bones[] = "bonesnn.xxx";
-char lock[PL_NSIZ+25];		/* long enough for username+.+name+.99 */
+char lock[PL_NSIZ+25];		/* long enough for username+-+name+.99 */
 # endif
 #endif
 
@@ -66,6 +75,12 @@ char SAVEF[SAVESIZE];	/* holds relative path of save file from playground */
 char SAVEP[SAVESIZE];	/* holds path of directory for save file */
 #endif
 
+#ifdef WIZARD
+#define WIZKIT_MAX 128
+static char wizkit[WIZKIT_MAX];
+STATIC_DCL FILE *NDECL(fopen_wizkit_file);
+#endif
+
 #ifdef AMIGA
 extern char PATH[];	/* see sys/amiga/amidos.c */
 extern char bbs_id[];
@@ -75,6 +90,7 @@ static int lockptr;
 # endif
 
 #include <libraries/dos.h>
+extern void FDECL(amii_set_text_font, ( char *, int ));
 #endif
 
 #if defined(WIN32) || defined(MSDOS)
@@ -86,41 +102,79 @@ static int lockptr;
 #define DeleteFile unlink
 #endif
 
+#ifdef USER_SOUNDS
+extern char *sounddir;
+#endif
+
 extern int n_dgns;		/* from dungeon.c */
 
 STATIC_DCL char *FDECL(set_bonesfile_name, (char *,d_level*));
 STATIC_DCL char *NDECL(set_bonestemp_name);
+#ifdef COMPRESS
+STATIC_DCL void FDECL(redirect, (const char *,const char *,FILE *,BOOLEAN_P));
+STATIC_DCL void FDECL(docompress_file, (const char *,BOOLEAN_P));
+#endif
 STATIC_DCL char *FDECL(make_lockname, (const char *,char *));
 STATIC_DCL FILE *FDECL(fopen_config_file, (const char *));
 STATIC_DCL int FDECL(get_uchars, (FILE *,char *,char *,uchar *,int,const char *));
 int FDECL(parse_config_line, (FILE *,char *,char *,char *));
-
-#ifdef WIN32
-extern void NDECL(dircheck);
+#ifdef NOCWD_ASSUMPTIONS
+STATIC_DCL void FDECL(adjust_prefix, (char *, int));
 #endif
 
+#ifndef PREFIXES_IN_USE
+/*ARGSUSED*/
+#endif
+const char *
+fqname(basename, whichprefix, buffnum)
+const char *basename;
+int whichprefix, buffnum;
+{
+#ifndef PREFIXES_IN_USE
+	return basename;
+#else
+	if (!basename || whichprefix < 0 || whichprefix >= PREFIX_COUNT)
+		return basename;
+	if (!fqn_prefix[whichprefix])
+		return basename;
+	if (buffnum < 0 || buffnum >= FQN_NUMBUF) {
+		impossible("Invalid fqn_filename_buffer specified: %d",
+								buffnum);
+		buffnum = 0;
+	}
+	if (strlen(fqn_prefix[whichprefix]) + strlen(basename) >=
+						    FQN_MAX_FILENAME) {
+		impossible("fqname too long: %s + %s", fqn_prefix[whichprefix],
+						basename);
+		return basename;	/* XXX */
+	}
+	Strcpy(fqn_filename_buffer[buffnum], fqn_prefix[whichprefix]);
+	return strcat(fqn_filename_buffer[buffnum], basename);
+#endif
+}
+
+
 /* fopen a file, with OS-dependent bells and whistles */
 /* NOTE: a simpler version of this routine also exists in util/dlb_main.c */
 FILE *
-fopen_datafile(filename, mode)
+fopen_datafile(filename, mode, use_scoreprefix)
 const char *filename, *mode;
+boolean use_scoreprefix;
 {
 	FILE *fp;
-#ifdef WIN32
-	dircheck();
-#endif
-#ifdef AMIGA
-	fp = fopenp(filename, mode);
-#else
-# ifdef VMS	/* essential to have punctuation, to avoid logical names */
+
+	filename = fqname(filename,
+				use_scoreprefix ? SCOREPREFIX : DATAPREFIX, 0);
+#ifdef VMS	/* essential to have punctuation, to avoid logical names */
+    {
 	char tmp[BUFSIZ];
 
 	if (!index(filename, '.') && !index(filename, ';'))
 		filename = strcat(strcpy(tmp, filename), ";0");
 	fp = fopen(filename, mode, "mbc=16");
-# else
+    }
+#else
 	fp = fopen(filename, mode);
-# endif
 #endif
 	return fp;
 }
@@ -144,7 +198,9 @@ set_lock_and_bones()
 	append_slash(bones);
 	Strcat(bones, "bonesnn.*");
 	Strcpy(lock, levels);
+#ifndef AMIGA
 	Strcat(lock, alllevels);
+#endif
 	return;
 }
 #endif /* MFLOPPY */
@@ -177,22 +233,21 @@ create_levelfile(lev)
 int lev;
 {
 	int fd;
+	const char *fq_lock;
 
-#ifdef WIN32
-	dircheck();
-#endif
 	set_levelfile_name(lock, lev);
+	fq_lock = fqname(lock, LEVELPREFIX, 0);
 
 #if defined(MICRO)
 	/* Use O_TRUNC to force the file to be shortened if it already
 	 * exists and is currently longer.
 	 */
-	fd = open(lock, O_WRONLY |O_CREAT | O_TRUNC | O_BINARY, FCMASK);
+	fd = open(fq_lock, O_WRONLY |O_CREAT | O_TRUNC | O_BINARY, FCMASK);
 #else
 # ifdef MAC
-	fd = maccreat(lock, LEVL_TYPE);
+	fd = maccreat(fq_lock, LEVL_TYPE);
 # else
-	fd = creat(lock, FCMASK);
+	fd = creat(fq_lock, FCMASK);
 # endif
 #endif /* MICRO */
 
@@ -208,20 +263,19 @@ open_levelfile(lev)
 int lev;
 {
 	int fd;
+	const char *fq_lock;
 
-#ifdef WIN32
-	dircheck();
-#endif
 	set_levelfile_name(lock, lev);
+	fq_lock = fqname(lock, LEVELPREFIX, 0);
 #ifdef MFLOPPY
 	/* If not currently accessible, swap it in. */
 	if (level_info[lev].where != ACTIVE)
 		swapin_file(lev);
 #endif
 #ifdef MAC
-	fd = macopen(lock, O_RDONLY | O_BINARY, LEVL_TYPE);
+	fd = macopen(fq_lock, O_RDONLY | O_BINARY, LEVL_TYPE);
 #else
-	fd = open(lock, O_RDONLY | O_BINARY, 0);
+	fd = open(fq_lock, O_RDONLY | O_BINARY, 0);
 #endif
 	return fd;
 }
@@ -231,16 +285,13 @@ void
 delete_levelfile(lev)
 int lev;
 {
-#ifdef WIN32
-	dircheck();
-#endif
 	/*
 	 * Level 0 might be created by port specific code that doesn't
 	 * call create_levfile(), so always assume that it exists.
 	 */
 	if (lev == 0 || (level_info[lev].flags & LFILE_EXISTS)) {
 		set_levelfile_name(lock, lev);
-		(void) unlink(lock);
+		(void) unlink(fqname(lock, LEVELPREFIX, 0));
 		level_info[lev].flags &= ~LFILE_EXISTS;
 	}
 }
@@ -249,7 +300,7 @@ int lev;
 void
 clearlocks()
 {
-#if defined(MFLOPPY) && !defined(AMIGA)
+#if !defined(PC_LOCKING) && defined(MFLOPPY) && !defined(AMIGA)
 	eraseall(levels, alllevels);
 	if (ramdisk)
 		eraseall(permbones, alllevels);
@@ -319,14 +370,12 @@ create_bonesfile(lev, bonesid)
 d_level *lev;
 char **bonesid;
 {
-	char *file;
+	const char *file;
 	int fd;
 
-#ifdef WIN32
-	dircheck();
-#endif
 	*bonesid = set_bonesfile_name(bones, lev);
 	file = set_bonestemp_name();
+	file = fqname(file, BONESPREFIX, 0);
 
 #ifdef MICRO
 	/* Use O_TRUNC to force the file to be shortened if it already
@@ -360,9 +409,10 @@ char **bonesid;
 void
 cancel_bonesfile()
 {
-	char *tempname;
+	const char *tempname;
 
 	tempname = set_bonestemp_name();
+	tempname = fqname(tempname, BONESPREFIX, 0);
 	(void) unlink(tempname);
 }
 #endif /* MFLOPPY */
@@ -372,25 +422,27 @@ void
 commit_bonesfile(lev)
 d_level *lev;
 {
-	char *tempname;
+	const char *fq_bones, *tempname;
 	int ret;
 
 	(void) set_bonesfile_name(bones, lev);
+	fq_bones = fqname(bones, BONESPREFIX, 0);
 	tempname = set_bonestemp_name();
+	tempname = fqname(tempname, BONESPREFIX, 1);
 
 #if (defined(SYSV) && !defined(SVR4)) || defined(GENIX)
 	/* old SYSVs don't have rename.  Some SVR3's may, but since they
 	 * also have link/unlink, it doesn't matter. :-)
 	 */
-	(void) unlink(bones);
-	ret = link(tempname, bones);
+	(void) unlink(fq_bones);
+	ret = link(tempname, fq_bones);
 	ret += unlink(tempname);
 #else
-	ret = rename(tempname, bones);
+	ret = rename(tempname, fq_bones);
 #endif
 #ifdef WIZARD
 	if (wizard && ret != 0)
-		pline("couldn't rename %s to %s", tempname, bones);
+		pline("couldn't rename %s to %s", tempname, fq_bones);
 #endif
 }
 
@@ -400,17 +452,16 @@ open_bonesfile(lev, bonesid)
 d_level *lev;
 char **bonesid;
 {
+	const char *fq_bones;
 	int fd;
 
-#ifdef WIN32
-	dircheck();
-#endif
 	*bonesid = set_bonesfile_name(bones, lev);
-	uncompress(bones);	/* no effect if nonexistent */
+	fq_bones = fqname(bones, BONESPREFIX, 0);
+	uncompress(fq_bones);	/* no effect if nonexistent */
 #ifdef MAC
-	fd = macopen(bones, O_RDONLY | O_BINARY, BONE_TYPE);
+	fd = macopen(fq_bones, O_RDONLY | O_BINARY, BONE_TYPE);
 #else
-	fd = open(bones, O_RDONLY | O_BINARY, 0);
+	fd = open(fq_bones, O_RDONLY | O_BINARY, 0);
 #endif
 	return fd;
 }
@@ -420,11 +471,8 @@ int
 delete_bonesfile(lev)
 d_level *lev;
 {
-#ifdef WIN32
-	dircheck();
-#endif
 	(void) set_bonesfile_name(bones, lev);
-	return !(unlink(bones) < 0);
+	return !(unlink(fqname(bones, BONESPREFIX, 0)) < 0);
 }
 
 
@@ -433,7 +481,7 @@ d_level *lev;
 void
 compress_bonesfile()
 {
-	compress(bones);
+	compress(fqname(bones, BONESPREFIX, 0));
 }
 
 /* ----------  END BONES FILE HANDLING ----------- */
@@ -514,34 +562,30 @@ set_error_savefile()
 int
 create_savefile()
 {
+	const char *fq_save;
 	int fd;
-#ifdef WIN32
-	dircheck();
-#endif
-#ifdef AMIGA
-	fd = ami_wbench_getsave(O_WRONLY | O_CREAT | O_TRUNC);
+
+	fq_save = fqname(SAVEF, SAVEPREFIX, 0);
+#ifdef MICRO
+	fd = open(fq_save, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, FCMASK);
 #else
-# ifdef MICRO
-	fd = open(SAVEF, O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, FCMASK);
+# ifdef MAC
+	fd = maccreat(fq_save, SAVE_TYPE);
 # else
-#  ifdef MAC
-	fd = maccreat(SAVEF, SAVE_TYPE);
-#  else
-	fd = creat(SAVEF, FCMASK);
-#  endif
-#  if defined(VMS) && !defined(SECURE)
+	fd = creat(fq_save, FCMASK);
+# endif
+# if defined(VMS) && !defined(SECURE)
 	/*
 	   Make sure the save file is owned by the current process.  That's
 	   the default for non-privileged users, but for priv'd users the
 	   file will be owned by the directory's owner instead of the user.
 	 */
-#   ifdef getuid	/*(see vmsunix.c)*/
-#    undef getuid
-#   endif
-	(void) chown(SAVEF, getuid(), getgid());
-#  endif /* VMS && !SECURE */
-# endif	/* MICRO */
-#endif	/* AMIGA */
+#  ifdef getuid	/*(see vmsunix.c)*/
+#   undef getuid
+#  endif
+	(void) chown(fq_save, getuid(), getgid());
+# endif /* VMS && !SECURE */
+#endif	/* MICRO */
 
 	return fd;
 }
@@ -551,20 +595,15 @@ create_savefile()
 int
 open_savefile()
 {
+	const char *fq_save;
 	int fd;
 
-#ifdef WIN32
-	dircheck();
-#endif
-#ifdef AMIGA
-	fd = ami_wbench_getsave(O_RDONLY);
+	fq_save = fqname(SAVEF, SAVEPREFIX, 0);
+#ifdef MAC
+	fd = macopen(fq_save, O_RDONLY | O_BINARY, SAVE_TYPE);
 #else
-# ifdef MAC
-	fd = macopen(SAVEF, O_RDONLY | O_BINARY, SAVE_TYPE);
-# else
-	fd = open(SAVEF, O_RDONLY | O_BINARY, 0);
-# endif
-#endif /* AMIGA */
+	fd = open(fq_save, O_RDONLY | O_BINARY, 0);
+#endif
 	return fd;
 }
 
@@ -573,10 +612,7 @@ open_savefile()
 int
 delete_savefile()
 {
-#ifdef AMIGA
-	ami_wbench_unlink(SAVEF);
-#endif
-	(void) unlink(SAVEF);
+	(void) unlink(fqname(SAVEF, SAVEPREFIX, 0));
 	return 0;	/* for restore_saved_game() (ex-xxxmain.c) test */
 }
 
@@ -585,26 +621,20 @@ delete_savefile()
 int
 restore_saved_game()
 {
+	const char *fq_save;
 	int fd;
-#ifdef WIN32
-	dircheck();
-#endif
 
 	set_savefile_name();
 #ifdef MFLOPPY
-	if (
-# ifdef AMIGA
-	    !(FromWBench || saveDiskPrompt(1))
-# else
-	    !saveDiskPrompt(1)
-# endif
-	  ) return -1;
+	if (!saveDiskPrompt(1))
+	    return -1;
 #endif /* MFLOPPY */
+	fq_save = fqname(SAVEF, SAVEPREFIX, 0);
 
-	uncompress(SAVEF);
+	uncompress(fq_save);
 	if ((fd = open_savefile()) < 0) return fd;
 
-	if (!uptodate(fd, SAVEF)) {
+	if (!uptodate(fd, fq_save)) {
 	    (void) close(fd),  fd = -1;
 	    (void) delete_savefile();
 	}
@@ -618,15 +648,12 @@ restore_saved_game()
 
 #ifdef COMPRESS
 
-void
+STATIC_OVL void
 redirect(filename, mode, stream, uncomp)
-char *filename, *mode;
+const char *filename, *mode;
 FILE *stream;
 boolean uncomp;
 {
-#ifdef WIN32
-	dircheck();
-#endif
 	if (freopen(filename, mode, stream) == (FILE *)0) {
 		(void) fprintf(stderr, "freopen of %s for %scompress failed\n",
 			filename, uncomp ? "un" : "");
@@ -641,9 +668,9 @@ boolean uncomp;
  *
  * cf. child() in unixunix.c.
  */
-void
+STATIC_OVL void
 docompress_file(filename, uncomp)
-char *filename;
+const char *filename;
 boolean uncomp;
 {
 	char cfn[80];
@@ -654,10 +681,10 @@ boolean uncomp;
 # endif
 	int i = 0;
 	int f;
+# ifdef TTY_GRAPHICS
+	boolean istty = !strncmpi(windowprocs.name, "tty", 3);
+# endif
 
-#ifdef WIN32
-	dircheck();
-#endif
 	Strcpy(cfn, filename);
 # ifdef COMPRESS_EXTENSION
 	Strcat(cfn, COMPRESS_EXTENSION);
@@ -696,7 +723,24 @@ boolean uncomp;
 	args[++i] = (char *)0;
 
 	f = fork();
+# ifdef TTY_GRAPHICS
+	/* If we don't do this and we are right after a y/n question *and*
+	 * there is an error message from the compression, the 'y' or 'n' can
+	 * end up being displayed after the error message.
+	 */
+	if (istty)
+	    mark_synch();
+# endif
 	if (f == 0) {	/* child */
+# ifdef TTY_GRAPHICS
+		/* any error messages from the compression must come out after
+		 * the first line, because the more() to let the user read
+		 * them will have to clear the first line.  This should be
+		 * invisible if there are no error messages.
+		 */
+		if (istty)
+		    raw_print("");
+# endif
 		/* run compressor without privileges, in case other programs
 		 * have surprises along the line of gzip once taking filenames
 		 * in GZIP.
@@ -747,6 +791,21 @@ boolean uncomp;
 		/* no message needed for compress case; life will go on */
 		(void) unlink(cfn);
 	    }
+#ifdef TTY_GRAPHICS
+	    /* Give them a chance to read any error messages from the
+	     * compression--these would go to stdout or stderr and would get
+	     * overwritten only in tty mode.  It's still ugly, since the
+	     * messages are being written on top of the screen, but at least
+	     * the user can read them.
+	     */
+	    if (istty)
+	    {
+		clear_nhwindow(WIN_MESSAGE);
+		more();
+		/* No way to know if this is feasible */
+		/* doredraw(); */
+	    }
+#endif
 	}
 }
 #endif	/* COMPRESS */
@@ -755,11 +814,12 @@ boolean uncomp;
 void
 compress(filename)
 const char *filename;
-#ifdef applec
+{
+#ifndef COMPRESS
+#if (defined(macintosh) && (defined(__SC__) || defined(__MRC__))) || defined(__MWERKS__)
 # pragma unused(filename)
 #endif
-{
-#ifdef COMPRESS
+#else
 	docompress_file(filename, FALSE);
 #endif
 }
@@ -769,11 +829,12 @@ const char *filename;
 void
 uncompress(filename)
 const char *filename;
-#ifdef applec
+{
+#ifndef COMPRESS
+#if (defined(macintosh) && (defined(__SC__) || defined(__MRC__))) || defined(__MWERKS__)
 # pragma unused(filename)
 #endif
-{
-#ifdef COMPRESS
+#else
 	docompress_file(filename, TRUE);
 #endif
 }
@@ -796,45 +857,49 @@ make_lockname(filename, lockname)
 const char *filename;
 char *lockname;
 {
-#if defined(UNIX) || defined(VMS) || defined(AMIGA) || defined(WIN32) || defined(MSDOS)
-# ifdef NO_FILE_LINKS
+#if (defined(macintosh) && (defined(__SC__) || defined(__MRC__))) || defined(__MWERKS__)
+# pragma unused(filename,lockname)
+	return (char*)0;
+#else
+# if defined(UNIX) || defined(VMS) || defined(AMIGA) || defined(WIN32) || defined(MSDOS)
+#  ifdef NO_FILE_LINKS
 	Strcpy(lockname, LOCKDIR);
 	Strcat(lockname, "/");
 	Strcat(lockname, filename);
-# else
+#  else
 	Strcpy(lockname, filename);
-# endif
-# ifdef VMS
+#  endif
+#  ifdef VMS
       {
 	char *semi_colon = rindex(lockname, ';');
 	if (semi_colon) *semi_colon = '\0';
       }
 	Strcat(lockname, ".lock;1");
-# else
+#  else
 	Strcat(lockname, "_lock");
-# endif
+#  endif
 	return lockname;
-#else
+# else
 	lockname[0] = '\0';
 	return (char*)0;
-#endif  /* UNIX || VMS || AMIGA || WIN32 || MSDOS */
+# endif  /* UNIX || VMS || AMIGA || WIN32 || MSDOS */
+#endif
 }
 
 
 /* lock a file */
 boolean
-lock_file(filename, retryct)
+lock_file(filename, whichprefix, retryct)
 const char *filename;
+int whichprefix;
 int retryct;
-#ifdef applec
+{
+#if (defined(macintosh) && (defined(__SC__) || defined(__MRC__))) || defined(__MWERKS__)
 # pragma unused(filename, retryct)
 #endif
-{
-	char *lockname, locknambuf[BUFSZ];
+	char locknambuf[BUFSZ];
+	const char *lockname;
 
-#ifdef WIN32
-	dircheck();
-#endif
 	nesting++;
 	if (nesting > 1) {
 	    impossible("TRIED TO NEST LOCKS");
@@ -842,6 +907,10 @@ int retryct;
 	}
 
 	lockname = make_lockname(filename, locknambuf);
+	filename = fqname(filename, whichprefix, 0);
+#ifndef NO_FILE_LINKS	/* LOCKDIR should be subsumed by LOCKPREFIX */
+	lockname = fqname(lockname, LOCKPREFIX, 1);
+#endif
 
 #if defined(UNIX) || defined(VMS)
 # ifdef NO_FILE_LINKS
@@ -934,18 +1003,19 @@ int retryct;
 void
 unlock_file(filename)
 const char *filename;
-#if defined(applec)
+#if defined(macintosh) && (defined(__SC__) || defined(__MRC__))
 # pragma unused(filename)
 #endif
 {
-	char *lockname, locknambuf[BUFSZ];
+	char locknambuf[BUFSZ];
+	const char *lockname;
 
 	if (nesting == 1) {
 		lockname = make_lockname(filename, locknambuf);
-
-#ifdef WIN32
-	dircheck();
+#ifndef NO_FILE_LINKS	/* LOCKDIR should be subsumed by LOCKPREFIX */
+		lockname = fqname(lockname, LOCKPREFIX, 1);
 #endif
+
 #if defined(UNIX) || defined(VMS)
 		if (unlink(lockname) < 0)
 			HUP raw_printf("Can't unlink %s.", lockname);
@@ -1008,12 +1078,11 @@ const char *filename;
 	FILE *fp;
 #if defined(UNIX) || defined(VMS)
 	char	tmp_config[BUFSZ];
+	char *envp;
 #endif
 
-#ifdef WIN32
-	dircheck();
-#endif
 	/* "filename" is an environment variable, so it should hang around */
+	/* if set, it is expected to be a full path name (if relevant) */
 	if (filename) {
 #ifdef UNIX
 		if (access(filename, 4) == -1) {
@@ -1030,21 +1099,33 @@ const char *filename;
 		} else
 #endif
 		if ((fp = fopenp(filename, "r")) != (FILE *)0) {
-			configfile = filename;
-			return(fp);
+		    configfile = filename;
+		    return(fp);
+#if defined(UNIX) || defined(VMS)
+		} else {
+		    /* access() above probably caught most problems for UNIX */
+		    raw_printf("Couldn't open requested config file %s (%d).",
+					filename, errno);
+		    wait_synch();
+		    /* fall through to standard names */
+#endif
 		}
 	}
 
 #if defined(MICRO) || defined(MAC) || defined(__BEOS__)
-	if ((fp = fopenp(configfile, "r")) != (FILE *)0)
+	if ((fp = fopenp(fqname(configfile, CONFIGPREFIX, 0), "r"))
+								!= (FILE *)0)
 		return(fp);
 # ifdef MSDOS
-	else if ((fp = fopenp(backward_compat_configfile, "r")) != (FILE *)0)
+	else if ((fp = fopenp(fqname(backward_compat_configfile,
+					CONFIGPREFIX, 0), "r")) != (FILE *)0)
 		return(fp);
 # endif
 #else
+	/* constructed full path names don't need fqname() */
 # ifdef VMS
-	if ((fp = fopenp("nethackini", "r")) != (FILE *)0) {
+	if ((fp = fopenp(fqname("nethackini", CONFIGPREFIX, 0), "r"))
+								!= (FILE *)0) {
 		configfile = "nethackini";
 		return(fp);
 	}
@@ -1052,13 +1133,29 @@ const char *filename;
 		configfile = "nethack.ini";
 		return(fp);
 	}
-	Sprintf(tmp_config, "%s%s", getenv("HOME"), "NetHack.cnf");
+
+	envp = nh_getenv("HOME");
+	if (!envp)
+		Strcpy(tmp_config, "NetHack.cnf");
+	else
+		Sprintf(tmp_config, "%s%s", envp, "NetHack.cnf");
 	if ((fp = fopenp(tmp_config, "r")) != (FILE *)0)
 		return(fp);
 # else	/* should be only UNIX left */
-	Sprintf(tmp_config, "%s/%s", getenv("HOME"), ".nethackrc");
+	envp = nh_getenv("HOME");
+	if (!envp)
+		Strcpy(tmp_config, ".nethackrc");
+	else
+		Sprintf(tmp_config, "%s/%s", envp, ".nethackrc");
 	if ((fp = fopenp(tmp_config, "r")) != (FILE *)0)
 		return(fp);
+	else if (errno != ENOENT) {
+		/* e.g., problems when setuid NetHack can't search home
+		 * directory restricted to user */
+		raw_printf("Couldn't open default config file %s (%d).",
+					tmp_config, errno);
+		wait_synch();
+	}
 # endif
 #endif
 	return (FILE *)0;
@@ -1121,6 +1218,25 @@ gi_error:
     /*NOTREACHED*/
 }
 
+#ifdef NOCWD_ASSUMPTIONS
+STATIC_OVL void
+adjust_prefix(bufp, prefixid)
+char *bufp;
+int prefixid;
+{
+	char *ptr;
+
+	if (!bufp) return;
+	/* Backward compatibility, ignore trailing ;n */ 
+	if ((ptr = index(bufp, ';')) != 0) *ptr = '\0';
+	if (strlen(bufp) > 0) {
+		fqn_prefix[prefixid] = (char *)alloc(strlen(bufp)+2);
+		Strcpy(fqn_prefix[prefixid], bufp);
+		append_slash(fqn_prefix[prefixid]);
+	}
+}
+#endif
+
 #define match_varname(INP,NAM,LEN) match_optname(INP, NAM, LEN, TRUE)
 
 /*ARGSUSED*/
@@ -1130,10 +1246,10 @@ FILE		*fp;
 char		*buf;
 char		*tmp_ramdisk;
 char		*tmp_levels;
-#if defined(applec)
+{
+#if (defined(macintosh) && (defined(__SC__) || defined(__MRC__))) || defined(__MWERKS__)
 # pragma unused(tmp_ramdisk,tmp_levels)
 #endif
-{
 	char		*bufp, *altp;
 	uchar   translate[MAXPCHARS];
 	int   len;
@@ -1161,46 +1277,69 @@ char		*tmp_levels;
 	do { ++bufp; } while (isspace(*bufp));
 
 	/* Go through possible variables */
+	/* some of these (at least LEVELS and SAVE) should now set the
+	 * appropriate fqn_prefix[] rather than specialized variables
+	 */
 	if (match_varname(buf, "OPTIONS", 4)) {
 		parseoptions(bufp, TRUE, TRUE);
 		if (plname[0])		/* If a name was given */
 			plnamesuffix();	/* set the character class */
-#ifdef MICRO
+#ifdef NOCWD_ASSUMPTIONS
 	} else if (match_varname(buf, "HACKDIR", 4)) {
-		(void) strncpy(hackdir, bufp, PATHLEN);
-# ifdef MFLOPPY
+		adjust_prefix(bufp, HACKPREFIX);
+	} else if (match_varname(buf, "LEVELDIR", 4) ||
+		   match_varname(buf, "LEVELS", 4)) {
+		adjust_prefix(bufp, LEVELPREFIX);
+	} else if (match_varname(buf, "SAVE", 4)) {
+		adjust_prefix(bufp, SAVEPREFIX);
+	} else if (match_varname(buf, "BONESDIR", 5)) {
+		adjust_prefix(bufp, BONESPREFIX);
+	} else if (match_varname(buf, "DATADIR", 4)) {
+		adjust_prefix(bufp, DATAPREFIX);
+	} else if (match_varname(buf, "SCOREDIR", 4)) {
+		adjust_prefix(bufp, SCOREPREFIX);
+	} else if (match_varname(buf, "LOCKDIR", 4)) {
+		adjust_prefix(bufp, LOCKPREFIX);
+	} else if (match_varname(buf, "CONFIGDIR", 4)) {
+		adjust_prefix(bufp, CONFIGPREFIX);
+#else /*NOCWD_ASSUMPTIONS*/
+# ifdef MICRO
+	} else if (match_varname(buf, "HACKDIR", 4)) {
+		(void) strncpy(hackdir, bufp, PATHLEN-1);
+#  ifdef MFLOPPY
 	} else if (match_varname(buf, "RAMDISK", 3)) {
 				/* The following ifdef is NOT in the wrong
 				 * place.  For now, we accept and silently
 				 * ignore RAMDISK */
-#  ifndef AMIGA
-		(void) strncpy(tmp_ramdisk, bufp, PATHLEN);
+#   ifndef AMIGA
+		(void) strncpy(tmp_ramdisk, bufp, PATHLEN-1);
+#   endif
 #  endif
-# endif
 	} else if (match_varname(buf, "LEVELS", 4)) {
-		(void) strncpy(tmp_levels, bufp, PATHLEN);
+		(void) strncpy(tmp_levels, bufp, PATHLEN-1);
 
 	} else if (match_varname(buf, "SAVE", 4)) {
-# ifdef MFLOPPY
+#  ifdef MFLOPPY
 		extern	int saveprompt;
-#endif
+#  endif
 		char *ptr;
 		if ((ptr = index(bufp, ';')) != 0) {
 			*ptr = '\0';
-# ifdef MFLOPPY
+#  ifdef MFLOPPY
 			if (*(ptr+1) == 'n' || *(ptr+1) == 'N') {
 				saveprompt = FALSE;
 			}
-# endif
+#  endif
 		}
-#ifdef	MFLOPPY
+# ifdef	MFLOPPY
 		else
 		    saveprompt = flags.asksavedisk;
-#endif
+# endif
 
-		(void) strncpy(SAVEP, bufp, PATHLEN);
+		(void) strncpy(SAVEP, bufp, SAVESIZE-1);
 		append_slash(SAVEP);
-#endif /* MICRO */
+# endif /* MICRO */
+#endif /*NOCWD_ASSUMPTIONS*/
 
 	} else if (match_varname(buf, "NAME", 4)) {
 	    (void) strncpy(plname, bufp, PL_NSIZ-1);
@@ -1214,6 +1353,8 @@ char		*tmp_levels;
 	} else if (match_varname(buf, "CATNAME", 3)) {
 	    (void) strncpy(catname, bufp, PL_PSIZ-1);
 
+	} else if (match_varname(buf, "BOULDER", 3)) {
+	    (void) get_uchars(fp, buf, bufp, &iflags.bouldersym, 1, "BOULDER");
 	} else if (match_varname(buf, "GRAPHICS", 4)) {
 	    len = get_uchars(fp, buf, bufp, translate, MAXPCHARS, "GRAPHICS");
 	    assign_graphics(translate, len, MAXPCHARS, 0);
@@ -1235,10 +1376,17 @@ char		*tmp_levels;
 	    /* monsyms[0] is unused */
 	    (void) get_uchars(fp, buf, bufp, &(monsyms[1]),
 					MAXMCLASSES-1, "MONSTERS");
+	} else if (match_varname(buf, "WARNINGS", 5)) {
+	    (void) get_uchars(fp, buf, bufp, translate,
+					WARNCOUNT, "WARNINGS");
+	    assign_warnings(translate);
+#ifdef WIZARD
+	} else if (match_varname(buf, "WIZKIT", 6)) {
+	    (void) strncpy(wizkit, bufp, WIZKIT_MAX-1);
+#endif
 #ifdef AMIGA
 	} else if (match_varname(buf, "FONT", 4)) {
 		char *t;
-		extern void amii_set_text_font( char *, int );
 
 		if( t = strchr( buf+5, ':' ) )
 		{
@@ -1247,9 +1395,7 @@ char		*tmp_levels;
 		    *t = ':';
 		}
 	} else if (match_varname(buf, "PATH", 4)) {
-		(void) strncpy(PATH, bufp, PATHLEN);
-#endif
-#ifdef AMIGA
+		(void) strncpy(PATH, bufp, PATHLEN-1);
 	} else if (match_varname(buf, "DEPTH", 5)) {
 		extern int amii_numcolors;
 		int val = atoi( bufp );
@@ -1264,8 +1410,10 @@ char		*tmp_levels;
 		}
 	} else if (match_varname(buf, "SCREENMODE", 10 )) {
 		extern long amii_scrnmode;
-		if( sscanf(bufp, "%x", &amii_scrnmode) != 1 )
-			amii_scrnmode = 0;
+		if (!stricmp(bufp,"req"))
+		    amii_scrnmode = 0xffffffff; /* Requester */
+		else if( sscanf(bufp, "%x", &amii_scrnmode) != 1 )
+		    amii_scrnmode = 0;
 	} else if (match_varname(buf, "MSGPENS", 7)) {
 		extern int amii_msgAPen, amii_msgBPen;
 		char *t = strtok(bufp, ",/");
@@ -1312,15 +1460,45 @@ char		*tmp_levels;
 				sscanf(t, "%d", &amii_otherBPen);
 		}
 	} else if (match_varname(buf, "PENS", 4)) {
+		extern unsigned short amii_init_map[ AMII_MAXCOLORS ];
 		int i;
 		char *t;
 
-		for (i = 0, t = strtok(bufp, ",/"); t != (char *)0;
-				    t = strtok((char *)0, ",/"), ++i)
+		for (i = 0, t = strtok(bufp, ",/");
+			i < AMII_MAXCOLORS && t != (char *)0;
+			t = strtok((char *)0, ",/"), ++i)
 		{
-			sscanf(t, "%hx", &flags.amii_curmap[i]);
+			sscanf(t, "%hx", &amii_init_map[i]);
 		}
 		amii_setpens( amii_numcolors = i );
+	} else if (match_varname(buf, "FGPENS", 6)) {
+		extern int foreg[ AMII_MAXCOLORS ];
+		int i;
+		char *t;
+
+		for (i = 0, t = strtok(bufp, ",/");
+			i < AMII_MAXCOLORS && t != (char *)0;
+			t = strtok((char *)0, ",/"), ++i)
+		{
+			sscanf(t, "%d", &foreg[i]);
+		}
+	} else if (match_varname(buf, "BGPENS", 6)) {
+		extern int backg[ AMII_MAXCOLORS ];
+		int i;
+		char *t;
+
+		for (i = 0, t = strtok(bufp, ",/");
+			i < AMII_MAXCOLORS && t != (char *)0;
+			t = strtok((char *)0, ",/"), ++i)
+		{
+			sscanf(t, "%d", &backg[i]);
+		}
+#endif
+#ifdef USER_SOUNDS
+	} else if (match_varname(buf, "SOUNDDIR", 8)) {
+		sounddir = (char *)strdup(bufp);
+	} else if (match_varname(buf, "SOUND", 5)) {
+		add_sound_mapping(bufp);
 #endif
 #ifdef QT_GRAPHICS
 	} else if (match_varname(buf, "QT_TILEWIDTH", 12)) {
@@ -1335,12 +1513,24 @@ char		*tmp_levels;
 		extern char *qt_fontsize;
 		if (qt_fontsize == NULL)
 			qt_fontsize=(char *)strdup(bufp);
+	} else if (match_varname(buf, "QT_COMPACT", 10)) {
+		extern int qt_compact_mode;
+		qt_compact_mode = atoi(bufp);
 #endif
 	} else
 		return 0;
 	return 1;
 }
 
+#ifdef USER_SOUNDS
+boolean
+can_read_file(filename)
+const char *filename;
+{
+	return (access(filename, 4) == 0);
+}
+#endif /* USER_SOUNDS */
+
 void
 read_config_file(filename)
 const char *filename;
@@ -1360,9 +1550,6 @@ const char *filename;
 #endif
 	char	buf[4*BUFSZ];
 	FILE	*fp;
-#ifdef WIN32
-	dircheck();
-#endif
 
 	if (!(fp = fopen_config_file(filename))) return;
 
@@ -1374,16 +1561,22 @@ const char *filename;
 # endif
 	tmp_levels[0] = 0;
 #endif
+	/* begin detection of duplicate configfile options */
+	set_duplicate_opt_detection(1);
 
 	while (fgets(buf, 4*BUFSZ, fp)) {
 		if (!parse_config_line(fp, buf, tmp_ramdisk, tmp_levels)) {
-			raw_printf("Bad option line:  \"%s\"", buf);
+			raw_printf("Bad option line:  \"%.50s\"", buf);
 			wait_synch();
 		}
 	}
 	(void) fclose(fp);
+	
+	/* turn off detection of duplicate configfile options */
+	set_duplicate_opt_detection(0);
 
-#ifdef MICRO
+#if defined(MICRO) && !defined(NOCWD_ASSUMPTIONS)
+	/* should be superseded by fqn_prefix[] */
 # ifdef MFLOPPY
 	Strcpy(permbones, tmp_levels);
 #  ifndef AMIGA
@@ -1401,6 +1594,104 @@ const char *filename;
 	return;
 }
 
+#ifdef WIZARD
+STATIC_OVL FILE *
+fopen_wizkit_file()
+{
+	FILE *fp;
+#if defined(VMS) || defined(UNIX)
+	char	tmp_wizkit[BUFSZ];
+#endif
+	char *envp;
+
+	envp = nh_getenv("WIZKIT");
+	if (envp && *envp) (void) strncpy(wizkit, envp, WIZKIT_MAX - 1);
+	if (!wizkit[0]) return (FILE *)0;
+
+#ifdef UNIX
+	if (access(wizkit, 4) == -1) {
+		/* 4 is R_OK on newer systems */
+		/* nasty sneaky attempt to read file through
+		 * NetHack's setuid permissions -- this is a
+		 * place a file name may be wholly under the player's
+		 * control
+		 */
+		raw_printf("Access to %s denied (%d).",
+				wizkit, errno);
+		wait_synch();
+		/* fall through to standard names */
+	} else
+#endif
+	if ((fp = fopenp(wizkit, "r")) != (FILE *)0) {
+	    return(fp);
+#if defined(UNIX) || defined(VMS)
+	} else {
+	    /* access() above probably caught most problems for UNIX */
+	    raw_printf("Couldn't open requested config file %s (%d).",
+				wizkit, errno);
+	    wait_synch();
+#endif
+	}
+
+#if defined(MICRO) || defined(MAC) || defined(__BEOS__) || defined(WIN32)
+	if ((fp = fopenp(fqname(wizkit, CONFIGPREFIX, 0), "r"))
+								!= (FILE *)0)
+		return(fp);
+#else
+# ifdef VMS
+	envp = nh_getenv("HOME");
+	if (envp)
+		Sprintf(tmp_wizkit, "%s%s", envp, wizkit);
+	else
+		Sprintf(tmp_wizkit, "%s%s", "sys$login:", wizkit);
+	if ((fp = fopenp(tmp_wizkit, "r")) != (FILE *)0)
+		return(fp);
+# else	/* should be only UNIX left */
+	envp = nh_getenv("HOME");
+	if (envp)
+		Sprintf(tmp_wizkit, "%s/%s", envp, wizkit);
+	else 	Strcpy(tmp_wizkit, wizkit);
+	if ((fp = fopenp(tmp_wizkit, "r")) != (FILE *)0)
+		return(fp);
+	else if (errno != ENOENT) {
+		/* e.g., problems when setuid NetHack can't search home
+		 * directory restricted to user */
+		raw_printf("Couldn't open default wizkit file %s (%d).",
+					tmp_wizkit, errno);
+		wait_synch();
+	}
+# endif
+#endif
+	return (FILE *)0;
+}
+
+void
+read_wizkit()
+{
+	FILE *fp;
+	char *ep, buf[BUFSZ];
+	struct obj *otmp;
+	if (!wizard || !(fp = fopen_wizkit_file())) return;
+
+	while (fgets(buf, 4*BUFSZ, fp)) {
+		if ((ep = index(buf, '\n'))) *ep = '\0';
+		if (buf[0]) {
+			otmp = readobjnam(buf, (struct obj *)0, FALSE);
+			if (otmp) {
+			    if (otmp != &zeroobj)
+				otmp = addinv(otmp);
+			} else {
+			    raw_printf("Bad wizkit item: \"%.50s\"", buf);
+			    wait_synch();
+			}
+		}
+	}
+	(void) fclose(fp);
+	return;
+}
+
+#endif /*WIZARD*/
+
 /* ----------  END CONFIG FILE HANDLING ----------- */
 
 /* ----------  BEGIN SCOREBOARD CREATION ----------- */
@@ -1409,58 +1700,63 @@ const char *filename;
 void
 check_recordfile(dir)
 const char *dir;
-#if defined(applec)
+{
+#if (defined(macintosh) && (defined(__SC__) || defined(__MRC__))) || defined(__MWERKS__)
 # pragma unused(dir)
 #endif
-{
-#if defined(UNIX) || defined(VMS)
-	int fd = open(RECORD, O_RDWR, 0);
+	const char *fq_record;
+	int fd;
 
+#if defined(UNIX) || defined(VMS)
+	fq_record = fqname(RECORD, SCOREPREFIX, 0);
+	fd = open(fq_record, O_RDWR, 0);
 	if (fd >= 0) {
 # ifdef VMS	/* must be stream-lf to use UPDATE_RECORD_IN_PLACE */
 		if (!file_is_stmlf(fd)) {
-		    raw_printf(	/* note: assume VMS dir has trailing punct */
-		  "Warning: scoreboard file %s%s is not in stream_lf format",
-				(dir ? dir : "[]"), RECORD);
+		    raw_printf(
+		  "Warning: scoreboard file %s is not in stream_lf format",
+				fq_record);
 		    wait_synch();
 		}
 # endif
 	    (void) close(fd);	/* RECORD is accessible */
-	} else if ((fd = open(RECORD, O_CREAT|O_RDWR, FCMASK)) >= 0) {
+	} else if ((fd = open(fq_record, O_CREAT|O_RDWR, FCMASK)) >= 0) {
 	    (void) close(fd);	/* RECORD newly created */
 # if defined(VMS) && !defined(SECURE)
 	    /* Re-protect RECORD with world:read+write+execute+delete access. */
-	    (void) chmod(RECORD, FCMASK | 007); /* allow everyone full access */
+	    (void) chmod(fq_record, FCMASK | 007);
 # endif /* VMS && !SECURE */
 	} else {
-	    raw_printf("Warning: cannot write scoreboard file %s/%s",
-			(dir ? dir : "."), RECORD);
+	    raw_printf("Warning: cannot write scoreboard file %s", fq_record);
 	    wait_synch();
 	}
 #endif  /* !UNIX && !VMS */
 #ifdef MICRO
-	int fd;
 	char tmp[PATHLEN];
-# ifdef WIN32
-	dircheck();
-# endif
 
 # ifdef OS2_CODEVIEW   /* explicit path on opening for OS/2 */
-	Strcpy(tmp, dir);
-	append_slash(tmp);
-	Strcat(tmp, RECORD);
+	/* how does this work when there isn't an explicit path or fopenp
+	 * for later access to the file via fopen_datafile? ? */
+	(void) strncpy(tmp, dir, PATHLEN - 1);
+	tmp[PATHLEN-1] = '\0';
+	if ((strlen(tmp) + 1 + strlen(RECORD)) < (PATHLEN - 1)) {
+		append_slash(tmp);
+		Strcat(tmp, RECORD);
+	}
+	fq_record = tmp;
 # else
 	Strcpy(tmp, RECORD);
+	fq_record = fqname(RECORD, SCOREPREFIX, 0);
 # endif
 
-	if ((fd = open(tmp, O_RDWR)) < 0) {
+	if ((fd = open(fq_record, O_RDWR)) < 0) {
 	    /* try to create empty record */
-# if defined(AZTEC_C) || defined(_DCC)
+# if defined(AZTEC_C) || defined(_DCC) || (defined(__GNUC__) && defined(__AMIGA__))
 	    /* Aztec doesn't use the third argument */
 	    /* DICE doesn't like it */
-	    if ((fd = open(tmp, O_CREAT|O_RDWR)) < 0) {
+	    if ((fd = open(fq_record, O_CREAT|O_RDWR)) < 0) {
 # else
-	    if ((fd = open(tmp, O_CREAT|O_RDWR, S_IREAD|S_IWRITE)) < 0) {
+	    if ((fd = open(fq_record, O_CREAT|O_RDWR, S_IREAD|S_IWRITE)) < 0) {
 # endif
 	raw_printf("Warning: cannot write record %s", tmp);
 		wait_synch();
@@ -1471,13 +1767,10 @@ const char *dir;
 #else /* MICRO */
 
 # ifdef MAC
-	int fd = macopen ( RECORD , O_RDWR | O_CREAT , TEXT_TYPE ) ;
-
-	if ( fd < 0 ) {
-		raw_printf ( "Warning: cannot write %s" , RECORD ) ;
-	} else {
-		close ( fd ) ;
-	}
+	/* Create the "record" file, if necessary */
+	fq_record = fqname(RECORD, SCOREPREFIX, 0);
+	fd = macopen (fq_record, O_RDWR | O_CREAT, TEXT_TYPE);
+	if (fd != -1) macclose (fd);
 # endif /* MAC */
 
 #endif /* MICRO */
