@@ -1,14 +1,18 @@
-/*	SCCS Id: @(#)hack.c	3.3	1999/08/16	*/
+/*	SCCS Id: @(#)hack.c	3.4	2002/03/09	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
 
+#ifdef OVL1
+STATIC_DCL void NDECL(maybe_wail);
+#endif /*OVL1*/
 STATIC_DCL int NDECL(moverock);
 STATIC_DCL int FDECL(still_chewing,(XCHAR_P,XCHAR_P));
 #ifdef SINKS
 STATIC_DCL void NDECL(dosinkfall);
 #endif
+STATIC_DCL void NDECL(findtravelpath);
 STATIC_DCL boolean FDECL(monstinroom, (struct permonst *,int));
 
 STATIC_DCL void FDECL(move_update, (BOOLEAN_P));
@@ -129,16 +133,14 @@ moverock()
 		switch(ttmp->ttyp) {
 		case LANDMINE:
 		    if (rn2(10)) {
-			pline("KAABLAMM!!!  %s triggers %s land mine.",
-				The(xname(otmp)),
+			pline("KAABLAMM!!!  %s %s land mine.",
+				Tobjnam(otmp, "trigger"),
 				ttmp->madeby_u ? "your" : "a");
 			obj_extract_self(otmp);
 			place_object(otmp, rx, ry);
-			deltrap(ttmp);
-			del_engr_at(rx,ry);
-			scatter(rx,ry, 4,
-				MAY_DESTROY|MAY_HIT|MAY_FRACTURE|VIS_EFFECTS,
-				(struct obj *)0);
+			blow_up_landmine(ttmp);
+			/* if the boulder remains, it should fill the pit */
+			fill_pit(u.ux, u.uy);
 			if (cansee(rx,ry)) newsym(rx,ry);
 			continue;
 		    }
@@ -157,9 +159,15 @@ moverock()
 		    continue;
 		case HOLE:
 		case TRAPDOOR:
-		    pline("%s %s and plugs a %s in the %s!",
-			  The(xname(otmp)),
-			  (ttmp->ttyp == TRAPDOOR) ? "triggers" : "falls into",
+		    if (Blind)
+			pline("Kerplunk!  You no longer feel %s.",
+				the(xname(otmp)));
+		    else
+			pline("%s%s and %s a %s in the %s!",
+			  Tobjnam(otmp,
+			   (ttmp->ttyp == TRAPDOOR) ? "trigger" : "fall"),
+			  (ttmp->ttyp == TRAPDOOR) ? nul : " into",
+			  otense(otmp, "plug"),
 			  (ttmp->ttyp == TRAPDOOR) ? "trap door" : "hole",
 			  surface(rx, ry));
 		    deltrap(ttmp);
@@ -255,14 +263,21 @@ moverock()
 	cannot_push:
 	    if (throws_rocks(youmonst.data)) {
 #ifdef STEED
-		if (u.usteed && P_SKILL(P_RIDING) < P_BASIC)
-			You("aren't skilled enough to %s %s from %s.",
-				flags.pickup ? "pick up" : "push aside",
-				the(xname(otmp)), mon_nam(u.usteed));
-		else
+		if (u.usteed && P_SKILL(P_RIDING) < P_BASIC) {
+		    You("aren't skilled enough to %s %s from %s.",
+			(flags.pickup && !In_sokoban(&u.uz))
+			    ? "pick up" : "push aside",
+			the(xname(otmp)), mon_nam(u.usteed));
+		} else
 #endif
-		  pline("However, you can easily %s.",
-			flags.pickup ? "pick it up" : "push it aside");
+		{
+		    pline("However, you can easily %s.",
+			(flags.pickup && !In_sokoban(&u.uz))
+			    ? "pick it up" : "push it aside");
+		    if (In_sokoban(&u.uz))
+			change_luck(-1);	/* Sokoban guilt */
+		    break;
+		}
 		break;
 	    }
 
@@ -275,6 +290,8 @@ moverock()
 				     && IS_ROCK(levl[sx][u.uy].typ))))
 		|| verysmall(youmonst.data))) {
 		pline("However, you can squeeze yourself into a small opening.");
+		if (In_sokoban(&u.uz))
+		    change_luck(-1);	/* Sokoban guilt */
 		break;
 	    } else
 		return (-1);
@@ -301,31 +318,43 @@ still_chewing(x,y)
 	(void) memset((genericptr_t)&digging, 0, sizeof digging);
 
     if (!boulder && IS_ROCK(lev->typ) && !may_dig(x,y)) {
-	You("hurt your teeth on the hard stone.");
+	You("hurt your teeth on the %s.",
+	    IS_TREE(lev->typ) ? "tree" : "hard stone");
 	nomul(0);
 	return 1;
     } else if (digging.pos.x != x || digging.pos.y != y ||
 		!on_level(&digging.level, &u.uz)) {
 	digging.down = FALSE;
 	digging.chew = TRUE;
+	digging.warned = FALSE;
 	digging.pos.x = x;
 	digging.pos.y = y;
 	assign_level(&digging.level, &u.uz);
 	/* solid rock takes more work & time to dig through */
-	digging.effort = (IS_ROCK(lev->typ) ? 30 : 60) + u.udaminc;
+	digging.effort =
+	    (IS_ROCK(lev->typ) && !IS_TREE(lev->typ) ? 30 : 60) + u.udaminc;
 	You("start chewing %s %s.",
-	    boulder ? "on a" : "a hole in the",
-	    boulder ? "boulder" : IS_ROCK(lev->typ) ? "rock" : "door");
+	    (boulder || IS_TREE(lev->typ)) ? "on a" : "a hole in the",
+	    boulder ? "boulder" :
+	    IS_TREE(lev->typ) ? "tree" : IS_ROCK(lev->typ) ? "rock" : "door");
+	watch_dig((struct monst *)0, x, y, FALSE);
 	return 1;
     } else if ((digging.effort += (30 + u.udaminc)) <= 100)  {
 	if (flags.verbose)
 	    You("%s chewing on the %s.",
 		digging.chew ? "continue" : "begin",
-		boulder ? "boulder" : IS_ROCK(lev->typ) ? "rock" : "door");
+		boulder ? "boulder" :
+		IS_TREE(lev->typ) ? "tree" :
+		IS_ROCK(lev->typ) ? "rock" : "door");
 	digging.chew = TRUE;
+	watch_dig((struct monst *)0, x, y, FALSE);
 	return 1;
     }
 
+    /* Okay, you've chewed through something */
+    u.uconduct.food++;
+    u.uhunger += rnd(20);
+
     if (boulder) {
 	delobj(boulder);		/* boulder goes bye-bye */
 	You("eat the boulder.");	/* yum */
@@ -418,18 +447,19 @@ dosinkfall()
 	register struct obj *obj;
 
 	if (is_floater(youmonst.data) || (HLevitation & FROMOUTSIDE)) {
-		You("wobble unsteadily for a moment.");
+	    You("wobble unsteadily for a moment.");
 	} else {
-		You("crash to the floor!");
-		losehp((rn1(10, 20 - (int)ACURR(A_CON))),
-			fell_on_sink, NO_KILLER_PREFIX);
-		exercise(A_DEX, FALSE);
-		for(obj = level.objects[u.ux][u.uy]; obj; obj = obj->nexthere)
-		    if(obj->oclass == WEAPON_CLASS) {
-			You("fell on %s.",doname(obj));
-			losehp(rnd(3), fell_on_sink, NO_KILLER_PREFIX);
-			exercise(A_CON, FALSE);
-		    }
+	    You("crash to the floor!");
+	    losehp(rn1(8, 25 - (int)ACURR(A_CON)),
+		   fell_on_sink, NO_KILLER_PREFIX);
+	    exercise(A_DEX, FALSE);
+	    selftouch("Falling, you");
+	    for (obj = level.objects[u.ux][u.uy]; obj; obj = obj->nexthere)
+		if (obj->oclass == WEAPON_CLASS || is_weptool(obj)) {
+		    You("fell on %s.", doname(obj));
+		    losehp(rnd(3), fell_on_sink, NO_KILLER_PREFIX);
+		    exercise(A_CON, FALSE);
+		}
 	}
 
 	ELevitation &= ~W_ARTI;
@@ -495,11 +525,205 @@ xchar x, y;
 #endif /* OVL1 */
 #ifdef OVL3
 
+/* return TRUE if (dx,dy) is an OK place to move */
+boolean 
+test_move(ux, uy, dx, dy, test_only)
+int ux, uy, dx, dy;
+boolean test_only;
+{
+    int x = ux+dx;
+    int y = uy+dy;
+    register struct rm *tmpr = &levl[x][y];
+    register struct rm *ust;
+
+    /*
+     *  Check for physical obstacles.  First, the place we are going.
+     */
+    if (IS_ROCK(tmpr->typ) || tmpr->typ == IRONBARS) {
+	if (Blind && !test_only) feel_location(x,y);
+	if (Passes_walls && may_passwall(x,y)) {
+	    ;	/* do nothing */
+	} else if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
+	    /* Eat the rock. */
+	    if (!test_only && still_chewing(x,y)) return FALSE;
+	} else if (flags.autodig && !flags.run && !flags.nopick &&
+		   uwep && is_pick(uwep)) {
+	/* MRKR: Automatic digging when wielding the appropriate tool */
+	    if (!test_only)
+		(void) use_pick_axe2(uwep);
+	    return FALSE;
+	} else {
+	    if ( !test_only ) {
+		if (Is_stronghold(&u.uz) && is_db_wall(x,y))
+		    pline_The("drawbridge is up!");
+		if (Passes_walls && !may_passwall(x,y) && In_sokoban(&u.uz))
+	    		pline_The("Sokoban walls resist your ability.");
+	    }
+	    return FALSE;
+	}
+    } else if (IS_DOOR(tmpr->typ)) {
+	if (closed_door(x,y)) {
+	    if (Blind && !test_only) feel_location(x,y);
+	    if (Passes_walls)
+		;	/* do nothing */
+	    else if (can_ooze(&youmonst)) {
+		if ( !test_only ) You("ooze under the door.");
+	    } else if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
+		/* Eat the door. */
+		if (!test_only && still_chewing(x,y)) return FALSE;
+	    } else {
+		if ( !test_only ) {
+		    if (amorphous(youmonst.data))
+			You("try to ooze under the door, but can't squeeze your possessions through.");
+		    else if (x == ux || y == uy) {
+			if (Blind || Stunned || ACURR(A_DEX) < 10 || Fumbling) {
+#ifdef STEED
+			    if (u.usteed)
+				You_cant("lead %s through that closed door.",
+				      x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE));
+		 	    else {
+#else
+			        pline("Ouch!  You bump into a door.");
+			        exercise(A_DEX, FALSE);
+#endif
+#ifdef STEED
+			    }
+#endif
+			} else pline("That door is closed.");
+		    }
+		}
+		return FALSE;
+	    }
+	} else if (dx && dy && !Passes_walls
+		    && ((tmpr->doormask & ~D_BROKEN)
+#ifdef REINCARNATION
+				    || Is_rogue_level(&u.uz)
+#endif
+				    || block_door(x,y))) {
+	    /* Diagonal moves into a door are not allowed. */
+	    if ( Blind && !test_only )
+		feel_location(x,y);
+	    return FALSE;
+	}
+    }
+    if (dx && dy
+	    && bad_rock(youmonst.data,ux,y) && bad_rock(youmonst.data,x,uy)) {
+	/* Move at a diagonal. */
+	if (In_sokoban(&u.uz)) {
+	    if ( !test_only )
+		You("cannot pass that way.");
+	    return FALSE;
+	}
+	if (bigmonst(youmonst.data)) {
+	    if ( !test_only )
+		Your("body is too large to fit through.");
+	    return FALSE;
+	}
+	if (invent && (inv_weight() + weight_cap() > 600)) {
+	    if ( !test_only )
+		You("are carrying too much to get through.");
+	    return FALSE;
+	}
+    }
+
+    ust = &levl[ux][uy];
+
+    /* Now see if other things block our way . . */
+    if (dx && dy && !Passes_walls
+		     && (IS_DOOR(ust->typ) && ((ust->doormask & ~D_BROKEN)
+#ifdef REINCARNATION
+			     || Is_rogue_level(&u.uz)
+#endif
+			     || block_entry(x, y))
+			 )) {
+	/* Can't move at a diagonal out of a doorway with door. */
+	return FALSE;
+    }
+
+    if (sobj_at(BOULDER,x,y) && (In_sokoban(&u.uz) || !Passes_walls)) {
+	if (!(Blind || Hallucination) && (flags.run >= 2))
+	    return FALSE;
+	if (!test_only) {
+	    /* tunneling monsters will chew before pushing */
+	    if (tunnels(youmonst.data) && !needspick(youmonst.data) &&
+		!In_sokoban(&u.uz)) {
+		if (still_chewing(x,y)) return FALSE;
+	    } else
+		if (moverock() < 0) return FALSE;
+	}
+	/* test_only will assume you'll be able to push it when you get there... */
+    }
+
+    /* OK, it is a legal place to move. */
+    return TRUE;
+}
+
+static void findtravelpath()
+{
+    if ( u.tx != u.ux || u.ty != u.uy ) {
+	xchar travel[COLNO][ROWNO];
+	xchar travelstepx[2][COLNO*ROWNO];
+	xchar travelstepy[2][COLNO*ROWNO];
+	int n=1;
+	int set=0;
+	int dia=1;
+
+	(void) memset((genericptr_t)travel,0,sizeof(travel));
+
+	travelstepx[0][0] = u.tx;
+	travelstepy[0][0] = u.ty;
+	while ( n ) {
+	    int i;
+	    int nn=0;
+	    for (i=0; i<n; i++) {
+		int dir;
+		int x = travelstepx[set][i];
+		int y = travelstepy[set][i];
+		static int ordered[] = { 0, 2, 4, 6, 1, 3, 5, 7 };
+		for (dir=0; dir<8; dir++) {
+		    int nx = x+xdir[ordered[dir]];
+		    int ny = y+ydir[ordered[dir]];
+    /*printf("try %d,%d\n",nx,ny);*/
+		    if ( isok(nx,ny) && test_move( x, y, nx-x, ny-y, 1 ) ) {
+			if ( nx == u.ux && ny == u.uy ) {
+			    u.dx = x-u.ux;
+			    u.dy = y-u.uy;
+	/*printf("found\n");*/
+	/*printf("findtravelpath %d,%d -> %d,%d by %d,%d\n",u.ux,u.uy,u.tx,u.ty,u.dx,u.dy);
+	*/
+			    return;
+			} else {
+	/*printf("%d %d %d",isok(nx,ny), !travel[nx][ny], ACCESSIBLE(levl[nx][ny].typ));*/
+			    if ( !travel[nx][ny] ) {
+				travelstepx[1-set][nn]=nx;
+				travelstepy[1-set][nn]=ny;
+				travel[nx][ny]=dia;
+				nn++;
+			    }
+			}
+		    }
+		}
+	    }
+	    n = nn;
+	    set = 1-set;
+	    dia++;
+	}
+
+	/* give up */
+    }
+
+    u.dx = 0;
+    u.dy = 0;
+    nomul(0);
+}
+
 void
 domove()
 {
 	register struct monst *mtmp;
-	register struct rm *tmpr,*ust;
+	register struct rm *tmpr;
 	register xchar x,y;
 	struct trap *trap;
 	int wtcap;
@@ -510,6 +734,9 @@ domove()
 
 	u_wipe_engr(rnd(5));
 
+	if ( flags.travel )
+	    findtravelpath();
+
 	if(((wtcap = near_capacity()) >= OVERLOADED
 	    || (wtcap > SLT_ENCUMBER &&
 		(Upolyd ? (u.mh < 5 && u.mh != u.mhmax)
@@ -620,9 +847,12 @@ domove()
 			 * otherwise:
 			 *	 7.5% chance of getting away.
 			 * [strength ought to be a factor]
+			 * If holder is tame and there is no conflict,
+			 * guaranteed escape.
 			 */
 			switch (rn2(!u.ustuck->mcanmove ? 8 : 40)) {
 			case 0: case 1: case 2:
+			pull_free:
 			    You("pull free from %s.", mon_nam(u.ustuck));
 			    u.ustuck = 0;
 			    break;
@@ -634,6 +864,9 @@ domove()
 			    }
 			    /*FALLTHRU*/
 			default:
+			    if (u.ustuck->mtame &&
+				!Conflict && !u.ustuck->mconf)
+				goto pull_free;
 			    You("cannot escape from %s!", mon_nam(u.ustuck));
 			    nomul(0);
 			    return;
@@ -686,6 +919,8 @@ domove()
 		if(mtmp->m_ap_type && !Protection_from_shape_changers
 						    && !sensemon(mtmp))
 		    stumble_onto_mimic(mtmp);
+		else if (mtmp->mpeaceful && !Hallucination)
+		    pline("Pardon me, %s.", m_monnam(mtmp));
 		else
 		    You("move right into %s.", mon_nam(mtmp));
 		return;
@@ -717,7 +952,10 @@ domove()
 	if (flags.forcefight ||
 	    /* remembered an 'I' && didn't use a move command */
 	    (glyph_is_invisible(levl[x][y].glyph) && !flags.nopick)) {
-		You("attack %s.", Underwater ? "empty water" : "thin air");
+	    	char buf[BUFSZ];
+		Sprintf(buf,"a vacant spot on the %s", surface(x,y));
+		You("attack %s.",
+		    !Underwater ? "thin air" : is_pool(x,y) ? "empty water" : buf);
 		unmap_object(x, y); /* known empty -- remove 'I' if present */
 		newsym(x, y);
 		nomul(0);
@@ -727,11 +965,13 @@ domove()
 	    unmap_object(x, y);
 	    newsym(x, y);
 	}
-
 	/* not attacking an animal, so we try to move */
 #ifdef STEED
 	if (u.usteed && !u.usteed->mcanmove && (u.dx || u.dy)) {
-		pline("%s won't move!", Monnam(u.usteed));
+		pline("%s won't move!",
+			upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
 		nomul(0);
 		return;
 	} else
@@ -751,20 +991,54 @@ domove()
 			clear_nhwindow(WIN_MESSAGE);
 			You("free your %s.", body_part(LEG));
 		    } else if (!(--u.utrap)) {
-			You("crawl to the edge of the pit.");
+			You("%s to the edge of the pit.",
+				(In_sokoban(&u.uz) && Levitation) ?
+				"struggle against the air currents and float" :
+#ifdef STEED
+				u.usteed ? "ride" :
+#endif
+				"crawl");
 			fill_pit(u.ux, u.uy);
 			vision_full_recalc = 1;	/* vision limits change */
-		    } else if (flags.verbose)
+		    } else if (flags.verbose) {
+#ifdef STEED
+			if (u.usteed)
+			    	Norep("%s is still in a pit.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+			else
+#endif
 			Norep( (Hallucination && !rn2(5)) ?
 				"You've fallen, and you can't get up." :
 				"You are still in a pit." );
+		    }
 		} else if (u.utraptype == TT_LAVA) {
-		    if(flags.verbose)
-			Norep("You are stuck in the lava.");
+		    if(flags.verbose) {
+			char *predicament = "stuck in the lava";
+#ifdef STEED
+			if (u.usteed)
+			    	Norep("%s is %s.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
+					 predicament);
+			else
+#endif
+			Norep("You are %s.", predicament);
+		    }
 		    if(!is_lava(x,y)) {
 			u.utrap--;
 			if((u.utrap & 0xff) == 0) {
-			    You("pull yourself to the edge of the lava.");
+#ifdef STEED
+			    if (u.usteed)
+				You("lead %s to the edge of the lava.",
+				      x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE));
+			    else
+#endif
+			     You("pull yourself to the edge of the lava.");
 			    u.utrap = 0;
 			}
 		    }
@@ -776,127 +1050,81 @@ domove()
 			return;
 		    }
 		    if(--u.utrap) {
-			if(flags.verbose)
-			    Norep("You are stuck to the web.");
-		    } else You("disentangle yourself.");
+			if(flags.verbose) {
+			    char *predicament = "stuck to the web";
+#ifdef STEED
+			    if (u.usteed)
+			    	Norep("%s is %s.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
+					 predicament);
+			    else
+#endif
+			    Norep("You are %s.", predicament);
+			}
+		    } else {
+#ifdef STEED
+			if (u.usteed)
+				pline("%s breaks out of the web.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+			else
+#endif
+			You("disentangle yourself.");
+		    }
 		} else if (u.utraptype == TT_INFLOOR) {
 		    if(--u.utrap) {
-			if(flags.verbose)
-			    Norep("You are stuck in the floor.");
-		    } else You("finally wiggle free.");
+			if(flags.verbose) {
+			    char *predicament = "stuck in the";
+#ifdef STEED
+			    if (u.usteed)
+			    	Norep("%s is %s %s.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
+					 predicament, surface(u.ux, u.uy));
+			    else
+#endif
+			    Norep("You are %s %s.", predicament, surface(u.ux, u.uy));
+			}
+		    } else {
+#ifdef STEED
+			if (u.usteed)
+				pline("%s finally wiggles free.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)));
+			else
+#endif
+			You("finally wiggle free.");
+		    }
 		} else {
-		    if(flags.verbose)
-			Norep("You are caught in a bear trap.");
+		    if(flags.verbose) {
+			char *predicament = "caught in a bear trap";
+#ifdef STEED
+			if (u.usteed)
+			    	Norep("%s is %s.",
+				      upstart(x_monnam(u.usteed,
+					 u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				 	 (char *)0, SUPPRESS_SADDLE, FALSE)),
+					 predicament);
+			else
+#endif
+			Norep("You are %s.", predicament);
+		    }
 		    if((u.dx && u.dy) || !rn2(5)) u.utrap--;
 		}
 		return;
 	}
 
-
-	/*
-	 *  Check for physical obstacles.  First, the place we are going.
-	 */
-	if (IS_ROCK(tmpr->typ) || tmpr->typ == IRONBARS) {
-	    if (Blind) feel_location(x,y);
-	    if (Passes_walls && may_passwall(x,y)) {
-		;	/* do nothing */
-	    } else if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
-		/* Eat the rock. */
-		if (still_chewing(x,y)) return;
-	    } else {
-		if (Is_stronghold(&u.uz) && is_db_wall(x,y))
-		    pline_The("drawbridge is up!");
-		flags.move = 0;
-		nomul(0);
-		return;
-	    }
-	} else if (IS_DOOR(tmpr->typ)) {
-	    if (closed_door(x,y)) {
-		if (Blind) feel_location(x,y);
-		if (Passes_walls)
-		    ;	/* do nothing */
-		else if (can_ooze(&youmonst))
-		    You("ooze under the door.");
-		else if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
-		    /* Eat the door. */
-		    if (still_chewing(x,y)) return;
-		} else {
-		    flags.move = 0;
-		    if (amorphous(youmonst.data))
-			You("try to ooze under the door, but can't squeeze your possessions through.");
-		    else if (x == u.ux || y == u.uy) {
-			if (Blind || Stunned || ACURR(A_DEX) < 10 || Fumbling) {
-			    pline("Ouch!  You bump into a door.");
-			    exercise(A_DEX, FALSE);
-			} else pline("That door is closed.");
-		    }
-		    nomul(0);
-		    return;
-		}
-	    } else if (u.dx && u.dy && !Passes_walls
-			&& ((tmpr->doormask & ~D_BROKEN)
-#ifdef REINCARNATION
-					|| Is_rogue_level(&u.uz)
-#endif
-					|| block_door(x,y))) {
-		/* Diagonal moves into a door are not allowed. */
-		if (Blind) feel_location(x,y);	/* ?? */
-		flags.move = 0;
-		nomul(0);
-		return;
-	    }
-	}
-	if (u.dx && u.dy
-		&& bad_rock(youmonst.data,u.ux,y) && bad_rock(youmonst.data,x,u.uy)) {
-	    /* Move at a diagonal. */
-	    if (In_sokoban(&u.uz)) {
-	    	You("cannot pass that way.");
-	    	nomul(0);
-	    	return;
-	    }
-	    if (bigmonst(youmonst.data)) {
-		Your("body is too large to fit through.");
-		nomul(0);
-		return;
-	    }
-	    if (invent && (inv_weight() + weight_cap() > 600)) {
-		You("are carrying too much to get through.");
-		nomul(0);
-		return;
-	    }
-	}
-
-	ust = &levl[u.ux][u.uy];
-
-	/* Now see if other things block our way . . */
-	if (u.dx && u.dy && !Passes_walls
-			 && (IS_DOOR(ust->typ) && ((ust->doormask & ~D_BROKEN)
-#ifdef REINCARNATION
-				 || Is_rogue_level(&u.uz)
-#endif
-				 || block_entry(x, y))
-			     )) {
-	    /* Can't move at a diagonal out of a doorway with door. */
+	if ( !test_move( u.ux, u.uy, x-u.ux, y-u.uy, 0 ) ) {
 	    flags.move = 0;
 	    nomul(0);
 	    return;
 	}
 
-	if (sobj_at(BOULDER,x,y) && (In_sokoban(&u.uz) || !Passes_walls)) {
-	    if (!(Blind || Hallucination) && (flags.run >= 2)) {
-		nomul(0);
-		flags.move = 0;
-		return;
-	    }
-	    /* tunneling monsters will chew before pushing */
-	    if (tunnels(youmonst.data) && !needspick(youmonst.data)) {
-		if (still_chewing(x,y)) return;
-	    } else
-		if (moverock() < 0) return;
-	}
-
-	/* OK, it is a legal place to move. */
-
 	/* Move ball and chain.  */
 	if (Punished)
 	    if (!drag_ball(x,y, &bc_control, &ballx, &bally, &chainx, &chainy,
@@ -938,6 +1166,7 @@ domove()
 	    if (mtmp->mtrapped) {
 		if (!rn2(mtmp->mtame)) {
 		    mtmp->mtame = mtmp->mpeaceful = mtmp->msleeping = 0;
+		    if (mtmp->mleashed) m_unleash(mtmp, TRUE);
 		    growl(mtmp);
 		} else {
 		    yelp(mtmp);
@@ -954,15 +1183,19 @@ domove()
 		/* can't swap places with pet pinned in a pit by a boulder */
 		u.ux = u.ux0,  u.uy = u.uy0;	/* didn't move after all */
 	    } else {
+		char pnambuf[BUFSZ];
+
+		/* save its current description in case of polymorph */
+		Strcpy(pnambuf, y_monnam(mtmp));
 		mtmp->mtrapped = 0;
 		remove_monster(x, y);
 		place_monster(mtmp, u.ux0, u.uy0);
 
 		/* check for displacing it into pools and traps */
-		switch (minwater(mtmp) ? 2 : mintrap(mtmp)) {
+		switch (minliquid(mtmp) ? 2 : mintrap(mtmp)) {
 		case 0:
 		    You("%s %s.", mtmp->mtame ? "displaced" : "frightened",
-			y_monnam(mtmp));
+			pnambuf);
 		    break;
 		case 1:		/* trapped */
 		case 3:		/* changed levels */
@@ -979,6 +1212,11 @@ domove()
 			u.ugangr++;
 			adjalign(-15);
 		    }
+
+		    /* you killed your pet by direct action.
+		     * minliquid and mintrap don't know to do this
+		     */
+		    u.uconduct.killer++;
 		    break;
 		default:
 		    pline("that's strange, unknown mintrap result!");
@@ -989,6 +1227,7 @@ domove()
 
 	reset_occupations();
 	if (flags.run) {
+	    if ( flags.run < 8 )
 		if (IS_DOOR(tmpr->typ) || IS_ROCK(tmpr->typ) ||
 			IS_FURNITURE(tmpr->typ))
 		    nomul(0);
@@ -1024,7 +1263,7 @@ domove()
 	if (Punished)				/* put back ball and chain */
 	    move_bc(0,bc_control,ballx,bally,chainx,chainy);
 
-	spoteffects();
+	spoteffects(TRUE);
 
 	/* delay next move because of ball dragging */
 	/* must come after we finished picking up, in spoteffects() */
@@ -1053,7 +1292,8 @@ invocation_message()
 #ifdef OVL2
 
 void
-spoteffects()
+spoteffects(pick)
+boolean pick;
 {
 	register struct trap *trap;
 	register struct monst *mtmp;
@@ -1090,20 +1330,34 @@ spoteffects()
 stillinwater:;
 	if (!Levitation && !u.ustuck && !Flying) {
 	    /* limit recursive calls through teleds() */
-	    if(is_lava(u.ux,u.uy) && lava_effects())
-		    return;
-	    if(is_pool(u.ux,u.uy) && !Wwalking && drown())
+	    if (is_pool(u.ux, u.uy) || is_lava(u.ux, u.uy)) {
+#ifdef STEED
+		if (u.usteed && !is_flyer(u.usteed->data) &&
+			!is_floater(u.usteed->data) &&
+			!is_clinger(u.usteed->data)) {
+		    dismount_steed(Underwater ?
+			    DISMOUNT_FELL : DISMOUNT_GENERIC);
+		    /* dismount_steed() -> float_down() -> pickup() */
+		    if (!Is_airlevel(&u.uz) && !Is_waterlevel(&u.uz))
+			pick = FALSE;
+		} else
+#endif
+		if (is_lava(u.ux, u.uy)) {
+		    if (lava_effects()) return;
+		} else if (!Wwalking && drown())
 		    return;
+	    }
 	}
 	check_special_room(FALSE);
 #ifdef SINKS
 	if(IS_SINK(levl[u.ux][u.uy].typ) && Levitation)
 		dosinkfall();
 #endif
-	if (!in_steed_dismounting)
-		pickup(1);
-	if ((trap = t_at(u.ux,u.uy)) != 0)
-		dotrap(trap);	/* fall into pit, arrow trap, etc. */
+	if (pick && !in_steed_dismounting)
+		(void) pickup(1);
+	/* if dismounting, we'll check again later */
+	if ((trap = t_at(u.ux,u.uy)) != 0 && !in_steed_dismounting)
+		dotrap(trap, 0);	/* fall into pit, arrow trap, etc. */
 	if((mtmp = m_at(u.ux, u.uy)) && !u.uswallow) {
 		mtmp->mundetected = mtmp->msleeping = 0;
 		switch(mtmp->data->mlet) {
@@ -1116,11 +1370,11 @@ stillinwater:;
 			    pline("Its blow glances off your helmet.");
 			else if (u.uac + 3 <= rnd(20))
 			    You("are almost hit by %s!",
-				x_monnam(mtmp, 2, "falling", 1));
+				x_monnam(mtmp, ARTICLE_A, "falling", 0, TRUE));
 			else {
 			    int dmg;
 			    You("are hit by %s!",
-				x_monnam(mtmp, 2, "falling", 1));
+				x_monnam(mtmp, ARTICLE_A, "falling", 0, TRUE));
 			    dmg = d(4,6);
 			    if(Half_physical_damage) dmg = (dmg+1) / 2;
 			    mdamageu(mtmp, dmg);
@@ -1152,7 +1406,7 @@ int roomno;
 	register struct monst *mtmp;
 
 	for(mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-		if(mtmp->data == mdat &&
+		if(!DEADMONSTER(mtmp) && mtmp->data == mdat &&
 		   index(in_rooms(mtmp->mx, mtmp->my, 0), roomno + ROOMOFFSET))
 			return(TRUE);
 	return(FALSE);
@@ -1336,7 +1590,8 @@ register boolean newlev;
 		    break;
 		case DELPHI:
 		    if(monstinroom(&mons[PM_ORACLE], roomno))
-			verbalize("%s, %s, welcome to Delphi!", Hello(), plname);
+			verbalize("%s, %s, welcome to Delphi!",
+					Hello((struct monst *) 0), plname);
 		    break;
 		case TEMPLE:
 		    intemple(roomno + ROOMOFFSET);
@@ -1375,7 +1630,7 @@ register boolean newlev;
 		}
 		if (rt == COURT || rt == SWAMP || rt == MORGUE || rt == ZOO)
 		    for(mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-			if (!Stealth && !rn2(3)) mtmp->msleeping = 0;
+			if (!DEADMONSTER(mtmp) && !Stealth && !rn2(3)) mtmp->msleeping = 0;
 	    }
 	}
 
@@ -1394,6 +1649,7 @@ dopickup()
 	multi = 0;	/* always reset */
 	/* uswallow case added by GAN 01/29/87 */
 	if(u.uswallow) {
+	    if (!u.ustuck->minvent) {
 		if (is_animal(u.ustuck->data)) {
 		    You("pick up %s tongue.", s_suffix(mon_nam(u.ustuck)));
 		    pline("But it's kind of slimy, so you drop it.");
@@ -1401,20 +1657,34 @@ dopickup()
 		    You("don't %s anything in here to pick up.",
 			  Blind ? "feel" : "see");
 		return(1);
+	    } else {
+	    	int tmpcount = -count;
+		return loot_mon(u.ustuck, &tmpcount, (boolean *)0);
+	    }
 	}
 	if(is_pool(u.ux, u.uy)) {
 	    if (Wwalking || is_floater(youmonst.data) || is_clinger(youmonst.data)
 			|| (Flying && !Breathless)) {
 		You("cannot dive into the water to pick things up.");
-		return(1);
+		return(0);
 	    } else if (!Underwater) {
 		You_cant("even see the bottom, let alone pick up %s.",
 				something);
-		return(1);
+		return(0);
+	    }
+	}
+	if (is_lava(u.ux, u.uy)) {
+	    if (Wwalking || is_floater(youmonst.data) || is_clinger(youmonst.data)
+			|| (Flying && !Breathless)) {
+		You_cant("reach the bottom to pick things up.");
+		return(0);
+	    } else if (!likes_lava(youmonst.data)) {
+		You("would burn to a crisp trying to pick things up.");
+		return(0);
 	    }
 	}
 	if(!OBJ_AT(u.ux, u.uy)) {
-		pline("There is nothing here to pick up.");
+		There("is nothing here to pick up.");
 		return(0);
 	}
 	if (!can_reach_floor()) {
@@ -1425,10 +1695,9 @@ dopickup()
 		else
 #endif
 		You("cannot reach the %s.", surface(u.ux,u.uy));
-		return(1);
+		return(0);
 	}
-	pickup(-count);
-	return(1);
+	return (pickup(-count));
 }
 
 #endif /* OVLB */
@@ -1445,16 +1714,16 @@ lookaround()
     register struct monst *mtmp;
     register struct trap *trap;
 
-	/* Grid bugs stop if trying to move diagonal, even if blind.  Maybe */
-	/* they polymorphed while in the middle of a long move. */
-	if (u.umonnum == PM_GRID_BUG && u.dx && u.dy) {
-		nomul(0);
-		return;
-	}
+    /* Grid bugs stop if trying to move diagonal, even if blind.  Maybe */
+    /* they polymorphed while in the middle of a long move. */
+    if (u.umonnum == PM_GRID_BUG && u.dx && u.dy) {
+	nomul(0);
+	return;
+    }
 
-	if(Blind || flags.run == 0) return;
-	for(x = u.ux-1; x <= u.ux+1; x++) for(y = u.uy-1; y <= u.uy+1; y++) {
-		if(!isok(x,y)) continue;
+    if(Blind || flags.run == 0) return;
+    for(x = u.ux-1; x <= u.ux+1; x++) for(y = u.uy-1; y <= u.uy+1; y++) {
+	if(!isok(x,y)) continue;
 
 	if(u.umonnum == PM_GRID_BUG && x != u.ux && y != u.uy) continue;
 
@@ -1482,7 +1751,7 @@ lookaround()
 	} else if (levl[x][y].typ == CORR) {
 bcorr:
 	    if(levl[u.ux][u.uy].typ != ROOM) {
-		if(flags.run == 1 || flags.run == 3) {
+		if(flags.run == 1 || flags.run == 3 || flags.run == 8) {
 		    i = dist2(x,y,u.ux+u.dx,u.uy+u.dy);
 		    if(i > 2) continue;
 		    if(corrct == 1 && dist2(x,y,x0,y0) != 1)
@@ -1515,6 +1784,7 @@ bcorr:
 	    continue;
 	} else {		/* e.g. objects or trap or stairs */
 	    if(flags.run == 1) goto bcorr;
+	    if(flags.run == 8) continue;
 	    if(mtmp) continue;		/* d */
 	    if(((x == u.ux - u.dx) && (y != u.uy + u.dy)) ||
 	       ((y == u.uy - u.dy) && (x != u.ux + u.dx)))
@@ -1526,8 +1796,9 @@ stop:
     } /* end for loops */
 
     if(corrct > 1 && flags.run == 2) goto stop;
-    if((flags.run == 1 || flags.run == 3) && !noturn && !m0 && i0 &&
-				(corrct == 1 || (corrct == 2 && i0 == 1))) {
+    if((flags.run == 1 || flags.run == 3 || flags.run == 8) &&
+	!noturn && !m0 && i0 && (corrct == 1 || (corrct == 2 && i0 == 1)))
+    {
 	/* make sure that we do not turn too far */
 	if(i0 == 2) {
 	    if(u.dx == y0-u.uy && u.dy == u.ux-x0)
@@ -1590,7 +1861,7 @@ nomul(nval)
 	u.uinvulnerable = FALSE;	/* Kludge to avoid ctrl-C bug -dlc */
 	u.usleep = 0;
 	multi = nval;
-	flags.mv = flags.run = 0;
+	flags.travel = flags.mv = flags.run = 0;
 }
 
 /* called when a non-movement, multi-turn action has completed */
@@ -1610,6 +1881,37 @@ const char *msg_override;
 #endif /* OVL2 */
 #ifdef OVL1
 
+STATIC_OVL void
+maybe_wail()
+{
+    static short powers[] = { TELEPORT, SEE_INVIS, POISON_RES, COLD_RES,
+			      SHOCK_RES, FIRE_RES, SLEEP_RES, DISINT_RES,
+			      TELEPORT_CONTROL, STEALTH, FAST, INVIS };
+
+    if (moves <= wailmsg + 50) return;
+
+    wailmsg = moves;
+    if (Role_if(PM_WIZARD) || Race_if(PM_ELF) || Role_if(PM_VALKYRIE)) {
+	const char *who;
+	int i, powercnt;
+
+	who = (Role_if(PM_WIZARD) || Role_if(PM_VALKYRIE)) ?
+		urole.name.m : "Elf";
+	if (u.uhp == 1) {
+	    pline("%s is about to die.", who);
+	} else {
+	    for (i = 0, powercnt = 0; i < SIZE(powers); ++i)
+		if (u.uprops[powers[i]].intrinsic & INTRINSIC) ++powercnt;
+
+	    pline(powercnt >= 4 ? "%s, all your powers will be lost..."
+				: "%s, your life force is running out.", who);
+	}
+    } else {
+	You_hear(u.uhp == 1 ? "the wailing of the Banshee..."
+			    : "the howling of the CwnAnnwn...");
+    }
+}
+
 void
 losehp(n, knam, k_format)
 register int n;
@@ -1620,7 +1922,10 @@ boolean k_format;
 		u.mh -= n;
 		if (u.mhmax < u.mh) u.mhmax = u.mh;
 		flags.botl = 1;
-		if (u.mh < 1) rehumanize();
+		if (u.mh < 1)
+		    rehumanize();
+		else if (n > 0 && u.mh*10 < u.mhmax && Unchanging)
+		    maybe_wail();
 		return;
 	}
 
@@ -1633,38 +1938,8 @@ boolean k_format;
 		killer = knam;		/* the thing that killed you */
 		You("die...");
 		done(DIED);
-	} else if (u.uhp*10 < u.uhpmax && moves-wailmsg > 50 && n > 0) {
-		wailmsg = moves;
-		if (Role_if(PM_WIZARD) || Race_if(PM_ELF) || Role_if(PM_VALKYRIE)) {
-			if (u.uhp == 1)
-				pline("%s is about to die.",
-			    		(Role_if(PM_WIZARD) || Role_if(PM_VALKYRIE)) ?
-			    		urole.name.m : "Elf");
-			else if (4 <= (!!(HTeleportation & INTRINSIC)) +
-				    (!!(HSee_invisible & INTRINSIC)) +
-				    (!!(HPoison_resistance & INTRINSIC)) +
-				    (!!(HCold_resistance & INTRINSIC)) +
-				    (!!(HShock_resistance & INTRINSIC)) +
-				    (!!(HFire_resistance & INTRINSIC)) +
-				    (!!(HSleep_resistance & INTRINSIC)) +
-				    (!!(HDisint_resistance & INTRINSIC)) +
-				    (!!(HTeleport_control & INTRINSIC)) +
-				    (!!(HStealth & INTRINSIC)) +
-				    (!!(HFast & INTRINSIC)) +
-				    (!!(HInvis & INTRINSIC)))
-				pline("%s, all your powers will be lost...",
-			    		(Role_if(PM_WIZARD) || Role_if(PM_VALKYRIE)) ?
-			    		urole.name.m : "Elf");
-			else
-				pline("%s, your life force is running out.",
-			    		(Role_if(PM_WIZARD) || Role_if(PM_VALKYRIE)) ?
-			    		urole.name.m : "Elf");
-		} else {
-			if(u.uhp == 1)
-				You_hear("the wailing of the Banshee...");
-			else
-				You_hear("the howling of the CwnAnnwn...");
-		}
+	} else if (n > 0 && u.uhp*10 < u.uhpmax) {
+		maybe_wail();
 	}
 }
 
@@ -1710,17 +1985,24 @@ int
 inv_weight()
 {
 	register struct obj *otmp = invent;
-	register int wt;
+	register int wt = 0;
 
+#ifndef GOLDOBJ
 	/* when putting stuff into containers, gold is inserted at the head
 	   of invent for easier manipulation by askchain & co, but it's also
 	   retained in u.ugold in order to keep the status line accurate; we
 	   mustn't add its weight in twice under that circumstance */
 	wt = (otmp && otmp->oclass == GOLD_CLASS) ? 0 :
 		(int)((u.ugold + 50L) / 100L);
-
+#endif
 	while (otmp) {
+#ifndef GOLDOBJ
 		if (otmp->otyp != BOULDER || !throws_rocks(youmonst.data))
+#else
+		if (otmp->oclass == GOLD_CLASS)
+			wt += (int)(((long)otmp->quan + 50L) / 100L);
+		else if (otmp->otyp != BOULDER || !throws_rocks(youmonst.data))
+#endif
 			wt += otmp->owt;
 		otmp = otmp->nobj;
 	}
@@ -1788,6 +2070,22 @@ inv_cnt()
 	return(ct);
 }
 
+#ifdef GOLDOBJ
+/* Counts the money in an object chain. */
+/* Intended use is for your or some monsters inventory, */
+/* now that u.gold/m.gold is gone.*/
+/* Counting money in a container might be possible too. */
+long money_cnt(otmp)
+struct obj *otmp;
+{
+        while(otmp) {
+	        /* Must change when silver & copper is implemented: */
+ 	        if (otmp->oclass == GOLD_CLASS) return otmp->quan;
+  	        otmp = otmp->nobj;
+	}
+	return 0;
+}
+#endif
 #endif /* OVLB */
 
 /*hack.c*/
