@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)light.c	3.3	97/04/10	*/
+/*	SCCS Id: @(#)light.c	3.4	1997/04/10	*/
 /* Copyright (c) Dean Luick, 1994					*/
 /* NetHack may be freely redistributed.  See license for details.	*/
 
@@ -40,22 +40,12 @@
 
 #ifdef OVL3
 
-typedef struct ls_t {
-    struct ls_t *next;
-    xchar x, y;		/* source's position */
-    short range;	/* source's current range */
-    short flags;
-    short type;		/* type of light source */
-    genericptr_t id;	/* source's identifier */
-} light_source;
-
 /* flags */
 #define LSF_SHOW	0x1		/* display the light source */
 #define LSF_NEEDS_FIXUP	0x2		/* need oid fixup */
 
 static light_source *light_base = 0;
 
-struct monst *FDECL(find_mid, (unsigned));
 STATIC_DCL void FDECL(write_ls, (int, light_source *));
 STATIC_DCL int FDECL(maybe_write_ls, (int, int, BOOLEAN_P));
 
@@ -216,19 +206,23 @@ do_light_sources(cs_rows)
 #define mon_is_local(mon)	((mon)->mx > 0)
 
 struct monst *
-find_mid(nid)
+find_mid(nid, fmflags)
 unsigned nid;
+unsigned fmflags;
 {
 	struct monst *mtmp;
 
 	if (!nid)
 	    return &youmonst;
-	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-	    if (mtmp->m_id == nid) return mtmp;
-	for (mtmp = migrating_mons; mtmp; mtmp = mtmp->nmon)
-	    if (mtmp->m_id == nid) return mtmp;
-	for (mtmp = mydogs; mtmp; mtmp = mtmp->nmon)
-	    if (mtmp->m_id == nid) return mtmp;
+	if (fmflags & FM_FMON)
+		for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+		    if (!DEADMONSTER(mtmp) && mtmp->m_id == nid) return mtmp;
+	if (fmflags & FM_MIGRATE)
+		for (mtmp = migrating_mons; mtmp; mtmp = mtmp->nmon)
+	    	    if (mtmp->m_id == nid) return mtmp;
+	if (fmflags & FM_MYDOGS)
+		for (mtmp = mydogs; mtmp; mtmp = mtmp->nmon)
+	    	    if (mtmp->m_id == nid) return mtmp;
 	return (struct monst *) 0;
 }
 
@@ -323,7 +317,7 @@ relink_light_sources(ghostly)
 		    ls->id = (genericptr_t) find_oid(nid);
 		} else {
 		    which = 'm';
-		    ls->id = (genericptr_t) find_mid(nid);
+		    ls->id = (genericptr_t) find_mid(nid, FM_EVERYWHERE);
 		}
 		if (!ls->id)
 		    impossible("relink_light_sources: cant find %c_id %d",
@@ -404,7 +398,7 @@ write_ls(fd, ls)
 		mtmp = (struct monst *)ls->id;
 		ls->id = (genericptr_t)mtmp->m_id;
 #ifdef DEBUG
-		if (find_mid((unsigned)ls->id) != mtmp)
+		if (find_mid((unsigned)ls->id, FM_EVERYWHERE) != mtmp)
 		    panic("write_ls: can't find mon #%u!", (unsigned)ls->id);
 #endif
 	    }
@@ -459,6 +453,12 @@ snuff_light_source(x, y)
 	if (ls->type == LS_OBJECT && ls->x == x && ls->y == y) {
 	    obj = (struct obj *) ls->id;
 	    if (obj_is_burning(obj)) {
+		/* The only way to snuff Sunsword is to unwield it.  Darkness
+		 * scrolls won't affect it.  (If we got here because it was
+		 * dropped or thrown inside a monster, this won't matter anyway
+		 * because it will go out when dropped.)
+		 */
+		if (artifact_light(obj)) continue;
 		end_burn(obj, obj->otyp != MAGIC_LAMP);
 		/*
 		 * The current ls element has just been removed (and
@@ -491,7 +491,8 @@ obj_is_burning(obj)
 		|| obj->otyp == CANDELABRUM_OF_INVOCATION
 		|| obj->otyp == TALLOW_CANDLE
 		|| obj->otyp == WAX_CANDLE
-		|| obj->otyp == POT_OIL));
+		|| obj->otyp == POT_OIL
+		|| artifact_light(obj)));
 }
 
 /* copy the light source(s) attachted to src, and attach it/them to dest */
