@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)mhitm.c	3.3	98/08/15	*/
+/*	SCCS Id: @(#)mhitm.c	3.4	2002/02/17	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -123,7 +123,12 @@ fightm(mtmp)		/* have monsters fight each other */
 	for(mon = fmon; mon; mon = nmon) {
 	    nmon = mon->nmon;
 	    if(nmon == mtmp) nmon = mtmp->nmon;
-	    if(mon != mtmp) {
+	    /* Be careful to ignore monsters that are already dead, since we
+	     * might be calling this before we've cleaned them up.  This can
+	     * happen if the monster attacked a cockatrice bare-handedly, for
+	     * instance.
+	     */
+	    if(mon != mtmp && !DEADMONSTER(mon)) {
 		if(monnear(mtmp,mon->mx,mon->my)) {
 		    if(!u.uswallow && (mtmp == u.ustuck)) {
 			if(!rn2(4)) {
@@ -158,7 +163,7 @@ fightm(mtmp)		/* have monsters fight each other */
  *
  * This function returns a result bitfield:
  *
- *	    --------- agressor died
+ *	    --------- aggressor died
  *	   /  ------- defender died
  *	  /  /  ----- defender was hit
  *	 /  /  /
@@ -184,7 +189,7 @@ mattackm(magr, mdef)
 		    attk,	/* attack attempted this time */
 		    struck = 0,	/* hit at least once */
 		    res[NATTK];	/* results of all attacks */
-    struct attack   *mattk;
+    struct attack   *mattk, alt_attk;
     struct permonst *pa, *pd;
 
     if (!magr || !mdef) return(MM_MISS);		/* mike@genat */
@@ -228,7 +233,7 @@ mattackm(magr, mdef)
     /* Now perform all attacks for the monster. */
     for (i = 0; i < NATTK; i++) {
 	res[i] = MM_MISS;
-	mattk = &(pa->mattk[i]);
+	mattk = getmattk(pa, i, res, &alt_attk);
 	otmp = (struct obj *)0;
 	attk = 1;
 	switch (mattk->aatyp) {
@@ -252,6 +257,9 @@ mattackm(magr, mdef)
 	    case AT_TUCH:
 	    case AT_BUTT:
 	    case AT_TENT:
+		/* Nymph that teleported away on first attack? */
+		if (distmin(magr->mx,magr->my,mdef->mx,mdef->my) > 1)
+		    return MM_MISS;
 		/* Monsters won't attack cockatrices physically if they
 		 * have a weapon instead.  This instinct doesn't work for
 		 * players, or under conflict or confusion. 
@@ -266,9 +274,22 @@ mattackm(magr, mdef)
 		/* KMH -- don't accumulate to-hit bonuses */
 		if (otmp)
 		    tmp -= hitval(otmp, mdef);
-		if (strike)
+		if (strike) {
 		    res[i] = hitmm(magr, mdef, mattk);
-		else
+		    if((mdef->data == &mons[PM_BLACK_PUDDING] || mdef->data == &mons[PM_BROWN_PUDDING])
+		       && otmp && objects[otmp->otyp].oc_material == IRON
+		       && mdef->mhp > 1 && !mdef->mcan)
+		    {
+			if (clone_mon(mdef)) {
+			    if (vis) {
+				char buf[BUFSZ];
+
+				Strcpy(buf, Monnam(mdef));
+				pline("%s divides as %s hits it!", buf, mon_nam(magr));
+			    }
+			}
+		    }
+		} else
 		    missmm(magr, mdef, mattk);
 		break;
 
@@ -290,6 +311,12 @@ mattackm(magr, mdef)
 		break;
 
 	    case AT_ENGL:
+#ifdef STEED
+		if (u.usteed && (mdef == u.usteed)) {
+		    strike = 0;
+		    break;
+		} 
+#endif
 		/* Engulfing attacks are directed at the hero if
 		 * possible. -dlc
 		 */
@@ -379,8 +406,8 @@ hitmm(magr, mdef, mattk)
 			default:
 				Sprintf(buf,"%s hits", magr_name);
 		    }
+		    pline("%s %s.", buf, mon_nam_too(mdef_name, mdef, magr));
 		}
-		pline("%s %s.", buf, mon_nam_too(mdef_name, mdef, magr));
 	} else  noises(magr, mattk);
 	return(mdamagem(magr, mdef, mattk));
 }
@@ -398,10 +425,38 @@ gazemm(magr, mdef, mattk)
 		pline("%s %s...", buf, mon_nam(mdef));
 	}
 
-	if (!mdef->mcansee || mdef->msleeping) {
+	if (magr->mcan || !magr->mcansee ||
+	    (magr->minvis && !perceives(mdef->data)) ||
+	    !mdef->mcansee || mdef->msleeping) {
 	    if(vis) pline("but nothing happens.");
 	    return(MM_MISS);
 	}
+	/* call mon_reflects 2x, first test, then, if visible, print message */
+	if (magr->data == &mons[PM_MEDUSA] && mon_reflects(mdef, (char *)0)) {
+	    if (canseemon(mdef))
+		(void) mon_reflects(mdef,
+				    "The gaze is reflected away by %s %s.");
+	    if (mdef->mcansee) {
+		if (mon_reflects(magr, (char *)0)) {
+		    if (canseemon(magr))
+			(void) mon_reflects(magr,
+					"The gaze is reflected away by %s %s.");
+		    return (MM_MISS);
+		}
+		if (mdef->minvis && !perceives(magr->data)) {
+		    if (canseemon(magr)) {
+			pline("%s doesn't seem to notice that %s gaze was reflected.",
+			      Monnam(magr), mhis(magr));
+		    }
+		    return (MM_MISS);
+		}
+		if (canseemon(magr))
+		    pline("%s is turned to stone!", Monnam(magr));
+		monstone(magr);
+		if (magr->mhp > 0) return (MM_MISS);
+		return (MM_AGR_DIED);
+	    }
+	}
 
 	return(mdamagem(magr, mdef, mattk));
 }
@@ -510,12 +565,17 @@ mdamagem(magr, mdef, mattk)
 {
 	struct	permonst *pa = magr->data, *pd = mdef->data;
 	int	tmp = d((int)mattk->damn,(int)mattk->damd);
+	struct obj *obj;
 	char buf[BUFSZ];
+	int protector =
+	    mattk->aatyp == AT_TENT ? 0 :
+	    mattk->aatyp == AT_KICK ? W_ARMF : W_ARMG;
+	int num;
 
 	if (touch_petrifies(pd) && !resists_ston(magr) &&
 	   (mattk->aatyp != AT_WEAP || !otmp) &&
 	   (mattk->aatyp != AT_GAZE && mattk->aatyp != AT_EXPL) &&
-	   !(magr->misc_worn_check & W_ARMG)) {
+	   !(magr->misc_worn_check & protector)) {
 		if (poly_when_stoned(pa)) {
 		    mon_to_stone(magr);
 		    return MM_HIT; /* no damage during the polymorph */
@@ -547,10 +607,39 @@ mdamagem(magr, mdef, mattk)
 		}
 		if(flags.verbose && flags.soundok) verbalize("Burrrrp!");
 		tmp = mdef->mhp;
+		/* Use up amulet of life saving */
+		if (!!(obj = mlifesaver(mdef))) m_useup(mdef, obj);
+
+		/* Is a corpse for nutrition possible?  It may kill magr */
+		if (!corpse_chance(mdef, magr, TRUE) || magr->mhp < 1)
+		    break;
+
+		/* Pets get nutrition from swallowing monster whole.
+		 * No nutrition from G_NOCORPSE monster, eg, undead.
+		 * DGST monsters don't die from undead corpses
+		 */
+		num = monsndx(mdef->data);
+		if (magr->mtame && !magr->isminion &&
+		    !(mvitals[num].mvflags & G_NOCORPSE)) {
+		    struct obj *virtualcorpse = mksobj(CORPSE, FALSE, FALSE);
+		    int nutrit;
+
+		    virtualcorpse->corpsenm = num;
+		    virtualcorpse->owt = weight(virtualcorpse);
+		    nutrit = dog_nutrition(magr, virtualcorpse);
+		    dealloc_obj(virtualcorpse);
+
+		    /* only 50% nutrition, 25% of normal eating time */
+		    if (magr->meating > 1) magr->meating = (magr->meating+3)/4;
+		    if (nutrit > 1) nutrit /= 2;
+		    EDOG(magr)->hungrytime += nutrit;
+		}
 		break;
 	    case AD_STUN:
 		if (magr->mcan) break;
-		if(vis) pline("%s staggers for a moment.", Monnam(mdef));
+		if (canseemon(mdef))
+		    pline("%s %s for a moment.", Monnam(mdef),
+			  makeplural(stagger(mdef->data, "stagger")));
 		mdef->mstun = 1;
 		/* fall through */
 	    case AD_WERE:
@@ -590,6 +679,7 @@ mdamagem(magr, mdef, mattk)
 		}
 		if (vis)
 		    pline("%s is %s!", Monnam(mdef),
+			  mdef->data == &mons[PM_WATER_ELEMENTAL] ? "boiling" :
 			  mattk->aatyp == AT_HUGS ?
 				"being roasted" : "on fire");
 		if (pd == &mons[PM_STRAW_GOLEM] ||
@@ -661,8 +751,8 @@ mdamagem(magr, mdef, mattk)
 		    pline("%s is covered in acid!", Monnam(mdef));
 		    pline("It burns %s!", mon_nam(mdef));
 		}
-		if(!rn2(30)) erode_armor(mdef, TRUE);
-		if(!rn2(6)) erode_weapon(mdef, TRUE);
+		if (!rn2(30)) erode_armor(mdef, TRUE);
+		if (!rn2(6)) erode_obj(MON_WEP(mdef), TRUE, TRUE);
 		break;
 	    case AD_RUST:
 		if (!magr->mcan && pd == &mons[PM_IRON_GOLEM]) {
@@ -674,7 +764,11 @@ mdamagem(magr, mdef, mattk)
 			return (MM_DEF_DIED | (grow_up(magr,mdef) ?
 							0 : MM_AGR_DIED));
 		}
-		hurtmarmor(magr->data, mdef, AD_RUST);
+		hurtmarmor(mdef, AD_RUST);
+		tmp = 0;
+		break;
+	    case AD_CORR:
+		hurtmarmor(mdef, AD_CORR);
 		tmp = 0;
 		break;
 	    case AD_DCAY:
@@ -688,7 +782,7 @@ mdamagem(magr, mdef, mattk)
 			return (MM_DEF_DIED | (grow_up(magr,mdef) ?
 							0 : MM_AGR_DIED));
 		}
-		hurtmarmor(magr->data, mdef, AD_DCAY);
+		hurtmarmor(mdef, AD_DCAY);
 		tmp = 0;
 		break;
 	    case AD_STON:
@@ -718,7 +812,11 @@ label2:			if (mdef->mhp > 0) return 0;
 		       we'll get "it" in the suddenly disappears message */
 		    if (vis) Strcpy(mdef_Monnam, Monnam(mdef));
 		    rloc(mdef);
-		    if (vis && !cansee(mdef->mx, mdef->my))
+		    if (vis && !canspotmon(mdef)
+#ifdef STEED
+		    	&& mdef != u.usteed
+#endif
+		    	)
 			pline("%s suddenly disappears!", mdef_Monnam);
 		}
 		break;
@@ -746,7 +844,7 @@ label2:			if (mdef->mhp > 0) return 0;
 		if (!magr->mcan && vis && mdef->mspeed != MSLOW) {
 		    unsigned int oldspeed = mdef->mspeed;
 
-		    mon_adjust_speed(mdef, -1);
+		    mon_adjust_speed(mdef, -1, (struct obj *)0);
 		    if (mdef->mspeed != oldspeed && vis)
 			pline("%s slows down.", Monnam(mdef));
 		}
@@ -762,10 +860,10 @@ label2:			if (mdef->mhp > 0) return 0;
 		}
 		break;
 	    case AD_BLND:
-		if (!magr->mcan && !resists_blnd(mdef)) {
+		if (can_blnd(magr, mdef, mattk->aatyp, (struct obj*)0)) {
 		    register unsigned rnd_tmp;
 
-		    if (vis)
+		    if (vis && mdef->mcansee)
 			pline("%s is blinded.", Monnam(mdef));
 		    rnd_tmp = d((int)mattk->damn, (int)mattk->damd);
 		    if ((rnd_tmp += mdef->mblinded) > 127) rnd_tmp = 127;
@@ -809,16 +907,33 @@ label2:			if (mdef->mhp > 0) return 0;
 		break;
 	    case AD_SGLD:
 		tmp = 0;
+#ifndef GOLDOBJ
 		if (magr->mcan || !mdef->mgold) break;
 		/* technically incorrect; no check for stealing gold from
 		 * between mdef's feet...
 		 */
 		magr->mgold += mdef->mgold;
 		mdef->mgold = 0;
+#else
+                if (magr->mcan) break;
+		/* technically incorrect; no check for stealing gold from
+		 * between mdef's feet...
+		 */
+                {
+		    struct obj *gold = findgold(mdef->minvent);
+		    if (!gold) break;
+                    obj_extract_self(gold);
+		    add_to_minv(magr, gold);
+                }
+#endif
 		if (vis) {
-			Strcpy(buf, Monnam(magr));
-			pline("%s steals some gold from %s.", buf,
-								mon_nam(mdef));
+		    Strcpy(buf, Monnam(magr));
+		    pline("%s steals some gold from %s.", buf, mon_nam(mdef));
+		}
+		if (!tele_restrict(magr)) {
+		    rloc(magr);
+		    if (vis && !canspotmon(magr))
+			pline("%s suddenly disappears!", buf);
 		}
 		break;
 	    case AD_DRLI:
@@ -838,30 +953,54 @@ label2:			if (mdef->mhp > 0) return 0;
 #endif
 	    case AD_SITM:	/* for now these are the same */
 	    case AD_SEDU:
-		if (!magr->mcan && mdef->minvent) {
-			char onambuf[BUFSZ];
+		/* find an object to steal, non-cursed if magr is tame */
+		for (obj = mdef->minvent; obj; obj = obj->nobj) {
+		    if (!magr->mtame || !obj->cursed)
+			break;
+		}
 
-			otmp = mdef->minvent;
+		if (!magr->mcan && obj) {
+			char onambuf[BUFSZ], mdefnambuf[BUFSZ];
+
+			/* make a special x_monnam() call that never omits
+			   the saddle, and save it for later messages */
+			Strcpy(mdefnambuf, x_monnam(mdef, ARTICLE_THE, (char *)0, 0, FALSE));
+
+			otmp = obj;
+#ifdef STEED
+			if (u.usteed == mdef &&
+					otmp == which_armor(mdef, W_SADDLE))
+				/* "You can no longer ride <steed>." */
+				dismount_steed(DISMOUNT_POLY);
+#endif
 			obj_extract_self(otmp);
 			if (otmp->owornmask) {
 				mdef->misc_worn_check &= ~otmp->owornmask;
+				if (otmp->owornmask & W_WEP)
+				    setmnotwielded(mdef,otmp);
 				otmp->owornmask = 0L;
 				update_mon_intrinsics(mdef, otmp, FALSE);
 			}
 			/* add_to_minv() might free otmp [if it merges] */
 			if (vis)
 				Strcpy(onambuf, doname(otmp));
-			add_to_minv(magr, otmp);
+			(void) add_to_minv(magr, otmp);
 			if (vis) {
 				Strcpy(buf, Monnam(magr));
 				pline("%s steals %s from %s!", buf,
-				      onambuf, mon_nam(mdef));
+				    onambuf, mdefnambuf);
 			}
 			possibly_unwield(mdef);
 			mselftouch(mdef, (const char *)0, FALSE);
 			if (mdef->mhp <= 0)
 				return (MM_DEF_DIED | (grow_up(magr,mdef) ?
 							0 : MM_AGR_DIED));
+			if (magr->data->mlet == S_NYMPH &&
+			    !tele_restrict(magr)) {
+			    rloc(magr);
+			    if (vis && !canspotmon(magr))
+				pline("%s suddenly disappears!", buf);
+			}
 		}
 		tmp = 0;
 		break;
@@ -888,6 +1027,7 @@ label2:			if (mdef->mhp > 0) return 0;
 	    case AD_DRIN:
 		if (notonhead || !has_head(pd)) {
 		    if (vis) pline("%s doesn't seem harmed.", Monnam(mdef));
+		    /* Not clear what to do for green slimes */
 		    tmp = 0;
 		    break;
 		}
@@ -896,7 +1036,7 @@ label2:			if (mdef->mhp > 0) return 0;
 			Strcpy(buf, s_suffix(Monnam(mdef)));
 			pline("%s helmet blocks %s attack to %s head.",
 				buf, s_suffix(mon_nam(magr)),
-				his[pronoun_gender(mdef)]);
+				mhis(mdef));
 		    }
 		    break;
 		}
@@ -917,9 +1057,10 @@ label2:			if (mdef->mhp > 0) return 0;
 	    case AD_SLIM:
 	    	if (!rn2(4) && mdef->data != &mons[PM_FIRE_VORTEX] &&
 	    			mdef->data != &mons[PM_FIRE_ELEMENTAL] &&
+	    			mdef->data != &mons[PM_SALAMANDER] &&
 	    			mdef->data != &mons[PM_GREEN_SLIME]) {
 	    	    if (vis) pline("%s turns into slime.", Monnam(mdef));
-	    	    (void) newcham(mdef, &mons[PM_GREEN_SLIME]);
+	    	    (void) newcham(mdef, &mons[PM_GREEN_SLIME], FALSE);
 	    	    tmp = 0;
 	    	}
 	    	break;
@@ -935,7 +1076,9 @@ label2:			if (mdef->mhp > 0) return 0;
 	if((mdef->mhp -= tmp) < 1) {
 	    if (m_at(mdef->mx, mdef->my) == magr) {  /* see gulpmm() */
 		remove_monster(mdef->mx, mdef->my);
+		mdef->mhp = 1;	/* otherwise place_monster will complain */
 		place_monster(mdef, mdef->mx, mdef->my);
+		mdef->mhp = 0;
 	    }
 	    monkilled(mdef, "", (int)mattk->adtyp);
 	    if (mdef->mhp > 0) return 0; /* mdef lifesaved */
@@ -1003,9 +1146,20 @@ mrustm(magr, mdef, obj)
 register struct monst *magr, *mdef;
 register struct obj *obj;
 {
+	boolean is_acid;
+
 	if (!magr || !mdef || !obj) return; /* just in case */
-	if (mdef->data == &mons[PM_RUST_MONSTER] && !mdef->mcan &&
-	    is_rustprone(obj) && obj->oeroded < MAX_ERODE) {
+
+	if (dmgtype(mdef->data, AD_CORR))
+	    is_acid = TRUE;
+	else if (dmgtype(mdef->data, AD_RUST))
+	    is_acid = FALSE;
+	else
+	    return;
+
+	if (!mdef->mcan &&
+	    (is_acid ? is_corrodeable(obj) : is_rustprone(obj)) &&
+	    (is_acid ? obj->oeroded2 : obj->oeroded) < MAX_ERODE) {
 		if (obj->greased || obj->oerodeproof || (obj->blessed && rn2(3))) {
 		    if (cansee(mdef->mx, mdef->my) && flags.verbose)
 			pline("%s weapon is not affected.",
@@ -1014,10 +1168,12 @@ register struct obj *obj;
 		} else {
 		    if (cansee(mdef->mx, mdef->my)) {
 			pline("%s %s%s!", s_suffix(Monnam(magr)),
-			      aobjnam(obj, "rust"),
-			      obj->oeroded ? " further" : "");
+			    aobjnam(obj, (is_acid ? "corrode" : "rust")),
+			    (is_acid ? obj->oeroded2 : obj->oeroded)
+				? " further" : "");
 		    }
-		    obj->oeroded++;
+		    if (is_acid) obj->oeroded2++;
+		    else obj->oeroded++;
 		}
 	}
 }
@@ -1028,11 +1184,11 @@ register struct monst *magr, *mdef;
 register struct obj *otemp;
 {
 	char buf[BUFSZ];
+	if (!flags.verbose || Blind || !mon_visible(magr)) return;
 	Strcpy(buf, mon_nam(mdef));
-	if (!flags.verbose || Blind) return;
 	pline("%s %s %s %s at %s.", Monnam(magr),
 	      (objects[otemp->otyp].oc_dir & PIERCE) ? "thrusts" : "swings",
-	      his[pronoun_gender(magr)], xname(otemp), buf);
+	      mhis(magr), singular(otemp, xname), buf);
 }
 
 /*
@@ -1138,7 +1294,8 @@ int mdead;
 		if (!magr->mstun) {
 		    magr->mstun = 1;
 		    if (canseemon(magr))
-			pline("%s staggers...", Monnam(magr));
+			pline("%s %s...", Monnam(magr),
+			      makeplural(stagger(magr->data, "stagger")));
 		}
 		tmp = 0;
 		break;
