@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)mklev.c	3.3	99/04/22	*/
+/*	SCCS Id: @(#)mklev.c	3.4	2001/11/29	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -34,7 +34,9 @@ STATIC_DCL struct mkroom *FDECL(pos_to_room, (XCHAR_P, XCHAR_P));
 STATIC_DCL boolean FDECL(place_niche,(struct mkroom *,int*,int*,int*));
 STATIC_DCL void FDECL(makeniche,(int));
 STATIC_DCL void NDECL(make_niches);
+
 STATIC_PTR int FDECL(do_comp,(const genericptr,const genericptr));
+
 STATIC_DCL void FDECL(dosdoor,(XCHAR_P,XCHAR_P,struct mkroom *,int));
 STATIC_DCL void FDECL(join,(int,int,BOOLEAN_P));
 STATIC_DCL void FDECL(do_room_or_subroom, (struct mkroom *,int,int,int,int,
@@ -450,7 +452,7 @@ static NEARDATA const char *trap_engravings[TRAPNUM] = {
 			(char *)0, (char *)0, (char *)0, (char *)0, (char *)0,
 			(char *)0, (char *)0, (char *)0, (char *)0, (char *)0,
 			(char *)0, (char *)0, (char *)0, (char *)0,
-			/* 14..16: trapdoor, teleport, level-teleport */
+			/* 14..16: trap door, teleport, level-teleport */
 			"Vlad was here", "ad aerarium", "ad aerarium",
 			(char *)0, (char *)0, (char *)0, (char *)0, (char *)0,
 			(char *)0,
@@ -498,8 +500,9 @@ int trap_type;
 		    dosdoor(xx, yy, aroom, rn2(5) ? SDOOR : DOOR);
 		else {
 		    if (!level.flags.noteleport)
-			(void) mksobj_at(SCR_TELEPORTATION, xx, yy+dy, TRUE);
-		    if(!rn2(3)) (void) mkobj_at(0, xx, yy+dy, TRUE);
+			(void) mksobj_at(SCR_TELEPORTATION,
+					 xx, yy+dy, TRUE, FALSE);
+		    if (!rn2(3)) (void) mkobj_at(0, xx, yy+dy, TRUE);
 		}
 	    }
 	    return;
@@ -719,7 +722,7 @@ makelevel()
 	register int u_depth = depth(&u.uz);
 
 #ifdef WIZARD
-	if(wizard && getenv("SHOPTYPE")) mkroom(SHOPBASE); else
+	if(wizard && nh_getenv("SHOPTYPE")) mkroom(SHOPBASE); else
 #endif
 	if (u_depth > 1 &&
 	    u_depth < depth(&medusa_level) &&
@@ -761,8 +764,8 @@ skip0:
 		    x = somex(croom); y = somey(croom);
 		    tmonst = makemon((struct permonst *) 0, x,y,NO_MM_FLAGS);
 		    if (tmonst && tmonst->data == &mons[PM_GIANT_SPIDER] &&
-			!is_pool(x,y))
-			(void) maketrap (x,y,WEB);
+			    !occupied(x, y))
+			(void) maketrap(x, y, WEB);
 		}
 		/* put traps and mimics inside */
 		goldseen = FALSE;
@@ -796,7 +799,7 @@ skip0:
 		 */
 		if(!rn2(nroom * 5 / 2))
 		    (void) mksobj_at((rn2(3)) ? LARGE_BOX : CHEST,
-				     somex(croom), somey(croom), TRUE);
+				     somex(croom), somey(croom), TRUE, FALSE);
 
 		/* maybe make some graffiti */
 		if(!rn2(27 + 3 * abs(depth(&u.uz)))) {
@@ -846,8 +849,9 @@ mineralize()
 	if (In_endgame(&u.uz)) return;
 	for (x = 2; x < (COLNO - 2); x++)
 	    for (y = 1; y < (ROWNO - 1); y++)
-	    	if ((levl[x][y].typ == POOL || levl[x][y].typ == MOAT) && !rn2(3))
-	    	    (void)mksobj_at(KELP_FROND, x, y, TRUE);
+		if ((levl[x][y].typ == POOL && !rn2(10)) ||
+			(levl[x][y].typ == MOAT && !rn2(30)))
+		    (void) mksobj_at(KELP_FROND, x, y, TRUE, FALSE);
 
 	/* determine if it is even allowed;
 	   almost all special levels are excluded */
@@ -1376,6 +1380,7 @@ struct mkroom *croom;
 	coord m;
 	register int tryct = 0;
 	register struct obj *otmp;
+	boolean dobell = !rn2(10);
 
 
 	if(croom->rtype != OROOM) return;
@@ -1387,7 +1392,7 @@ struct mkroom *croom;
 	} while (occupied(m.x, m.y) || bydoor(m.x, m.y));
 
 	/* Put a grave at m.x, m.y */
-	levl[m.x][m.y].typ = GRAVE;
+	make_grave(m.x, m.y, dobell ? "Saved by the bell!" : (char *) 0);
 
 	/* Possibly fill it with objects */
 	if (!rn2(3)) (void) mkgold(0L, m.x, m.y);
@@ -1401,7 +1406,7 @@ struct mkroom *croom;
 	}
 
 	/* Leave a bell, in case we accidentally buried someone alive */
-	if (!rn2(25)) (void) mksobj_at(BELL, m.x, m.y, TRUE);
+	if (dobell) (void) mksobj_at(BELL, m.x, m.y, TRUE, FALSE);
 	return;
 }
 
