@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)mkmaze.c	3.2	95/09/06	*/
+/*	SCCS Id: @(#)mkmaze.c	3.4	2002/03/12	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -11,49 +11,52 @@ extern char *lev_message;
 extern lev_region *lregions;
 extern int num_lregions;
 
-static boolean FDECL(iswall,(int,int));
-static boolean FDECL(iswall_or_stone,(int,int));
-static boolean FDECL(is_solid,(int,int));
-static int FDECL(extend_spine, (int [3][3], int, int, int));
-static boolean FDECL(okay,(int,int,int));
-static void FDECL(maze0xy,(coord *));
-static boolean FDECL(put_lregion_here,(XCHAR_P,XCHAR_P,XCHAR_P,
+STATIC_DCL boolean FDECL(iswall,(int,int));
+STATIC_DCL boolean FDECL(iswall_or_stone,(int,int));
+STATIC_DCL boolean FDECL(is_solid,(int,int));
+STATIC_DCL int FDECL(extend_spine, (int [3][3], int, int, int));
+STATIC_DCL boolean FDECL(okay,(int,int,int));
+STATIC_DCL void FDECL(maze0xy,(coord *));
+STATIC_DCL boolean FDECL(put_lregion_here,(XCHAR_P,XCHAR_P,XCHAR_P,
 	XCHAR_P,XCHAR_P,XCHAR_P,XCHAR_P,BOOLEAN_P,d_level *));
-static void NDECL(fixup_special);
-static void FDECL(move, (int *,int *,int));
-static void NDECL(setup_waterlevel);
-static void NDECL(unsetup_waterlevel);
+STATIC_DCL void NDECL(fixup_special);
+STATIC_DCL void FDECL(move, (int *,int *,int));
+STATIC_DCL void NDECL(setup_waterlevel);
+STATIC_DCL void NDECL(unsetup_waterlevel);
 
-#define OUT_OF_BOUNDS(x,y) ((x)<=0 || (y)<0 || (x)>COLNO-1 || (y)>ROWNO-1)
 
-static boolean
+STATIC_OVL boolean
 iswall(x,y)
 int x,y;
 {
-	if (OUT_OF_BOUNDS(x,y)) return FALSE;
-	return (IS_WALL(levl[x][y].typ) || IS_DOOR(levl[x][y].typ)
-		|| levl[x][y].typ == SDOOR);
+    register int type;
+
+    if (!isok(x,y)) return FALSE;
+    type = levl[x][y].typ;
+    return (IS_WALL(type) || IS_DOOR(type) ||
+	    type == SDOOR || type == IRONBARS);
 }
 
-static boolean
+STATIC_OVL boolean
 iswall_or_stone(x,y)
     int x,y;
 {
     register int type;
 
     /* out of bounds = stone */
-    if (OUT_OF_BOUNDS(x,y)) return TRUE;
+    if (!isok(x,y)) return TRUE;
 
     type = levl[x][y].typ;
-    return (type == STONE || IS_WALL(type) || IS_DOOR(type) || type == SDOOR);
+    return (type == STONE || IS_WALL(type) || IS_DOOR(type) ||
+	    type == SDOOR || type == IRONBARS);
 }
 
 /* return TRUE if out of bounds, wall or rock */
-static boolean
+STATIC_OVL boolean
 is_solid(x,y)
     int x, y;
 {
-    return (OUT_OF_BOUNDS(x,y) || IS_STWALL(levl[x][y].typ));
+    return (!isok(x,y) || IS_STWALL(levl[x][y].typ));
 }
 
 
@@ -75,7 +78,7 @@ is_solid(x,y)
  *		W x W		This would extend a spine from x down.
  *		. W W
  */
-static int
+STATIC_OVL int
 extend_spine(locale, wall_there, dx, dy)
     int locale[3][3];
     int wall_there, dx, dy;
@@ -191,7 +194,7 @@ int x1, y1, x2, y2;
 	    }
 }
 
-static boolean
+STATIC_OVL boolean
 okay(x,y,dir)
 int x,y;
 register int dir;
@@ -203,7 +206,7 @@ register int dir;
 	return(TRUE);
 }
 
-static void
+STATIC_OVL void
 maze0xy(cc)	/* find random starting point for maze generation */
 	coord	*cc;
 {
@@ -259,11 +262,9 @@ place_lregion(lx, ly, hx, hy, nlx, nly, nhx, nhy, rtype, lev)
     /* first a probabilistic approach */
 
     oneshot = (lx == hx && ly == hy);
-    for(trycnt = 0; trycnt < 100; trycnt ++) {
-
+    for (trycnt = 0; trycnt < 200; trycnt++) {
 	x = rn1((hx - lx) + 1, lx);
 	y = rn1((hy - ly) + 1, ly);
-
 	if (put_lregion_here(x,y,nlx,nly,nhx,nhy,rtype,oneshot,lev))
 	    return;
     }
@@ -279,7 +280,7 @@ place_lregion(lx, ly, hx, hy, nlx, nly, nhx, nhy, rtype, lev)
     impossible("Couldn't place lregion type %d!", rtype);
 }
 
-static boolean
+STATIC_OVL boolean
 put_lregion_here(x,y,nlx,nly,nhx,nhy,rtype,oneshot,lev)
 xchar x, y;
 xchar nlx, nly, nhx, nhy;
@@ -287,14 +288,14 @@ xchar rtype;
 boolean oneshot;
 d_level *lev;
 {
-    if(oneshot) {
+    if (bad_location(x, y, nlx, nly, nhx, nhy)) return FALSE;
+    if (oneshot) {
 	/* must make due with the only location possible */
 	/* avoid failure due to a misplaced trap */
 	/* it might still fail if there's a dungeon feature here */
 	struct trap *t = t_at(x,y);
 	if (t) deltrap(t);
     }
-    if(bad_location(x, y, nlx, nly, nhx, nhy)) return(FALSE);
     switch (rtype) {
     case LR_TELE:
     case LR_UPTELE:
@@ -324,7 +325,7 @@ d_level *lev;
 static boolean was_waterlevel; /* ugh... this shouldn't be needed */
 
 /* this is special stuff that the level compiler cannot (yet) handle */
-static void
+STATIC_OVL void
 fixup_special()
 {
     register lev_region *r = lregions;
@@ -399,6 +400,10 @@ fixup_special()
 	place_lregion(0,0,0,0,0,0,0,0,LR_BRANCH,(d_level *)0);
     }
 
+	/* KMH -- Sokoban levels */
+	if(In_sokoban(&u.uz))
+		sokoban_detect();
+
     /* Still need to add some stuff to level file */
     if (Is_medusa_level(&u.uz)) {
 	struct obj *otmp;
@@ -407,7 +412,7 @@ fixup_special()
 	croom = &rooms[0]; /* only one room on the medusa level */
 	for (tryct = rnd(4); tryct; tryct--) {
 	    x = somex(croom); y = somey(croom);
-	    if (goodpos(x, y, (struct monst *)0, (struct permonst *)0)) {
+	    if (goodpos(x, y, (struct monst *)0)) {
 		otmp = mk_tt_object(STATUE, x, y);
 		while (otmp && (poly_when_stoned(&mons[otmp->corpsenm]) ||
 				pm_resistance(&mons[otmp->corpsenm],MR_STONE))) {
@@ -437,20 +442,20 @@ fixup_special()
 	/* using an unfilled morgue for rm id */
 	croom = search_special(MORGUE);
 	/* avoid inappropriate morgue-related messages */
-	level.flags.graveyard = level.flags.has_morgue = FALSE;
+	level.flags.graveyard = level.flags.has_morgue = 0;
 	croom->rtype = OROOM;	/* perhaps it should be set to VAULT? */
 	/* stock the main vault */
 	for(x = croom->lx; x <= croom->hx; x++)
 	    for(y = croom->ly; y <= croom->hy; y++) {
-		mkgold((long) rn1(300, 600), x, y);
+		(void) mkgold((long) rn1(300, 600), x, y);
 		if (!rn2(3) && !is_pool(x,y))
 		    (void)maketrap(x, y, rn2(3) ? LANDMINE : SPIKED_PIT);
 	    }
-    } else if (Role_is('P') && In_quest(&u.uz)) {
+    } else if (Role_if(PM_PRIEST) && In_quest(&u.uz)) {
 	/* less chance for undead corpses (lured from lower morgues) */
-	level.flags.graveyard = TRUE;
+	level.flags.graveyard = 1;
     } else if (Is_stronghold(&u.uz)) {
-	level.flags.graveyard = TRUE;
+	level.flags.graveyard = 1;
     } else if(Is_sanctum(&u.uz)) {
 	croom = search_special(TEMPLE);
 
@@ -510,10 +515,37 @@ register const char *s;
 
 	} else Strcpy(protofile, "");
 
+#ifdef WIZARD
+	/* SPLEVTYPE format is "level-choice,level-choice"... */
+	if (wizard && *protofile && sp && sp->rndlevs) {
+	    char *ep = getenv("SPLEVTYPE");	/* not nh_getenv */
+	    if (ep) {
+		/* rindex always succeeds due to code in prior block */
+		int len = (rindex(protofile, '-') - protofile) + 1;
+
+		while (ep && *ep) {
+		    if (!strncmp(ep, protofile, len)) {
+			int pick = atoi(ep + len);
+			/* use choice only if valid */
+			if (pick > 0 && pick <= (int) sp->rndlevs)
+			    Sprintf(protofile + len, "%d", pick);
+			break;
+		    } else {
+			ep = index(ep, ',');
+			if (ep) ++ep;
+		    }
+		}
+	    }
+	}
+#endif
+
 	if(*protofile) {
 	    Strcat(protofile, LEV_EXT);
 	    if(load_special(protofile)) {
 		fixup_special();
+		/* some levels can end up with monsters
+		   on dead mon list, including light source monsters */
+		dmonsfree();
 		return;	/* no mazification right now */
 	    }
 	    impossible("Couldn't load \"%s\" - making a maze.", protofile);
@@ -534,7 +566,7 @@ register const char *s;
 	maze0xy(&mm);
 	walkfrom((int) mm.x, (int) mm.y);
 	/* put a boulder at the maze center */
-	(void) mksobj_at(BOULDER, (int) mm.x, (int) mm.y, TRUE);
+	(void) mksobj_at(BOULDER, (int) mm.x, (int) mm.y, TRUE, FALSE);
 
 #ifdef WALLIFIED_MAZE
 	wallification(2, 2, x_maze_max, y_maze_max);
@@ -597,7 +629,7 @@ register const char *s;
 	}
 	for(x = rn1(10,2); x; x--) {
 		mazexy(&mm);
-		(void) mksobj_at(BOULDER, mm.x, mm.y, TRUE);
+		(void) mksobj_at(BOULDER, mm.x, mm.y, TRUE, FALSE);
 	}
 	for (x = rn2(3); x; x--) {
 		mazexy(&mm);
@@ -609,7 +641,7 @@ register const char *s;
 	}
 	for(x = rn1(6,7); x; x--) {
 		mazexy(&mm);
-		mkgold(0L,mm.x,mm.y);
+		(void) mkgold(0L,mm.x,mm.y);
 	}
 	for(x = rn1(6,7); x; x--)
 		mktrap(0,1,(struct mkroom *) 0, (coord*) 0);
@@ -704,7 +736,7 @@ int x,y;
 }
 #endif /* MICRO */
 
-static void
+STATIC_OVL void
 move(x,y,dir)
 register int *x, *y;
 register int dir;
@@ -874,24 +906,12 @@ register xchar x, y, todnum, todlevel;
 /* to ease the work of debuggers at this stage */
 #define register
 
-struct container {
-	struct container *next;
-	xchar x, y;
-	short what;
-	genericptr_t list;
-};
 #define CONS_OBJ   0
 #define CONS_MON   1
 #define CONS_HERO  2
 #define CONS_TRAP  3
 
-static struct bubble {
-	xchar x, y;	/* coordinates of the upper left corner */
-	schar dx, dy;	/* the general direction of the bubble's movement */
-	uchar *bm;	/* pointer to the bubble bit mask */
-	struct bubble *prev, *next; /* need to traverse the list up and down */
-	struct container *cons;
-} *bbubbles, *ebubbles;
+static struct bubble *bbubbles, *ebubbles;
 
 static struct trap *wportal;
 static int xmin, ymin, xmax, ymax;	/* level boundaries */
@@ -901,9 +921,9 @@ static int xmin, ymin, xmax, ymax;	/* level boundaries */
 #define bxmax (xmax - 1)
 #define bymax (ymax - 1)
 
-static void NDECL(set_wportal);
-static void FDECL(mk_bubble, (int,int,int));
-static void FDECL(mv_bubble, (struct bubble *,int,int,BOOLEAN_P));
+STATIC_DCL void NDECL(set_wportal);
+STATIC_DCL void FDECL(mk_bubble, (int,int,int));
+STATIC_DCL void FDECL(mv_bubble, (struct bubble *,int,int,BOOLEAN_P));
 
 void
 movebubbles()
@@ -919,6 +939,8 @@ movebubbles()
 	if (!wportal) set_wportal();
 
 	vision_recalc(2);
+	/* keep attached ball&chain separate from bubble objects */
+	if (Punished) unplacebc();
 
 	/*
 	 * Pick up everything inside of a bubble then fill all bubble
@@ -1021,6 +1043,8 @@ movebubbles()
 			    FALSE);
 	}
 
+	/* put attached ball&chain back */
+	if (Punished) placebc();
 	vision_full_recalc = 1;
 }
 
@@ -1031,7 +1055,7 @@ water_friction()
 	register int x, y, dx, dy;
 	register boolean eff = FALSE;
 
-	if (is_swimmer(uasmon) && rn2(4))
+	if (Swimming && rn2(4))
 		return;		/* natural swimmers have advantage */
 
 	if (u.dx && !rn2(!u.dy ? 3 : 6)) {	/* 1/3 chance or half that */
@@ -1115,7 +1139,7 @@ register int fd;
 	was_waterlevel = TRUE;
 }
 
-static void
+STATIC_OVL void
 set_wportal()
 {
 	/* there better be only one magic portal on water level... */
@@ -1124,7 +1148,7 @@ set_wportal()
 	impossible("set_wportal(): no portal!");
 }
 
-static void
+STATIC_OVL void
 setup_waterlevel()
 {
 	register int x, y;
@@ -1153,7 +1177,7 @@ setup_waterlevel()
 			mk_bubble(x,y,rn2(7));
 }
 
-static void
+STATIC_OVL void
 unsetup_waterlevel()
 {
 	register struct bubble *b, *bb;
@@ -1167,7 +1191,7 @@ unsetup_waterlevel()
 	bbubbles = ebubbles = (struct bubble *)0;
 }
 
-static void
+STATIC_OVL void
 mk_bubble(x,y,n)
 register int x, y, n;
 {
@@ -1224,7 +1248,7 @@ register int x, y, n;
  * in the immediate neighborhood of one, he/she may get sucked inside.
  * This property also makes leaving a bubble slightly difficult.
  */
-static void
+STATIC_OVL void
 mv_bubble(b,dx,dy,ini)
 register struct bubble *b;
 register int dx, dy;
@@ -1321,7 +1345,6 @@ register boolean ini;
 		    if (MON_AT(cons->x, cons->y)) {
 				mnexto(m_at(cons->x,cons->y));
 			}
-		    if (Punished) placebc();	/* do this for now */
 		    break;
 		}
 
