@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)mplayer.c	3.3	97/02/04	*/
+/*	SCCS Id: @(#)mplayer.c	3.4	1997/02/04	*/
 /*	Copyright (c) Izchak Miller, 1992.			  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -9,7 +9,7 @@ STATIC_DCL void FDECL(get_mplname, (struct monst *, char *));
 STATIC_DCL void FDECL(mk_mplayer_armor, (struct monst *, SHORT_P));
 
 /* These are the names of those who
- * contributed to the development of NetHack 3.2/3.3.
+ * contributed to the development of NetHack 3.2/3.3/3.4.
  *
  * Keep in alphabetical order within teams.
  * Same first name is entered once within each team.
@@ -18,17 +18,18 @@ static const char *developers[] = {
 	/* devteam */
 	"Dave", "Dean", "Eric", "Izchak", "Janet", "Jessie",
 	"Ken", "Kevin", "Michael", "Mike", "Pat", "Paul", "Steve", "Timo",
+	"Warwick",
 	/* PC team */
 	"Bill", "Eric", "Keizo", "Ken", "Kevin", "Michael", "Mike", "Paul",
 	"Stephen", "Steve", "Timo", "Yitzhak",
 	/* Amiga team */
-	"Andy", "Gregg", "Keni", "Mike", "Olaf", "Richard",
+	"Andy", "Gregg", "Janne", "Keni", "Mike", "Olaf", "Richard",
 	/* Mac team */
 	"Andy", "Chris", "Dean", "Jon", "Jonathan", "Kevin", "Wang",
 	/* Atari team */
-	"Eric", "Warwick",
+	"Eric", "Marvin", "Warwick",
 	/* NT team */
-	"Michael",
+	"Alex", "Dion", "Michael",
 	/* OS/2 team */
 	"Helge", "Ron", "Timo",
 	/* VMS team */
@@ -104,7 +105,7 @@ short typ;
 	 * chances of having plusses is the same as usual....
 	 */
 	obj->spe = rn2(10) ? (rn2(3) ? rn2(5) : rn1(4,4)) : -rnd(3);
-	mpickobj(mon, obj);
+	(void) mpickobj(mon, obj);
 }
 
 struct monst *
@@ -224,41 +225,45 @@ register boolean special;
 	    }
 
 	    if (weapon != STRANGE_OBJECT) {
-			otmp = mksobj(weapon, TRUE, FALSE);
-	    	otmp->spe = (special ? rn1(5,4) : rn2(4));
-	    	if (!rn2(3)) otmp->oerodeproof = 1;
-	    	else if (!rn2(2)) otmp->greased = 1;
-	    	if (special && rn2(2))
-	    	otmp = mk_artifact(otmp, A_NONE);
-	    	/* mplayers knew better than to overenchant Magicbane */
-	    	if (otmp->oartifact == ART_MAGICBANE)
-	    	otmp->spe = rnd(4);
-	    	mpickobj(mtmp, otmp);
+		otmp = mksobj(weapon, TRUE, FALSE);
+		otmp->spe = (special ? rn1(5,4) : rn2(4));
+		if (!rn2(3)) otmp->oerodeproof = 1;
+		else if (!rn2(2)) otmp->greased = 1;
+		if (special && rn2(2))
+		    otmp = mk_artifact(otmp, A_NONE);
+		/* mplayers knew better than to overenchant Magicbane */
+		if (otmp->oartifact == ART_MAGICBANE)
+		    otmp->spe = rnd(4);
+		(void) mpickobj(mtmp, otmp);
 	    }
 
 	    if(special) {
-	        if (!rn2(10))
+		if (!rn2(10))
 		    (void) mongets(mtmp, rn2(3) ? LUCKSTONE : LOADSTONE);
-		    mk_mplayer_armor(mtmp, armor);
-		    mk_mplayer_armor(mtmp, cloak);
-		    mk_mplayer_armor(mtmp, helm);
-		    mk_mplayer_armor(mtmp, shield);
-	        if (rn2(8))
+		mk_mplayer_armor(mtmp, armor);
+		mk_mplayer_armor(mtmp, cloak);
+		mk_mplayer_armor(mtmp, helm);
+		mk_mplayer_armor(mtmp, shield);
+		if (rn2(8))
 		    mk_mplayer_armor(mtmp, rnd_class(LEATHER_GLOVES,
-				               GAUNTLETS_OF_DEXTERITY));
-	        if (rn2(8))
+					       GAUNTLETS_OF_DEXTERITY));
+		if (rn2(8))
 		    mk_mplayer_armor(mtmp, rnd_class(LOW_BOOTS, LEVITATION_BOOTS));
-	        m_dowear(mtmp, TRUE);
+		m_dowear(mtmp, TRUE);
 
-	        quan = rn2(3) ? rn2(3) : rn2(16);
-	        while(quan--)
+		quan = rn2(3) ? rn2(3) : rn2(16);
+		while(quan--)
 		    (void)mongets(mtmp, rnd_class(DILITHIUM_CRYSTAL, JADE));
-	        /* To get the gold "right" would mean a player can double his */
-	        /* gold supply by killing one mplayer.  Not good. */
-	        mtmp->mgold = rn2(1000);
-	        quan = rn2(10);
-	        while(quan--)
-		    mpickobj(mtmp, mkobj(RANDOM_CLASS, FALSE));
+		/* To get the gold "right" would mean a player can double his */
+		/* gold supply by killing one mplayer.  Not good. */
+#ifndef GOLDOBJ
+		mtmp->mgold = rn2(1000);
+#else
+		mkmonmoney(mtmp, rn2(1000));
+#endif
+		quan = rn2(10);
+		while(quan--)
+		    (void) mpickobj(mtmp, mkobj(RANDOM_CLASS, FALSE));
 	    }
 	    quan = rnd(3);
 	    while(quan--)
@@ -300,7 +305,7 @@ boolean special;
 		do {
 		    x = rn1(COLNO-4, 2);
 		    y = rnd(ROWNO-2);
-		} while(!goodpos(x, y, &fakemon) || tryct++ >= 50);
+		} while(!goodpos(x, y, &fakemon) && tryct++ <= 50);
 
 		/* if pos not found in 50 tries, don't bother to continue */
 		if(tryct > 50) return;
