@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)muse.c	3.3	1999/12/03	*/
+/*	SCCS Id: @(#)muse.c	3.4	2002/02/07	*/
 /*	Copyright (C) 1990 by Ken Arromdee			   */
 /* NetHack may be freely redistributed.  See license for details.  */
 
@@ -19,6 +19,7 @@ boolean m_using = FALSE;
  * don't know not to read scrolls, etc....
  */
 
+STATIC_DCL struct permonst *FDECL(muse_newcham_mon, (struct monst *));
 STATIC_DCL int FDECL(precheck, (struct monst *,struct obj *));
 STATIC_DCL void FDECL(mzapmsg, (struct monst *,struct obj *,BOOLEAN_P));
 STATIC_DCL void FDECL(mreadmsg, (struct monst *,struct obj *));
@@ -28,6 +29,8 @@ STATIC_DCL void FDECL(mbhit,
 	(struct monst *,int,int FDECL((*),(MONST_P,OBJ_P)),
 	int FDECL((*),(OBJ_P,OBJ_P)),struct obj *));
 STATIC_DCL void FDECL(you_aggravate, (struct monst *));
+STATIC_DCL void FDECL(mon_consume_unstone, (struct monst *,struct obj *,
+	BOOLEAN_P,BOOLEAN_P));
 
 static struct musable {
 	struct obj *offensive;
@@ -68,27 +71,28 @@ struct obj *obj;
 #define POTION_OCCUPANT_CHANCE(n) (13 + 2*(n))	/* also in potion.c */
 
 	    potion_descr = OBJ_DESCR(objects[obj->otyp]);
-	    if (potion_descr && !strcmp(potion_descr, "milky") &&
-		    flags.ghost_count < MAXMONNO &&
+	    if (potion_descr && !strcmp(potion_descr, "milky")) {
+	        if ( flags.ghost_count < MAXMONNO &&
 		    !rn2(POTION_OCCUPANT_CHANCE(flags.ghost_count))) {
-		if (!enexto(&cc, mon->mx, mon->my, &mons[PM_GHOST])) return 0;
-		mquaffmsg(mon, obj);
-		m_useup(mon, obj);
-		mtmp = makemon(&mons[PM_GHOST], cc.x, cc.y, NO_MM_FLAGS);
-		if (!mtmp) {
-		    if (vis) pline(empty);
-		} else {
-		    if (vis) {
-			pline("As %s opens the bottle, an enormous %s emerges!",
-			   mon_nam(mon),
-			   Hallucination ? rndmonnam() : (const char *)"ghost");
-			pline("%s is frightened to death, and unable to move.",
-				Monnam(mon));
+		    if (!enexto(&cc, mon->mx, mon->my, &mons[PM_GHOST])) return 0;
+		    mquaffmsg(mon, obj);
+		    m_useup(mon, obj);
+		    mtmp = makemon(&mons[PM_GHOST], cc.x, cc.y, NO_MM_FLAGS);
+		    if (!mtmp) {
+			if (vis) pline(empty);
+		    } else {
+			if (vis) {
+			    pline("As %s opens the bottle, an enormous %s emerges!",
+			       mon_nam(mon),
+			       Hallucination ? rndmonnam() : (const char *)"ghost");
+			    pline("%s is frightened to death, and unable to move.",
+				    Monnam(mon));
+			}
+			mon->mcanmove = 0;
+			mon->mfrozen = 3;
 		    }
-		    mon->mcanmove = 0;
-		    mon->mfrozen = 3;
+		    return 2;
 		}
-		return 2;
 	    }
 	    if (potion_descr && !strcmp(potion_descr, "smoky") &&
 		    flags.djinni_count < MAXMONNO &&
@@ -152,7 +156,7 @@ boolean self;
 					"nearby" : "distant");
 	} else if (self)
 		pline("%s zaps %sself with %s!",
-		      Monnam(mtmp), him[pronoun_gender(mtmp)], doname(otmp));
+		      Monnam(mtmp), mhim(mtmp), doname(otmp));
 	else {
 		pline("%s zaps %s!", Monnam(mtmp), an(xname(otmp)));
 		stop_occupation();
@@ -189,12 +193,13 @@ struct obj *otmp;
 	    pline("%s reads %s!", Monnam(mtmp), onambuf);
 	else
 	    You_hear("%s reading %s.",
-		     an(Hallucination ? rndmonnam() : mtmp->data->mname),
-		     onambuf);
+		x_monnam(mtmp, ARTICLE_A, (char *)0,
+		    (SUPPRESS_IT|SUPPRESS_INVISIBLE|SUPPRESS_SADDLE), FALSE),
+		onambuf);
 
 	if (mtmp->mconf)
 	    pline("Being confused, %s mispronounces the magic words...",
-		  vismon ? mon_nam(mtmp) : he[pronoun_gender(mtmp)]);
+		  vismon ? mon_nam(mtmp) : mhe(mtmp));
 }
 
 STATIC_OVL void
@@ -232,6 +237,7 @@ struct obj *otmp;
 #define MUSE_BUGLE 16
 #define MUSE_UNICORN_HORN 17
 #define MUSE_POT_FULL_HEALING 18
+#define MUSE_LIZARD_CORPSE 19
 /*
 #define MUSE_INNATE_TPT 9999
  * We cannot use this.  Since monsters get unlimited teleportation, if they
@@ -279,14 +285,26 @@ struct monst *mtmp;
 	    }
 	}
 
+	if (mtmp->mconf) {
+	    for(obj = mtmp->minvent; obj; obj = obj->nobj) {
+		if (obj->otyp == CORPSE && obj->corpsenm == PM_LIZARD) {
+		    m.defensive = obj;
+		    m.has_defense = MUSE_LIZARD_CORPSE;
+		    return TRUE;
+		}
+	    }
+	}
+
 	/* It so happens there are two unrelated cases when we might want to
 	 * check specifically for healing alone.  The first is when the monster
 	 * is blind (healing cures blindness).  The second is when the monster
 	 * is peaceful; then we don't want to flee the player, and by
 	 * coincidence healing is all there is that doesn't involve fleeing.
 	 * These would be hard to combine because of the control flow.
+	 * Pestilence won't use healing even when blind.
 	 */
-	if (!mtmp->mcansee && !nohands(mtmp->data)) {
+	if (!mtmp->mcansee && !nohands(mtmp->data) &&
+		mtmp->data != &mons[PM_PESTILENCE]) {
 	    if ((obj = m_carrying(mtmp, POT_FULL_HEALING)) != 0) {
 		m.defensive = obj;
 		m.has_defense = MUSE_POT_FULL_HEALING;
@@ -345,7 +363,7 @@ struct monst *mtmp;
 	} else if (sstairs.sx && sstairs.sx == x && sstairs.sy == y) {
 		m.has_defense = MUSE_SSTAIRS;
 	} else if (!stuck && !immobile) {
-	/* Note: trapdoors take precedence over teleport traps. */
+	/* Note: trap doors take precedence over teleport traps. */
 		int xx, yy;
 
 		for(xx = x-1; xx <= x+1; xx++) for(yy = y-1; yy <= y+1; yy++)
@@ -396,7 +414,7 @@ struct monst *mtmp;
 	}
 
 	/* use immediate physical escape prior to attempting magic */
-	if (m.has_defense)	/* stairs, trapdoor or tele-trap, bugle alert */
+	if (m.has_defense)    /* stairs, trap door or tele-trap, bugle alert */
 		goto botm;
 
 	/* kludge to cut down on trap destruction (particularly portals) */
@@ -415,6 +433,8 @@ struct monst *mtmp;
 		if (obj->otyp == WAN_DIGGING && obj->spe > 0 && !stuck && !t
 		    && !mtmp->isshk && !mtmp->isgd && !mtmp->ispriest
 		    && !is_floater(mtmp->data)
+		    /* monsters digging in Sokoban can ruin things */
+		    && !In_sokoban(&u.uz)
 		    /* digging wouldn't be effective; assume they know that */
 		    && !(levl[x][y].wall_info & W_NONDIGGABLE)
 		    && !(Is_botlevel(&u.uz) || In_endgame(&u.uz))
@@ -439,6 +459,8 @@ struct monst *mtmp;
 			m.defensive = obj;
 			m.has_defense = MUSE_SCR_TELEPORTATION;
 		}
+
+	    if (mtmp->data != &mons[PM_PESTILENCE]) {
 		nomore(MUSE_POT_FULL_HEALING);
 		if(obj->otyp == POT_FULL_HEALING) {
 			m.defensive = obj;
@@ -459,6 +481,18 @@ struct monst *mtmp;
 			m.defensive = obj;
 			m.has_defense = MUSE_POT_HEALING;
 		}
+	    } else {	/* Pestilence */
+		nomore(MUSE_POT_FULL_HEALING);
+		if (obj->otyp == POT_SICKNESS) {
+			m.defensive = obj;
+			m.has_defense = MUSE_POT_FULL_HEALING;
+		}
+		nomore(MUSE_WAN_CREATE_MONSTER);
+		if (obj->otyp == WAN_CREATE_MONSTER && obj->spe > 0) {
+			m.defensive = obj;
+			m.has_defense = MUSE_WAN_CREATE_MONSTER;
+		}
+	    }
 		nomore(MUSE_SCR_CREATE_MONSTER);
 		if(obj->otyp == SCR_CREATE_MONSTER) {
 			m.defensive = obj;
@@ -491,7 +525,7 @@ struct monst *mtmp;
 	   rushing right straight back; don't override if already scared */
 	fleetim = !mtmp->mflee ? (33 - (30 * mtmp->mhp / mtmp->mhpmax)) : 0;
 #define m_flee(m)	if (fleetim && !m->iswiz) \
-			{ m->mflee = 1; m->mfleetim = fleetim; }
+			{ monflee(m, fleetim, FALSE, FALSE); }
 
 	switch(m.has_defense) {
 	case MUSE_UNICORN_HORN:
@@ -499,7 +533,7 @@ struct monst *mtmp;
 		    if (otmp)
 			pline("%s uses a unicorn horn!", Monnam(mtmp));
 		    else
-			pline("The tip of %s's horn glows!", mon_nam(mtmp));
+			pline_The("tip of %s's horn glows!", mon_nam(mtmp));
 		}
 		if (!mtmp->mcansee) {
 		    mtmp->mcansee = 1;
@@ -602,18 +636,21 @@ mon_tele:
 		}
 		if (!Can_dig_down(&u.uz)) {
 		    if(canseemon(mtmp))
-			pline_The("floor here is too hard to dig in.");
+			pline_The("%s here is too hard to dig in.",
+					surface(mtmp->mx, mtmp->my));
 		    return 2;
 		}
 		ttmp = maketrap(mtmp->mx, mtmp->my, HOLE);
 		if (!ttmp) return 2;
 		seetrap(ttmp);
 		if (vis) {
-		    pline("%s has made a hole in the floor.", Monnam(mtmp));
+		    pline("%s has made a hole in the %s.", Monnam(mtmp),
+				surface(mtmp->mx, mtmp->my));
 		    pline("%s %s through...", Monnam(mtmp),
 			  is_flyer(mtmp->data) ? "dives" : "falls");
 		} else if (flags.soundok)
-			You_hear("%s crash through the floor.", something);
+			You_hear("%s crash through the %s.", something,
+				surface(mtmp->mx, mtmp->my));
 		/* we made sure that there is a level for mtmp to go to */
 		migrate_to_level(mtmp, ledger_no(&u.uz) + 1,
 				 MIGR_RANDOM, (coord *)0);
@@ -668,8 +705,8 @@ mon_tele:
 		return 2;
 	    }
 	case MUSE_TRAPDOOR:
-		/* trapdoors on "bottom" levels of dungeons are rock-drop
-		 * trapdoors, not holes in the floor.  We check here for
+		/* trap doors on "bottom" levels of dungeons are rock-drop
+		 * trap doors, not holes in the floor.  We check here for
 		 * safety.
 		 */
 		if (Is_botlevel(&u.uz)) return 0;
@@ -679,7 +716,11 @@ mon_tele:
 			t = t_at(trapx,trapy);
 			pline("%s %s into a %s!", Monnam(mtmp),
 			makeplural(locomotion(mtmp->data, "jump")),
-			t->ttyp == TRAPDOOR ? "trapdoor" : "hole");
+			t->ttyp == TRAPDOOR ? "trap door" : "hole");
+			if (levl[trapx][trapy].typ == SCORR) {
+			    levl[trapx][trapy].typ = CORR;
+			    unblock_point(trapx, trapy);
+			}
 			seetrap(t_at(trapx,trapy));
 		}
 
@@ -713,7 +754,7 @@ mon_tele:
 			(dunlev(&u.uz) < dunlevs_in_dungeon(&u.uz) - 3)) {
 		    if (vismon) pline(
      "As %s climbs the stairs, a mysterious force momentarily surrounds %s...",
-				     mon_nam(mtmp), him[pronoun_gender(mtmp)]);
+				     mon_nam(mtmp), mhim(mtmp));
 		    /* simpler than for the player; this will usually be
 		       the Wizard and he'll immediately go right to the
 		       upstairs, so there's not much point in having any
@@ -766,6 +807,10 @@ mon_tele:
 		if (vis) {
 			pline("%s %s onto a teleport trap!", Monnam(mtmp),
 				makeplural(locomotion(mtmp->data, "jump")));
+			if (levl[trapx][trapy].typ == SCORR) {
+			    levl[trapx][trapy].typ = CORR;
+			    unblock_point(trapx, trapy);
+			}
 			seetrap(t_at(trapx,trapy));
 		}
 		/*  don't use rloc_to() because worm tails must "move" */
@@ -807,16 +852,21 @@ mon_tele:
 		return 2;
 	case MUSE_POT_FULL_HEALING:
 		mquaffmsg(mtmp, otmp);
+		if (otmp->otyp == POT_SICKNESS) unbless(otmp); /* Pestilence */
 		mtmp->mhp = (mtmp->mhpmax += (otmp->blessed ? 8 : 4));
-		if (!mtmp->mcansee) {
+		if (!mtmp->mcansee && otmp->otyp != POT_SICKNESS) {
 			mtmp->mcansee = 1;
 			mtmp->mblinded = 0;
 			if (vismon) pline(mcsa, Monnam(mtmp));
 		}
 		if (vismon) pline("%s looks completely healed.", Monnam(mtmp));
-		if (oseen) makeknown(POT_FULL_HEALING);
+		if (oseen) makeknown(otmp->otyp);
 		m_useup(mtmp, otmp);
 		return 2;
+	case MUSE_LIZARD_CORPSE:
+		/* not actually called for its unstoning effect */
+		mon_consume_unstone(mtmp, otmp, FALSE, FALSE);
+		return 2;
 	case 0: return 0; /* i.e. an exploded wand */
 	default: impossible("%s wanted to perform action %d?", Monnam(mtmp),
 			m.has_defense);
@@ -852,7 +902,8 @@ struct monst *mtmp;
 		case 2: return SCR_CREATE_MONSTER;
 		case 3: return POT_HEALING;
 		case 4: return POT_EXTRA_HEALING;
-		case 5: return POT_FULL_HEALING;
+		case 5: return (mtmp->data != &mons[PM_PESTILENCE]) ?
+				POT_FULL_HEALING : POT_SICKNESS;
 		case 7: if (is_floater(pm) || mtmp->isshk || mtmp->isgd
 						|| mtmp->ispriest
 									)
@@ -892,6 +943,7 @@ struct monst *mtmp;
 	register struct obj *obj;
 	boolean ranged_stuff = lined_up(mtmp);
 	boolean reflection_skip = (Reflecting && rn2(2));
+	struct obj *helmet = which_armor(mtmp, W_ARMH);
 
 	m.offensive = (struct obj *)0;
 	m.has_offense = 0;
@@ -963,7 +1015,7 @@ struct monst *mtmp;
 			m.has_offense = MUSE_POT_PARALYSIS;
 		}
 		nomore(MUSE_POT_BLINDNESS);
-		if(obj->otyp == POT_BLINDNESS) {
+		if(obj->otyp == POT_BLINDNESS && !attacktype(mtmp->data, AT_GAZE)) {
 			m.offensive = obj;
 			m.has_offense = MUSE_POT_BLINDNESS;
 		}
@@ -986,21 +1038,21 @@ struct monst *mtmp;
 		 * are in a 1 square radius are a subset of the locations that
 		 * are in wand range
 		 */
-		{
-		    struct obj *helmet = which_armor(mtmp, W_ARMH);
-
-		    nomore(MUSE_SCR_EARTH);
-		    if (obj->otyp == SCR_EARTH
-		       && ((helmet && is_metallic(helmet)) || mtmp->mconf || amorphous(mtmp->data) || passes_walls(mtmp->data) || noncorporeal(mtmp->data) || unsolid(mtmp->data) || !rn2(10))
+		nomore(MUSE_SCR_EARTH);
+		if (obj->otyp == SCR_EARTH
+		       && ((helmet && is_metallic(helmet)) ||
+				mtmp->mconf || amorphous(mtmp->data) ||
+				passes_walls(mtmp->data) ||
+				noncorporeal(mtmp->data) ||
+				unsolid(mtmp->data) || !rn2(10))
 		       && dist2(mtmp->mx,mtmp->my,mtmp->mux,mtmp->muy) <= 2
 		       && mtmp->mcansee && haseyes(mtmp->data)
 #ifdef REINCARNATION
 		       && !Is_rogue_level(&u.uz)
 #endif
 		       && (!In_endgame(&u.uz) || Is_earthlevel(&u.uz))) {
-			    m.offensive = obj;
-			    m.has_offense = MUSE_SCR_EARTH;
-		    }
+		    m.offensive = obj;
+		    m.has_offense = MUSE_SCR_EARTH;
 		}
 #if 0
 		nomore(MUSE_SCR_FIRE);
@@ -1034,17 +1086,22 @@ register struct obj *otmp;
 		reveal_invis = TRUE;
 		if (mtmp == &youmonst) {
 			if (zap_oseen) makeknown(WAN_STRIKING);
-			if (rnd(20) < 10 + u.uac) {
-				pline_The("wand hits you!");
-				tmp = d(2,12);
-				if(Half_spell_damage) tmp = (tmp+1) / 2;
-				losehp(tmp, "wand", KILLED_BY_AN);
+			if (Antimagic) {
+			    shieldeff(u.ux, u.uy);
+			    pline("Boing!");
+			} else if (rnd(20) < 10 + u.uac) {
+			    pline_The("wand hits you!");
+			    tmp = d(2,12);
+			    if(Half_spell_damage) tmp = (tmp+1) / 2;
+			    losehp(tmp, "wand", KILLED_BY_AN);
 			} else pline_The("wand misses you.");
 			stop_occupation();
 			nomul(0);
+		} else if (resists_magm(mtmp)) {
+			shieldeff(mtmp->mx, mtmp->my);
+			pline("Boing!");
 		} else if (rnd(20) < 10+find_mac(mtmp)) {
 			tmp = d(2,12);
-			if(Half_spell_damage) tmp = (tmp+1) / 2;
 			hit("wand", mtmp, exclam(tmp));
 			(void) resist(mtmp, otmp->oclass, tmp, TELL);
 			if (cansee(mtmp->mx, mtmp->my) && zap_oseen)
@@ -1067,7 +1124,7 @@ register struct obj *otmp;
 				pline("%s resists the magic!", Monnam(mtmp));
 			    mtmp->msleeping = 0;
 			    if(mtmp->m_ap_type) seemimic(mtmp);
-			} else
+			} else if (!tele_restrict(mtmp))
 			    rloc(mtmp);
 		}
 		break;
@@ -1303,7 +1360,7 @@ struct monst *mtmp;
 					if (canspotmon(mtmp2))
 					    pline("%s's %s does not protect %s.",
 						Monnam(mtmp2), xname(helmet),
-						him[pronoun_gender(mtmp2)]);
+						mhim(mtmp2));
 				    }
 				}
 	    	    	    	mtmp2->mhp -= mdmg;
@@ -1323,14 +1380,14 @@ struct monst *mtmp;
 		}
 		m_useup(mtmp, otmp);
 		/* Attack the player */
-		if (dist2(mmx, mmy, u.ux, u.uy) == 1 && !otmp->cursed) {
+		if (distmin(mmx, mmy, u.ux, u.uy) == 1 && !otmp->cursed) {
 		    int dmg;
 		    struct obj *otmp2;
 
 		    /* Okay, _you_ write this without repeating the code */
 		    otmp2 = mksobj(confused ? ROCK : BOULDER,
 				FALSE, FALSE);
-		    if (!otmp2) break;
+		    if (!otmp2) goto xxx_noobj;  /* Shouldn't happen */
 		    otmp2->quan = confused ? rn1(5,2) : 1;
 		    otmp2->owt = weight(otmp2);
 		    if (!amorphous(youmonst.data) &&
@@ -1357,9 +1414,10 @@ struct monst *mtmp;
 		    }
 		    if (dmg) losehp(dmg, "scroll of earth", KILLED_BY_AN);
 		}
+	    xxx_noobj:
 
+		return (mtmp->mhp <= 0) ? 1 : 2;
 	    }
-	    break;
 #if 0
 	case MUSE_SCR_FIRE:
 	      {
@@ -1387,6 +1445,7 @@ struct monst *mtmp;
 			if (Half_spell_damage) num = (num+1) / 2;
 			else losehp(num, "scroll of fire", KILLED_BY_AN);
 			for(mtmp2 = fmon; mtmp2; mtmp2 = mtmp2->nmon) {
+			   if(DEADMONSTER(mtmp2)) continue;
 			   if(mtmp == mtmp2) continue;
 			   if(dist2(mtmp2->mx,mtmp2->my,mtmp->mx,mtmp->my) < 3){
 				if (resists_fire(mtmp2)) continue;
@@ -1548,14 +1607,15 @@ struct monst *mtmp;
 		if(obj->otyp == WAN_MAKE_INVISIBLE && obj->spe > 0 &&
 		    !mtmp->minvis && !mtmp->invis_blkd &&
 		    (!mtmp->mpeaceful || See_invisible) &&
-		    (mdat != &mons[PM_MEDUSA] || mtmp->mcan)) {
+		    (!attacktype(mtmp->data, AT_GAZE) || mtmp->mcan)) {
 			m.misc = obj;
 			m.has_misc = MUSE_WAN_MAKE_INVISIBLE;
 		}
 		nomore(MUSE_POT_INVISIBILITY);
 		if(obj->otyp == POT_INVISIBILITY &&
 		    !mtmp->minvis && !mtmp->invis_blkd &&
-		    (!mtmp->mpeaceful || See_invisible)) {
+		    (!mtmp->mpeaceful || See_invisible) &&
+		    (!attacktype(mtmp->data, AT_GAZE) || mtmp->mcan)) {
 			m.misc = obj;
 			m.has_misc = MUSE_POT_INVISIBILITY;
 		}
@@ -1588,6 +1648,23 @@ struct monst *mtmp;
 #undef nomore
 }
 
+/* type of monster to polymorph into; defaults to one suitable for the
+   current level rather than the totally arbitrary choice of newcham() */
+static struct permonst *
+muse_newcham_mon(mon)
+struct monst *mon;
+{
+	struct obj *m_armr;
+
+	if ((m_armr = which_armor(mon, W_ARM)) != 0) {
+	    if (Is_dragon_scales(m_armr))
+		return Dragon_scales_to_pm(m_armr);
+	    else if (Is_dragon_mail(m_armr))
+		return Dragon_mail_to_pm(m_armr);
+	}
+	return rndmonst();
+}
+
 int
 use_misc(mtmp)
 struct monst *mtmp;
@@ -1669,7 +1746,7 @@ skipmsg:
 	case MUSE_WAN_SPEED_MONSTER:
 		mzapmsg(mtmp, otmp, TRUE);
 		otmp->spe--;
-		mon_adjust_speed(mtmp, 1);
+		mon_adjust_speed(mtmp, 1, otmp);
 		return 2;
 	case MUSE_POT_SPEED:
 		mquaffmsg(mtmp, otmp);
@@ -1677,22 +1754,19 @@ skipmsg:
 		   different methods of maintaining speed ratings:
 		   player's character becomes "very fast" temporarily;
 		   monster becomes "one stage faster" permanently */
-		if (vismon)
-		    pline("%s is suddenly moving faster.", Monnam(mtmp));
-		if (oseen) makeknown(POT_SPEED);
-		mon_adjust_speed(mtmp, 1);
+		mon_adjust_speed(mtmp, 1, otmp);
 		m_useup(mtmp, otmp);
 		return 2;
 	case MUSE_WAN_POLYMORPH:
 		mzapmsg(mtmp, otmp, TRUE);
 		otmp->spe--;
-		(void) newcham(mtmp, rndmonst());
+		(void) newcham(mtmp, muse_newcham_mon(mtmp), TRUE);
 		if (oseen) makeknown(WAN_POLYMORPH);
 		return 2;
 	case MUSE_POT_POLYMORPH:
 		mquaffmsg(mtmp, otmp);
 		if (vismon) pline("%s suddenly mutates!", Monnam(mtmp));
-		(void) newcham(mtmp, rndmonst());
+		(void) newcham(mtmp, muse_newcham_mon(mtmp), FALSE);
 		if (oseen) makeknown(POT_POLYMORPH);
 		m_useup(mtmp, otmp);
 		return 2;
@@ -1710,7 +1784,7 @@ skipmsg:
 		if (mtmp->wormno) worm_move(mtmp);
 		newsym(trapx, trapy);
 
-		(void) newcham(mtmp, (struct permonst *)0);
+		(void) newcham(mtmp, (struct permonst *)0, FALSE);
 		return 2;
 	case MUSE_BULLWHIP:
 		/* attempt to disarm hero */
@@ -1735,16 +1809,21 @@ skipmsg:
 		    }
 		    pline("%s wraps around %s you're wielding!",
 			  The_whip, the_weapon);
-		    if (obj->cursed) {
-			pline("%s is welded to your %s%c",
-			      (obj->quan == 1L) ? "It" : "They",
+		    if (welded(obj)) {
+			pline("%s welded to your %s%c",
+			      !is_plural(obj) ? "It is" : "They are",
 			      hand, !obj->bknown ? '!' : '.');
-			obj->bknown = 1;
+			/* obj->bknown = 1; */ /* welded() takes care of this */
 			where_to = 0;
 		    }
 		    if (!where_to) {
-			pline("The whip slips free.");  /* not `The_whip' */
+			pline_The("whip slips free.");  /* not `The_whip' */
 			return 1;
+		    } else if (where_to == 3 && hates_silver(mtmp->data) &&
+			    objects[obj->otyp].oc_material == SILVER) {
+			/* this monster won't want to catch a silver
+			   weapon; drop it at hero's feet instead */
+			where_to = 2;
 		    }
 		    freeinv(obj);
 		    uwepgone();
@@ -1752,10 +1831,6 @@ skipmsg:
 			case 1:		/* onto floor beneath mon */
 			    pline("%s yanks %s from your %s!", Monnam(mtmp),
 				  the_weapon, hand);
-			    if (obj->otyp == CRYSKNIFE && (!obj->oerodeproof || !rn2(10))) {
-			    	obj->otyp = WORM_TOOTH;
-			    	obj->oerodeproof = 0;
-			    }
 			    place_object(obj, mtmp->mx, mtmp->my);
 			    break;
 			case 2:		/* onto floor beneath you */
@@ -1766,7 +1841,7 @@ skipmsg:
 			case 3:		/* into mon's inventory */
 			    pline("%s snatches %s!", Monnam(mtmp),
 				  the_weapon);
-			    mpickobj(mtmp,obj);
+			    (void) mpickobj(mtmp,obj);
 			    break;
 		    }
 		    return 1;
@@ -1784,16 +1859,15 @@ STATIC_OVL void
 you_aggravate(mtmp)
 struct monst *mtmp;
 {
-	killer = ""; /* hack: prevent "it" in names */
 	pline("For some reason, %s presence is known to you.",
-		s_suffix(mon_nam(mtmp)));
+		s_suffix(noit_mon_nam(mtmp)));
 	cls();
 #ifdef CLIPPING
 	cliparound(mtmp->mx, mtmp->my);
 #endif
 	show_glyph(mtmp->mx, mtmp->my, mon_to_glyph(mtmp));
 	display_self();
-	You_feel("aggravated at %s.", mon_nam(mtmp));
+	You_feel("aggravated at %s.", noit_mon_nam(mtmp));
 	display_nhwindow(WIN_MAP, TRUE);
 	docrt();
 	if (unconscious()) {
@@ -1801,8 +1875,9 @@ struct monst *mtmp;
 		nomovemsg =
 		      "Aggravated, you are jolted into full consciousness.";
 	}
-	killer = 0;
 	newsym(mtmp->mx,mtmp->my);
+	if (!canspotmon(mtmp))
+	    map_invisible(mtmp->mx, mtmp->my);
 }
 
 int
@@ -1854,12 +1929,14 @@ struct obj *obj;
 	    return FALSE;
 
 	if (typ == WAN_MAKE_INVISIBLE || typ == POT_INVISIBILITY)
-	    return (boolean)(!mon->minvis && !mon->invis_blkd);
+	    return (boolean)(!mon->minvis && !mon->invis_blkd && !attacktype(mon->data, AT_GAZE));
 	if (typ == WAN_SPEED_MONSTER || typ == POT_SPEED)
 	    return (boolean)(mon->mspeed != MFAST);
 
 	switch (obj->oclass) {
 	case WAND_CLASS:
+	    if (obj->spe <= 0)
+		return FALSE;
 	    if (typ == WAN_DIGGING)
 		return (boolean)(!is_floater(mon->data));
 	    if (typ == WAN_POLYMORPH)
@@ -1879,9 +1956,10 @@ struct obj *obj;
 		    typ == POT_PARALYSIS ||
 		    typ == POT_SLEEPING ||
 		    typ == POT_ACID ||
-		    typ == POT_BLINDNESS ||
 		    typ == POT_CONFUSION)
 		return TRUE;
+	    if (typ == POT_BLINDNESS && !attacktype(mon->data, AT_GAZE))
+		return TRUE;
 	    break;
 	case SCROLL_CLASS:
 	    if (typ == SCR_TELEPORTATION || typ == SCR_CREATE_MONSTER
@@ -1900,7 +1978,7 @@ struct obj *obj;
 	    if (typ == UNICORN_HORN)
 		return (boolean)(!obj->cursed && !is_unicorn(mon->data));
 	    if (typ == FROST_HORN || typ == FIRE_HORN)
-		return TRUE;
+		return (obj->spe > 0);
 	    break;
 	case FOOD_CLASS:
 	    if (typ == CORPSE)
@@ -1908,7 +1986,8 @@ struct obj *obj;
 				    touch_petrifies(&mons[obj->corpsenm])) ||
 				(!resists_ston(mon) &&
 				    (obj->corpsenm == PM_LIZARD ||
-					acidic(&mons[obj->corpsenm]))));
+					(acidic(&mons[obj->corpsenm]) &&
+					 obj->corpsenm != PM_GREEN_SLIME))));
 	    if (typ == EGG)
 		return (boolean)(touch_petrifies(&mons[obj->corpsenm]));
 	    break;
@@ -1944,7 +2023,8 @@ const char *str;
 	    if (str)
 		pline(str, s_suffix(mon_nam(mon)), "armor");
 	    return TRUE;
-	} else if (mon->data == &mons[PM_SILVER_DRAGON]) {
+	} else if (mon->data == &mons[PM_SILVER_DRAGON] ||
+		mon->data == &mons[PM_CHROMATIC_DRAGON]) {
 	    /* Silver dragons only reflect when mature; babies do not */
 	    if (str)
 		pline(str, s_suffix(mon_nam(mon)), "scales");
@@ -2002,51 +2082,67 @@ boolean by_you;
 	for(obj = mon->minvent; obj; obj = obj->nobj) {
 	    /* Monsters can also use potions of acid */
 	    if ((obj->otyp == POT_ACID) || (obj->otyp == CORPSE &&
-	    		(obj->corpsenm == PM_LIZARD || acidic(&mons[obj->corpsenm])))) {
-		int nutrit = dog_nutrition(mon, obj); /* also sets meating */
-
-		if (canseemon(mon)) {
-		    long save_quan = obj->quan;
-
-		    obj->quan = 1L;
-		    pline("%s %ss %s.", Monnam(mon),
-		    		(obj->otyp == POT_ACID) ? "quaff" : "eat",
-		    		distant_name(obj,doname));
-		    obj->quan = save_quan;
-		} else if (flags.soundok)
-		    You("hear %s.", (obj->otyp == POT_ACID) ? "drinking" : "chewing");
-		m_useup(mon, obj);
-		if (((obj->otyp == POT_ACID) || acidic(&mons[obj->corpsenm])) &&
-				!resists_acid(mon)) {
-		    mon->mhp -= rnd(15);
-		    pline("%s has a very bad case of stomach acid.",
-			Monnam(mon));
-		}
-		if (mon->mhp <= 0) {
-		    pline("%s dies!", Monnam(mon));
-		    if (by_you) xkilled(mon, 0);
-		    else mondead(mon);
-		    return TRUE;
-		}
-		if (canseemon(mon)) {
-		    if (Hallucination)
-		pline("What a pity - %s just ruined a future piece of art!",
-			mon_nam(mon));
-		    else
-			pline("%s seems limber!", Monnam(mon));
-		}
-		if (mon->mtame && !mon->isminion) {
-		    struct edog *edog = EDOG(mon);
-
-		    if (edog->hungrytime < moves) edog->hungrytime = moves;
-		    edog->hungrytime += nutrit;
-		    mon->mconf = 0;
-		}
-		mon->mlstmv = monstermoves; /* it takes a turn */
+	    		(obj->corpsenm == PM_LIZARD || (acidic(&mons[obj->corpsenm]) && obj->corpsenm != PM_GREEN_SLIME)))) {
+		mon_consume_unstone(mon, obj, by_you, TRUE);
 		return TRUE;
 	    }
 	}
 	return FALSE;
 }
 
+STATIC_OVL void
+mon_consume_unstone(mon, obj, by_you, stoning)
+struct monst *mon;
+struct obj *obj;
+boolean by_you;
+boolean stoning;
+{
+    int nutrit = (obj->otyp == CORPSE) ? dog_nutrition(mon, obj) : 0;
+    /* also sets meating */
+
+    if (canseemon(mon)) {
+	long save_quan = obj->quan;
+
+	obj->quan = 1L;
+	pline("%s %ss %s.", Monnam(mon),
+		    (obj->otyp == POT_ACID) ? "quaff" : "eat",
+		    distant_name(obj,doname));
+	obj->quan = save_quan;
+    } else if (flags.soundok)
+	You_hear("%s.", (obj->otyp == POT_ACID) ? "drinking" : "chewing");
+    m_useup(mon, obj);
+    if (((obj->otyp == POT_ACID) || acidic(&mons[obj->corpsenm])) &&
+		    !resists_acid(mon)) {
+	mon->mhp -= rnd(15);
+	pline("%s has a very bad case of stomach acid.",
+	    Monnam(mon));
+    }
+    if (mon->mhp <= 0) {
+	pline("%s dies!", Monnam(mon));
+	if (by_you) xkilled(mon, 0);
+	else mondead(mon);
+	return;
+    }
+    if (stoning && canseemon(mon)) {
+	if (Hallucination)
+    pline("What a pity - %s just ruined a future piece of art!",
+	    mon_nam(mon));
+	else
+	    pline("%s seems limber!", Monnam(mon));
+    }
+    if (obj->otyp == CORPSE && obj->corpsenm == PM_LIZARD && mon->mconf) {
+	mon->mconf = 0;
+	if (canseemon(mon))
+	    pline("%s seems steadier now.", Monnam(mon));
+    }
+    if (mon->mtame && !mon->isminion && nutrit > 0) {
+	struct edog *edog = EDOG(mon);
+
+	if (edog->hungrytime < monstermoves) edog->hungrytime = monstermoves;
+	edog->hungrytime += nutrit;
+	mon->mconf = 0;
+    }
+    mon->mlstmv = monstermoves; /* it takes a turn */
+}
+
 /*muse.c*/
