@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)music.c	3.3	1999/08/19	*/
+/*	SCCS Id: @(#)music.c	3.4	2001/12/03	*/
 /*	Copyright (c) 1989 by Jean-Christophe Collet */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -65,6 +65,7 @@ int distance;
 	register int distm;
 
 	while(mtmp) {
+	    if (!DEADMONSTER(mtmp)) {
 		distm = distu(mtmp->mx, mtmp->my);
 		if (distm < distance) {
 		    mtmp->msleeping = 0;
@@ -72,10 +73,11 @@ int distance;
 		    mtmp->mfrozen = 0;
 		    /* May scare some monsters */
 		    if (distm < distance/3 &&
-			    !resist(mtmp, SCROLL_CLASS, 0, NOTELL))
-			mtmp->mflee = 1;
+			    !resist(mtmp, TOOL_CLASS, 0, NOTELL))
+			monflee(mtmp, 0, FALSE, TRUE);
 		}
-		mtmp = mtmp->nmon;
+	    }
+	    mtmp = mtmp->nmon;
 	}
 }
 
@@ -90,8 +92,8 @@ int distance;
 	register struct monst *mtmp = fmon;
 
 	while(mtmp) {
-		if (distu(mtmp->mx, mtmp->my) < distance &&
-			sleep_monst(mtmp, d(10,10), WAND_CLASS)) {
+		if (!DEADMONSTER(mtmp) && distu(mtmp->mx, mtmp->my) < distance &&
+			sleep_monst(mtmp, d(10,10), TOOL_CLASS)) {
 		    mtmp->msleeping = 1; /* 10d10 turns + wake_nearby to rouse */
 		    slept_monst(mtmp);
 		}
@@ -111,17 +113,18 @@ int distance;
 	int could_see_mon, was_peaceful;
 
 	while (mtmp) {
-	    if (mtmp->data->mlet == S_SNAKE && mtmp->mcanmove &&
+	    if (!DEADMONSTER(mtmp) && mtmp->data->mlet == S_SNAKE && mtmp->mcanmove &&
 		    distu(mtmp->mx, mtmp->my) < distance) {
 		was_peaceful = mtmp->mpeaceful;
 		mtmp->mpeaceful = 1;
+		mtmp->mavenge = 0;
 		could_see_mon = canseemon(mtmp);
 		mtmp->mundetected = 0;
 		newsym(mtmp->mx, mtmp->my);
 		if (canseemon(mtmp)) {
 		    if (!could_see_mon)
 			You("notice %s, swaying with the music.",
-			    an(mon_nam(mtmp)));
+			    a_monnam(mtmp));
 		    else
 			pline("%s freezes, then sways with the music%s.",
 			      Monnam(mtmp),
@@ -143,10 +146,11 @@ int distance;
 	register struct monst *mtmp = fmon;
 
 	while (mtmp) {
-	    if (mtmp->data->mlet == S_NYMPH && mtmp->mcanmove &&
+	    if (!DEADMONSTER(mtmp) && mtmp->data->mlet == S_NYMPH && mtmp->mcanmove &&
 		    distu(mtmp->mx, mtmp->my) < distance) {
 		mtmp->msleeping = 0;
 		mtmp->mpeaceful = 1;
+		mtmp->mavenge = 0;
 		if (canseemon(mtmp))
 		    pline(
 		     "%s listens cheerfully to the music, then seems quieter.",
@@ -164,7 +168,8 @@ awaken_soldiers()
 	register struct monst *mtmp = fmon;
 
 	while(mtmp) {
-	    if (is_mercenary(mtmp->data) && mtmp->data != &mons[PM_GUARD]) {
+	    if (!DEADMONSTER(mtmp) &&
+			is_mercenary(mtmp->data) && mtmp->data != &mons[PM_GUARD]) {
 		mtmp->mpeaceful = mtmp->msleeping = mtmp->mfrozen = 0;
 		mtmp->mcanmove = 1;
 		if (canseemon(mtmp))
@@ -176,20 +181,29 @@ awaken_soldiers()
 	}
 }
 
-/* Charm monsters in range.  Note that they may resist the spell. */
+/* Charm monsters in range.  Note that they may resist the spell.
+ * If swallowed, range is reduced to 0.
+ */
 
 STATIC_OVL void
 charm_monsters(distance)
 int distance;
 {
-	register struct monst *mtmp = fmon, *mtmp2;
+	struct monst *mtmp, *mtmp2;
 
-	while(mtmp) {
+	if (u.uswallow) {
+	    if (!resist(u.ustuck, TOOL_CLASS, 0, NOTELL))
+		(void) tamedog(u.ustuck, (struct obj *) 0);
+	} else {
+	    for (mtmp = fmon; mtmp; mtmp = mtmp2) {
 		mtmp2 = mtmp->nmon;
-		if (distu(mtmp->mx, mtmp->my) <= distance)
-		    if(!resist(mtmp, SCROLL_CLASS, 0, NOTELL))
+		if (DEADMONSTER(mtmp)) continue;
+
+		if (distu(mtmp->mx, mtmp->my) <= distance) {
+		    if (!resist(mtmp, TOOL_CLASS, 0, NOTELL))
 			(void) tamedog(mtmp, (struct obj *) 0);
-		mtmp = mtmp2;
+		}
+	    }
 	}
 
 }
@@ -251,7 +265,7 @@ int force;
 			goto do_pit;
 		  case GRAVE :
 			if (cansee(x,y))
-				pline("The headstone topples into a chasm.");
+				pline_The("headstone topples into a chasm.");
 			goto do_pit;
 		  case THRONE :
 			if (cansee(x,y))
@@ -295,7 +309,8 @@ do_pit:		    chasm = maketrap(x,y,PIT);
 					pline("It is destroyed!");
 				    else {
 					You("destroy %s!", mtmp->mtame ?
-					    x_monnam(mtmp, 0, "poor", 0) :
+					    x_monnam(mtmp, ARTICLE_THE, "poor",
+				mtmp->mnamelth ? SUPPRESS_SADDLE : 0, FALSE):
 					    mon_nam(mtmp));
 				    }
 				    xkilled(mtmp,0);
@@ -323,6 +338,7 @@ do_pit:		    chasm = maketrap(x,y,PIT);
 		    if (*in_rooms(x, y, SHOPBASE))
 			add_damage(x, y, 0L);
 		    levl[x][y].doormask = D_NODOOR;
+		    unblock_point(x,y);
 		    newsym(x,y);
 		    break;
 	    }
@@ -377,8 +393,7 @@ struct obj *instr;
 	    } /* else FALLTHRU */
 	case WOODEN_FLUTE:		/* May charm snakes */
 	    do_spec &= (rn2(ACURR(A_DEX)) + u.ulevel > 25);
-	    pline("%s %s.", The(xname(instr)),
-		  do_spec ? "trills" : "toots");
+	    pline("%s.", Tobjnam(instr, do_spec ? "trill" : "toot"));
 	    if (do_spec) charm_snakes(u.ulevel * 3);
 	    exercise(A_DEX, TRUE);
 	    break;
@@ -388,14 +403,14 @@ struct obj *instr;
 		check_unpaid(instr);
 		instr->spe--;
 		if (!getdir((char *)0)) {
-		    pline("%s vibrates.", The(xname(instr)));
+		    pline("%s.", Tobjnam(instr, "vibrate"));
 		    break;
 		} else if (!u.dx && !u.dy && !u.dz) {
-		    if ((damage = zapyourself(instr, TRUE)) != 0)
-			losehp(damage,
-			       self_pronoun("using a magical horn on %sself",
-					    "him"),
-			       NO_KILLER_PREFIX);
+		    if ((damage = zapyourself(instr, TRUE)) != 0) {
+			char buf[BUFSZ];
+			Sprintf(buf, "using a magical horn on %sself", uhim());
+			losehp(damage, buf, NO_KILLER_PREFIX);
+		    }
 		} else {
 		    buzz((instr->otyp == FROST_HORN) ? AD_COLD-1 : AD_FIRE-1,
 			 rn1(6,6), u.ux, u.uy, u.dx, u.dy);
@@ -417,8 +432,7 @@ struct obj *instr;
 	    if (do_spec && instr->spe > 0) {
 		check_unpaid(instr);
 		instr->spe--;
-		pline("%s produces very attractive music.",
-		      The(xname(instr)));
+		pline("%s very attractive music.", Tobjnam(instr, "produce"));
 		charm_monsters((u.ulevel - 1) / 3 + 1);
 		exercise(A_DEX, TRUE);
 		break;
@@ -607,7 +621,7 @@ atconsole()
      * difficult given the presence of virtual consoles and other modern
      * UNIX impedimenta...
      */
-    char	*termtype = getenv("TERM");
+    char	*termtype = nh_getenv("TERM");
 
      return(!strcmp(termtype, "AT386") || !strcmp(termtype, "xterm"));
 }
