@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)polyself.c	3.3	1999/08/16	*/
+/*	SCCS Id: @(#)polyself.c	3.4	2002/01/15	*/
 /*	Copyright (C) 1987, 1988, 1989 by Ken Arromdee */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -16,7 +16,6 @@
 STATIC_DCL void FDECL(polyman, (const char *,const char *));
 STATIC_DCL void NDECL(break_armor);
 STATIC_DCL void FDECL(drop_weapon,(int));
-STATIC_DCL void NDECL(skinback);
 STATIC_DCL void NDECL(uunstick);
 STATIC_DCL int FDECL(armor_to_dragon,(int));
 STATIC_DCL void NDECL(newman);
@@ -48,7 +47,7 @@ const char *fmt, *arg;
 
 	u.mh = u.mhmax = 0;
 	u.mtimedone = 0;
-	skinback();
+	skinback(FALSE);
 	u.uundetected = 0;
 
 	if (sticky) uunstick();
@@ -77,6 +76,7 @@ const char *fmt, *arg;
 			(urace.femalenum != NON_PM &&
 			(mvitals[urace.femalenum].mvflags & G_GENOD))) {
 	    /* intervening activity might have clobbered genocide info */
+	    killer = delayed_killer;
 	    if (!killer || !strstri(killer, "genocid")) {
 		killer_format = KILLED_BY;
 		killer = "self-genocide";
@@ -90,7 +90,7 @@ const char *fmt, *arg;
 
 	if(!Levitation && !u.ustuck &&
 	   (is_pool(u.ux,u.uy) || is_lava(u.ux,u.uy)))
-		spoteffects();
+		spoteffects(TRUE);
 
 	see_monsters();
 }
@@ -102,19 +102,25 @@ change_sex()
 	   swap unintentionally makes `Upolyd' appear to be true */
 	boolean already_polyd = (boolean) Upolyd;
 
-	flags.female = !flags.female;
+	/* Some monsters are always of one sex and their sex can't be changed */
+	/* succubi/incubi can change, but are handled below */
+	/* !already_polyd check necessary because is_male() and is_female()
+           are true if the player is a priest/priestess */
+	if (!already_polyd || (!is_male(youmonst.data) && !is_female(youmonst.data) && !is_neuter(youmonst.data)))
+	    flags.female = !flags.female;
 	if (already_polyd)	/* poly'd: also change saved sex */
-		u.mfemale = !u.mfemale;
+	    u.mfemale = !u.mfemale;
 	max_rank_sz();		/* [this appears to be superfluous] */
-	if (flags.female && urole.name.f)
+	if ((already_polyd ? u.mfemale : flags.female) && urole.name.f)
 	    Strcpy(pl_character, urole.name.f);
 	else
 	    Strcpy(pl_character, urole.name.m);
-	u.umonster = (flags.female && urole.femalenum != NON_PM) ?
+	u.umonster = ((already_polyd ? u.mfemale : flags.female) && urole.femalenum != NON_PM) ?
 			urole.femalenum : urole.malenum;
 	if (!already_polyd) {
 	    u.umonnum = u.umonster;
 	} else if (u.umonnum == PM_SUCCUBUS || u.umonnum == PM_INCUBUS) {
+	    flags.female = !flags.female;
 	    /* change monster type to match new sex */
 	    u.umonnum = (u.umonnum == PM_SUCCUBUS) ? PM_INCUBUS : PM_SUCCUBUS;
 	    set_uasmon();
@@ -124,24 +130,34 @@ change_sex()
 STATIC_OVL void
 newman()
 {
-	int tmp, tmp2;
+	int tmp, oldlvl;
+
+	tmp = u.uhpmax;
+	oldlvl = u.ulevel;
+	u.ulevel = u.ulevel + rn1(5, -2);
+	if (u.ulevel > 127 || u.ulevel < 1) { /* level went below 0? */
+	    u.ulevel = oldlvl; /* restore old level in case they lifesave */
+	    goto dead;
+	}
+	if (u.ulevel > MAXULEV) u.ulevel = MAXULEV;
+	/* If your level goes down, your peak level goes down by
+	   the same amount so that you can't simply use blessed
+	   full healing to undo the decrease.  But if your level
+	   goes up, your peak level does *not* undergo the same
+	   adjustment; you might end up losing out on the chance
+	   to regain some levels previously lost to other causes. */
+	if (u.ulevel < oldlvl) u.ulevelmax -= (oldlvl - u.ulevel);
+	if (u.ulevelmax < u.ulevel) u.ulevelmax = u.ulevel;
 
 	if (!rn2(10)) change_sex();
 
-	tmp = u.uhpmax;
-	tmp2 = u.ulevel;
-	u.ulevel = u.ulevel + rn1(5, -2);
-	if (u.ulevel > 127 || u.ulevel < 1) u.ulevel = 1;
-	if (u.ulevel > MAXULEV) u.ulevel = MAXULEV;
-	if (u.ulevelmax < u.ulevel) u.ulevelmax = u.ulevel;
-
-	adjabil(tmp2, (int)u.ulevel);
+	adjabil(oldlvl, (int)u.ulevel);
 	reset_rndmonst(NON_PM);	/* new monster generation criteria */
 
 	/* random experience points for the new experience level */
 	u.uexp = rndexp();
 
-	/* u.uhpmax * u.ulevel / tmp2: proportionate hit points to new level
+	/* u.uhpmax * u.ulevel / oldlvl: proportionate hit points to new level
 	 * -10 and +10: don't apply proportionate HP to 10 of a starting
 	 *   character's hit points (since a starting character's hit points
 	 *   are not on the same scale with hit points obtained through level
@@ -149,7 +165,7 @@ newman()
 	 * 9 - rn2(19): random change of -9 to +9 hit points
 	 */
 #ifndef LINT
-	u.uhpmax = ((u.uhpmax - 10) * (long)u.ulevel / tmp2 + 10) +
+	u.uhpmax = ((u.uhpmax - 10) * (long)u.ulevel / oldlvl + 10) +
 		(9 - rn2(19));
 #endif
 
@@ -161,7 +177,7 @@ newman()
 
 	tmp = u.uenmax;
 #ifndef LINT
-	u.uenmax = u.uenmax * (long)u.ulevel / tmp2 + 9 - rn2(19);
+	u.uenmax = u.uenmax * (long)u.ulevel / oldlvl + 9 - rn2(19);
 #endif
 	if (u.uenmax < 0) u.uenmax = 0;
 #ifndef LINT
@@ -170,30 +186,39 @@ newman()
 
 	redist_attr();
 	u.uhunger = rn1(500,500);
-	newuhs(FALSE);
 	if (Sick) make_sick(0L, (char *) 0, FALSE, SICK_ALL);
 	Stoned = 0;
+	delayed_killer = 0;
 	if (u.uhp <= 0 || u.uhpmax <= 0) {
 		if (Polymorph_control) {
 		    if (u.uhp <= 0) u.uhp = 1;
 		    if (u.uhpmax <= 0) u.uhpmax = 1;
 		} else {
+dead: /* we come directly here if their experience level went to 0 or less */
 		    Your("new form doesn't seem healthy enough to survive.");
 		    killer_format = KILLED_BY_AN;
 		    killer="unsuccessful polymorph";
 		    done(DIED);
+		    newuhs(FALSE);
+		    return; /* lifesaved */
 		}
 	}
+	newuhs(FALSE);
 	polyman("feel like a new %s!",
 		(flags.female && urace.individual.f) ? urace.individual.f :
 		(urace.individual.m) ? urace.individual.m : urace.noun);
+	if (Slimed) {
+		Your("body transforms, but there is still slime on you.");
+		Slimed = 10L;
+	}
 	flags.botl = 1;
 	see_monsters();
 	(void) encumber_msg();
 }
 
 void
-polyself()
+polyself(forcecontrol)
+boolean forcecontrol;     
 {
 	char buf[BUFSZ];
 	int old_light, new_light;
@@ -205,7 +230,7 @@ polyself()
 	boolean iswere = (u.ulycn >= LOW_PM || is_were(youmonst.data));
 	boolean isvamp = (youmonst.data->mlet == S_VAMPIRE || u.umonnum == PM_VAMPIRE_BAT);
 
-	if(!Polymorph_control && !draconian && !iswere && !isvamp) {
+        if(!Polymorph_control && !forcecontrol && !draconian && !iswere && !isvamp) {
 	    if (rn2(20) > ACURR(A_CON)) {
 		You(shudder_for_moment);
 		losehp(rnd(30), "system shock", KILLED_BY_AN);
@@ -215,7 +240,7 @@ polyself()
 	}
 	old_light = Upolyd ? emits_light(youmonst.data) : 0;
 
-	if (Polymorph_control) {
+	if (Polymorph_control || forcecontrol) {
 		do {
 			getlin("Become what kind of monster? [type the name]",
 				buf);
@@ -258,8 +283,11 @@ polyself()
 			else
 				mntmp = PM_VAMPIRE;
 		}
-		if (polymon(mntmp))
-			goto made_change;
+		/* if polymon fails, "you feel" message has been given
+		   so don't follow up with another polymon or newman */
+		if (mntmp == PM_HUMAN) newman();	/* werecritter */
+		else (void) polymon(mntmp);
+		goto made_change;    /* maybe not, but this is right anyway */
 	}
 
 	if (mntmp < LOW_PM) {
@@ -338,9 +366,10 @@ int	mntmp;
 	}
 	if (dochange) {
 		flags.female = !flags.female;
-		You("%s %s %s!",
+		You("%s %s%s!",
 		    (u.umonnum != mntmp) ? "turn into a" : "feel like a new",
-		    flags.female ? "female" : "male",
+		    (is_male(&mons[mntmp]) || is_female(&mons[mntmp])) ? "" :
+			flags.female ? "female " : "male ",
 		    mons[mntmp].mname);
 	} else {
 		if (u.umonnum != mntmp)
@@ -353,6 +382,7 @@ int	mntmp;
 		You("turn to stone!");
 		mntmp = PM_STONE_GOLEM;
 		Stoned = 0;
+		delayed_killer = 0;
 	}
 
 	u.mtimedone = rn1(500, 500);
@@ -366,12 +396,24 @@ int	mntmp;
 
 	if (Stone_resistance && Stoned) { /* parnes@eniac.seas.upenn.edu */
 		Stoned = 0;
+		delayed_killer = 0;
 		You("no longer seem to be petrifying.");
 	}
 	if (Sick_resistance && Sick) {
 		make_sick(0L, (char *) 0, FALSE, SICK_ALL);
 		You("no longer feel sick.");
 	}
+	if (Slimed) {
+	    if (mntmp == PM_FIRE_VORTEX || mntmp == PM_FIRE_ELEMENTAL || mntmp == PM_SALAMANDER) {
+		pline_The("slime burns away!");
+		Slimed = 0L;
+		flags.botl = 1;
+	    } else if (mntmp == PM_GREEN_SLIME) {
+		/* do it silently */
+		Slimed = 0L;
+		flags.botl = 1;
+	    }
+	}
 	if (nohands(youmonst.data)) Glib = 0;
 
 	/*
@@ -404,7 +446,7 @@ int	mntmp;
 	}
 
 	if (uskin && mntmp != armor_to_dragon(uskin->otyp))
-		skinback();
+		skinback(FALSE);
 	break_armor();
 	drop_weapon(1);
 	if (hides_under(youmonst.data))
@@ -446,8 +488,8 @@ int	mntmp;
 		pline(use_thec,monsterc,"spit venom");
 	    if (youmonst.data->mlet == S_NYMPH)
 		pline(use_thec,monsterc,"remove an iron ball");
-	    if (youmonst.data->mlet == S_UMBER)
-		pline(use_thec,monsterc,"confuse monsters");
+	    if (attacktype(youmonst.data, AT_GAZE))
+		pline(use_thec,monsterc,"gaze at monsters");
 	    if (is_hider(youmonst.data))
 		pline(use_thec,monsterc,"hide");
 	    if (is_were(youmonst.data))
@@ -475,7 +517,7 @@ int	mntmp;
 	if((!Levitation && !u.ustuck && !Flying &&
 	    (is_pool(u.ux,u.uy) || is_lava(u.ux,u.uy))) ||
 	   (Underwater && !Swimming))
-	    spoteffects();
+	    spoteffects(TRUE);
 	if (Passes_walls && u.utrap && u.utraptype == TT_INFLOOR) {
 	    u.utrap = 0;
 	    pline_The("rock seems to no longer trap you.");
@@ -497,7 +539,7 @@ int	mntmp;
 	    /* probably should burn webs too if PM_FIRE_ELEMENTAL */
 	    u.utrap = 0;
 	}
-	if (youmonst.data->mlet == S_SPIDER && u.utrap && u.utraptype == TT_WEB) {
+	if (webmaker(youmonst.data) && u.utrap && u.utraptype == TT_WEB) {
 	    You("orient yourself on the web.");
 	    u.utrap = 0;
 	}
@@ -525,11 +567,11 @@ break_armor()
 	}
 	if ((otmp = uarmc) != 0) {
 	    if(otmp->oartifact) {
-		Your("cloak falls off!");
+		Your("%s falls off!", cloak_simple_name(otmp));
 		(void) Cloak_off();
 		dropx(otmp);
 	    } else {
-		Your("cloak tears apart!");
+		Your("%s tears apart!", cloak_simple_name(otmp));
 		(void) Cloak_off();
 		useup(otmp);
 	    }
@@ -549,8 +591,8 @@ break_armor()
 	}
 	if ((otmp = uarmc) != 0) {
 		if (is_whirly(youmonst.data))
-			Your("cloak falls, unsupported!");
-		else You("shrink out of your cloak!");
+			Your("%s falls, unsupported!", cloak_simple_name(otmp));
+		else You("shrink out of your %s!", cloak_simple_name(otmp));
 		(void) Cloak_off();
 		dropx(otmp);
 	}
@@ -604,6 +646,8 @@ drop_weapon(alone)
 int alone;
 {
     struct obj *otmp;
+    struct obj *otmp2;
+
     if ((otmp = uwep) != 0) {
 	/* !alone check below is currently superfluous but in the
 	 * future it might not be so if there are monsters which cannot
@@ -614,10 +658,16 @@ int alone;
 
 	    if (alone) You("find you must drop your weapon%s!",
 			   	u.twoweap ? "s" : "");
+	    otmp2 = u.twoweap ? uswapwep : 0;
 	    uwepgone();
 	    if (!wep->cursed || wep->otyp != LOADSTONE)
 		dropx(otmp);
-		untwoweapon();
+	    if (otmp2 != 0) {
+		uswapwepgone();
+		if (!otmp2->cursed || otmp2->otyp != LOADSTONE)
+		    dropx(otmp2);
+	    }
+	    untwoweapon();
 	}
     }
 }
@@ -690,7 +740,7 @@ dospit()
 	otmp = mksobj(u.umonnum==PM_COBRA ? BLINDING_VENOM : ACID_VENOM,
 			TRUE, FALSE);
 	otmp->spe = 1; /* to indicate it's yours */
-	throwit(otmp);
+	throwit(otmp, 0L, FALSE);
 	return(1);
 }
 
@@ -782,9 +832,16 @@ dospinweb()
 			deltrap(ttmp);
 			if (Invisible) newsym(u.ux, u.uy);
 			return 1;
+		case ROLLING_BOULDER_TRAP:
+			You("spin a web, jamming the trigger.");
+			deltrap(ttmp);
+			if (Invisible) newsym(u.ux, u.uy);
+			return(1);
 		case ARROW_TRAP:
 		case DART_TRAP:
 		case BEAR_TRAP:
+		case ROCKTRAP:
+		case FIRE_TRAP:
 		case LANDMINE:
 		case SLP_GAS_TRAP:
 		case RUST_TRAP:
@@ -792,7 +849,7 @@ dospinweb()
 		case ANTI_MAGIC:
 		case POLY_TRAP:
 			You("have triggered a trap!");
-			dotrap(ttmp);
+			dotrap(ttmp, 0);
 			return(1);
 		default:
 			impossible("Webbing over trap type %d?", ttmp->ttyp);
@@ -825,15 +882,29 @@ dosummon()
 }
 
 int
-doconfuse()
+dogaze()
 {
 	register struct monst *mtmp;
 	int looked = 0;
 	char qbuf[QBUFSZ];
+	int i;
+	uchar adtyp = 0;
+
+	for (i = 0; i < NATTK; i++) {
+	    if(youmonst.data->mattk[i].aatyp == AT_GAZE) {
+		adtyp = youmonst.data->mattk[i].adtyp;
+		break;
+	    }
+	}
+	if (adtyp != AD_CONF && adtyp != AD_FIRE) {
+	    impossible("gaze attack %d?", adtyp);
+	    return 0;
+	}
+
 
 	if (Blind) {
-		You_cant("see anything to gaze at.");
-		return 0;
+	    You_cant("see anything to gaze at.");
+	    return 0;
 	}
 	if (u.uen < 15) {
 	    You("lack the energy to use your special gaze!");
@@ -843,7 +914,8 @@ doconfuse()
 	flags.botl = 1;
 
 	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
-	    if (canseemon(mtmp)) {
+	    if (DEADMONSTER(mtmp)) continue;
+	    if (canseemon(mtmp) && couldsee(mtmp->mx, mtmp->my)) {
 		looked++;
 		if (Invis && !perceives(mtmp->data))
 		    pline("%s seems not to notice your gaze.", Monnam(mtmp));
@@ -859,7 +931,9 @@ doconfuse()
 		} else {
 		    if (flags.confirm && mtmp->mpeaceful && !Confusion
 							&& !Hallucination) {
-			Sprintf(qbuf, "Really confuse %s?", mon_nam(mtmp));
+			Sprintf(qbuf, "Really %s %s?",
+			    (adtyp == AD_CONF) ? "confuse" : "attack",
+			    mon_nam(mtmp));
 			if (yn(qbuf) != 'y') continue;
 			setmangry(mtmp);
 		    }
@@ -868,15 +942,37 @@ doconfuse()
 			looked--;
 			continue;
 		    }
-		    if (!mon_reflects(mtmp,"Your gaze is reflected by %s %s.")){
+		    /* No reflection check for consistency with when a monster
+		     * gazes at *you*--only medusa gaze gets reflected then.
+		     */
+		    if (adtyp == AD_CONF) {
 			if (!mtmp->mconf)
 			    Your("gaze confuses %s!", mon_nam(mtmp));
 			else
 			    pline("%s is getting more and more confused.",
-							    Monnam(mtmp));
+							Monnam(mtmp));
 			mtmp->mconf = 1;
+		    } else if (adtyp == AD_FIRE) {
+			int dmg = d(2,6);
+			You("attack %s with a fiery gaze!", mon_nam(mtmp));
+			if (resists_fire(mtmp)) {
+			    pline_The("fire doesn't burn %s!", mon_nam(mtmp));
+			    dmg = 0;
+			}
+			if((int) u.ulevel > rn2(20))
+			    (void) destroy_mitem(mtmp, SCROLL_CLASS, AD_FIRE);
+			if((int) u.ulevel > rn2(20))
+			    (void) destroy_mitem(mtmp, POTION_CLASS, AD_FIRE);
+			if((int) u.ulevel > rn2(25))
+			    (void) destroy_mitem(mtmp, SPBOOK_CLASS, AD_FIRE);
+			if (dmg && !DEADMONSTER(mtmp)) mtmp->mhp -= dmg;
+			if (mtmp->mhp <= 0) killed(mtmp);
 		    }
-		    if ((mtmp->data==&mons[PM_FLOATING_EYE]) && !mtmp->mcan) {
+		    /* For consistency with passive() in uhitm.c, this only
+		     * affects you if the monster is still alive.
+		     */
+		    if (!DEADMONSTER(mtmp) &&
+			  (mtmp->data==&mons[PM_FLOATING_EYE]) && !mtmp->mcan) {
 			if (!Free_action) {
 			    You("are frozen by %s gaze!",
 					     s_suffix(mon_nam(mtmp)));
@@ -889,7 +985,13 @@ doconfuse()
 			    You("stiffen momentarily under %s gaze.",
 				    s_suffix(mon_nam(mtmp)));
 		    }
-		    if ((mtmp->data == &mons[PM_MEDUSA]) && !mtmp->mcan) {
+		    /* Technically this one shouldn't affect you at all because
+		     * the Medusa gaze is an active monster attack that only
+		     * works on the monster's turn, but for it to *not* have an
+		     * effect would be too weird.
+		     */
+		    if (!DEADMONSTER(mtmp) &&
+			    (mtmp->data == &mons[PM_MEDUSA]) && !mtmp->mcan) {
 			pline(
 			 "Gazing at the awake %s is not a very good idea.",
 			    l_monnam(mtmp));
@@ -944,6 +1046,8 @@ domindblast()
 		int u_sen;
 
 		nmon = mtmp->nmon;
+		if (DEADMONSTER(mtmp))
+			continue;
 		if (distu(mtmp->mx, mtmp->my) > BOLT_LIM * BOLT_LIM)
 			continue;
 		if(mtmp->mpeaceful)
@@ -969,11 +1073,12 @@ uunstick()
 	u.ustuck = 0;
 }
 
-STATIC_OVL void
-skinback()
+void
+skinback(silently)
+boolean silently;
 {
 	if (uskin) {
-		Your("skin returns to its original form.");
+		if (!silently) Your("skin returns to its original form.");
 		uarm = uskin;
 		uskin = (struct obj *)0;
 		/* undo save/restore hack */
@@ -992,34 +1097,44 @@ int part;
 	static NEARDATA const char
 	*humanoid_parts[] = { "arm", "eye", "face", "finger",
 		"fingertip", "foot", "hand", "handed", "head", "leg",
-		"light headed", "neck", "spine", "toe", "hair" },
+		"light headed", "neck", "spine", "toe", "hair",
+		"blood", "lung", "nose", "stomach"},
 	*jelly_parts[] = { "pseudopod", "dark spot", "front",
 		"pseudopod extension", "pseudopod extremity",
 		"pseudopod root", "grasp", "grasped", "cerebral area",
 		"lower pseudopod", "viscous", "middle", "surface",
-		"pseudopod extremity", "ripples" },
+		"pseudopod extremity", "ripples", "juices",
+		"surface", "sensor", "stomach" },
 	*animal_parts[] = { "forelimb", "eye", "face", "foreclaw", "claw tip",
 		"rear claw", "foreclaw", "clawed", "head", "rear limb",
-		"light headed", "neck", "spine", "rear claw tip", "fur" },
+		"light headed", "neck", "spine", "rear claw tip",
+		"fur", "blood", "lung", "nose", "stomach" },
 	*horse_parts[] = { "foreleg", "eye", "face", "forehoof", "hoof tip",
 		"rear hoof", "foreclaw", "hooved", "head", "rear leg",
-		"light headed", "neck", "backbone", "rear hoof tip", "mane" },
+		"light headed", "neck", "backbone", "rear hoof tip",
+		"mane", "blood", "lung", "nose", "stomach"},
 	*sphere_parts[] = { "appendage", "optic nerve", "body", "tentacle",
 		"tentacle tip", "lower appendage", "tentacle", "tentacled",
 		"body", "lower tentacle", "rotational", "equator", "body",
-		"lower tentacle tip", "cilia" },
+		"lower tentacle tip", "cilia", "life force", "retina",
+		"olfactory nerve", "interior" },
 	*fungus_parts[] = { "mycelium", "visual area", "front", "hypha",
 		"hypha", "root", "strand", "stranded", "cap area",
 		"rhizome", "sporulated", "stalk", "root", "rhizome tip",
-		"spores" },
+		"spores", "juices", "gill", "gill", "interior" },
 	*vortex_parts[] = { "region", "eye", "front", "minor current",
 		"minor current", "lower current", "swirl", "swirled",
 		"central core", "lower current", "addled", "center",
-		"currents", "edge", "currents" },
+		"currents", "edge", "currents", "life force",
+		"center", "leading edge", "interior" },
 	*snake_parts[] = { "vestigial limb", "eye", "face", "large scale",
 		"large scale tip", "rear region", "scale gap", "scale gapped",
 		"head", "rear region", "light headed", "neck", "length",
-		"rear scale", "scales" };
+		"rear scale", "scales", "blood", "lung", "forked tongue", "stomach" },
+	*fish_parts[] = { "fin", "eye", "premaxillary", "pelvic axillary",
+		"pelvic fin", "anal fin", "pectoral fin", "finned", "head", "peduncle",
+		"played out", "gills", "dorsal fin", "caudal fin",
+		"scales", "blood", "gill", "nostril", "stomach" };
 	/* claw attacks are overloaded in mons[]; most humanoids with
 	   such attacks should still reference hands rather than claws */
 	static const char not_claws[] = {
@@ -1040,6 +1155,16 @@ int part;
 		    mptr != &mons[PM_INCUBUS] && mptr != &mons[PM_SUCCUBUS])
 		return part == HAND ? "claw" : "clawed";
 	}
+	if ((mptr == &mons[PM_MUMAK] || mptr == &mons[PM_MASTODON]) &&
+		part == NOSE)
+	    return "trunk";
+	if (mptr == &mons[PM_SHARK] && part == HAIR)
+	    return "skin";	/* sharks don't have scales */
+	if (mptr == &mons[PM_JELLYFISH] && (part == ARM || part == FINGER ||
+	    part == HAND || part == FOOT || part == TOE))
+	    return "tentacle";
+	if (mptr == &mons[PM_FLOATING_EYE] && part == EYE)
+	    return "cornea";
 	if (humanoid(mptr) &&
 		(part == ARM || part == FINGER || part == FINGERTIP ||
 		    part == HAND || part == HANDED))
@@ -1047,12 +1172,14 @@ int part;
 	if (mptr->mlet == S_CENTAUR || mptr->mlet == S_UNICORN ||
 		(mptr == &mons[PM_ROTHE] && part != HAIR))
 	    return horse_parts[part];
-	if (slithy(mptr))
+	if (mptr->mlet == S_EEL && mptr != &mons[PM_JELLYFISH])
+	    return fish_parts[part];
+	if (slithy(mptr) || (mptr->mlet == S_DRAGON && part == HAIR))
 	    return snake_parts[part];
 	if (mptr->mlet == S_EYE)
 	    return sphere_parts[part];
 	if (mptr->mlet == S_JELLY || mptr->mlet == S_PUDDING ||
-		mptr->mlet == S_BLOB)
+		mptr->mlet == S_BLOB || mptr == &mons[PM_JELLYFISH])
 	    return jelly_parts[part];
 	if (mptr->mlet == S_VORTEX || mptr->mlet == S_ELEMENTAL)
 	    return vortex_parts[part];
@@ -1078,20 +1205,8 @@ poly_gender()
 {
 /* Returns gender of polymorphed player; 0/1=same meaning as flags.female,
  * 2=none.
- * Used in:
- *	- Seduction by succubus/incubus
- *	- Talking to nymphs (sounds.c)
- * Not used in:
- *	- Messages given by nymphs stealing armor (they can't steal from
- *	  incubi/succubi/nymphs, and nonhumanoids can't wear armor).
- *	- Amulet of change (must refer to real gender no matter what
- *	  polymorphed into).
- *	- Priest/Priestess, Caveman/Cavewoman (ditto)
- *	- Polymorph self (only happens when human)
- *	- Shopkeeper messages (since referred to as "creature" and not "sir"
- *	  or "lady" when polymorphed)
  */
-	if (!humanoid(youmonst.data)) return 2;
+	if (is_neuter(youmonst.data) || !humanoid(youmonst.data)) return 2;
 	return flags.female;
 }
 
