@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)potion.c	3.3	99/03/13	*/
+/*	SCCS Id: @(#)potion.c	3.4	2002/03/05	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -83,7 +83,14 @@ boolean talk;
 			Hallucination ? "less wobbly" : "a bit steadier");
 	}
 	if (xtime && !old) {
-		if (talk) You("stagger...");
+		if (talk) {
+#ifdef STEED
+			if (u.usteed)
+				You("wobble in the saddle.");
+			else
+#endif
+			You("%s...", stagger(youmonst.data, "stagger"));
+		}
 	}
 	if ((!xtime && old) || (xtime && !old)) flags.botl = TRUE;
 
@@ -157,34 +164,77 @@ long xtime;
 boolean talk;
 {
 	long old = Blinded;
-	boolean changed = FALSE;
+	boolean u_could_see, can_see_now;
+	int eyecnt;
+	char buf[BUFSZ];
+	static const char
+		vismsg[] = "vision seems to %s for a moment but is %s now.",
+		eyemsg[] = "%s momentarily %s.";
+
+	/* we need to probe ahead in case the Eyes of the Overworld
+	   are or will be overriding blindness */
+	u_could_see = !Blind;
+	Blinded = xtime ? 1L : 0L;
+	can_see_now = !Blind;
+	Blinded = old;		/* restore */
 
 	if (u.usleep) talk = FALSE;
 
-	if (!xtime && old && !Blindfolded && haseyes(youmonst.data)) {
+	if (can_see_now && !u_could_see) {	/* regaining sight */
 	    if (talk) {
 		if (Hallucination)
 		    pline("Far out!  Everything is all cosmic again!");
-		else		   You("can see again.");
+		else
+		    You("can see again.");
+	    }
+	} else if (old && !xtime) {
+	    /* clearing temporary blindness without toggling blindness */
+	    if (talk) {
+		if (!haseyes(youmonst.data)) {
+		    strange_feeling((struct obj *)0, (char *)0);
+		} else if (Blindfolded) {
+		    Strcpy(buf, body_part(EYE));
+		    eyecnt = eyecount(youmonst.data);
+		    Your(eyemsg, (eyecnt == 1) ? buf : makeplural(buf),
+			 (eyecnt == 1) ? "itches" : "itch");
+		} else {	/* Eyes of the Overworld */
+		    Your(vismsg, "brighten",
+			 Hallucination ? "sadder" : "normal");
+		}
 	    }
-	    changed = TRUE;
 	}
-	if (xtime && !old && !Blindfolded && haseyes(youmonst.data)) {
+
+	if (u_could_see && !can_see_now) {	/* losing sight */
 	    if (talk) {
 		if (Hallucination)
-			pline("Oh, bummer!  Everything is dark!  Help!");
+		    pline("Oh, bummer!  Everything is dark!  Help!");
 		else
-			pline("A cloud of darkness falls upon you.");
+		    pline("A cloud of darkness falls upon you.");
 	    }
-	    changed = TRUE;
-
 	    /* Before the hero goes blind, set the ball&chain variables. */
 	    if (Punished) set_bc(0);
+	} else if (!old && xtime) {
+	    /* setting temporary blindness without toggling blindness */
+	    if (talk) {
+		if (!haseyes(youmonst.data)) {
+		    strange_feeling((struct obj *)0, (char *)0);
+		} else if (Blindfolded) {
+		    Strcpy(buf, body_part(EYE));
+		    eyecnt = eyecount(youmonst.data);
+		    Your(eyemsg, (eyecnt == 1) ? buf : makeplural(buf),
+			 (eyecnt == 1) ? "twitches" : "twitch");
+		} else {	/* Eyes of the Overworld */
+		    Your(vismsg, "dim",
+			 Hallucination ? "happier" : "normal");
+		}
+	    }
 	}
+
 	set_itimeout(&Blinded, xtime);
-	if (changed) {
+
+	if (u_could_see ^ can_see_now) {  /* one or the other but not both */
 	    flags.botl = 1;
-	    vision_full_recalc = 1;
+	    vision_full_recalc = 1;	/* blindness just got toggled */
 	    if (Blind_telepat || Infravision) see_monsters();
 	}
 }
@@ -225,6 +275,7 @@ long mask;	/* nonzero if resistance status should change by mask */
 		/* The see_* routines should be called *before* the pline. */
 		see_monsters();
 		see_objects();
+		see_traps();
 	    }
 	    flags.botl = 1;
 	    if (!Blind && talk) pline(message);
@@ -293,18 +344,20 @@ dodrink() {
 #define POTION_OCCUPANT_CHANCE(n) (13 + 2*(n))	/* also in muse.c */
 
 	potion_descr = OBJ_DESCR(objects[otmp->otyp]);
-	if (potion_descr && !strcmp(potion_descr, "milky") &&
+	if (potion_descr) {
+	    if (!strcmp(potion_descr, "milky") &&
 		    flags.ghost_count < MAXMONNO &&
 		    !rn2(POTION_OCCUPANT_CHANCE(flags.ghost_count))) {
 		ghost_from_bottle();
 		useup(otmp);
 		return(1);
-	} else if (potion_descr && !strcmp(potion_descr, "smoky") &&
+	    } else if (!strcmp(potion_descr, "smoky") &&
 		    flags.djinni_count < MAXMONNO &&
 		    !rn2(POTION_OCCUPANT_CHANCE(flags.djinni_count))) {
 		djinni_from_bottle(otmp);
 		useup(otmp);
 		return(1);
+	    }
 	}
 	return dopotion(otmp);
 }
@@ -349,7 +402,9 @@ peffects(otmp)
 		    break;
 		} else {
 		    pline("Wow!  This makes you feel %s!",
-			  (otmp->blessed) ? "great" : "good");
+			  (otmp->blessed) ?
+				(unfixable_trouble_count(FALSE) ? "better" : "great")
+			  : "good");
 		    i = rn2(A_MAX);		/* start at a random point */
 		    for (ii = 0; ii < A_MAX; ii++) {
 			lim = AMAX(i);
@@ -372,10 +427,10 @@ peffects(otmp)
 		break;
 	case POT_WATER:
 		if(!otmp->blessed && !otmp->cursed) {
-			pline("This tastes like water.");
-			u.uhunger += rnd(10);
-			newuhs(FALSE);
-			break;
+		    pline("This tastes like water.");
+		    u.uhunger += rnd(10);
+		    newuhs(FALSE);
+		    break;
 		}
 		unkn++;
 		if(is_undead(youmonst.data) || is_demon(youmonst.data) ||
@@ -461,7 +516,7 @@ peffects(otmp)
 		}
 		/* FALLTHRU */
 	case POT_INVISIBILITY:
-		if (Invisible || Blind || BInvis) {
+		if (Invis || Blind || BInvis) {
 		    nothing++;
 		} else {
 		    self_invis_message();
@@ -514,10 +569,14 @@ peffects(otmp)
 	    }
 	case POT_PARALYSIS:
 		if (Free_action)
-		    You("stiffen momentarily.");             
+		    You("stiffen momentarily.");
 		else {
 		    if (Levitation||Is_airlevel(&u.uz)||Is_waterlevel(&u.uz))
 			You("are motionlessly suspended.");
+#ifdef STEED
+		    else if (u.usteed)
+			You("are frozen in place!");
+#endif
 		    else
 			Your("%s are frozen to the %s!",
 			     makeplural(body_part(FOOT)), surface(u.ux, u.uy));
@@ -526,11 +585,11 @@ peffects(otmp)
 		    exercise(A_DEX, FALSE);
 		}
 		break;
-	case POT_SLEEPING:        
+	case POT_SLEEPING:
 		if(Sleep_resistance || Free_action)
 		    You("yawn.");
 		else {
-		    pline("You suddenly fall asleep!");
+		    You("suddenly fall asleep!");
 		    fall_asleep(-rn1(10, 25 - 12*bcsign(otmp)), TRUE);
 		}
 		break;
@@ -541,7 +600,12 @@ peffects(otmp)
 
 		    if (Detect_monsters) nothing++;
 		    unkn++;
-		    set_itimeout(&HDetect_monsters, 20L+rnd(40));
+		    /* after a while, repeated uses become less effective */
+		    if (HDetect_monsters >= 300L)
+			i = 1;
+		    else
+			i = rn1(40,21);
+		    incr_itimeout(&HDetect_monsters, i);
 		    for (x = 1; x < COLNO; x++) {
 			for (y = 0; y < ROWNO; y++) {
 			    if (levl[x][y].glyph == GLYPH_INVISIBLE) {
@@ -552,6 +616,7 @@ peffects(otmp)
 			}
 		    }
 		    see_monsters();
+		    if (unkn) You_feel("lonely.");
 		    break;
 		}
 		if (monster_detect(otmp, 0))
@@ -569,8 +634,14 @@ peffects(otmp)
 		if (otmp->blessed) {
 		    pline("(But in fact it was mildly stale %s juice.)",
 			  pl_fruit);
-		    if (!Role_if(PM_HEALER))
-			losehp(1, "mildly contaminated potion", KILLED_BY_AN);
+		    if (!Role_if(PM_HEALER)) {
+			if (otmp->corpsenm)
+			    losehp(1,
+				   "mildly contaminated tap water", KILLED_BY);
+			else
+			    losehp(1,
+				   "mildly contaminated potion", KILLED_BY_AN);
+		    }
 		} else {
 		    if(Poison_resistance)
 			pline(
@@ -587,9 +658,14 @@ peffects(otmp)
 			    		Poison_resistance ? -1 : -rn1(4,3),
 			    		TRUE);
 			}
-			if(!Poison_resistance)
+			if(!Poison_resistance) {
+			    if (otmp->corpsenm)
+				losehp(rnd(10)+5*!!(otmp->cursed),
+				       "contaminated tap water", KILLED_BY);
+			    else
 				losehp(rnd(10)+5*!!(otmp->cursed),
 				       "contaminated potion", KILLED_BY_AN);
+			}
 			exercise(A_CON, FALSE);
 		    }
 		}
@@ -614,6 +690,8 @@ peffects(otmp)
 		if(otmp->cursed) {
 		    pline("Ulch!  That potion tasted foul!");
 		    unkn++;
+		} else if (Fixed_abil) {
+		    nothing++;
 		} else {      /* If blessed, increase all; if not, try up to */
 		    int itmp; /* 6 times to find one which can be increased. */
 		    i = -1;		/* increment to 0 */
@@ -628,11 +706,15 @@ peffects(otmp)
 		}
 		break;
 	case POT_SPEED:
-		if(Wounded_legs && !otmp->cursed) {
+		if(Wounded_legs && !otmp->cursed
+#ifdef STEED
+		   && !u.usteed	/* heal_legs() would heal steeds legs */
+#endif
+						) {
 			heal_legs();
 			unkn++;
 			break;
-		}		/* and fall through */
+		} /* and fall through */
 	case SPE_HASTE_SELF:
 		if(!Very_fast) /* wwf@doe.carleton.ca */
 			You("are suddenly moving %sfaster.",
@@ -649,7 +731,7 @@ peffects(otmp)
 		if(Blind) nothing++;
 		make_blinded(itimeout_incr(Blinded,
 					   rn1(200, 250 - 125 * bcsign(otmp))),
-			     TRUE);
+			     (boolean)!Blind);
 		break;
 	case POT_GAIN_LEVEL:
 		if (otmp->cursed) {
@@ -698,12 +780,16 @@ peffects(otmp)
 		exercise(A_CON, TRUE);
 		exercise(A_STR, TRUE);
 		break;
-	case POT_FULL_HEALING:        
+	case POT_FULL_HEALING:
 		You_feel("completely healed.");
 		healup(400, 4+4*bcsign(otmp), !otmp->cursed, TRUE);
 		/* Restore one lost level if blessed */
-		if (otmp->blessed && (u.ulevel < u.ulevelmax))
-			pluslvl(FALSE);
+		if (otmp->blessed && u.ulevel < u.ulevelmax) {
+		    /* when multiple levels have been lost, drinking
+		       multiple potions will only get half of them back */
+		    u.ulevelmax -= 1;
+		    pluslvl(FALSE);
+		}
 		make_hallucinated(0L,TRUE,0L);
 		exercise(A_STR, TRUE);
 		exercise(A_CON, TRUE);
@@ -736,6 +822,7 @@ peffects(otmp)
 		    incr_itimeout(&HLevitation, rn1(50,250));
 		    HLevitation |= I_SPECIAL;
 		} else incr_itimeout(&HLevitation, rn1(140,10));
+		spoteffects(FALSE);	/* for sinks */
 		break;
 	case POT_GAIN_ENERGY:			/* M. Stephenson */
 		{	register int num;
@@ -778,16 +865,17 @@ peffects(otmp)
 			pline("This tastes %s.", Hallucination ? "tangy" : "sour");
 		else {
 			pline("This burns%s!", otmp->blessed ? " a little" :
-					otmp->cursed ? " a lot" : "");
+					otmp->cursed ? " a lot" : " like acid");
 			losehp(d(otmp->cursed ? 2 : 1, otmp->blessed ? 4 : 8),
 					"potion of acid", KILLED_BY_AN);
 			exercise(A_CON, FALSE);
 		}
 		if (Stoned) fix_petrification();
+		unkn++; /* holy/unholy water can burn like acid too */
 		break;
 	case POT_POLYMORPH:
 		You_feel("a little %s.", Hallucination ? "normal" : "strange");
-		if (!Unchanging) polyself();
+		if (!Unchanging) polyself(FALSE);
 		break;
 	default:
 		impossible("What a funny potion! (%u)", otmp->otyp);
@@ -821,7 +909,7 @@ strange_feeling(obj,txt)
 register struct obj *obj;
 register const char *txt;
 {
-	if(flags.beginner)
+	if (flags.beginner || !txt)
 		You("have a %s feeling for a moment, then it passes.",
 		Hallucination ? "normal" : "strange");
 	else
@@ -840,13 +928,20 @@ const char *bottlenames[] = {
 	"bottle", "phial", "flagon", "carafe", "flask", "jar", "vial"
 };
 
+
+const char *
+bottlename()
+{
+	return bottlenames[rn2(SIZE(bottlenames))];
+}
+
 void
 potionhit(mon, obj, your_fault)
 register struct monst *mon;
 register struct obj *obj;
 boolean your_fault;
 {
-	register const char *botlnam = bottlenames[rn2(SIZE(bottlenames))];
+	register const char *botlnam = bottlename();
 	boolean isyou = (mon == &youmonst);
 	int distance;
 
@@ -878,7 +973,7 @@ boolean your_fault;
 
 	/* oil doesn't instantly evaporate */
 	if (obj->otyp != POT_OIL && cansee(mon->mx,mon->my))
-		pline("%s evaporates.", The(xname(obj)));
+		pline("%s.", Tobjnam(obj, "evaporate"));
 
     if (isyou) {
 	switch (obj->otyp) {
@@ -888,7 +983,15 @@ boolean your_fault;
 		break;
 	case POT_POLYMORPH:
 		You_feel("a little %s.", Hallucination ? "normal" : "strange");
-		if (!Unchanging && !Antimagic) polyself();
+		if (!Unchanging && !Antimagic) polyself(FALSE);
+		break;
+	case POT_ACID:
+		if (!Acid_resistance) {
+		    pline("This burns%s!", obj->blessed ? " a little" :
+				    obj->cursed ? " a lot" : "");
+		    losehp(d(obj->cursed ? 2 : 1, obj->blessed ? 4 : 8),
+				    "potion of acid", KILLED_BY_AN);
+		}
 		break;
 	}
     } else {
@@ -896,11 +999,14 @@ boolean your_fault;
 
 	if (!your_fault) angermon = FALSE;
 	switch (obj->otyp) {
-	case POT_RESTORE_ABILITY:
-	case POT_GAIN_ABILITY:
 	case POT_HEALING:
 	case POT_EXTRA_HEALING:
 	case POT_FULL_HEALING:
+		if (mon->data == &mons[PM_PESTILENCE]) goto do_illness;
+		/*FALLTHRU*/
+	case POT_RESTORE_ABILITY:
+	case POT_GAIN_ABILITY:
+ do_healing:
 		angermon = FALSE;
 		if(mon->mhp < mon->mhpmax) {
 		    mon->mhp = mon->mhpmax;
@@ -909,6 +1015,15 @@ boolean your_fault;
 		}
 		break;
 	case POT_SICKNESS:
+		if (mon->data == &mons[PM_PESTILENCE]) goto do_healing;
+		if (dmgtype(mon->data, AD_DISE) ||
+			   dmgtype(mon->data, AD_PEST) || /* won't happen, see prior goto */
+			   resists_poison(mon)) {
+		    if (canseemon(mon))
+			pline("%s looks unharmed.", Monnam(mon));
+		    break;
+		}
+ do_illness:
 		if((mon->mhpmax > 3) && !resist(mon, POTION_CLASS, 0, NOTELL))
 			mon->mhpmax /= 2;
 		if((mon->mhp > 2) && !resist(mon, POTION_CLASS, 0, NOTELL))
@@ -922,6 +1037,7 @@ boolean your_fault;
 		if(!resist(mon, POTION_CLASS, 0, NOTELL))  mon->mconf = TRUE;
 		break;
 	case POT_INVISIBILITY:
+		angermon = FALSE;
 		mon_set_minvis(mon);
 		break;
 	case POT_SLEEPING:
@@ -942,7 +1058,7 @@ boolean your_fault;
 		break;
 	case POT_SPEED:
 		angermon = FALSE;
-		mon_adjust_speed(mon, 1);
+		mon_adjust_speed(mon, 1, obj);
 		break;
 	case POT_BLINDNESS:
 		if(haseyes(mon->data)) {
@@ -957,12 +1073,15 @@ boolean your_fault;
 		if (is_undead(mon->data) || is_demon(mon->data) ||
 			is_were(mon->data)) {
 		    if (obj->blessed) {
-			pline("%s shrieks in pain!", Monnam(mon));
+			pline("%s %s in pain!", Monnam(mon),
+			      is_silent(mon->data) ? "writhes" : "shrieks");
 			mon->mhp -= d(2,6);
+			/* should only be by you */
 			if (mon->mhp < 1) killed(mon);
 			else if (is_were(mon->data) && !is_human(mon->data))
 			    new_were(mon);	/* revert to human */
 		    } else if (obj->cursed) {
+			angermon = FALSE;
 			if (canseemon(mon))
 			    pline("%s looks healthier.", Monnam(mon));
 			mon->mhp += d(2,6);
@@ -972,7 +1091,14 @@ boolean your_fault;
 			    new_were(mon);	/* transform into beast */
 		    }
 		} else if(mon->data == &mons[PM_GREMLIN]) {
+		    angermon = FALSE;
 		    (void)split_mon(mon, (struct monst *)0);
+		} else if(mon->data == &mons[PM_IRON_GOLEM]) {
+		    if (canseemon(mon))
+			pline("%s rusts.", Monnam(mon));
+		    mon->mhp -= d(1,6);
+		    /* should only be by you */
+		    if (mon->mhp < 1) killed(mon);
 		}
 		break;
 	case POT_OIL:
@@ -981,9 +1107,15 @@ boolean your_fault;
 		break;
 	case POT_ACID:
 		if (!resists_acid(mon) && !resist(mon, POTION_CLASS, 0, NOTELL)) {
-		    pline("%s shrieks in pain!", Monnam(mon));
+		    pline("%s %s in pain!", Monnam(mon),
+			  is_silent(mon->data) ? "writhes" : "shrieks");
 		    mon->mhp -= d(obj->cursed ? 2 : 1, obj->blessed ? 4 : 8);
-		    if (mon->mhp < 1) killed(mon);
+		    if (mon->mhp < 1) {
+			if (your_fault)
+			    killed(mon);
+			else
+			    monkilled(mon, "", AD_ACID);
+		    }
 		}
 		break;
 	case POT_POLYMORPH:
@@ -1005,7 +1137,8 @@ boolean your_fault;
     }
 
 	/* Note: potionbreathe() does its own docall() */
-	if (distance==0 || ((distance < 3) && rn2(5)))
+	if ((distance==0 || ((distance < 3) && rn2(5))) &&
+	    (!breathless(youmonst.data) || haseyes(youmonst.data)))
 		potionbreathe(obj);
 	else if (obj->dknown && !objects[obj->otyp].oc_name_known &&
 		   !objects[obj->otyp].oc_uname && cansee(mon->mx,mon->my))
@@ -1025,6 +1158,7 @@ boolean your_fault;
 	obfree(obj, (struct obj *)0);
 }
 
+/* vapors are inhaled or get in your eyes */
 void
 potionbreathe(obj)
 register struct obj *obj;
@@ -1035,7 +1169,14 @@ register struct obj *obj;
 	case POT_RESTORE_ABILITY:
 	case POT_GAIN_ABILITY:
 		if(obj->cursed) {
-		    pline("Ulch!  That potion smells terrible!");
+		    if (!breathless(youmonst.data))
+			pline("Ulch!  That potion smells terrible!");
+		    else if (haseyes(youmonst.data)) {
+			int numeyes = eyecount(youmonst.data);
+			Your("%s sting%s!",
+			     (numeyes == 1) ? body_part(EYE) : makeplural(body_part(EYE)),
+			     (numeyes == 1) ? "s" : "");
+		    }
 		    break;
 		} else {
 		    i = rn2(A_MAX);		/* start at a random point */
@@ -1093,15 +1234,21 @@ register struct obj *obj;
 		break;
 	case POT_PARALYSIS:
 		kn++;
-		if (!Free_action) {                
+		if (!Free_action) {
 		    pline("%s seems to be holding you.", Something);
 		    nomul(-rnd(5));
 		    nomovemsg = You_can_move_again;
 		    exercise(A_DEX, FALSE);
-		} else You("stiffen momentarily.");                
+		} else You("stiffen momentarily.");
 		break;
 	case POT_SLEEPING:
-		You("yawn.");
+		kn++;
+		if (!Free_action && !Sleep_resistance) {
+		    You_feel("rather tired.");
+		    nomul(-rnd(5));
+		    nomovemsg = You_can_move_again;
+		    exercise(A_DEX, FALSE);
+		} else You("yawn.");
 		break;
 	case POT_SPEED:
 		if (!Fast) Your("knees seem more flexible now.");
@@ -1114,6 +1261,7 @@ register struct obj *obj;
 		    pline("It suddenly gets dark.");
 		}
 		make_blinded(itimeout_incr(Blinded, rnd(5)), FALSE);
+		if (!Blind && !u.usleep) Your(vision_clears);
 		break;
 	case POT_WATER:
 		if(u.umonnum == PM_GREMLIN) {
@@ -1126,6 +1274,7 @@ register struct obj *obj;
 		    else if (obj->cursed && !Upolyd)
 			you_were();
 		}
+		break;
 	case POT_ACID:
 	case POT_POLYMORPH:
 		exercise(A_CON, FALSE);
@@ -1141,12 +1290,13 @@ register struct obj *obj;
 */
 	}
 	/* note: no obfree() */
-	if (obj->dknown)
+	if (obj->dknown) {
 	    if (kn)
 		makeknown(obj->otyp);
 	    else if (!objects[obj->otyp].oc_name_known &&
 						!objects[obj->otyp].oc_uname)
 		docall(obj);
+	}
 }
 
 STATIC_OVL short
@@ -1182,7 +1332,7 @@ register struct obj *o1, *o2;
 			    case POT_GAIN_ENERGY:
 				return POT_FULL_HEALING;
 			}
-		case POT_FULL_HEALING:                
+		case POT_FULL_HEALING:
 			switch (o2->otyp) {
 			    case POT_GAIN_LEVEL:
 			    case POT_GAIN_ENERGY:
@@ -1257,19 +1407,20 @@ register struct obj *obj;
 	if (snuff_lit(obj)) return(TRUE);
 
 	if (obj->greased) {
-		grease_protect(obj,(char *)0,FALSE,&youmonst);
+		grease_protect(obj,(char *)0,&youmonst);
 		return(FALSE);
 	}
 	(void) Shk_Your(Your_buf, obj);
-	/* (Rusting and diluting unpaid shop goods ought to be charged for.) */
+	/* (Rusting shop goods ought to be charged for.) */
 	switch (obj->oclass) {
 	    case WEAPON_CLASS:
 		if (!obj->oerodeproof && is_rustprone(obj) &&
-		    (obj->oeroded < MAX_ERODE) && !rn2(10)) {
+		    (obj->oeroded < MAX_ERODE) && !rn2(2)) {
 			pline("%s %s some%s.",
 			      Your_buf, aobjnam(obj, "rust"),
 			      obj->oeroded ? " more" : "what");
 			obj->oeroded++;
+			update_inventory();
 			return TRUE;
 		} else break;
 	    case POTION_CLASS:
@@ -1279,10 +1430,15 @@ register struct obj *obj;
 			pline("It boils vigorously!");
 			losehp(rnd(10), "elementary chemistry", KILLED_BY);
 			makeknown(obj->otyp);
+			update_inventory();
 			return (TRUE);
 		}
 		pline("%s %s%s.", Your_buf, aobjnam(obj,"dilute"),
 		      obj->odiluted ? " further" : "");
+		if(obj->unpaid && costly_spot(u.ux, u.uy)) {
+		    You("dilute it, you pay for it.");
+		    bill_dummy_object(obj);
+		}
 		if (obj->odiluted) {
 			obj->odiluted = 0;
 #ifdef UNIXPC
@@ -1293,6 +1449,7 @@ register struct obj *obj;
 #endif
 			obj->otyp = POT_WATER;
 		} else obj->odiluted++;
+		update_inventory();
 		return TRUE;
 	    case SCROLL_CLASS:
 		if (obj->otyp != SCR_BLANK_PAPER
@@ -1302,17 +1459,16 @@ register struct obj *obj;
 		    ) {
 			if (!Blind) {
 				boolean oq1 = obj->quan == 1L;
-				pline_The("scroll%s fade%s.",
-					oq1 ? "" : "s",
-					oq1 ? "s" : "");
+				pline_The("scroll%s %s.",
+					  oq1 ? "" : "s", otense(obj, "fade"));
 			}
-			if(obj->unpaid) {
-			    subfrombill(obj, shop_keeper(*u.ushops));
+			if(obj->unpaid && costly_spot(u.ux, u.uy)) {
 			    You("erase it, you pay for it.");
 			    bill_dummy_object(obj);
 			}
 			obj->otyp = SCR_BLANK_PAPER;
 			obj->spe = 0;
+			update_inventory();
 			return TRUE;
 		} else break;
 	    case SPBOOK_CLASS:
@@ -1325,14 +1481,14 @@ register struct obj *obj;
 			    if (!Blind) {
 				    boolean oq1 = obj->quan == 1L;
 				    pline_The("spellbook%s fade%s.",
-					oq1 ? "" : "s", oq1 ? "s" : "");
+					oq1 ? "" : "s", otense(obj, "fade"));
 			    }
-			    if(obj->unpaid) {
-			        subfrombill(obj, shop_keeper(*u.ushops));
+			    if(obj->unpaid && costly_spot(u.ux, u.uy)) {
 			        You("erase it, you pay for it.");
 			        bill_dummy_object(obj);
 			    }
 			    obj->otyp = SPE_BLANK_PAPER;
+			    update_inventory();
 			}
 			return TRUE;
 		}
@@ -1345,6 +1501,7 @@ int
 dodip()
 {
 	register struct obj *potion, *obj;
+	struct obj *singlepotion;
 	const char *tmp;
 	uchar here;
 	char allowall[2];
@@ -1437,20 +1594,41 @@ dodip()
 		} else
 			if (get_wet(obj))
 			    goto poof;
-	} else if ((obj->otyp == POT_POLYMORPH || potion->otyp == POT_POLYMORPH)
-			&& obj->otyp != potion->otyp) {
-	    if (obj->oclass != POTION_CLASS ||
-	    	(obj->otyp == POT_POLYMORPH &&
-		 objects[potion->otyp].oc_name_known) ||
-	    	(potion->otyp == POT_POLYMORPH &&
-		 objects[obj->otyp].oc_name_known))
-	    	makeknown(POT_POLYMORPH);
- 
-	    /* KMH, conduct */
-	    u.uconduct.polypiles++;
- 
-	    poly_obj(obj, STRANGE_OBJECT);                
-	    useup(potion);
+	} else if (obj->otyp == POT_POLYMORPH ||
+		potion->otyp == POT_POLYMORPH) {
+	    /* some objects can't be polymorphed */
+	    if (obj->otyp == potion->otyp ||	/* both POT_POLY */
+		    obj->otyp == WAN_POLYMORPH ||
+		    obj->otyp == SPE_POLYMORPH ||
+		    obj == uball || obj == uskin ||
+		    obj_resists(obj->otyp == POT_POLYMORPH ?
+				potion : obj, 5, 95)) {
+		pline(nothing_happens);
+	    } else {
+	    	boolean was_wep = FALSE, was_swapwep = FALSE, was_quiver = FALSE;
+		short save_otyp = obj->otyp;
+		/* KMH, conduct */
+		u.uconduct.polypiles++;
+
+		if (obj == uwep) was_wep = TRUE;
+		else if (obj == uswapwep) was_swapwep = TRUE;
+		else if (obj == uquiver) was_quiver = TRUE;
+
+		obj = poly_obj(obj, STRANGE_OBJECT);
+
+		if (was_wep) setuwep(obj);
+		else if (was_swapwep) setuswapwep(obj);
+		else if (was_quiver) setuqwep(obj);
+
+		if (obj->otyp != save_otyp) {
+			makeknown(POT_POLYMORPH);
+			useup(potion);
+			prinv((char *)0, obj, 0L);
+		} else {
+			pline("Nothing seems to happen.");
+			useup(potion);
+		}
+	    }
 	    return(1);
 	} else if(obj->oclass == POTION_CLASS && obj->otyp != potion->otyp) {
 		/* Mixing potions is dangerous... */
@@ -1459,7 +1637,8 @@ dodip()
 		if (obj->cursed || obj->otyp == POT_ACID || !rn2(10)) {
 			pline("BOOM!  They explode!");
 			exercise(A_STR, FALSE);
-			potionbreathe(obj);
+			if (!breathless(youmonst.data) || haseyes(youmonst.data))
+				potionbreathe(obj);
 			useup(obj);
 			useup(potion);
 			losehp(rnd(10), "alchemic blast", KILLED_BY_AN);
@@ -1497,7 +1676,6 @@ dodip()
 		    }
 		}
 
-
 		obj->odiluted = (obj->otyp != POT_WATER);
 
 		if (obj->otyp == POT_WATER && !Hallucination) {
@@ -1518,7 +1696,7 @@ dodip()
 		if (!Blind) {
 		    if (!See_invisible) pline("Where did %s go?",
 		    		the(xname(obj)));
-		    else pline("You notice a little haziness around %s.",
+		    else You("notice a little haziness around %s.",
 		    		the(xname(obj)));
 		}
 		goto poof;
@@ -1527,7 +1705,7 @@ dodip()
 		if (!Blind) {
 		    if (!See_invisible) pline("So that's where %s went!",
 		    		the(xname(obj)));
-		    else pline("The haziness around %s disappears.",
+		    else pline_The("haziness around %s disappears.",
 		    		the(xname(obj)));
 		}
 		goto poof;
@@ -1537,9 +1715,12 @@ dodip()
 	if(is_poisonable(obj)) {
 	    if(potion->otyp == POT_SICKNESS && !obj->opoisoned) {
 		char buf[BUFSZ];
-		Strcpy(buf, The(xname(potion)));
-		pline("%s form%s a coating on %s.",
-			buf, potion->quan == 1L ? "s" : "", the(xname(obj)));
+		if (potion->quan > 1L)
+		    Sprintf(buf, "One of %s", the(xname(potion)));
+		else
+		    Strcpy(buf, The(xname(potion)));
+		pline("%s forms a coating on %s.",
+		      buf, the(xname(obj)));
 		obj->opoisoned = TRUE;
 		goto poof;
 	    } else if(obj->opoisoned &&
@@ -1552,19 +1733,20 @@ dodip()
 	    }
 	}
 
-	if (potion->otyp == POT_OIL &&
-		(obj->oclass == WEAPON_CLASS || is_weptool(obj))) {
+	if (potion->otyp == POT_OIL) {
 	    boolean wisx = FALSE;
 	    if (potion->lamplit) {	/* burning */
 		int omat = objects[obj->otyp].oc_material;
-		if (obj->oerodeproof || obj_resists(obj, 5, 95) ||
-			/* `METAL' should not be confused with is_metallic() */
-			omat == METAL || omat == MITHRIL || omat == BONE) {
-		    pline("%s seem%s to burn for a moment.",
-			  Yname2(obj),
-			  (obj->quan > 1L) ? "" : "s");
+		/* the code here should be merged with fire_damage */
+		if (catch_lit(obj)) {
+		    /* catch_lit does all the work if true */
+		} else if (obj->oerodeproof || obj_resists(obj, 5, 95) ||
+			   !is_flammable(obj) || obj->oclass == FOOD_CLASS) {
+		    pline("%s %s to burn for a moment.",
+			  Yname2(obj), otense(obj, "seem"));
 		} else {
-		    if (omat == PLASTIC) obj->oeroded = MAX_ERODE;
+		    if ((omat == PLASTIC || omat == PAPER) && !obj->oartifact)
+			obj->oeroded = MAX_ERODE;
 		    pline_The("burning oil %s %s.",
 			    obj->oeroded == MAX_ERODE ? "destroys" : "damages",
 			    yname(obj));
@@ -1573,29 +1755,34 @@ dodip()
 			obfree(obj, (struct obj *)0);
 			obj = (struct obj *) 0;
 		    } else {
-			/* should check for and do something about
-			   damaging unpaid shop goods here */
+			/* we know it's carried */
+			if (obj->unpaid) {
+			    /* create a dummy duplicate to put on bill */
+			    verbalize("You burnt it, you bought it!");
+			    bill_dummy_object(obj);
+			}
 			obj->oeroded++;
 		    }
 		}
-	    } else if (potion->cursed || !is_metallic(obj) ||
-		    /* arrows,&c are classed as metallic due to arrowhead
-		       material, but dipping in oil shouldn't repair them */
-		    is_ammo(obj)) {
+	    } else if (potion->cursed) {
 		pline_The("potion spills and covers your %s with oil.",
 			  makeplural(body_part(FINGER)));
 		incr_itimeout(&Glib, d(2,10));
-	    /* oil removes rust and corrosion, but doesn't unburn */
-	    } else if (!is_flammable(obj) ||
-					(!obj->oeroded && !obj->oeroded2)) {
+	    } else if (obj->oclass != WEAPON_CLASS && !is_weptool(obj)) {
+		/* the following cases apply only to weapons */
+		goto more_dips;
+	    /* Oil removes rust and corrosion, but doesn't unburn.
+	     * Arrows, etc are classed as metallic due to arrowhead
+	     * material, but dipping in oil shouldn't repair them.
+	     */
+	    } else if ((!is_rustprone(obj) && !is_corrodeable(obj)) ||
+			is_ammo(obj) || (!obj->oeroded && !obj->oeroded2)) {
 		/* uses up potion, doesn't set obj->greased */
-		pline("%s gleam%s with an oily sheen.",
-		      Yname2(obj),
-		      (obj->quan > 1L) ? "" : "s");
+		pline("%s %s with an oily sheen.",
+		      Yname2(obj), otense(obj, "gleam"));
 	    } else {
 		pline("%s %s less %s.",
-		      Yname2(obj),
-		      (obj->quan > 1L) ? "are" : "is",
+		      Yname2(obj), otense(obj, "are"),
 		      (obj->oeroded && obj->oeroded2) ? "corroded and rusty" :
 			obj->oeroded ? "rusty" : "corroded");
 		if (obj->oeroded > 0) obj->oeroded--;
@@ -1607,6 +1794,7 @@ dodip()
 	    useup(potion);
 	    return 1;
 	}
+    more_dips:
 
 	/* Allow filling of MAGIC_LAMPs to prevent identification by player */
 	if ((obj->otyp == OIL_LAMP || obj->otyp == MAGIC_LAMP) &&
@@ -1614,7 +1802,7 @@ dodip()
 	    /* Turn off engine before fueling, turn off fuel too :-)  */
 	    if (obj->lamplit || potion->lamplit) {
 		useup(potion);
-		explode(u.ux, u.uy, 11, d(6,6), 0);
+		explode(u.ux, u.uy, 11, d(6,6), 0, EXPL_FIERY);
 		exercise(A_WIS, FALSE);
 		return 1;
 	    }
@@ -1624,15 +1812,16 @@ dodip()
 		obj->age = 0;
 	    }
 	    if (obj->age > 1000L) {
-		pline("%s is full.", Yname2(obj));
+		pline("%s %s full.", Yname2(obj), otense(obj, "are"));
 	    } else {
 		You("fill %s with oil.", yname(obj));
-		check_unpaid(potion);	/* surcharge for using unpaid item */
+		check_unpaid(potion);	/* Yendorian Fuel Tax */
 		obj->age += 2*potion->age;	/* burns more efficiently */
 		if (obj->age > 1500L) obj->age = 1500L;
 		useup(potion);
 		exercise(A_WIS, TRUE);
 	    }
+	    makeknown(POT_OIL);
 	    obj->spe = 1;
 	    update_inventory();
 	    return 1;
@@ -1640,34 +1829,46 @@ dodip()
 
 	if ((obj->otyp == UNICORN_HORN || obj->otyp == AMETHYST) &&
 	    (mixture = mixtype(obj, potion)) != 0) {
-		/* with multiple merged potions, we should split off one and
-		   just clear it, but clearing them all together is easier */
-		boolean more_than_one = potion->quan > 1L;
-		potion->otyp = mixture;
-		potion->blessed = 0;
+		boolean more_than_one = potion->quan > 1;
+		/* with multiple merged potions, split off one and
+		   just clear it */
+		if (potion->quan > 1L) {
+		    singlepotion = splitobj(potion, 1L);
+		} else singlepotion = potion;
+		
+		if(singlepotion->unpaid && costly_spot(u.ux, u.uy)) {
+		    You("use it, you pay for it.");
+		    bill_dummy_object(singlepotion);
+		}
+		singlepotion->otyp = mixture;
+		singlepotion->blessed = 0;
 		if (mixture == POT_WATER)
-		    potion->cursed = potion->odiluted = 0;
+		    singlepotion->cursed = singlepotion->odiluted = 0;
 		else
-		    potion->cursed = obj->cursed;  /* odiluted left as-is */
+		    singlepotion->cursed = obj->cursed;  /* odiluted left as-is */
+		singlepotion->bknown = FALSE;
 		if (Blind)
-			potion->dknown = FALSE;
+			singlepotion->dknown = FALSE;
 		else {
 			if (mixture == POT_WATER &&
 #ifdef DCC30_BUG
-			    (potion->dknown = !Hallucination,
-			     potion->dknown != 0))
+			    (singlepotion->dknown = !Hallucination,
+			     singlepotion->dknown != 0))
 #else
-			    (potion->dknown = !Hallucination) != 0)
+			    (singlepotion->dknown = !Hallucination) != 0)
 #endif
-				pline_The("potion%s clear%s.",
-					more_than_one ? "s" : "",
-					more_than_one ? "" : "s");
+				pline_The("potion%s clears.",
+					more_than_one ? " that you dipped into" : "");
 			else
-				pline_The("potion%s turn%s %s.",
-					more_than_one ? "s" : "",
-					more_than_one ? "" : "s",
+				pline_The("potion%s turns %s.",
+					more_than_one ? " that you dipped into" : "",
 					hcolor(OBJ_DESCR(objects[mixture])));
 		}
+	    	obj_extract_self(singlepotion);
+		singlepotion = hold_another_object(singlepotion,
+					"You juggle and drop %s!",
+					doname(singlepotion), (const char *)0);
+		update_inventory();
 		return(1);
 	}
 
