@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)priest.c	3.2	96/05/03	*/
+/*	SCCS Id: @(#)priest.c	3.4	2001/11/07	*/
 /* Copyright (c) Izchak Miller, Steve Linhart, 1989.		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -8,10 +8,13 @@
 #include "epri.h"
 #include "emin.h"
 
+/* this matches the categorizations shown by enlightenment */
+#define ALGN_SINNED	(-4)	/* worse than strayed */
+
 #ifdef OVLB
 
-static boolean FDECL(histemple_at,(struct monst *,XCHAR_P,XCHAR_P));
-static boolean FDECL(has_shrine,(struct monst *));
+STATIC_DCL boolean FDECL(histemple_at,(struct monst *,XCHAR_P,XCHAR_P));
+STATIC_DCL boolean FDECL(has_shrine,(struct monst *));
 
 /*
  * Move for priests and shopkeepers.  Called from shk_move() and pri_move().
@@ -47,7 +50,8 @@ register xchar omx,omy,gx,gy;
 	if (passes_walls(mtmp->data)) allowflags |= (ALLOW_ROCK|ALLOW_WALL);
 	if (throws_rocks(mtmp->data)) allowflags |= ALLOW_ROCK;
 	if (tunnels(mtmp->data) &&
-		    (!needspick(mtmp->data) || m_carrying(mtmp, PICK_AXE)))
+		    (!needspick(mtmp->data) || m_carrying(mtmp, PICK_AXE) ||
+		     m_carrying(mtmp, DWARVISH_MATTOCK)))
 		allowflags |= ALLOW_DIG;
 	if (!nohands(mtmp->data) && !verysmall(mtmp->data)) {
 		allowflags |= OPENDOOR;
@@ -101,7 +105,7 @@ pick_move:
 			    pline("%s picks up %s.", Monnam(mtmp),
 				distant_name(ib,doname));
 			obj_extract_self(ib);
-			mpickobj(mtmp, ib);
+			(void) mpickobj(mtmp, ib);
 		}
 		return(1);
 	}
@@ -127,7 +131,7 @@ register char *array;
 #endif /* OVL0 */
 #ifdef OVLB
 
-static boolean
+STATIC_OVL boolean
 histemple_at(priest, x, y)
 register struct monst *priest;
 register xchar x, y;
@@ -189,9 +193,9 @@ struct mkroom *sroom;
 int sx, sy;
 boolean sanctum;   /* is it the seat of the high priest? */
 {
-	register struct monst *priest;
-	register struct obj *otmp;
-	register int cnt;
+	struct monst *priest;
+	struct obj *otmp;
+	int cnt;
 
 	if(MON_AT(sx+1, sy))
 		rloc(m_at(sx+1, sy)); /* insurance */
@@ -207,35 +211,25 @@ boolean sanctum;   /* is it the seat of the high priest? */
 		priest->mtrapseen = ~0;	/* traps are known */
 		priest->mpeaceful = 1;
 		priest->ispriest = 1;
-		priest->msleep = 0;
+		priest->msleeping = 0;
 		set_malign(priest); /* mpeaceful may have changed */
 
 		/* now his/her goodies... */
 		if(sanctum && EPRI(priest)->shralign == A_NONE &&
-		     on_level(&sanctum_level, &u.uz))
+		     on_level(&sanctum_level, &u.uz)) {
 			(void) mongets(priest, AMULET_OF_YENDOR);
-		/* Do NOT put the rest in m_initinv.    */
-		/* Priests created elsewhere than in a  */
-		/* temple should not carry these items, */
-		cnt = rn1(2,3);
-		while(cnt) {
-		    otmp = mkobj(SPBOOK_CLASS, FALSE);
-		    if(otmp) mpickobj(priest, otmp);
-		    cnt--;
 		}
-		if(p_coaligned(priest))
-		    (void) mongets(priest, rn2(2) ? CLOAK_OF_PROTECTION
-						  : CLOAK_OF_MAGIC_RESISTANCE);
-		else {
-		    if(!rn2(5))
-			otmp = mksobj(CLOAK_OF_MAGIC_RESISTANCE, TRUE, FALSE);
-		    else otmp = mksobj(CLOAK_OF_PROTECTION, TRUE, FALSE);
-		    if(otmp) {
-			if(!rn2(2)) curse(otmp);
-			mpickobj(priest, otmp);
-		    }
+		/* 2 to 4 spellbooks */
+		for (cnt = rn1(3,2); cnt > 0; --cnt) {
+		    (void) mpickobj(priest, mkobj(SPBOOK_CLASS, FALSE));
+		}
+		/* robe [via makemon()] */
+		if (rn2(2) && (otmp = which_armor(priest, W_ARMC)) != 0) {
+		    if (p_coaligned(priest))
+			uncurse(otmp);
+		    else
+			curse(otmp);
 		}
-		m_dowear(priest, TRUE);
 	}
 }
 
@@ -248,12 +242,15 @@ boolean sanctum;   /* is it the seat of the high priest? */
  *		(coaligned Angels are also created as minions, but they
  *		use the same naming convention)
  *	- minions do not have ispriest but have isminion and emin
+ *	- caller needs to inhibit Hallucination if it wants to force
+ *		the true name even when under that influence
  */
 char *
-priestname(mon)
+priestname(mon, pname)
 register struct monst *mon;
+char *pname;		/* caller-supplied output buffer */
 {
-	static NEARDATA char pname[PL_NSIZ];
+	const char *what = Hallucination ? rndmonnam() : mon->data->mname;
 
 	Strcpy(pname, "the ");
 	if (mon->minvis) Strcat(pname, "invisible ");
@@ -264,7 +261,7 @@ register struct monst *mon;
 			Strcat(pname, "guardian ");
 		if (mon->data != &mons[PM_ALIGNED_PRIEST] &&
 				mon->data != &mons[PM_HIGH_PRIEST]) {
-			Strcat(pname, mon->data->mname);
+			Strcat(pname, what);
 			Strcat(pname, " ");
 		}
 		if (mon->data != &mons[PM_ANGEL]) {
@@ -272,19 +269,21 @@ register struct monst *mon;
 				Strcat(pname, "renegade ");
 			if (mon->data == &mons[PM_HIGH_PRIEST])
 				Strcat(pname, "high ");
-			if (mon->female)
+			if (Hallucination)
+				Strcat(pname, "poohbah ");
+			else if (mon->female)
 				Strcat(pname, "priestess ");
 			else
 				Strcat(pname, "priest ");
 		}
 		Strcat(pname, "of ");
-		Strcat(pname, align_gname((int)EPRI(mon)->shralign));
+		Strcat(pname, halu_gname((int)EPRI(mon)->shralign));
 		return(pname);
 	}
 	/* use emin instead of epri */
-	Strcat(pname, mon->data->mname);
+	Strcat(pname, what);
 	Strcat(pname, " of ");
-	Strcat(pname, align_gname(EMIN(mon)->min_align));
+	Strcat(pname, halu_gname(EMIN(mon)->min_align));
 	return(pname);
 }
 
@@ -295,7 +294,7 @@ struct monst *priest;
 	return((boolean)(u.ualign.type == ((int)EPRI(priest)->shralign)));
 }
 
-static boolean
+STATIC_OVL boolean
 has_shrine(pri)
 struct monst *pri;
 {
@@ -315,10 +314,12 @@ char roomno;
 {
 	register struct monst *mtmp;
 
-	for(mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+	for(mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	    if (DEADMONSTER(mtmp)) continue;
 	    if(mtmp->ispriest && (EPRI(mtmp)->shroom == roomno) &&
 	       histemple_at(mtmp,mtmp->mx,mtmp->my))
 		return(mtmp);
+	}
 	return (struct monst *)0;
 }
 
@@ -338,7 +339,7 @@ register int roomno;
 		shrined = has_shrine(priest);
 		sanctum = (priest->data == &mons[PM_HIGH_PRIEST] &&
 			   (Is_sanctum(&u.uz) || In_endgame(&u.uz)));
-		can_speak = (priest->mcanmove && !priest->msleep);
+		can_speak = (priest->mcanmove && !priest->msleeping);
 		if (can_speak)
 		    pline("%s intones:",
 			  (!Blind ? Monnam(priest) : "A nearby voice"));
@@ -362,8 +363,8 @@ register int roomno;
 		}
 		if(!sanctum) {
 		    /* !tended -> !shrined */
-		    if(!shrined || !p_coaligned(priest) ||
-						   u.ualign.record < -5)
+		    if (!shrined || !p_coaligned(priest) ||
+			    u.ualign.record <= ALGN_SINNED)
 			You("have a%s forbidding feeling...",
 				(!shrined) ? "" : " strange");
 		    else You("experience a strange sense of peace.");
@@ -399,6 +400,9 @@ register struct monst *priest;
 	boolean coaligned = p_coaligned(priest);
 	boolean strayed = (u.ualign.record < 0);
 
+	/* KMH, conduct */
+	u.uconduct.gnostic++;
+
 	if(priest->mflee || (!priest->ispriest && coaligned && strayed)) {
 	    pline("%s doesn't want anything to do with you!",
 				Monnam(priest));
@@ -408,17 +412,17 @@ register struct monst *priest;
 
 	/* priests don't chat unless peaceful and in their own temple */
 	if(!histemple_at(priest,priest->mx,priest->my) ||
-		 !priest->mpeaceful || !priest->mcanmove || priest->msleep) {
+		 !priest->mpeaceful || !priest->mcanmove || priest->msleeping) {
 	    static const char *cranky_msg[3] = {
 		"Thou wouldst have words, eh?  I'll give thee a word or two!",
 		"Talk?  Here is what I have to say!",
 		"Pilgrim, I would speak no longer with thee."
 	    };
 
-	    if(!priest->mcanmove || priest->msleep) {
+	    if(!priest->mcanmove || priest->msleeping) {
 		pline("%s breaks out of %s reverie!",
-		      Monnam(priest), his[pronoun_gender(priest)]);
-		priest->mfrozen = priest->msleep = 0;
+		      Monnam(priest), mhis(priest));
+		priest->mfrozen = priest->msleeping = 0;
 		priest->mcanmove = 1;
 	    }
 	    priest->mpeaceful = 0;
@@ -433,7 +437,7 @@ register struct monst *priest;
 	    priest->mpeaceful = 0;
 	    return;
 	}
-
+#ifndef GOLDOBJ
 	if(!u.ugold) {
 	    if(coaligned && !strayed) {
 		if (priest->mgold > 0L) {
@@ -446,6 +450,16 @@ register struct monst *priest;
 			u.ugold = 1L;
 		    priest->mgold -= u.ugold;
 		    flags.botl = 1;
+#else
+	if(!money_cnt(invent)) {
+	    if(coaligned && !strayed) {
+                long pmoney = money_cnt(priest->minvent);
+		if (pmoney > 0L) {
+		    /* Note: two bits is actually 25 cents.  Hmm. */
+		    pline("%s gives you %s for an ale.", Monnam(priest),
+			(pmoney == 1L) ? "one bit" : "two bits");
+		     money2u(priest, pmoney > 1L ? 2 : 1);
+#endif
 		} else
 		    pline("%s preaches the virtues of poverty.", Monnam(priest));
 		exercise(A_WIS, TRUE);
@@ -461,7 +475,11 @@ register struct monst *priest;
 		verbalize("Thou shalt regret thine action!");
 		if(coaligned) adjalign(-1);
 	    } else if(offer < (u.ulevel * 200)) {
+#ifndef GOLDOBJ
 		if(u.ugold > (offer * 2L)) verbalize("Cheapskate.");
+#else
+		if(money_cnt(invent) > (offer * 2L)) verbalize("Cheapskate.");
+#endif
 		else {
 		    verbalize("I thank thee for thy contribution.");
 		    /*  give player some token  */
@@ -469,22 +487,31 @@ register struct monst *priest;
 		}
 	    } else if(offer < (u.ulevel * 400)) {
 		verbalize("Thou art indeed a pious individual.");
+#ifndef GOLDOBJ
 		if(u.ugold < (offer * 2L)) {
-		    if(coaligned && u.ualign.record < -5) adjalign(1);
+#else
+		if(money_cnt(invent) < (offer * 2L)) {
+#endif
+		    if (coaligned && u.ualign.record <= ALGN_SINNED)
+			adjalign(1);
 		    verbalize("I bestow upon thee a blessing.");
-		    HClairvoyant += rn1(500,500);
+		    incr_itimeout(&HClairvoyant, rn1(500,500));
 		}
 	    } else if(offer < (u.ulevel * 600) &&
 		      u.ublessed < 20 &&
 		      (u.ublessed < 9 || !rn2(u.ublessed))) {
 		verbalize("Thy devotion has been rewarded.");
-		if (!(Protection & INTRINSIC))  {
-			Protection |= FROMOUTSIDE;
+		if (!(HProtection & INTRINSIC))  {
+			HProtection |= FROMOUTSIDE;
 			if (!u.ublessed)  u.ublessed = rn1(3, 2);
 		} else u.ublessed++;
 	    } else {
 		verbalize("Thy selfless generosity is deeply appreciated.");
+#ifndef GOLDOBJ
 		if(u.ugold < (offer * 2L) && coaligned) {
+#else
+		if(money_cnt(invent) < (offer * 2L) && coaligned) {
+#endif
 		    if(strayed && (moves - u.ucleansed) > 5000L) {
 			u.ualign.record = 0; /* cleanse thee */
 			u.ucleansed = moves;
@@ -521,7 +548,7 @@ boolean peaceful;
 	roamer->isminion = TRUE;	/* borrowing this bit */
 	roamer->mtrapseen = ~0;		/* traps are known */
 	roamer->mpeaceful = peaceful;
-	roamer->msleep = 0;
+	roamer->msleeping = 0;
 	set_malign(roamer); /* peaceful may have changed */
 
 	/* MORE TO COME */
@@ -544,17 +571,27 @@ register struct monst *roamer;
 }
 
 boolean
-in_your_sanctuary(x, y)
+in_your_sanctuary(mon, x, y)
+struct monst *mon;	/* if non-null, <mx,my> overrides <x,y> */
 xchar x, y;
 {
 	register char roomno;
 	register struct monst *priest;
 
-	if ((u.ualign.record < -5) || !(roomno = temple_occupied(u.urooms)) ||
-	    (roomno != *in_rooms(x, y, TEMPLE)) ||
-	    !(priest = findpriest(roomno)))
-		return(FALSE);
-	return((boolean)(has_shrine(priest) && p_coaligned(priest) && priest->mpeaceful));
+	if (mon) {
+	    if (is_minion(mon->data) || is_rider(mon->data)) return FALSE;
+	    x = mon->mx, y = mon->my;
+	}
+	if (u.ualign.record <= ALGN_SINNED)	/* sinned or worse */
+	    return FALSE;
+	if ((roomno = temple_occupied(u.urooms)) == 0 ||
+		roomno != *in_rooms(x, y, TEMPLE))
+	    return FALSE;
+	if ((priest = findpriest(roomno)) == 0)
+	    return FALSE;
+	return (boolean)(has_shrine(priest) &&
+			 p_coaligned(priest) &&
+			 priest->mpeaceful);
 }
 
 void
@@ -655,7 +692,7 @@ clearpriests()
 
     for(mtmp = fmon; mtmp; mtmp = mtmp2) {
 	mtmp2 = mtmp->nmon;
-	if (mtmp->ispriest && !on_level(&(EPRI(mtmp)->shrlevel), &u.uz))
+	if (!DEADMONSTER(mtmp) && mtmp->ispriest && !on_level(&(EPRI(mtmp)->shrlevel), &u.uz))
 	    mongone(mtmp);
     }
 }
