@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)quest.c	3.3	98/11/04	*/
+/*	SCCS Id: @(#)quest.c	3.4	2000/05/05	*/
 /*	Copyright 1991, M. Stephenson		  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -42,7 +42,7 @@ on_locate()
   if(!Qstat(first_locate)) {
     qt_pager(QT_FIRSTLOCATE);
     Qstat(first_locate) = TRUE;
-  } else if(u.uz0.dlevel < u.uz.dlevel)
+  } else if(u.uz0.dlevel < u.uz.dlevel && !Qstat(killed_nemesis))
 	qt_pager(QT_NEXTLOCATE);
 }
 
@@ -109,14 +109,14 @@ STATIC_OVL boolean
 is_pure(talk)
 boolean talk;
 {
-    aligntyp original_alignment = u.ualignbase[1];
+    aligntyp original_alignment = u.ualignbase[A_ORIGINAL];
 
 #ifdef WIZARD
     if (wizard && talk) {
 	if (u.ualign.type != original_alignment) {
 	    You("are currently %s instead of %s.",
 		align_str(u.ualign.type), align_str(original_alignment));
-	} else if (u.ualignbase[0] != original_alignment) {
+	} else if (u.ualignbase[A_CURRENT] != original_alignment) {
 	    You("have converted.");
 	} else if (u.ualign.record < MIN_QUEST_ALIGN) {
 	    You("are currently %d and require %d.",
@@ -128,7 +128,7 @@ boolean talk;
 #endif
     return (boolean)(u.ualign.record >= MIN_QUEST_ALIGN &&
 		     u.ualign.type == original_alignment &&
-		     u.ualignbase[0] == original_alignment);
+		     u.ualignbase[A_CURRENT] == original_alignment);
 }
 
 /*
@@ -152,6 +152,7 @@ boolean seal;
 		  !seal ? 1 : -1;
     schedule_goto(dest, FALSE, FALSE, portal_flag, (char *)0, (char *)0);
     if (seal) {	/* remove the portal to the quest - sealing it off */
+	int reexpelled = u.uevent.qexpelled;
 	u.uevent.qexpelled = 1;
 	/* Delete the near portal now; the far (main dungeon side)
 	   portal will be deleted as part of arrival on that level.
@@ -160,7 +161,7 @@ boolean seal;
 	for (t = ftrap; t; t = t->ntrap)
 	    if (t->ttyp == MAGIC_PORTAL) break;
 	if (t) deltrap(t);	/* (display might be briefly out of sync) */
-	else impossible("quest portal already gone?");
+	else if (!reexpelled) impossible("quest portal already gone?");
     }
 }
 
@@ -194,6 +195,7 @@ struct obj *obj;	/* quest artifact; possibly null if carrying Amulet */
 	    /* behave as if leader imparts sufficient info about the
 	       quest artifact */
 	    fully_identify_obj(obj);
+	    update_inventory();
 	}
 }
 
@@ -293,7 +295,8 @@ nemesis_speaks()
 {
 	if(!Qstat(in_battle)) {
 	  if(u.uhave.questart) qt_pager(QT_NEMWANTSIT);
-	  else if(Qstat(made_goal) == 1) qt_pager(QT_FIRSTNEMESIS);
+	  else if(Qstat(made_goal) == 1 || !Qstat(met_nemesis))
+	      qt_pager(QT_FIRSTNEMESIS);
 	  else if(Qstat(made_goal) < 4) qt_pager(QT_NEXTNEMESIS);
 	  else if(Qstat(made_goal) < 7) qt_pager(QT_OTHERNEMESIS);
 	  else if(!rn2(5))	qt_pager(rn1(10, QT_DISCOURAGE));
@@ -307,7 +310,10 @@ STATIC_OVL void
 chat_with_guardian()
 {
 /*	These guys/gals really don't have much to say... */
-	qt_pager(rn1(5, QT_GUARDTALK));
+	if (u.uhave.questart && Qstat(killed_nemesis))
+	    qt_pager(rn1(5, QT_GUARDTALK2));
+	else
+	    qt_pager(rn1(5, QT_GUARDTALK));
 }
 
 STATIC_OVL void
@@ -321,6 +327,7 @@ prisoner_speaks (mtmp)
 	    	pline("%s speaks:", Monnam(mtmp));
 	    verbalize("I'm finally free!");
 	    mtmp->mstrategy &= ~STRAT_WAITMASK;
+	    mtmp->mpeaceful = 1;
 
 	    /* Your god is happy... */
 	    adjalign(3);
@@ -335,8 +342,11 @@ void
 quest_chat(mtmp)
 	register struct monst *mtmp;
 {
+    if (mtmp->m_id == Qstat(leader_m_id)) {
+	chat_with_leader();
+	return;
+    }
     switch(mtmp->data->msound) {
-	    case MS_LEADER:	chat_with_leader(); break;
 	    case MS_NEMESIS:	chat_with_nemesis(); break;
 	    case MS_GUARDIAN:	chat_with_guardian(); break;
 	    default:	impossible("quest_chat: Unknown quest character %s.",
@@ -348,8 +358,11 @@ void
 quest_talk(mtmp)
 	register struct monst *mtmp;
 {
+    if (mtmp->m_id == Qstat(leader_m_id)) {
+	leader_speaks(mtmp);
+	return;
+    }
     switch(mtmp->data->msound) {
-	    case MS_LEADER:	leader_speaks(mtmp); break;
 	    case MS_NEMESIS:	nemesis_speaks(); break;
 	    case MS_DJINNI:	prisoner_speaks(mtmp); break;
 	    default:		break;
