@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)shknam.c	3.3	97/05/25	*/
+/*	SCCS Id: @(#)shknam.c	3.4	2001/09/06	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -115,7 +115,7 @@ static const char *shktools[] = {
     "Erreip", "Nehpets", "Mron", "Snivek", "Lapu", "Kahztiy",
 #endif
 #ifdef WIN32
-    "Lechaim",
+    "Lechaim", "Lexa", "Niod",
 #endif
 #ifdef MAC
     "Nhoj-lee", "Evad\'kh", "Ettaw-noj", "Tsew-mot", "Ydna-s",
@@ -123,7 +123,10 @@ static const char *shktools[] = {
 #endif
 #ifdef AMIGA
     "Falo", "Nosid-da\'r", "Ekim-p", "Rebrol-nek", "Noslo", "Yl-rednow",
-    "Mured-oog",
+    "Mured-oog", "Ivrajimsal",
+#endif
+#ifdef TOS
+    "Nivram",
 #endif
 #ifdef VMS
     "Lez-tneg", "Ytnu-haled", "Niknar",
@@ -243,21 +246,25 @@ mkshobj_at(shp, sx, sy)
 const struct shclass *shp;
 int sx, sy;
 {
-	register struct monst *mtmp;
+	struct monst *mtmp;
 	int atype;
 	struct permonst *ptr;
 
 	if (rn2(100) < depth(&u.uz) &&
-	    !MON_AT(sx, sy) && (ptr = mkclass(S_MIMIC,0)) &&
-	    (mtmp=makemon(ptr,sx,sy,NO_MM_FLAGS))) {
-		/* note: makemon will set the mimic symbol to a shop item */
-		if (rn2(10) >= depth(&u.uz)) {
-			mtmp->m_ap_type = M_AP_OBJECT;
-			mtmp->mappearance = STRANGE_OBJECT;
-		}
-	} else if ((atype = get_shop_item(shp - shtypes)) < 0)
-		(void) mksobj_at(-atype, sx, sy, TRUE);
-	else (void) mkobj_at(atype, sx, sy, TRUE);
+		!MON_AT(sx, sy) && (ptr = mkclass(S_MIMIC,0)) &&
+		(mtmp = makemon(ptr,sx,sy,NO_MM_FLAGS)) != 0) {
+	    /* note: makemon will set the mimic symbol to a shop item */
+	    if (rn2(10) >= depth(&u.uz)) {
+		mtmp->m_ap_type = M_AP_OBJECT;
+		mtmp->mappearance = STRANGE_OBJECT;
+	    }
+	} else {
+	    atype = get_shop_item(shp - shtypes);
+	    if (atype < 0)
+		(void) mksobj_at(-atype, sx, sy, TRUE, TRUE);
+	    else
+		(void) mkobj_at(atype, sx, sy, TRUE);
+	}
 }
 
 /* extract a shopkeeper name for the given shop type */
@@ -309,10 +316,9 @@ const char *nlp[];
 		    shname = shk->female ? "Lucrezia" : "Dirk";
 		}
 
-		/* is name already is use on this level? */
+		/* is name already in use on this level? */
 		for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
-		    if (mtmp == shk) continue;
-		    if (!mtmp->isshk) continue;
+		    if (DEADMONSTER(mtmp) || (mtmp == shk) || !mtmp->isshk) continue;
 		    if (strcmp(ESHK(mtmp)->shknam, shname)) continue;
 		    break;
 		}
@@ -400,7 +406,13 @@ struct mkroom	*sroom;
 	ESHK(shk)->visitct = 0;
 	ESHK(shk)->following = 0;
 	ESHK(shk)->billct = 0;
+#ifndef GOLDOBJ
 	shk->mgold = 1000L + 30L*(long)rnd(100);	/* initial capital */
+#else
+        mkmonmoney(shk, 1000L + 30L*(long)rnd(100));	/* initial capital */
+#endif
+	if (shp->shknms == shkrings)
+	    (void) mongets(shk, TOUCHSTONE);
 	nameshk(shk, shp->shknms);
 
 	return(sh);
