@@ -1,9 +1,12 @@
-/*	SCCS Id: @(#)sounds.c	3.3	97/05/25	*/
+/*	SCCS Id: @(#)sounds.c	3.4	2001/02/14	*/
 /*	Copyright (c) 1989 Janet Walz, Mike Threepoint */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
 #include "edog.h"
+#ifdef USER_SOUNDS
+#include <regex.h>
+#endif
 
 #ifdef OVLB
 
@@ -67,7 +70,8 @@ dosounds()
 		"Someone shouts \"Off with %s head!\"",
 		"Queen Beruthiel's cats!",
 	};
-	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	    if (DEADMONSTER(mtmp)) continue;
 	    if ((mtmp->msleeping ||
 			is_lord(mtmp->data) || is_prince(mtmp->data)) &&
 		!is_animal(mtmp->data) &&
@@ -76,9 +80,10 @@ dosounds()
 		int which = rn2(3)+hallu;
 
 		if (which != 2) You_hear(throne_msg[which]);
-		else		pline(throne_msg[2], his[flags.female]);
+		else		pline(throne_msg[2], uhis());
 		return;
 	    }
+	}
     }
     if (level.flags.has_swamp && !rn2(200)) {
 	static const char *swamp_msg[3] = {
@@ -133,7 +138,8 @@ dosounds()
 	return;
     }
     if (level.flags.has_beehive && !rn2(200)) {
-	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	    if (DEADMONSTER(mtmp)) continue;
 	    if ((mtmp->data->mlet == S_ANT && is_flyer(mtmp->data)) &&
 		mon_in_room(mtmp, BEEHIVE)) {
 		switch (rn2(2)+hallu) {
@@ -150,9 +156,11 @@ dosounds()
 		}
 		return;
 	    }
+	}
     }
     if (level.flags.has_morgue && !rn2(200)) {
-	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	    if (DEADMONSTER(mtmp)) continue;
 	    if (is_undead(mtmp->data) &&
 		mon_in_room(mtmp, MORGUE)) {
 		switch (rn2(2)+hallu) {
@@ -170,6 +178,7 @@ dosounds()
 		}
 		return;
 	    }
+	}
     }
     if (level.flags.has_barracks && !rn2(200)) {
 	static const char *barracks_msg[4] = {
@@ -180,7 +189,8 @@ dosounds()
 	};
 	int count = 0;
 
-	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	    if (DEADMONSTER(mtmp)) continue;
 	    if (is_mercenary(mtmp->data) &&
 #if 0		/* don't bother excluding these */
 		!strstri(mtmp->data->mname, "watch") &&
@@ -192,6 +202,7 @@ dosounds()
 		You_hear(barracks_msg[rn2(3)+hallu]);
 		return;
 	    }
+	}
     }
     if (level.flags.has_zoo && !rn2(200)) {
 	static const char *zoo_msg[3] = {
@@ -199,12 +210,14 @@ dosounds()
 		"a sound reminiscent of a seal barking.",
 		"Doctor Doolittle!",
 	};
-	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
+	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon) {
+	    if (DEADMONSTER(mtmp)) continue;
 	    if ((mtmp->msleeping || is_animal(mtmp->data)) &&
 		    mon_in_room(mtmp, ZOO)) {
 		You_hear(zoo_msg[rn2(2)+hallu]);
 		return;
 	    }
+	}
     }
     if (level.flags.has_shop && !rn2(200)) {
 	if (!(sroom = search_special(ANY_SHOP))) {
@@ -226,7 +239,7 @@ dosounds()
     if (Is_oracle_level(&u.uz) && !rn2(400)) {
 	/* make sure the Oracle is still here */
 	for (mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-	    if (mtmp->data == &mons[PM_ORACLE])
+	    if (!DEADMONSTER(mtmp) && mtmp->data == &mons[PM_ORACLE])
 		break;
 	/* and don't produce silly effects when she's clearly visible */
 	if (mtmp && (hallu || !canseemon(mtmp))) {
@@ -309,7 +322,7 @@ register struct monst *mtmp;
     else
 	growl_verb = growl_sound(mtmp);
     if (growl_verb) {
-	pline("%s %s!", Monnam(mtmp), makeplural(growl_verb));
+	pline("%s %s!", Monnam(mtmp), vtense((char *)0, growl_verb));
 	if(flags.run) nomul(0);
 	wake_nearto(mtmp->mx, mtmp->my, mtmp->data->mlevel * 18);
     }
@@ -350,7 +363,7 @@ register struct monst *mtmp;
 	    break;
     }
     if (yelp_verb) {
-	pline("%s %ss!", Monnam(mtmp), yelp_verb);
+	pline("%s %s!", Monnam(mtmp), vtense((char *)0, yelp_verb));
 	if(flags.run) nomul(0);
 	wake_nearto(mtmp->mx, mtmp->my, mtmp->data->mlevel * 12);
     }
@@ -382,7 +395,7 @@ register struct monst *mtmp;
 	    break;
     }
     if (whimper_verb) {
-	pline("%s %ss.", Monnam(mtmp), whimper_verb);
+	pline("%s %s.", Monnam(mtmp), vtense((char *)0, whimper_verb));
 	if(flags.run) nomul(0);
 	wake_nearto(mtmp->mx, mtmp->my, mtmp->data->mlevel * 6);
     }
@@ -398,7 +411,7 @@ register struct monst *mtmp;
 	return;
 
     /* presumably nearness and soundok checks have already been made */
-    if (mtmp->data->msound != MS_SILENT && mtmp->data->msound <= MS_ANIMAL)
+    if (!is_silent(mtmp->data) && mtmp->data->msound <= MS_ANIMAL)
 	(void) domonnoise(mtmp);
     else if (mtmp->data->msound >= MS_HUMANOID) {
 	if (!canspotmon(mtmp))
@@ -414,9 +427,17 @@ register struct monst *mtmp;
     register const char *pline_msg = 0,	/* Monnam(mtmp) will be prepended */
 			*verbl_msg = 0;	/* verbalize() */
     struct permonst *ptr = mtmp->data;
+    char verbuf[BUFSZ];
 
     /* presumably nearness and sleep checks have already been made */
     if (!flags.soundok) return(0);
+    if (is_silent(ptr)) return(0);
+
+    /* be sure to do this before talking; the monster might teleport away, in
+     * which case we want to check its pre-teleport position
+     */
+    if (!canspotmon(mtmp))
+	map_invisible(mtmp->mx, mtmp->my);
 
     switch (ptr->msound) {
 	case MS_ORACLE:
@@ -433,23 +454,87 @@ register struct monst *mtmp;
 	    shk_chat(mtmp);
 	    break;
 	case MS_VAMPIRE:
-	    if (mtmp->mpeaceful)
-	    	verbl_msg = "I only drink... potions.";
-	    else
-	    	verbl_msg = "I vant to suck your blood!";
+	    {
+	    /* vampire messages are varied by tameness, peacefulness, and time of night */
+		boolean isnight = night();
+		boolean kindred =    (Upolyd && (u.umonnum == PM_VAMPIRE ||
+				       u.umonnum == PM_VAMPIRE_LORD));
+		boolean nightchild = (Upolyd && (u.umonnum == PM_WOLF ||
+				       u.umonnum == PM_WINTER_WOLF ||
+	    			       u.umonnum == PM_WINTER_WOLF_CUB));
+		const char *racenoun = (flags.female && urace.individual.f) ?
+					urace.individual.f : (urace.individual.m) ?
+					urace.individual.m : urace.noun;
+
+		if (mtmp->mtame) {
+			if (kindred) {
+				Sprintf(verbuf, "Good %s to you Master%s",
+					isnight ? "evening" : "day",
+					isnight ? "!" : ".  Why do we not rest?");
+				verbl_msg = verbuf;
+		    	} else {
+		    	    Sprintf(verbuf,"%s%s",
+				nightchild ? "Child of the night, " : "",
+				midnight() ?
+					"I can stand this craving no longer!" :
+				isnight ?
+					"I beg you, help me satisfy this growing craving!" :
+					"I find myself growing a little weary.");
+				verbl_msg = verbuf;
+			}
+		} else if (mtmp->mpeaceful) {
+			if (kindred && isnight) {
+				Sprintf(verbuf, "Good feeding %s!",
+	    				flags.female ? "sister" : "brother");
+				verbl_msg = verbuf;
+ 			} else if (nightchild && isnight) {
+				Sprintf(verbuf,
+				    "How nice to hear you, child of the night!");
+				verbl_msg = verbuf;
+	    		} else
+		    		verbl_msg = "I only drink... potions.";
+    	        } else {
+			int vampindex;
+	    		static const char *vampmsg[] = {
+			       /* These first two (0 and 1) are specially handled below */
+	    			"I vant to suck your %s!",
+	    			"I vill come after %s without regret!",
+		    	       /* other famous vampire quotes can follow here if desired */
+	    		};
+			if (kindred)
+			    verbl_msg = "This is my hunting ground that you dare to prowl!";
+			else if (youmonst.data == &mons[PM_SILVER_DRAGON] ||
+				 youmonst.data == &mons[PM_BABY_SILVER_DRAGON]) {
+			    /* Silver dragons are silver in color, not made of silver */
+			    Sprintf(verbuf, "%s! Your silver sheen does not frighten me!",
+					youmonst.data == &mons[PM_SILVER_DRAGON] ?
+					"Fool" : "Young Fool");
+			    verbl_msg = verbuf; 
+			} else {
+			    vampindex = rn2(SIZE(vampmsg));
+			    if (vampindex == 0) {
+				Sprintf(verbuf, vampmsg[vampindex], body_part(BLOOD));
+	    			verbl_msg = verbuf;
+			    } else if (vampindex == 1) {
+				Sprintf(verbuf, vampmsg[vampindex],
+					Upolyd ? an(mons[u.umonnum].mname) : an(racenoun));
+	    			verbl_msg = verbuf;
+		    	    } else
+			    	verbl_msg = vampmsg[vampindex];
+			}
+	        }
+	    }
 	    break;
 	case MS_WERE:
 	    if (flags.moonphase == FULL_MOON && (night() ^ !rn2(13))) {
 		pline("%s throws back %s head and lets out a blood curdling %s!",
-		      Monnam(mtmp), his[pronoun_gender(mtmp)],
+		      Monnam(mtmp), mhis(mtmp),
 		      ptr == &mons[PM_HUMAN_WERERAT] ? "shriek" : "howl");
 		wake_nearto(mtmp->mx, mtmp->my, 11*11);
 	    } else
 		pline_msg =
 		     "whispers inaudibly.  All you can make out is \"moon\".";
 	    break;
-	case MS_SILENT:
-	    return 0;
 	case MS_BARK:
 	    if (flags.moonphase == FULL_MOON && night()) {
 		pline_msg = "howls.";
@@ -460,8 +545,10 @@ register struct monst *mtmp;
 		    pline_msg = "whines.";
 		else if (mtmp->mtame && EDOG(mtmp)->hungrytime > moves + 1000)
 		    pline_msg = "yips.";
-		else
-		    pline_msg = "barks.";
+		else {
+		    if (mtmp->data != &mons[PM_DINGO])	/* dingos do not actually bark */
+			    pline_msg = "barks.";
+		}
 	    } else {
 		pline_msg = "growls.";
 	    }
@@ -559,7 +646,7 @@ register struct monst *mtmp;
 	    if (!mtmp->mpeaceful) {
 		switch (rn2(4)) {
 		case 0: pline("%s boasts about %s gem collection.",
-			      Monnam(mtmp), his[pronoun_gender(mtmp)]);
+			      Monnam(mtmp), mhis(mtmp));
 			break;
 		case 1: pline_msg = "complains about a diet of mutton.";
 			break;
@@ -586,11 +673,15 @@ register struct monst *mtmp;
 		verbl_msg = !rn2(3) ? "Huh?" : rn2(2) ? "What?" : "Eh?";
 	    else if (!mtmp->mcansee)
 		verbl_msg = "I can't see!";
-	    else if (mtmp->mtrapped)
+	    else if (mtmp->mtrapped) {
+		struct trap *t = t_at(mtmp->mx, mtmp->my);
+
+		if (t) t->tseen = 1;
 		verbl_msg = "I'm trapped!";
-	    else if (mtmp->mhp < mtmp->mhpmax/2)
+	    } else if (mtmp->mhp < mtmp->mhpmax/2)
 		pline_msg = "asks for a potion of healing.";
-	    else if (mtmp->mtame && moves > EDOG(mtmp)->hungrytime)
+	    else if (mtmp->mtame && !mtmp->isminion &&
+						moves > EDOG(mtmp)->hungrytime)
 		verbl_msg = "I'm hungry.";
 	    /* Specific monsters' interests */
 	    else if (is_elf(ptr))
@@ -627,10 +718,11 @@ register struct monst *mtmp;
 			(void) doseduce(mtmp);
 			break;
 	    }
-	    switch ((poly_gender() != (int) mtmp->female) ? rn2(3) : 0) {
+	    switch ((poly_gender() != (int) mtmp->female) ? rn2(3) : 0)
 #else
-	    switch ((poly_gender() == 0) ? rn2(3) : 0) {
+	    switch ((poly_gender() == 0) ? rn2(3) : 0)
 #endif
+	    {
 		case 2:
 			verbl_msg = "Hello, sailor.";
 			break;
@@ -684,7 +776,11 @@ register struct monst *mtmp;
 	    else verbl_msg = "Relax, this won't hurt a bit.";
 	    break;
 	case MS_GUARD:
+#ifndef GOLDOBJ
 	    if (u.ugold)
+#else
+	    if (money_cnt(invent))
+#endif
 		verbl_msg = "Please drop that gold and follow me.";
 	    else
 		verbl_msg = "Please follow me.";
@@ -711,8 +807,6 @@ register struct monst *mtmp;
 	    break;
     }
 
-    if (!canspotmon(mtmp))
-	map_invisible(mtmp->mx, mtmp->my);
     if (pline_msg) pline("%s %s", Monnam(mtmp), pline_msg);
     else if (verbl_msg) verbalize(verbl_msg);
     return(1);
@@ -737,7 +831,7 @@ dochat()
     register int tx,ty;
     struct obj *otmp;
 
-    if (youmonst.data->msound == MS_SILENT) {
+    if (is_silent(youmonst.data)) {
 	pline("As %s, you cannot speak.", an(youmonst.data->mname));
 	return(0);
     }
@@ -766,8 +860,12 @@ dochat()
 	return(1);
     }
 
-    (void) getdir("Talk to whom? [in what direction]");
+    (void) getdir("Talk to whom? (in what direction)");
 
+#ifdef STEED
+    if (u.usteed && u.dz > 0)
+	return (domonnoise(u.usteed));
+#endif
     if (u.dz) {
 	pline("They won't hear you %s there.", u.dz < 0 ? "up" : "down");
 	return(0);
@@ -814,6 +912,92 @@ dochat()
     return domonnoise(mtmp);
 }
 
+#ifdef USER_SOUNDS
+
+extern void FDECL(play_usersound, (const char*, int));
+
+typedef struct audio_mapping_rec {
+	struct re_pattern_buffer regex;
+	char *filename;
+	int volume;
+	struct audio_mapping_rec *next;
+} audio_mapping;
+
+static audio_mapping *soundmap = 0;
+
+char* sounddir = ".";
+
+/* adds a sound file mapping, returns 0 on failure, 1 on success */
+int
+add_sound_mapping(mapping)
+const char *mapping;
+{
+	char text[256];
+	char filename[256];
+	char filespec[256];
+	int volume;
+
+	if (sscanf(mapping, "MESG \"%255[^\"]\"%*[\t ]\"%255[^\"]\" %d",
+		   text, filename, &volume) == 3) {
+	    const char *err;
+	    audio_mapping *new_map;
+
+	    if (strlen(sounddir) + strlen(filename) > 254) {
+		raw_print("sound file name too long");
+		return 0;
+	    }
+	    Sprintf(filespec, "%s/%s", sounddir, filename);
+
+	    if (can_read_file(filespec)) {
+		new_map = (audio_mapping *)alloc(sizeof(audio_mapping));
+		new_map->regex.translate = 0;
+		new_map->regex.fastmap = 0;
+		new_map->regex.buffer = 0;
+		new_map->regex.allocated = 0;
+		new_map->regex.regs_allocated = REGS_FIXED;
+		new_map->filename = strdup(filespec);
+		new_map->volume = volume;
+		new_map->next = soundmap;
+
+		err = re_compile_pattern(text, strlen(text), &new_map->regex);
+
+		if (err) {
+		    raw_print(err);
+		    free(new_map->filename);
+		    free(new_map);
+		    return 0;
+		} else {
+		    soundmap = new_map;
+		}
+	    } else {
+		Sprintf(text, "cannot read %.243s", filespec);
+		raw_print(text);
+		return 0;
+	    }
+	} else {
+	    raw_print("syntax error in SOUND");
+	    return 0;
+	}
+
+	return 1;
+}
+
+void
+play_sound_for_message(msg)
+const char* msg;
+{
+	audio_mapping* cursor = soundmap;
+
+	while (cursor) {
+	    if (re_search(&cursor->regex, msg, strlen(msg), 0, 9999, 0) >= 0) {
+		play_usersound(cursor->filename, cursor->volume);
+	    }
+	    cursor = cursor->next;
+	}
+}
+
+#endif /* USER_SOUNDS */
+
 #endif /* OVLB */
 
 /*sounds.c*/
