@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)spell.c	3.3	1999/11/01	*/
+/*	SCCS Id: @(#)spell.c	3.4	2002/02/12	*/
 /*	Copyright (c) M. Stephenson 1988			  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -20,17 +20,17 @@ static NEARDATA struct obj *book;	/* last/current book being xscribed */
 #define spellet(spell)	\
 	((char)((spell < 26) ? ('a' + spell) : ('A' + spell - 26)))
 
-static int FDECL(spell_let_to_idx, (CHAR_P));
-static void FDECL(cursed_book, (int));
-static void FDECL(deadbook, (struct obj *));
+STATIC_DCL int FDECL(spell_let_to_idx, (CHAR_P));
+STATIC_DCL void FDECL(cursed_book, (int));
+STATIC_DCL void FDECL(deadbook, (struct obj *));
 STATIC_PTR int NDECL(learn);
-static boolean FDECL(getspell, (int *));
-static boolean FDECL(dospellmenu, (const char *,int,int *));
-static int FDECL(percent_success, (int));
-static int NDECL(throwspell);
-static void NDECL(cast_protection);
-static const char *FDECL(spelltypemnemonic, (int));
-static int FDECL(isqrt, (int));
+STATIC_DCL boolean FDECL(getspell, (int *));
+STATIC_DCL boolean FDECL(dospellmenu, (const char *,int,int *));
+STATIC_DCL int FDECL(percent_success, (int));
+STATIC_DCL int NDECL(throwspell);
+STATIC_DCL void NDECL(cast_protection);
+STATIC_DCL const char *FDECL(spelltypemnemonic, (int));
+STATIC_DCL int FDECL(isqrt, (int));
 
 /* The roles[] table lists the role-specific values for tuning
  * percent_success().
@@ -53,7 +53,7 @@ static int FDECL(isqrt, (int));
  *
  *	The arms penalty is lessened for trained fighters Bar, Kni, Ran,
  *	Sam, Val -
- *	the penalty is its metal interference, not encumberance.
+ *	the penalty is its metal interference, not encumbrance.
  *	The `spelspec' is a single spell which is fundamentally easier
  *	 for that role to cast.
  *
@@ -89,7 +89,7 @@ static int FDECL(isqrt, (int));
 static const char explodes[] = "radiates explosive energy";
 
 /* convert a letter into a number in the range 0..51, or -1 if not a letter */
-static int
+STATIC_OVL int
 spell_let_to_idx(ilet)
 char ilet;
 {
@@ -102,7 +102,7 @@ char ilet;
     return -1;
 }
 
-static void
+STATIC_OVL void
 cursed_book(lev)
 	register int	lev;
 {
@@ -131,10 +131,15 @@ cursed_book(lev)
 		    if (uarmg->oerodeproof || !is_corrodeable(uarmg)) {
 			Your("gloves seem unaffected.");
 		    } else if (uarmg->oeroded2 < MAX_ERODE) {
-			Your("gloves corrode%s!",
-			     uarmg->oeroded2+1 == MAX_ERODE ? " completely" :
-			     uarmg->oeroded2 ? " further" : "");
-			uarmg->oeroded2++;
+			if (uarmg->greased) {
+			    grease_protect(uarmg, "gloves", &youmonst);
+			} else {
+			    Your("gloves corrode%s!",
+				 uarmg->oeroded2+1 == MAX_ERODE ?
+				 " completely" : uarmg->oeroded2 ?
+				 " further" : "");
+			    uarmg->oeroded2++;
+			}
 		    } else
 			Your("gloves %s completely corroded.",
 			     Blind ? "feel" : "look");
@@ -162,7 +167,7 @@ cursed_book(lev)
 }
 
 /* special effects for The Book of the Dead */
-static void
+STATIC_OVL void
 deadbook(book2)
 struct obj *book2;
 {
@@ -238,7 +243,9 @@ raise_dead:
     } else if(book2->blessed) {
 	for(mtmp = fmon; mtmp; mtmp = mtmp2) {
 	    mtmp2 = mtmp->nmon;		/* tamedog() changes chain */
-	    if(is_undead(mtmp->data) && cansee(mtmp->mx, mtmp->my)) {
+	    if (DEADMONSTER(mtmp)) continue;
+
+	    if (is_undead(mtmp->data) && cansee(mtmp->mx, mtmp->my)) {
 		mtmp->mpeaceful = TRUE;
 		if(sgn(mtmp->data->maligntyp) == sgn(u.ualign.type)
 		   && distu(mtmp->mx, mtmp->my) < 4)
@@ -247,7 +254,7 @@ raise_dead:
 			    mtmp->mtame++;
 		    } else
 			(void) tamedog(mtmp, (struct obj *)0);
-		else mtmp->mflee = TRUE;
+		else monflee(mtmp, 0, FALSE, TRUE);
 	    }
 	}
     } else {
@@ -273,6 +280,8 @@ learn()
 	char splname[BUFSZ];
 	boolean costly = TRUE;
 
+	/* JDS: lenses give 50% faster reading; 33% smaller read time */
+	if (delay && ublindf && ublindf->otyp == LENSES && rn2(2)) delay++;
 	if (delay) {	/* not if (delay++), so at end delay == 0 */
 		delay++;
 		return(1); /* still busy */
@@ -372,26 +381,31 @@ register struct obj *spellbook;
 
 		/* Books are often wiser than their readers (Rus.) */
 		spellbook->in_use = TRUE;
-		if (!spellbook->blessed && spellbook->otyp != SPE_BOOK_OF_THE_DEAD) {
-			if (spellbook->cursed) {
-			    too_hard = TRUE;
-			} else {
-			    /* uncursed - chance to fail */
-			    int read_ability = ACURR(A_INT) + 4 + u.ulevel/2
-				- 2*objects[booktype].oc_level;
-			    /* only wizards know if a spell is too difficult */
-			    if (Role_if(PM_WIZARD) && read_ability < 20) {
-				char qbuf[QBUFSZ];
-				Sprintf(qbuf,
+		if (!spellbook->blessed &&
+		    spellbook->otyp != SPE_BOOK_OF_THE_DEAD) {
+		    if (spellbook->cursed) {
+			too_hard = TRUE;
+		    } else {
+			/* uncursed - chance to fail */
+			int read_ability = ACURR(A_INT) + 4 + u.ulevel/2
+			    - 2*objects[booktype].oc_level
+			    + ((ublindf && ublindf->otyp == LENSES) ? 2 : 0);
+			/* only wizards know if a spell is too difficult */
+			if (Role_if(PM_WIZARD) && read_ability < 20) {
+			    char qbuf[QBUFSZ];
+			    Sprintf(qbuf,
 		      "This spellbook is %sdifficult to comprehend. Continue?",
-					(read_ability < 12 ? "very " : ""));
-				if (ynq(qbuf) != 'y') return(1);
-			    }
-			    /* its up to random luck now */
-			    if (rnd(20) > read_ability) {
-				too_hard = TRUE;
+				    (read_ability < 12 ? "very " : ""));
+			    if (yn(qbuf) != 'y') {
+				spellbook->in_use = FALSE;
+				return(1);
 			    }
 			}
+			/* its up to random luck now */
+			if (rnd(20) > read_ability) {
+			    too_hard = TRUE;
+			}
+		    }
 		}
 
 		if (too_hard) {
@@ -439,6 +453,15 @@ register struct obj *spellbook;
 	return(1);
 }
 
+/* a spellbook has been destroyed or the character has changed levels;
+   the stored address for the current book is no longer valid */
+void
+book_disappears(obj)
+struct obj *obj;
+{
+	if (obj == book) book = (struct obj *)0;
+}
+
 /* renaming an object usually results in it having a different address;
    so the sequence start reading, get interrupted, name the book, resume
    reading would read the "new" book from scratch */
@@ -470,7 +493,7 @@ age_spells()
  * Return TRUE if a spell was picked, with the spell index in the return
  * parameter.  Otherwise return FALSE.
  */
-static boolean
+STATIC_OVL boolean
 getspell(spell_no)
 	int *spell_no;
 {
@@ -523,7 +546,7 @@ docast()
 	return 0;
 }
 
-static const char *
+STATIC_OVL const char *
 spelltypemnemonic(skill)
 int skill;
 {
@@ -555,7 +578,7 @@ int booktype;
 	return (objects[booktype].oc_skill);
 }
 
-static void
+STATIC_OVL void
 cast_protection()
 {
 	int loglev = 0;
@@ -668,14 +691,6 @@ boolean atme;
 		if (spellid(spell) != SPE_DETECT_FOOD) {
 			int hungr = energy * 2;
 
-			/* don't put player (quite) into fainting from
-			 * casting a spell, particularly since they might
-			 * not even be hungry at the beginning; however,
-			 * this is low enough that they must eat before
-			 * casting anything else except detect food
-			 */
-			if (hungr > u.uhunger-3)
-				hungr = u.uhunger-3;
 			/* If hero is a wizard, their current intelligence
 			 * (bonuses + temporary + current)
 			 * affects hunger reduction in casting a spell.
@@ -690,12 +705,22 @@ boolean atme;
 			 * understand quite well how to cast spells.
 			 */
 			intell = acurr(A_INT);
+			if (!Role_if(PM_WIZARD)) intell = 10;
 			switch (intell) {
-				case 18:
+				case 25: case 24: case 23: case 22:
+				case 21: case 20: case 19: case 18:
 				case 17: hungr = 0; break;
 				case 16: hungr /= 4; break;
 				case 15: hungr /= 2; break;
 			}
+			/* don't put player (quite) into fainting from
+			 * casting a spell, particularly since they might
+			 * not even be hungry at the beginning; however,
+			 * this is low enough that they must eat before
+			 * casting anything else except detect food
+			 */
+			if (hungr > u.uhunger-3)
+				hungr = u.uhunger-3;
 			morehungry(hungr);
 		}
 	}
@@ -724,14 +749,10 @@ boolean atme;
 
 	switch(pseudo->otyp)  {
 	/*
-	 * At first these act as expected.  As the character increases in
-	 * experience the spell increases in its ability.  Initially the
-	 * spells have their expected levels of damage.  When the hero level
-	 * reaches three times the level of the spell the spell does special
-	 * damage.  This special damage is indicated before each spell.  Note
-	 * even when the hero reaches three times the level of the spell she
-	 * still has the choice of casting either spell. Also the new level
-	 * of spell has an increased cost in casting it.
+	 * At first spells act as expected.  As the hero increases in skill
+	 * with the appropriate spell type, some spells increase in their
+	 * effects, e.g. more damage, further distance, and so on, without
+	 * additional cost to the spellcaster.
 	 */
 	case SPE_CONE_OF_COLD:
 	case SPE_FIREBALL:
@@ -741,18 +762,21 @@ boolean atme;
 		    n=rnd(8)+1;
 		    while(n--) {
 			if(!u.dx && !u.dy && !u.dz) {
-			    if ((damage = zapyourself(pseudo, TRUE)) != 0)
-				losehp(damage,
-				     self_pronoun("zapped %sself with a spell",
-						"him"),
-				       NO_KILLER_PREFIX);
+			    if ((damage = zapyourself(pseudo, TRUE)) != 0) {
+				char buf[BUFSZ];
+				Sprintf(buf, "zapped %sself with a spell", uhim());
+				losehp(damage, buf, NO_KILLER_PREFIX);
+			    }
 			} else {
 			    explode(u.dx, u.dy,
 				    pseudo->otyp - SPE_MAGIC_MISSILE + 10,
-				    u.ulevel/2 + 1 + spell_damage_bonus(), 0);
+				    u.ulevel/2 + 1 + spell_damage_bonus(), 0,
+					(pseudo->otyp == SPE_CONE_OF_COLD) ?
+						EXPL_FROSTY : EXPL_FIERY);
 			}
 			u.dx = cc.x+rnd(3)-2; u.dy = cc.y+rnd(3)-2;
-			if (!cansee(u.dx,u.dy) || IS_STWALL(levl[u.dx][u.dy].typ)) {
+			if (!isok(u.dx,u.dy) || !cansee(u.dx,u.dy) ||
+			    IS_STWALL(levl[u.dx][u.dy].typ) || u.uswallow) {
 			    /* Spell is reflected back to center */
 			    u.dx = cc.x;
 			    u.dy = cc.y;
@@ -785,27 +809,24 @@ boolean atme;
 			if (atme) u.dx = u.dy = u.dz = 0;
 			else (void) getdir((char *)0);
 			if(!u.dx && !u.dy && !u.dz) {
-			    if ((damage = zapyourself(pseudo, TRUE)) != 0)
-				losehp(damage,
-				     self_pronoun("zapped %sself with a spell",
-						  "him"),
-				     NO_KILLER_PREFIX);
+			    if ((damage = zapyourself(pseudo, TRUE)) != 0) {
+				char buf[BUFSZ];
+				Sprintf(buf, "zapped %sself with a spell", uhim());
+				losehp(damage, buf, NO_KILLER_PREFIX);
+			    }
 			} else weffects(pseudo);
 		} else weffects(pseudo);
+		update_inventory();	/* spell may modify inventory */
 		break;
 
 	/* these are all duplicates of scroll effects */
 	case SPE_REMOVE_CURSE:
-		/*
-		 * When the hero is skilled enough the spell is equivalent
-		 * to a blessed scroll.
-		 */
-		if (role_skill >= P_SKILLED)
-		    pseudo->blessed=1;
-		/* fall through */
 	case SPE_CONFUSE_MONSTER:
 	case SPE_DETECT_FOOD:
 	case SPE_CAUSE_FEAR:
+		/* high skill yields effect equivalent to blessed scroll */
+		if (role_skill >= P_SKILLED) pseudo->blessed = 1;
+		/* fall through */
 	case SPE_CHARM_MONSTER:
 	case SPE_MAGIC_MAPPING:
 	case SPE_CREATE_MONSTER:
@@ -819,6 +840,9 @@ boolean atme;
 	case SPE_DETECT_MONSTERS:
 	case SPE_LEVITATION:
 	case SPE_RESTORE_ABILITY:
+		/* high skill yields effect equivalent to blessed potion */
+		if (role_skill >= P_SKILLED) pseudo->blessed = 1;
+		/* fall through */
 	case SPE_INVISIBILITY:
 		(void) peffects(pseudo);
 		break;
@@ -829,8 +853,9 @@ boolean atme;
 	case SPE_CURE_SICKNESS:
 		if (Sick) You("are no longer ill.");
 		if (Slimed) {
-		    pline("The slime disappears!");
+		    pline_The("slime disappears!");
 		    Slimed = 0;
+		 /* flags.botl = 1; -- healup() handles this */
 		}
 		healup(0, 0, TRUE, FALSE);
 		break;
@@ -849,8 +874,8 @@ boolean atme;
 		cast_protection();
 		break;
 	case SPE_JUMPING:
-		if (!jump(role_skill))
-			pline("Nothing happens.");
+		if (!jump(max(role_skill,1)))
+			pline(nothing_happens);
 		break;
 	default:
 		impossible("Unknown spell %d attempted.", spell);
@@ -867,7 +892,7 @@ boolean atme;
 }
 
 /* Choose location where spell takes effect. */
-static int
+STATIC_OVL int
 throwspell()
 {
 	coord cc;
@@ -885,10 +910,10 @@ throwspell()
 	    return 0;	/* user pressed ESC */
 	/* The number of moves from hero to where the spell drops.*/
 	if (distmin(u.ux, u.uy, cc.x, cc.y) > 10) {
-	    pline("The spell dissipates over the distance!");
+	    pline_The("spell dissipates over the distance!");
 	    return 0;
 	} else if (u.uswallow) {
-	    pline("The spell is cut short!");
+	    pline_The("spell is cut short!");
 	    exercise(A_WIS, FALSE); /* What were you THINKING! */
 	    u.dx = 0;
 	    u.dy = 0;
@@ -947,7 +972,7 @@ dovspell()
 	return 0;
 }
 
-static boolean
+STATIC_OVL boolean
 dospellmenu(prompt, splaction, spell_no)
 const char *prompt;
 int splaction;	/* SPELLMENU_CAST, SPELLMENU_VIEW, or spl_book[] index */
@@ -972,10 +997,14 @@ int *spell_no;
 	 * To do it right would require that we implement columns
 	 * in the window-ports (say via a tab character).
 	 */
-	Sprintf(buf, "%-20s     Level  %-12s Fail", "    Name", "Category");
+	if (!iflags.menu_tab_sep)
+		Sprintf(buf, "%-20s     Level  %-12s Fail", "    Name", "Category");
+	else
+		Sprintf(buf, "Name\tLevel\tCategory\tFail");
 	add_menu(tmpwin, NO_GLYPH, &any, 0, 0, ATR_NONE, buf, MENU_UNSELECTED);
 	for (i = 0; i < MAXSPELL && spellid(i) != NO_SPELL; i++) {
-	        Sprintf(buf, "%-20s  %2d%s   %-12s %3d%%",
+		Sprintf(buf, iflags.menu_tab_sep ?
+			"%s\t%-d%s\t%s\t%-d%%" : "%-20s  %2d%s   %-12s %3d%%",
 			spellname(i), spellev(i),
 			spellknow(i) ? " " : "*",
 			spelltypemnemonic(spell_skilltype(spellid(i))),
@@ -1014,7 +1043,7 @@ int *spell_no;
 }
 
 /* Integer square root function without using floating point. */
-static int
+STATIC_OVL int
 isqrt(val)
 int val;
 {
@@ -1028,7 +1057,7 @@ int val;
     return rt;
 }
 
-static int
+STATIC_OVL int
 percent_success(spell)
 int spell;
 {
@@ -1083,7 +1112,8 @@ int spell;
 	 * The difficulty is based on the hero's level and their skill level
 	 * in that spell type.
 	 */
-	skill = P_SKILL(spell_skilltype(spellid(spell)))-1;
+	skill = P_SKILL(spell_skilltype(spellid(spell)));
+	skill = max(skill,P_UNSKILLED) - 1;	/* unskilled => 0 */
 	difficulty= (spellev(spell)-1) * 4 - ((skill * 6) + (u.ulevel/3) + 1);
 
 	if (difficulty > 0) {
@@ -1121,8 +1151,8 @@ int spell;
 
 	/* Finally, chance (based on player intell/wisdom and level) is
 	 * combined with ability (based on player intrinsics and
-	 * encumberances).  No matter how intelligent/wise and advanced
-	 * a player is, intrinsics and encumberance can prevent casting;
+	 * encumbrances).  No matter how intelligent/wise and advanced
+	 * a player is, intrinsics and encumbrance can prevent casting;
 	 * and no matter how able, learning is always required.
 	 */
 	chance = chance * (20-splcaster) / 15 - splcaster;
@@ -1159,5 +1189,4 @@ struct obj *obj;
 	return;
 }
 
-
 /*spell.c*/
