@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)timeout.c	3.3	1999/02/13	*/
+/*	SCCS Id: @(#)timeout.c	3.4	2000/09/28	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -12,6 +12,7 @@ STATIC_DCL void NDECL(slime_dialogue);
 STATIC_DCL void NDECL(slip_or_trip);
 STATIC_DCL void FDECL(see_lamp_flicker, (struct obj *, const char *));
 STATIC_DCL void FDECL(lantern_message, (struct obj *));
+STATIC_DCL void FDECL(cleanup_burn, (genericptr_t,long));
 
 #ifdef OVLB
 
@@ -29,11 +30,11 @@ stoned_dialogue()
 {
 	register long i = (Stoned & TIMEOUT);
 
-	if(i > 0 && i <= SIZE(stoned_texts))
+	if (i > 0L && i <= SIZE(stoned_texts))
 		pline(stoned_texts[SIZE(stoned_texts) - i]);
-	if(i == 5)
+	if (i == 5L)
 		HFast = 0L;
-	if(i == 3)
+	if (i == 3L)
 		nomul(-3);
 	exercise(A_DEX, FALSE);
 }
@@ -95,18 +96,24 @@ choke_dialogue()
 	if(i > 0 && i <= SIZE(choke_texts)) {
 	    if (Breathless || !rn2(50))
 		pline(choke_texts2[SIZE(choke_texts2) - i], body_part(NECK));
-	    else
-		pline(choke_texts[SIZE(choke_texts) - i], hcolor(blue));
+	    else {
+		const char *str = choke_texts[SIZE(choke_texts)-i];
+
+		if (index(str, '%'))
+		    pline(str, hcolor(blue));
+		else
+		    pline(str);
+	    }
 	}
 	exercise(A_STR, FALSE);
 }
 
 static NEARDATA const char *slime_texts[] = {
-	"You are turning a little green.",           /* 5 */
+	"You are turning a little %s.",           /* 5 */
 	"Your limbs are getting oozy.",              /* 4 */
 	"Your skin begins to peel away.",            /* 3 */
-	"You are turning into a green slime.",       /* 2 */
-	"You have become a green slime."             /* 1 */
+	"You are turning into %s.",       /* 2 */
+	"You have become %s."             /* 1 */
 };
 
 STATIC_OVL void
@@ -114,11 +121,24 @@ slime_dialogue()
 {
 	register long i = (Slimed & TIMEOUT) / 2L;
 
-	if (((Slimed & TIMEOUT) % 2L) && i >= 0
-	    && i < SIZE(slime_texts))
-	    pline(slime_texts[SIZE(slime_texts) - i - 1]);
-	if(i == 4)
-	    HFast = 0;
+	if (((Slimed & TIMEOUT) % 2L) && i >= 0L
+		&& i < SIZE(slime_texts)) {
+	    const char *str = slime_texts[SIZE(slime_texts) - i - 1L];
+
+	    if (index(str, '%')) {
+		if (i == 4L) {	/* "you are turning green" */
+		    if (!Blind)	/* [what if you're already green?] */
+			pline(str, hcolor(green));
+		} else
+		    pline(str, an(Hallucination ? rndmonnam() : "green slime"));
+	    } else
+		pline(str);
+	}
+	if (i == 3L) {	/* limbs becoming oozy */
+	    HFast = 0L;	/* lose intrinsic speed */
+	    stop_occupation();
+	    if (multi > 0) nomul(0);
+	}
 	exercise(A_DEX, FALSE);
 }
 
@@ -128,6 +148,7 @@ burn_away_slime()
 	if (Slimed) {
 	    pline_The("slime that covers you is burned away!");
 	    Slimed = 0L;
+	    flags.botl = 1;
 	}
 	return;
 }
@@ -142,8 +163,12 @@ nh_timeout()
 	register struct prop *upp;
 	int sleeptime;
 	int m_idx;
+	int baseluck = (flags.moonphase == FULL_MOON) ? 1 : 0;
 
-	if(u.uluck && moves % (u.uhave.amulet || u.ugangr ? 300 : 600) == 0) {
+	if (flags.friday13) baseluck -= 1;
+
+	if (u.uluck != baseluck &&
+		moves % (u.uhave.amulet || u.ugangr ? 300 : 600) == 0) {
 	/* Cursed luckstones stop bad luck from timing out; blessed luckstones
 	 * stop good luck from timing out; normal luckstones stop both;
 	 * neither is stopped if you don't have a luckstone.
@@ -151,9 +176,6 @@ nh_timeout()
 	 */
 	    register int time_luck = stone_luck(FALSE);
 	    boolean nostone = !carrying(LUCKSTONE) && !stone_luck(TRUE);
-	    int baseluck = (flags.moonphase == FULL_MOON) ? 1 : 0;
-
-	    baseluck -= (flags.friday13 ? 1 : 0);
 
 	    if(u.uluck > baseluck && (nostone || time_luck < 0))
 		u.uluck--;
@@ -196,16 +218,28 @@ nh_timeout()
 	    if((upp->intrinsic & TIMEOUT) && !(--upp->intrinsic & TIMEOUT)) {
 		switch(upp - u.uprops){
 		case STONED:
+			if (delayed_killer && !killer) {
+				killer = delayed_killer;
+				delayed_killer = 0;
+			}
 			if (!killer) {
-				killer_format = KILLED_BY;
-				killer = "petrification";
-			} done(STONING);
+				/* leaving killer_format would make it
+				   "petrified by petrification" */
+				killer_format = NO_KILLER_PREFIX;
+				killer = "killed by petrification";
+			}
+			done(STONING);
 			break;
 		case SLIMED:
+			if (delayed_killer && !killer) {
+				killer = delayed_killer;
+				delayed_killer = 0;
+			}
 			if (!killer) {
-				killer_format = KILLED_BY;
-				killer = "turning into green slime";
-			} done(TURNED_SLIME);
+				killer_format = NO_KILLER_PREFIX;
+				killer = "turned into green slime";
+			}
+			done(TURNED_SLIME);
 			break;
 		case VOMITING:
 			make_vomiting(0L, TRUE);
@@ -277,11 +311,11 @@ nh_timeout()
 		case SLEEPING:
 			if (unconscious() || Sleep_resistance)
 				HSleeping += rnd(100);
-			else if (ESleeping) {
+			else if (Sleeping) {
 				You("fall asleep.");
 				sleeptime = rnd(20);
 				fall_asleep(-sleeptime, TRUE);
-				HSleeping = sleeptime + rnd(100);
+				HSleeping += sleeptime + rnd(100);
 			}
 			break;
 		case LEVITATION:
@@ -392,7 +426,8 @@ long timeout;
 
 	mon = mon2 = (struct monst *)0;
 	mnum = big_to_little(egg->corpsenm);
-	yours = egg->spe != 0;
+	/* The identity of one's father is learned, not innate */
+	yours = (egg->spe || (!flags.female && carried(egg) && !rn2(2)));
 	silent = (timeout != monstermoves);	/* hatched while away */
 
 	/* only can hatch when in INVENT, FLOOR, MINVENT */
@@ -409,12 +444,10 @@ long timeout;
 		       same dungeon level, or any dragon egg which hatches
 		       while it's in your inventory */
 		    if ((yours && !silent) ||
-			(mon->data->mlet == S_DRAGON &&
-				egg->where == OBJ_INVENT)) {
+			(carried(egg) && mon->data->mlet == S_DRAGON)) {
 			if ((mon2 = tamedog(mon, (struct obj *)0)) != 0) {
 			    mon = mon2;
-			    if (egg->where == OBJ_INVENT &&
-				    mon->data->mlet != S_DRAGON)
+			    if (carried(egg) && mon->data->mlet != S_DRAGON)
 				mon->mtame = 20;
 			}
 		    }
@@ -461,7 +494,8 @@ long timeout;
 	    if (cansee_hatchspot) {
 		Sprintf(monnambuf, "%s%s",
 			siblings ? "some " : "",
-			siblings ? makeplural(m_monnam(mon)) : a_monnam(mon));
+			siblings ?
+			makeplural(m_monnam(mon)) : an(m_monnam(mon)));
 		/* we don't learn the egg type here because learning
 		   an egg type requires either seeing the egg hatch
 		   or being familiar with the egg already,
@@ -472,16 +506,17 @@ long timeout;
 	    switch (egg->where) {
 		case OBJ_INVENT:
 		    knows_egg = TRUE; /* true even if you are blind */
-		    if (Blind)
+		    if (!cansee_hatchspot)
 			You_feel("%s %s from your pack!", something,
 			    locomotion(mon->data, "drop"));
 		    else
 			You("see %s %s out of your pack!",
 			    monnambuf, locomotion(mon->data, "drop"));
 		    if (yours) {
-			pline("%s cries sound like \"%s.\"",
+			pline("%s cries sound like \"%s%s\"",
 			    siblings ? "Their" : "Its",
-			    flags.female ? "mommy" : "daddy");
+			    flags.female ? "mommy" : "daddy",
+			    egg->spe ? "." : "?");
 		    } else if (mon->data->mlet == S_DRAGON) {
 			verbalize("Gleep!");		/* Mything eggs :-) */
 		    }
@@ -532,6 +567,8 @@ long timeout;
 		    (void) start_timer((long)rnd(12), TIMER_OBJECT,
 					HATCH_EGG, (genericptr_t)egg);
 		}
+	    } else if (carried(egg)) {
+		useup(egg);
 	    } else {
 		/* free egg here because we use it above */
 		obj_extract_self(egg);
@@ -579,8 +616,12 @@ slip_or_trip()
 	struct obj *otmp = vobj_at(u.ux, u.uy);
 	const char *what, *pronoun;
 	char buf[BUFSZ];
+	boolean on_foot = TRUE;
+#ifdef STEED
+	if (u.usteed) on_foot = FALSE;
+#endif
 
-	if (otmp) {		/* trip over something in particular */
+	if (otmp && on_foot) {		/* trip over something in particular */
 	    /*
 		If there is only one item, it will have just been named
 		during the move, so refer to by via pronoun; otherwise,
@@ -603,22 +644,51 @@ slip_or_trip()
 		You("trip over %s.", what);
 	    }
 	} else if (rn2(3) && is_ice(u.ux, u.uy)) {
-	    You("%s on the ice.", rn2(2) ? "slip" : "slide");
-	} else switch (rn2(4)) {
-	    case 1:
-		You("trip over your own %s.", Hallucination ?
-			"elbow" : makeplural(body_part(FOOT)));
-		break;
-	    case 2:
-		You("slip %s.", Hallucination ?
-			"on a banana peel" : "and nearly fall");
-		break;
-	    case 3:
-		You("flounder.");
-		break;
-	    default:
-		You("stumble.");
-		break;
+	    pline("%s %s%s on the ice.",
+#ifdef STEED
+		u.usteed ? upstart(x_monnam(u.usteed,
+				u.usteed->mnamelth ? ARTICLE_NONE : ARTICLE_THE,
+				(char *)0, SUPPRESS_SADDLE, FALSE)) :
+#endif
+		"You", rn2(2) ? "slip" : "slide", on_foot ? "" : "s");
+	} else {
+	    if (on_foot) {
+		switch (rn2(4)) {
+		  case 1:
+			You("trip over your own %s.", Hallucination ?
+				"elbow" : makeplural(body_part(FOOT)));
+			break;
+		  case 2:
+			You("slip %s.", Hallucination ?
+				"on a banana peel" : "and nearly fall");
+			break;
+		  case 3:
+			You("flounder.");
+			break;
+		  default:
+			You("stumble.");
+			break;
+		}
+	    }
+#ifdef STEED
+	    else {
+		switch (rn2(4)) {
+		  case 1:
+			Your("%s slip out of the stirrups.", makeplural(body_part(FOOT)));
+			break;
+		  case 2:
+			You("let go of the reins.");
+			break;
+		  case 3:
+			You("bang into the saddle-horn.");
+			break;
+		  default:
+			You("slide to one side of the saddle.");
+			break;
+		}
+		dismount_steed(DISMOUNT_FELL);
+	    }
+#endif
 	}
 }
 
@@ -704,7 +774,7 @@ long timeout;
 
 	/* only interested in INVENT, FLOOR, and MINVENT */
 	if (get_obj_location(obj, &x, &y, 0)) {
-	    canseeit = cansee(x, y);
+	    canseeit = !Blind && cansee(x, y);
 	    /* set up `whose[]' to be "Your" or "Fred's" or "The goblin's" */
 	    (void) Shk_Your(whose, obj);
 	} else {
@@ -973,7 +1043,8 @@ begin_burn(obj, already_lit)
 	long turns = 0;
 	boolean do_timer = TRUE;
 
-	if (obj->age == 0 && obj->otyp != MAGIC_LAMP) return;
+	if (obj->age == 0 && obj->otyp != MAGIC_LAMP && !artifact_light(obj))
+	    return;
 
 	switch (obj->otyp) {
 	    case MAGIC_LAMP:
@@ -1015,8 +1086,15 @@ begin_burn(obj, already_lit)
 		break;
 
 	    default:
-		impossible("begin burn: unexpected %s", xname(obj));
-		turns = obj->age;
+                /* [ALI] Support artifact light sources */
+                if (artifact_light(obj)) {
+		    obj->lamplit = 1;
+		    do_timer = FALSE;
+		    radius = 2;
+		} else {
+		    impossible("begin burn: unexpected %s", xname(obj));
+		    turns = obj->age;
+		}
 		break;
 	}
 
@@ -1025,11 +1103,14 @@ begin_burn(obj, already_lit)
 					BURN_OBJECT, (genericptr_t)obj)) {
 		obj->lamplit = 1;
 		obj->age -= turns;
-		if (obj->where == OBJ_INVENT && !already_lit)
+		if (carried(obj) && !already_lit)
 		    update_inventory();
 	    } else {
 		obj->lamplit = 0;
 	    }
+	} else {
+	    if (carried(obj) && !already_lit)
+		update_inventory();
 	}
 
 	if (obj->lamplit && !already_lit) {
@@ -1051,30 +1132,55 @@ end_burn(obj, timer_attached)
 	struct obj *obj;
 	boolean timer_attached;
 {
-	long expire_time;
-
 	if (!obj->lamplit) {
 	    impossible("end_burn: obj %s not lit", xname(obj));
 	    return;
 	}
 
-	del_light_source(LS_OBJECT, (genericptr_t) obj);
+	if (obj->otyp == MAGIC_LAMP || artifact_light(obj))
+	    timer_attached = FALSE;
 
-	if (obj->otyp == MAGIC_LAMP) timer_attached = FALSE;
-	if (timer_attached) {
-	    expire_time = stop_timer(BURN_OBJECT, (genericptr_t) obj);
-	    if (expire_time)
-		/* restore unused time */
-		obj->age += expire_time - monstermoves;
-	    else
-		impossible("end_burn: obj %s not timed!", xname(obj));
-	}
-	obj->lamplit = 0;
-
-	if (obj->where == OBJ_INVENT)
-	    update_inventory();
+	if (!timer_attached) {
+	    /* [DS] Cleanup explicitly, since timer cleanup won't happen */
+	    del_light_source(LS_OBJECT, (genericptr_t)obj);
+	    obj->lamplit = 0;
+	    if (obj->where == OBJ_INVENT)
+		update_inventory();
+	} else if (!stop_timer(BURN_OBJECT, (genericptr_t) obj))
+	    impossible("end_burn: obj %s not timed!", xname(obj));
 }
 
+#endif /* OVL1 */
+#ifdef OVL0
+
+/*
+ * Cleanup a burning object if timer stopped.
+ */
+static void
+cleanup_burn(arg, expire_time)
+    genericptr_t arg;
+    long expire_time;
+{
+    struct obj *obj = (struct obj *)arg;
+    if (!obj->lamplit) {
+	impossible("cleanup_burn: obj %s not lit", xname(obj));
+	return;
+    }
+
+    del_light_source(LS_OBJECT, arg);
+
+    /* restore unused time */
+    obj->age += expire_time - monstermoves;
+
+    obj->lamplit = 0;
+
+    if (obj->where == OBJ_INVENT)
+	update_inventory();
+}
+
+#endif /* OVL0 */
+#ifdef OVL1
+
 void
 do_storms()
 {
@@ -1171,17 +1277,6 @@ do_storms()
  *		Stop all timers attached to obj.
  */
 
-
-typedef struct fe {
-    struct fe *next;		/* next item in chain */
-    long timeout;		/* when we time out */
-    unsigned long tid;		/* timer ID */
-    short kind;			/* kind of use */
-    short func_index;		/* what to call when we time out */
-    genericptr_t arg;		/* pointer to timeout argument */
-    Bitfield (needs_fixup,1);	/* does arg need to be patched? */
-} timer_element;
-
 #ifdef WIZARD
 STATIC_DCL const char *FDECL(kind_name, (SHORT_P));
 STATIC_DCL void FDECL(print_queue, (winid, timer_element *));
@@ -1189,6 +1284,7 @@ STATIC_DCL void FDECL(print_queue, (winid, timer_element *));
 STATIC_DCL void FDECL(insert_timer, (timer_element *));
 STATIC_DCL timer_element *FDECL(remove_timer, (timer_element **, SHORT_P,
 								genericptr_t));
+STATIC_DCL void FDECL(write_timer, (int, timer_element *));
 STATIC_DCL boolean FDECL(mon_is_local, (struct monst *));
 STATIC_DCL boolean FDECL(timer_is_local, (timer_element *));
 STATIC_DCL int FDECL(maybe_write_timer, (int, int, BOOLEAN_P));
@@ -1201,23 +1297,23 @@ static unsigned long timer_id = 1;
 #define VERBOSE_TIMER
 
 typedef struct {
-    timeout_proc f;
+    timeout_proc f, cleanup;
 #ifdef VERBOSE_TIMER
     const char *name;
-# define TTAB(a, b) {a,b}
+# define TTAB(a, b, c) {a,b,c}
 #else
-# define TTAB(a, b) {a}
+# define TTAB(a, b, c) {a,b}
 #endif
 } ttable;
 
 /* table of timeout functions */
 static ttable timeout_funcs[NUM_TIME_FUNCS] = {
-    TTAB(rot_organic,	"rot_organic"),
-    TTAB(rot_corpse,	"rot_corpse"),
-    TTAB(revive_mon,	"revive_mon"),
-    TTAB(burn_object,	"burn_object"),
-    TTAB(hatch_egg,	"hatch_egg"),
-    TTAB(fig_transform,	"fig_transform")
+    TTAB(rot_organic,	(timeout_proc)0,	"rot_organic"),
+    TTAB(rot_corpse,	(timeout_proc)0,	"rot_corpse"),
+    TTAB(revive_mon,	(timeout_proc)0,	"revive_mon"),
+    TTAB(burn_object,	cleanup_burn,		"burn_object"),
+    TTAB(hatch_egg,	(timeout_proc)0,	"hatch_egg"),
+    TTAB(fig_transform,	(timeout_proc)0,	"fig_transform")
 };
 #undef TTAB
 
@@ -1384,6 +1480,8 @@ genericptr_t arg;
 	timeout = doomed->timeout;
 	if (doomed->kind == TIMER_OBJECT)
 	    ((struct obj *)arg)->timed--;
+	if (timeout_funcs[doomed->func_index].cleanup)
+	    (*timeout_funcs[doomed->func_index].cleanup)(arg, timeout);
 	free((genericptr_t) doomed);
 	return timeout;
     }
@@ -1449,6 +1547,9 @@ obj_stop_timers(obj)
 		prev->next = curr->next;
 	    else
 		timer_base = curr->next;
+	    if (timeout_funcs[curr->func_index].cleanup)
+		(*timeout_funcs[curr->func_index].cleanup)(curr->arg,
+			curr->timeout);
 	    free((genericptr_t) curr);
 	} else {
 	    prev = curr;
