@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)u_init.c	3.3	1999/12/05	*/
+/*	SCCS Id: @(#)u_init.c	3.4	2002/03/02	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -12,9 +12,10 @@ struct trobj {
 	Bitfield(trbless,2);
 };
 
-static void FDECL(ini_inv, (struct trobj *));
-static void FDECL(knows_object,(int));
-static void FDECL(knows_class,(CHAR_P));
+STATIC_DCL void FDECL(ini_inv, (struct trobj *));
+STATIC_DCL void FDECL(knows_object,(int));
+STATIC_DCL void FDECL(knows_class,(CHAR_P));
+STATIC_DCL boolean FDECL(restricted_spell_discipline, (int));
 
 #define UNDEF_TYP	0
 #define UNDEF_SPE	'\177'
@@ -32,6 +33,7 @@ static struct trobj Archeologist[] = {
 	{ FOOD_RATION, 0, FOOD_CLASS, 3, 0 },
 	{ PICK_AXE, UNDEF_SPE, TOOL_CLASS, 1, UNDEF_BLESS },
 	{ TINNING_KIT, UNDEF_SPE, TOOL_CLASS, 1, UNDEF_BLESS },
+	{ TOUCHSTONE, 0, GEM_CLASS, 1, 0 },
 	{ SACK, 0, TOOL_CLASS, 1, 0 },
 	{ 0, 0, 0, 0, 0 }
 };
@@ -68,8 +70,8 @@ static struct trobj Healer[] = {
 	{ 0, 0, 0, 0, 0 }
 };
 static struct trobj Knight[] = {
-	{ LONG_SWORD, 0, WEAPON_CLASS, 1, UNDEF_BLESS },
-	{ SPEAR, 2, WEAPON_CLASS, 1, UNDEF_BLESS },
+	{ LONG_SWORD, 1, WEAPON_CLASS, 1, UNDEF_BLESS },
+	{ LANCE, 1, WEAPON_CLASS, 1, UNDEF_BLESS },
 	{ RING_MAIL, 1, ARMOR_CLASS, 1, UNDEF_BLESS },
 	{ HELMET, 0, ARMOR_CLASS, 1, UNDEF_BLESS },
 	{ SMALL_SHIELD, 0, ARMOR_CLASS, 1, UNDEF_BLESS },
@@ -79,7 +81,7 @@ static struct trobj Knight[] = {
 	{ 0, 0, 0, 0, 0 }
 };
 static struct trobj Monk[] = {
-#define M_BOOK          2
+#define M_BOOK		2
 	{ LEATHER_GLOVES, 2, ARMOR_CLASS, 1, UNDEF_BLESS },
 	{ ROBE, 1, ARMOR_CLASS, 1, UNDEF_BLESS },
 	{ UNDEF_TYP, UNDEF_SPE, SPBOOK_CLASS, 1, 1 },
@@ -211,6 +213,12 @@ static struct trobj Wishing[] = {
 	{ WAN_WISHING, 3, WAND_CLASS, 1, 0 },
 	{ 0, 0, 0, 0, 0 }
 };
+#ifdef GOLDOBJ
+static struct trobj Money[] = {
+	{ GOLD_PIECE, 0 , GOLD_CLASS, 1, 0 },
+	{ 0, 0, 0, 0, 0 }
+};
+#endif
 
 /* race-based substitutions for initial inventory;
    the weaker cloak for elven rangers is intentional--they shoot better */
@@ -268,7 +276,7 @@ static struct def_skill Skill_B[] = {
     { P_TWO_HANDED_SWORD, P_EXPERT },	{ P_SCIMITAR, P_SKILLED },
     { P_SABER, P_BASIC },		{ P_CLUB, P_SKILLED },
     { P_MACE, P_SKILLED },		{ P_MORNING_STAR, P_SKILLED },
-    { P_FLAIL, P_BASIC },               { P_HAMMER, P_EXPERT },
+    { P_FLAIL, P_BASIC },		{ P_HAMMER, P_EXPERT },
     { P_QUARTERSTAFF, P_BASIC },	{ P_SPEAR, P_SKILLED },
     { P_TRIDENT, P_SKILLED },		{ P_BOW, P_BASIC },
     { P_ATTACK_SPELL, P_SKILLED },
@@ -381,15 +389,15 @@ static struct def_skill Skill_R[] = {
 
 static struct def_skill Skill_Ran[] = {
     { P_DAGGER, P_EXPERT },		 { P_KNIFE,  P_SKILLED },
-    { P_AXE, P_SKILLED },        { P_PICK_AXE, P_BASIC },
-    { P_SHORT_SWORD, P_BASIC },  { P_MORNING_STAR, P_BASIC },
-    { P_FLAIL, P_SKILLED },      { P_HAMMER, P_BASIC },
+    { P_AXE, P_SKILLED },	 { P_PICK_AXE, P_BASIC },
+    { P_SHORT_SWORD, P_BASIC },	 { P_MORNING_STAR, P_BASIC },
+    { P_FLAIL, P_SKILLED },	 { P_HAMMER, P_BASIC },
     { P_QUARTERSTAFF, P_BASIC }, { P_POLEARMS, P_SKILLED },
-    { P_SPEAR, P_SKILLED },      { P_JAVELIN, P_EXPERT },
-    { P_TRIDENT, P_BASIC },      { P_BOW, P_EXPERT },
-    { P_SLING, P_EXPERT },       { P_CROSSBOW, P_EXPERT },
-    { P_DART, P_EXPERT },        { P_SHURIKEN, P_SKILLED },
-    { P_BOOMERANG, P_EXPERT },   { P_WHIP, P_BASIC },
+    { P_SPEAR, P_SKILLED },	 { P_JAVELIN, P_EXPERT },
+    { P_TRIDENT, P_BASIC },	 { P_BOW, P_EXPERT },
+    { P_SLING, P_EXPERT },	 { P_CROSSBOW, P_EXPERT },
+    { P_DART, P_EXPERT },	 { P_SHURIKEN, P_SKILLED },
+    { P_BOOMERANG, P_EXPERT },	 { P_WHIP, P_BASIC },
     { P_HEALING_SPELL, P_BASIC },
     { P_DIVINATION_SPELL, P_EXPERT },
     { P_ESCAPE_SPELL, P_BASIC },
@@ -484,7 +492,7 @@ static struct def_skill Skill_W[] = {
 };
 
 
-static void
+STATIC_OVL void
 knows_object(obj)
 register int obj;
 {
@@ -495,7 +503,7 @@ register int obj;
 /* Know ordinary (non-magical) objects of a certain class,
  * like all gems except the loadstone and luckstone.
  */
-static void
+STATIC_OVL void
 knows_class(sym)
 register char sym;
 {
@@ -510,9 +518,6 @@ u_init()
 {
 	register int i;
 
-	flags.pantheon = -1;		/* role_init() will reset this */
-	/* Initialize the role, race, gender, and alignment */
-	role_init();
 	flags.female = flags.initgend;
 	flags.beginner = 1;
 
@@ -574,7 +579,7 @@ u_init()
 	init_uhunger();
 	for (i = 0; i <= MAXSPELL; i++) spl_book[i].sp_id = NO_SPELL;
 	u.ublesscnt = 300;			/* no prayers just yet */
-	u.ualignbase[0] = u.ualignbase[1] = u.ualign.type =
+	u.ualignbase[A_CURRENT] = u.ualignbase[A_ORIGINAL] = u.ualign.type =
 			aligns[flags.initalign].value;
 	u.ulycn = NON_PM;
 
@@ -603,8 +608,8 @@ u_init()
 		if(!rn2(10)) ini_inv(Tinopener);
 		else if(!rn2(4)) ini_inv(Lamp);
 		else if(!rn2(10)) ini_inv(Magicmarker);
-		knows_class(GEM_CLASS);
 		knows_object(SACK);
+		knows_object(TOUCHSTONE);
 		skill_init(Skill_A);
 		break;
 	case PM_BARBARIAN:
@@ -624,7 +629,11 @@ u_init()
 		skill_init(Skill_C);
 		break;
 	case PM_HEALER:
+#ifndef GOLDOBJ
 		u.ugold = u.ugold0 = rn1(1000, 1001);
+#else
+		u.umoney0 = rn1(1000, 1001);
+#endif
 		ini_inv(Healer);
 		if(!rn2(25)) ini_inv(Lamp);
 		knows_object(POT_FULL_HEALING);
@@ -666,38 +675,6 @@ u_init()
 		 */
 		break;
 	case PM_RANGER:
-#if 0		/* superseded by inv_subs[] */
-		switch (rn2(100) / 20) {
-		case 0:	/* Special racial bow */
-		case 1:
-		case 2:
-		    switch (Race_switch) {
-		    case PM_ELF:
-			Ranger[RAN_BOW].trotyp = ELVEN_BOW;
-			Ranger[RAN_TWO_ARROWS].trotyp =
-			Ranger[RAN_ZERO_ARROWS].trotyp = ELVEN_ARROW;
-			break;
-		    case PM_GNOME:
-			Ranger[RAN_BOW].trotyp = CROSSBOW;
-			Ranger[RAN_TWO_ARROWS].trotyp =
-			Ranger[RAN_ZERO_ARROWS].trotyp = CROSSBOW_BOLT;
-			break;
-		    case PM_ORC:
-			Ranger[RAN_BOW].trotyp = ORCISH_BOW;
-			Ranger[RAN_TWO_ARROWS].trotyp =
-			Ranger[RAN_ZERO_ARROWS].trotyp = ORCISH_ARROW;
-			break;
-		    default: break;	/* Use default bow + arrow */
-		    }
-		    break;
-		case 3:	/* Missiles */
-		    Ranger[RAN_BOW].trotyp = BOOMERANG;
-		    Ranger[RAN_TWO_ARROWS].trotyp =
-		    Ranger[RAN_ZERO_ARROWS].trotyp = DART;
-		    break;
-		default: break;	/* Use default bow + arrow */
-		}
-#endif	/*0*/
 		Ranger[RAN_TWO_ARROWS].trquan = rn1(10, 50);
 		Ranger[RAN_ZERO_ARROWS].trquan = rn1(10, 30);
 		ini_inv(Ranger);
@@ -705,7 +682,11 @@ u_init()
 		break;
 	case PM_ROGUE:
 		Rogue[R_DAGGERS].trquan = rn1(10, 6);
+#ifndef GOLDOBJ
 		u.ugold = u.ugold0 = 0;
+#else
+		u.umoney0 = 0;
+#endif
 		ini_inv(Rogue);
 		if(!rn2(5)) ini_inv(Blindfold);
 		knows_object(SACK);
@@ -722,7 +703,11 @@ u_init()
 #ifdef TOURIST
 	case PM_TOURIST:
 		Tourist[T_DARTS].trquan = rn1(20, 21);
+#ifndef GOLDOBJ
 		u.ugold = u.ugold0 = rnd(1000);
+#else
+		u.umoney0 = rnd(1000);
+#endif
 		ini_inv(Tourist);
 		if(!rn2(25)) ini_inv(Tinopener);
 		else if(!rn2(25)) ini_inv(Leash);
@@ -732,7 +717,6 @@ u_init()
 		break;
 #endif
 	case PM_VALKYRIE:
-		flags.female = TRUE;
 		ini_inv(Valkyrie);
 		if(!rn2(6)) ini_inv(Lamp);
 		knows_class(WEAPON_CLASS);
@@ -760,10 +744,10 @@ u_init()
 	case PM_ELF:
 	    /*
 	     * Elves are people of music and song, or they are warriors.
-	     * Non-warriors MAY get an instrument.  We use a kludge to
+	     * Non-warriors get an instrument.  We use a kludge to
 	     * get only non-magic instruments.
 	     */
-	    if (Role_if(PM_HEALER) || Role_if(PM_WIZARD)) {
+	    if (Role_if(PM_PRIEST) || Role_if(PM_WIZARD)) {
 		static int trotyp[] = {
 		    WOODEN_FLUTE, TOOLED_HORN, WOODEN_HARP,
 		    BELL, BUGLE, LEATHER_DRUM
@@ -821,11 +805,21 @@ u_init()
 	default:	/* impossible */
 		break;
 	}
-		
+
 	if (discover)
 		ini_inv(Wishing);
 
+#ifdef WIZARD
+	if (wizard)
+		read_wizkit();
+#endif
+
+#ifndef GOLDOBJ
 	u.ugold0 += hidden_gold();	/* in case sack has gold in it */
+#else
+	if (u.umoney0) ini_inv(Money);
+	u.umoney0 += hidden_gold();	/* in case sack has gold in it */
+#endif
 
 	find_ac();			/* get initial ac value */
 	init_attr(75);			/* init attribute values */
@@ -851,7 +845,41 @@ u_init()
 	return;
 }
 
-static void
+/* skills aren't initialized, so we use the role-specific skill lists */
+STATIC_OVL boolean
+restricted_spell_discipline(otyp)
+int otyp;
+{
+    struct def_skill *skills;
+    int this_skill = spell_skilltype(otyp);
+
+    switch (Role_switch) {
+     case PM_ARCHEOLOGIST:	skills = Skill_A; break;
+     case PM_BARBARIAN:		skills = Skill_B; break;
+     case PM_CAVEMAN:		skills = Skill_C; break;
+     case PM_HEALER:		skills = Skill_H; break;
+     case PM_KNIGHT:		skills = Skill_K; break;
+     case PM_MONK:		skills = Skill_Mon; break;
+     case PM_PRIEST:		skills = Skill_P; break;
+     case PM_RANGER:		skills = Skill_Ran; break;
+     case PM_ROGUE:		skills = Skill_R; break;
+     case PM_SAMURAI:		skills = Skill_S; break;
+#ifdef TOURIST
+     case PM_TOURIST:		skills = Skill_T; break;
+#endif
+     case PM_VALKYRIE:		skills = Skill_V; break;
+     case PM_WIZARD:		skills = Skill_W; break;
+     default:			skills = 0; break;	/* lint suppression */
+    }
+
+    while (skills->skill != P_NONE) {
+	if (skills->skill == this_skill) return FALSE;
+	++skills;
+    }
+    return TRUE;
+}
+
+STATIC_OVL void
 ini_inv(trop)
 register struct trobj *trop;
 {
@@ -875,6 +903,7 @@ register struct trobj *trop;
 			static NEARDATA short nocreate = STRANGE_OBJECT;
 			static NEARDATA short nocreate2 = STRANGE_OBJECT;
 			static NEARDATA short nocreate3 = STRANGE_OBJECT;
+			static NEARDATA short nocreate4 = STRANGE_OBJECT;
 		/*
 		 * For random objects, do not create certain overly powerful
 		 * items: wand of wishing, ring of levitation, or the
@@ -886,35 +915,45 @@ register struct trobj *trop;
 		 * weapon.)
 		 */
 			obj = mkobj(trop->trclass, FALSE);
-			while(obj->otyp == WAN_WISHING
-				|| obj->otyp == nocreate
-				|| obj->otyp == nocreate2
-				|| obj->otyp == nocreate3
+			otyp = obj->otyp;
+			while (otyp == WAN_WISHING
+				|| otyp == nocreate
+				|| otyp == nocreate2
+				|| otyp == nocreate3
+				|| otyp == nocreate4
 #ifdef ELBERETH
-				|| obj->otyp == RIN_LEVITATION
+				|| otyp == RIN_LEVITATION
 #endif
 				/* 'useless' items */
-				|| obj->otyp == POT_HALLUCINATION
-				|| obj->otyp == POT_ACID
-				|| obj->otyp == SCR_AMNESIA
-				|| obj->otyp == SCR_FIRE
-				|| obj->otyp == SCR_STINKING_CLOUD
-				|| obj->otyp == RIN_AGGRAVATE_MONSTER
-				|| obj->otyp == RIN_HUNGER
-				|| obj->otyp == WAN_NOTHING
+				|| otyp == POT_HALLUCINATION
+				|| otyp == POT_ACID
+				|| otyp == SCR_AMNESIA
+				|| otyp == SCR_FIRE
+				|| otyp == SCR_BLANK_PAPER
+				|| otyp == SPE_BLANK_PAPER
+				|| otyp == RIN_AGGRAVATE_MONSTER
+				|| otyp == RIN_HUNGER
+				|| otyp == WAN_NOTHING
+				/* Monks don't use weapons */
+				|| (otyp == SCR_ENCHANT_WEAPON &&
+				    Role_if(PM_MONK))
 				/* wizard patch -- they already have one */
-				|| obj->otyp == SPE_FORCE_BOLT
+				|| (otyp == SPE_FORCE_BOLT &&
+				    Role_if(PM_WIZARD))
 				/* powerful spells are either useless to
-				   low level players or unbalancing */
+				   low level players or unbalancing; also
+				   spells in restricted skill categories */
 				|| (obj->oclass == SPBOOK_CLASS &&
-				    objects[obj->otyp].oc_level > 3)
+				    (objects[otyp].oc_level > 3 ||
+				    restricted_spell_discipline(otyp)))
 							) {
 				dealloc_obj(obj);
 				obj = mkobj(trop->trclass, FALSE);
+				otyp = obj->otyp;
 			}
 
 			/* Don't start with +0 or negative rings */
-			if(objects[obj->otyp].oc_charged && obj->spe <= 0)
+			if (objects[otyp].oc_charged && obj->spe <= 0)
 				obj->spe = rne(3);
 
 			/* Heavily relies on the fact that 1) we create wands
@@ -923,49 +962,64 @@ register struct trobj *trop;
 			 * particular symbol is to be prohibited.  (For more
 			 * objects, we need more nocreate variables...)
 			 */
-			switch (obj->otyp) {
+			switch (otyp) {
 			    case WAN_POLYMORPH:
 			    case RIN_POLYMORPH:
+			    case POT_POLYMORPH:
 				nocreate = RIN_POLYMORPH_CONTROL;
 				break;
 			    case RIN_POLYMORPH_CONTROL:
 				nocreate = RIN_POLYMORPH;
 				nocreate2 = SPE_POLYMORPH;
+				nocreate3 = POT_POLYMORPH;
 			}
 			/* Don't have 2 of the same ring or spellbook */
 			if (obj->oclass == RING_CLASS ||
 			    obj->oclass == SPBOOK_CLASS)
-				nocreate3 = obj->otyp;
+				nocreate4 = otyp;
 		}
 
-		obj->dknown = obj->bknown = obj->rknown = 1;
-		if (objects[obj->otyp].oc_uses_known) obj->known = 1;
-		obj->cursed = 0;
-		if (obj->opoisoned && u.ualign.type != A_CHAOTIC)
-		    obj->opoisoned = 0;
-		if(obj->oclass == WEAPON_CLASS || obj->oclass == TOOL_CLASS) {
-			obj->quan = (long) trop->trquan;
-			trop->trquan = 1;
+#ifdef GOLDOBJ
+		if (trop->trclass == GOLD_CLASS) {
+			/* no "blessed" or "identified" money */
+			obj->quan = u.umoney0;
+		} else {
+#endif
+			obj->dknown = obj->bknown = obj->rknown = 1;
+			if (objects[otyp].oc_uses_known) obj->known = 1;
+			obj->cursed = 0;
+			if (obj->opoisoned && u.ualign.type != A_CHAOTIC)
+			    obj->opoisoned = 0;
+			if (obj->oclass == WEAPON_CLASS ||
+				obj->oclass == TOOL_CLASS) {
+			    obj->quan = (long) trop->trquan;
+			    trop->trquan = 1;
+			} else if (obj->oclass == GEM_CLASS &&
+				is_graystone(obj) && obj->otyp != FLINT) {
+			    obj->quan = 1L;
+			}
+			if (trop->trspe != UNDEF_SPE)
+			    obj->spe = trop->trspe;
+			if (trop->trbless != UNDEF_BLESS)
+			    obj->blessed = trop->trbless;
+#ifdef GOLDOBJ
 		}
-		if(trop->trspe != UNDEF_SPE)
-			obj->spe = trop->trspe;
-		if(trop->trbless != UNDEF_BLESS)
-			obj->blessed = trop->trbless;
-
+#endif
 		/* defined after setting otyp+quan + blessedness */
 		obj->owt = weight(obj);
 		obj = addinv(obj);
 
 		/* Make the type known if necessary */
-		if (OBJ_DESCR(objects[obj->otyp]) && obj->known)
-			discover_object(obj->otyp,TRUE,FALSE);
-		if (obj->otyp == OIL_LAMP)
-			discover_object(POT_OIL,TRUE,FALSE);
+		if (OBJ_DESCR(objects[otyp]) && obj->known)
+			discover_object(otyp, TRUE, FALSE);
+		if (otyp == OIL_LAMP)
+			discover_object(POT_OIL, TRUE, FALSE);
 
 		if(obj->oclass == ARMOR_CLASS){
-			if (is_shield(obj) && !uarms)
+			if (is_shield(obj) && !uarms) {
 				setworn(obj, W_ARMS);
-			else if (is_helmet(obj) && !uarmh)
+				if (uswapwep) setuswapwep((struct obj *) 0);
+			} else if (is_helmet(obj) && !uarmh)
 				setworn(obj, W_ARMH);
 			else if (is_gloves(obj) && !uarmg)
 				setworn(obj, W_ARMG);
@@ -980,19 +1034,13 @@ register struct trobj *trop;
 			else if (is_suit(obj) && !uarm)
 				setworn(obj, W_ARM);
 		}
-		/* below changed by GAN 01/09/87 to allow wielding of
-		 * pick-axe or can-opener if there is no weapon
-		 * Rocks are assumed to be for slings.
-		 */
-		if(obj->oclass == WEAPON_CLASS || is_weptool(obj) ||
-		   obj->otyp == TIN_OPENER || obj->otyp == ROCK) {
-			if(!uwep) setuwep(obj);
-			else if(!uswapwep) setuswapwep(obj);
-			else if (is_ammo(obj) || is_missile(obj) || is_spear(obj) ||
-				(is_blade(obj) && (objects[obj->otyp].oc_dir & PIERCE)) ||
-				obj->otyp == WAR_HAMMER || obj->otyp == AKLYS ||
-				obj->otyp == ROCK)
+
+		if (obj->oclass == WEAPON_CLASS || is_weptool(obj) ||
+			otyp == TIN_OPENER || otyp == FLINT || otyp == ROCK) {
+		    if (is_ammo(obj) || is_missile(obj)) {
 			if (!uquiver) setuqwep(obj);
+		    } else if (!uwep) setuwep(obj);
+		    else if (!uswapwep) setuswapwep(obj);
 		}
 		if (obj->oclass == SPBOOK_CLASS &&
 				obj->otyp != SPE_BLANK_PAPER)
@@ -1011,5 +1059,4 @@ register struct trobj *trop;
 	}
 }
 
-
 /*u_init.c*/
