@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)vault.c	3.3	96/06/05	*/
+/*	SCCS Id: @(#)vault.c	3.4	2001/05/24	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -7,6 +7,9 @@
 
 STATIC_DCL struct monst *NDECL(findgd);
 
+#define g_monnam(mtmp) \
+	x_monnam(mtmp, ARTICLE_NONE, (char *)0, SUPPRESS_IT, FALSE)
+
 #ifdef OVLB
 
 STATIC_DCL boolean FDECL(clear_fcorr, (struct monst *,BOOLEAN_P));
@@ -23,6 +26,8 @@ register boolean forceshow;
 	register int fcx, fcy, fcbeg;
 	register struct monst *mtmp;
 
+	if (!on_level(&(EGD(grd)->gdlevel), &u.uz)) return TRUE;
+
 	while((fcbeg = EGD(grd)->fcbeg) < EGD(grd)->fcend) {
 		fcx = EGD(grd)->fakecorr[fcbeg].fx;
 		fcy = EGD(grd)->fakecorr[fcbeg].fy;
@@ -101,7 +106,7 @@ findgd()
 	register struct monst *mtmp;
 
 	for(mtmp = fmon; mtmp; mtmp = mtmp->nmon)
-	    if(mtmp->isgd && on_level(&(EGD(mtmp)->gdlevel), &u.uz))
+	    if(mtmp->isgd && !DEADMONSTER(mtmp) && on_level(&(EGD(mtmp)->gdlevel), &u.uz))
 		return(mtmp);
 	return((struct monst *)0);
 }
@@ -142,7 +147,9 @@ invault()
 	char buf[BUFSZ];
 	register int x, y, dd, gx, gy;
 	int lx = 0, ly = 0;
-
+#ifdef GOLDOBJ
+        long umoney;
+#endif
 	/* first find the goal for the guard */
 	for(dd = 2; (dd < ROWNO || dd < COLNO); dd++) {
 	  for(y = u.uy-dd; y <= u.uy+dd; ly = y, y++) {
@@ -233,9 +240,17 @@ fnd:
 	}
 
 	reset_faint();			/* if fainted - wake up */
-	pline("Suddenly one of the Vault's guards enters!");
+	pline("Suddenly one of the Vault's %s enters!",
+	      makeplural(g_monnam(guard)));
 	newsym(guard->mx,guard->my);
-	if (Strangled || youmonst.data->msound == MS_SILENT) {
+	if ((youmonst.m_ap_type == M_AP_OBJECT &&
+		youmonst.mappearance == GOLD_PIECE) || u.uundetected) {
+	    /* You're mimicking a pile of gold or you're hidden. */
+	    pline("Puzzled, %s turns around and leaves.", mhe(guard));
+	    mongone(guard);
+	    return;
+	}
+	if (Strangled || is_silent(youmonst.data)) {
 	    verbalize("I'll be back when you're ready to speak to me!");
 	    mongone(guard);
 	    return;
@@ -271,6 +286,7 @@ fnd:
 	    return;
 	}
 	verbalize("I don't know you.");
+#ifndef GOLDOBJ
 	if (!u.ugold && !hidden_gold())
 	    verbalize("Please follow me.");
 	else {
@@ -279,6 +295,17 @@ fnd:
 	    verbalize("Most likely all your gold was stolen from this vault.");
 	    verbalize("Please drop that gold and follow me.");
 	}
+#else
+        umoney = money_cnt(invent);
+	if (!umoney && !hidden_gold())
+	    verbalize("Please follow me.");
+	else {
+	    if (!umoney)
+		verbalize("You have hidden money.");
+	    verbalize("Most likely all your money was stolen from this vault.");
+	    verbalize("Please drop that money and follow me.");
+	}
+#endif
 	EGD(guard)->gdx = gx;
 	EGD(guard)->gdy = gy;
 	EGD(guard)->fcbeg = 0;
@@ -404,7 +431,7 @@ struct monst *grd;
 
 	if(movedgold || fixed) {
 	    if(in_fcorridor(grd, grd->mx, grd->my) || cansee(grd->mx, grd->my))
-		pline_The("guard whispers an incantation.");
+		pline_The("%s whispers an incantation.", g_monnam(grd));
 	    else You_hear("a distant chant.");
 	    if(movedgold)
 		pline("A mysterious force moves the gold into the vault.");
@@ -431,8 +458,12 @@ register struct monst *grd;
 			 grd_in_vault = *in_rooms(grd->mx, grd->my, VAULT)?
 					TRUE : FALSE;
 	boolean disappear_msg_seen = FALSE, semi_dead = (grd->mhp <= 0);
+#ifndef GOLDOBJ
 	register boolean u_carry_gold = ((u.ugold + hidden_gold()) > 0L);
-
+#else
+        long umoney = money_cnt(invent);
+	register boolean u_carry_gold = ((umoney + hidden_gold()) > 0L);
+#endif
 	if(!on_level(&(egrd->gdlevel), &u.uz)) return(-1);
 	nx = ny = m = n = 0;
 	if(!u_in_vault && !grd_in_vault)
@@ -463,9 +494,16 @@ register struct monst *grd;
 			(u_carry_gold || um_dist(grd->mx, grd->my, 1))) {
 		if(egrd->warncnt == 3)
 			verbalize("I repeat, %sfollow me!",
-				u_carry_gold ? (!u.ugold ?
+				u_carry_gold ? (
+#ifndef GOLDOBJ
+					  !u.ugold ?
 					  "drop that hidden gold and " :
 					  "drop that gold and ") : "");
+#else
+					  !umoney ?
+					  "drop that hidden money and " :
+					  "drop that money and ") : "");
+#endif
 		if(egrd->warncnt == 7) {
 			m = grd->mx;
 			n = grd->my;
@@ -481,7 +519,7 @@ register struct monst *grd;
 		return(0);
 	    }
 
-	    if (!u_in_vault)
+	    if (!u_in_vault) {
 		if (u_carry_gold) {	/* player teleported */
 		    m = grd->mx;
 		    n = grd->my;
@@ -490,13 +528,13 @@ register struct monst *grd;
 		    newsym(m,n);
 		    grd->mpeaceful = 0;
 letknow:
-		    if(!cansee(grd->mx, grd->my))
+		    if (!cansee(grd->mx, grd->my) || !mon_visible(grd))
 			You_hear("the shrill sound of a guard's whistle.");
 		    else
 			You(um_dist(grd->mx, grd->my, 2) ?
 			    "see an angry %s approaching." :
 			    "are confronted by an angry %s.",
-			    l_monnam(grd));
+			    g_monnam(grd));
 		    return(-1);
 		} else {
 		    verbalize("Well, begone.");
@@ -504,6 +542,7 @@ letknow:
 		    egrd->gddone = 1;
 		    goto cleanup;
 		}
+	    }
 	}
 
 	if(egrd->fcend > 1) {
@@ -511,7 +550,7 @@ letknow:
 		  !egrd->gddone && !in_fcorridor(grd, u.ux, u.uy) &&
 		  levl[egrd->fakecorr[0].fx][egrd->fakecorr[0].fy].typ
 				 == egrd->fakecorr[0].ftyp) {
-		pline_The("guard, confused, disappears.");
+		pline_The("%s, confused, disappears.", g_monnam(grd));
 		disappear_msg_seen = TRUE;
 		goto cleanup;
 	    }
@@ -546,8 +585,13 @@ letknow:
 		if (m == u.ux && n == u.uy) {
 		    struct obj *gold = g_at(m,n);
 		    /* Grab the gold from between the hero's feet.  */
+#ifndef GOLDOBJ
 		    grd->mgold += gold->quan;
 		    delobj(gold);
+#else
+		    obj_extract_self(gold);
+		    add_to_minv(grd, gold);
+#endif
 		    newsym(m,n);
 		} else if (m == x && n == y) {
 		    mpickgold(grd);	/* does a newsym */
@@ -686,7 +730,7 @@ cleanup:
 		if(!semi_dead && (in_fcorridor(grd, u.ux, u.uy) ||
 				     cansee(x, y))) {
 		    if (!disappear_msg_seen)
-			pline("Suddenly, the guard disappears.");
+			pline("Suddenly, the %s disappears.", g_monnam(grd));
 		    return(1);
 		}
 		return(-2);
@@ -705,15 +749,30 @@ void
 paygd()
 {
 	register struct monst *grd = findgd();
+#ifndef GOLDOBJ
 	struct obj *gold;
+#else
+        long umoney = money_cnt(invent);
+	struct obj *coins, *nextcoins;
+#endif
 	int gx,gy;
 	char buf[BUFSZ];
 
+#ifndef GOLDOBJ
 	if (!u.ugold || !grd) return;
+#else
+	if (!umoney || !grd) return;
+#endif
 
 	if (u.uinvault) {
-	    Your("%ld zorkmid%s goes into the Magic Memory Vault.",
-		u.ugold, plur(u.ugold));
+	    Your("%ld %s goes into the Magic Memory Vault.",
+#ifndef GOLDOBJ
+		u.ugold,
+		currency(u.ugold));
+#else
+		umoney,
+		currency(umoney));
+#endif
 	    gx = u.ux;
 	    gy = u.uy;
 	} else {
@@ -728,10 +787,21 @@ paygd()
 	    Sprintf(buf,
 		"To Croesus: here's the gold recovered from %s the %s.",
 		plname, mons[u.umonster].mname);
-	    make_engr_at(gx, gy, buf, 0L, ENGRAVE);
+	    make_grave(gx, gy, buf);
 	}
+#ifndef GOLDOBJ
 	place_object(gold = mkgoldobj(u.ugold), gx, gy);
 	stackobj(gold);
+#else
+        for (coins = invent; coins; coins = nextcoins) {
+            nextcoins = coins->nobj;
+	    if (objects[coins->otyp].oc_class == GOLD_CLASS) {
+	        freeinv(coins);
+                place_object(coins, gx, gy);
+		stackobj(coins);
+	    }
+        }
+#endif
 	mongone(grd);
 }
 
