@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)version.c	3.2	99/05/19	*/
+/*	SCCS Id: @(#)version.c	3.4	1999/12/01	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -68,7 +68,7 @@ boolean complain;
 		   version_data->entity_count != VERSION_SANITY1 ||
 		   version_data->struct_sizes != VERSION_SANITY2) {
 	    if (complain)
-		pline("Configuration incompatability for file \"%s\".",
+		pline("Configuration incompatibility for file \"%s\".",
 		      filename);
 	    return FALSE;
 	}
@@ -122,4 +122,41 @@ int fd;
 const char amiga_version_string[] = AMIGA_VERSION_STRING;
 #endif
 
+unsigned long
+get_feature_notice_ver(str)
+char *str;
+{
+	char buf[BUFSZ];
+	int ver_maj, ver_min, patch;
+	char *istr[3];
+	int j = 0;
+
+	if (!str) return 0L;
+	str = strcpy(buf, str);
+	istr[j] = str;
+	while (*str) {
+		if (*str == '.') {
+			*str++ = '\0';
+			j++;
+			istr[j] = str;
+			if (j == 2) break;
+		} else if (index("0123456789", *str) != 0) {
+			str++;
+		} else 
+			return 0L;
+	}
+	if (j != 2) return 0L;
+	ver_maj = atoi(istr[0]);
+	ver_min = atoi(istr[1]);
+	patch = atoi(istr[2]);
+	return FEATURE_NOTICE_VER(ver_maj,ver_min,patch);
+	/* macro from hack.h */
+}
+
+unsigned long
+get_current_feature_ver()
+{
+	return FEATURE_NOTICE_VER(VERSION_MAJOR,VERSION_MINOR,PATCHLEVEL);
+}
+
 /*version.c*/
