@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)vision.c	3.2	96/02/14	*/
+/*	SCCS Id: @(#)vision.c	3.4	1999/02/18	*/
 /* Copyright (c) Dean Luick, with acknowledgements to Dave Cohrs, 1990.	*/
 /* NetHack may be freely redistributed.  See license for details.	*/
 
@@ -95,14 +95,14 @@ static char  left_ptrs[ROWNO][COLNO];		/* LOS algorithm helpers */
 static char right_ptrs[ROWNO][COLNO];
 
 /* Forward declarations. */
-static void FDECL(fill_point, (int,int));
-static void FDECL(dig_point, (int,int));
-static void NDECL(view_init);
-static void FDECL(view_from,(int,int,char **,char *,char *,int,
+STATIC_DCL void FDECL(fill_point, (int,int));
+STATIC_DCL void FDECL(dig_point, (int,int));
+STATIC_DCL void NDECL(view_init);
+STATIC_DCL void FDECL(view_from,(int,int,char **,char *,char *,int,
 			     void (*)(int,int,genericptr_t),genericptr_t));
-static void FDECL(get_unused_cs, (char ***,char **,char **));
+STATIC_DCL void FDECL(get_unused_cs, (char ***,char **,char **));
 #ifdef REINCARNATION
-static void FDECL(rogue_vision, (char **,char *,char *));
+STATIC_DCL void FDECL(rogue_vision, (char **,char *,char *));
 #endif
 
 /* Macro definitions that I can't find anywhere. */
@@ -163,7 +163,7 @@ does_block(x,y,lev)
     struct monst *mon;
 
     /* Features that block . . */
-    if (IS_ROCK(lev->typ) || (IS_DOOR(lev->typ) &&
+    if (IS_ROCK(lev->typ) || lev->typ == TREE || (IS_DOOR(lev->typ) &&
 			    (lev->doormask & (D_CLOSED|D_LOCKED|D_TRAPPED) )))
 	return 1;
 
@@ -242,6 +242,7 @@ vision_reset()
 	}
     }
 
+    iflags.vision_inited = 1;	/* vision is ready */
     vision_full_recalc = 1;	/* we want to run vision_recalc() */
 }
 
@@ -252,7 +253,7 @@ vision_reset()
  * Called from vision_recalc() and at least one light routine.  Get pointers
  * to the unused vision work area.
  */
-static void
+STATIC_OVL void
 get_unused_cs(rows, rmin, rmax)
     char ***rows;
     char **rmin, **rmax;
@@ -295,7 +296,7 @@ get_unused_cs(rows, rmin, rmax)
  * We set the in_sight bit here as well to escape a bug that shows up
  * due to the one-sided lit wall hack.
  */
-static void
+STATIC_OVL void
 rogue_vision(next, rmin, rmax)
     char **next;	/* could_see array pointers */
     char *rmin, *rmax;
@@ -348,11 +349,11 @@ rogue_vision(next, rmin, rmax)
 }
 #endif /* REINCARNATION */
 
-/*#define EXTEND_SPINE		/* possibly better looking wall-angle */
+/*#define EXTEND_SPINE*/	/* possibly better looking wall-angle */
 
 #ifdef EXTEND_SPINE
 
-static int FDECL(new_angle, (struct rm *, unsigned char *, int, int));
+STATIC_DCL int FDECL(new_angle, (struct rm *, unsigned char *, int, int));
 /*
  * new_angle()
  *
@@ -395,7 +396,7 @@ static int FDECL(new_angle, (struct rm *, unsigned char *, int, int));
  *	  many exceptions.  I may have to bite the bullet and do more
  *	  checks.	- Dean 2/11/93
  */
-static int
+STATIC_OVL int
 new_angle(lev, sv, row, col)
     struct rm *lev;
     unsigned char *sv;
@@ -510,7 +511,7 @@ vision_recalc(control)
     int oldseenv;				/* previous seenv value */
 
     vision_full_recalc = 0;			/* reset flag */
-    if (in_mklev) return;
+    if (in_mklev || !iflags.vision_inited) return;
 
 #ifdef GCC_WARN
     row = 0;
@@ -540,7 +541,7 @@ vision_recalc(control)
 	 *	+ Monsters can see you even when you're in a pit.
 	 */
 	view_from(u.uy, u.ux, next_array, next_rmin, next_rmax,
-					0,(void(*)())0,(genericptr_t)0);
+		0, (void FDECL((*),(int,int,genericptr_t)))0, (genericptr_t)0);
 
 	/*
 	 * Our own version of the update loop below.  We know we can't see
@@ -604,7 +605,7 @@ vision_recalc(control)
 	    }
 	} else
 	    view_from(u.uy, u.ux, next_array, next_rmin, next_rmax,
-					0,(void(*)())0,(genericptr_t)0);
+		0, (void FDECL((*),(int,int,genericptr_t)))0, (genericptr_t)0);
 
 	/*
 	 * Set the IN_SIGHT bit for xray and night vision.
@@ -621,8 +622,13 @@ vision_recalc(control)
 		    stop  = min(COLNO-1, u.ux + ranges[dy]);
 
 		    for (col = start; col <= stop; col++) {
+			char old_row_val = next_row[col];
 			next_row[col] |= IN_SIGHT;
+			oldseenv = levl[col][row].seenv;
 			levl[col][row].seenv = SVALL;	/* see all! */
+			/* Update if previously not in sight or new angle. */
+			if (!(old_row_val & IN_SIGHT) || oldseenv != SVALL)
+			    newsym(col,row);
 		    }
 
 		    next_rmin[row] = min(start, next_rmin[row]);
@@ -715,17 +721,18 @@ vision_recalc(control)
 		    newsym(col,row);
 	    }
 
-	    else if (next_row[col] & COULD_SEE
-				&& (lev->lit || next_row[col] & TEMP_LIT)) {
+	    else if ((next_row[col] & COULD_SEE)
+				&& (lev->lit || (next_row[col] & TEMP_LIT))) {
 		/*
 		 * We see this position because it is lit.
 		 */
-		if (IS_DOOR(lev->typ) && !viz_clear[row][col]) {
+		if ((IS_DOOR(lev->typ) || lev->typ == SDOOR ||
+		     IS_WALL(lev->typ)) && !viz_clear[row][col]) {
 		    /*
-		     * Make sure doors, boulders or mimics don't show up
+		     * Make sure doors, walls, boulders or mimics don't show up
 		     * at the end of dark hallways.  We do this by checking
 		     * the adjacent position.  If it is lit, then we can see
-		     * the door, otherwise we can't.
+		     * the door or wall, otherwise we can't.
 		     */
 		    dx = u.ux - col;	dx = sign(dx);
 		    flev = &(levl[col+dx][row+dy]);
@@ -751,7 +758,7 @@ vision_recalc(control)
 		    if ( !(old_row[col] & IN_SIGHT) || oldseenv != lev->seenv)
 			newsym(col,row);
 		}
-	    } else if (next_row[col] & COULD_SEE && lev->waslit) {
+	    } else if ((next_row[col] & COULD_SEE) && lev->waslit) {
 		/*
 		 * If we make it here, the hero _could see_ the location,
 		 * but doesn't see it (location is not lit).
@@ -763,13 +770,24 @@ vision_recalc(control)
 		newsym(col,row);
 	    }
 	    /*
-	     * At this point we know that the row position is *not* in
-	     * sight.  If the old one *was* in sight, then clean up the
-	     * position.
+	     * At this point we know that the row position is *not* in normal
+	     * sight.  That is, the position is could be seen, but is dark
+	     * or LOS is just plain blocked.
+	     *
+	     * Update the position if:
+	     * o If the old one *was* in sight.  We may need to clean up
+	     *   the glyph -- E.g. darken room spot, etc.
+	     * o If we now could see the location (yet the location is not
+	     *   lit), but previously we couldn't see the location, or vice
+	     *   versa.  Update the spot because there there may be an infared
+	     *   monster there.
 	     */
 	    else {
 not_in_sight:
-		if (old_row[col] & IN_SIGHT) newsym(col,row);
+		if ((old_row[col] & IN_SIGHT)
+			|| ((next_row[col] & COULD_SEE)
+				^ (old_row[col] & COULD_SEE)))
+		    newsym(col,row);
 	    }
 
 	} /* end for col . . */
@@ -777,7 +795,13 @@ not_in_sight:
     colbump[u.ux] = colbump[u.ux+1] = 0;
 
 skip:
-    newsym(u.ux,u.uy);		/* Make sure the hero shows up! */
+    /* This newsym() caused a crash delivering msg about failure to open
+     * dungeon file init_dungeons() -> panic() -> done(11) ->
+     * vision_recalc(2) -> newsym() -> crash!  u.ux and u.uy are 0 and
+     * program_state.panicking == 1 under those circumstances
+     */
+    if (!program_state.panicking)
+	newsym(u.ux, u.uy);		/* Make sure the hero shows up! */
 
     /* Set the new min and max pointers. */
     viz_rmin  = next_rmin;
@@ -873,7 +897,7 @@ unblock_point(x,y)
  *   This means that a right-edge (a blocked spot that has an open
  *    spot on its right) will point to itself.
  */
-static void
+STATIC_OVL void
 dig_point(row,col)
     int row,col;
 {
@@ -954,7 +978,7 @@ dig_point(row,col)
     }
 }
 
-static void
+STATIC_OVL void
 fill_point(row,col)
     int row, col;
 {
@@ -1279,10 +1303,10 @@ static genericptr_t varg;
 
 #else   /* quadrants are really functions */
 
-static int FDECL(_q1_path, (int,int,int,int));
-static int FDECL(_q2_path, (int,int,int,int));
-static int FDECL(_q3_path, (int,int,int,int));
-static int FDECL(_q4_path, (int,int,int,int));
+STATIC_DCL int FDECL(_q1_path, (int,int,int,int));
+STATIC_DCL int FDECL(_q2_path, (int,int,int,int));
+STATIC_DCL int FDECL(_q3_path, (int,int,int,int));
+STATIC_DCL int FDECL(_q4_path, (int,int,int,int));
 
 #define q1_path(sy,sx,y,x,dummy) result = _q1_path(sy,sx,y,x)
 #define q2_path(sy,sx,y,x,dummy) result = _q2_path(sy,sx,y,x)
@@ -1292,7 +1316,7 @@ static int FDECL(_q4_path, (int,int,int,int));
 /*
  * Quadrant I (step < 0).
  */
-static int
+STATIC_OVL int
 _q1_path(srow,scol,y2,x2)
     int scol, srow, y2, x2;
 {
@@ -1336,7 +1360,7 @@ _q1_path(srow,scol,y2,x2)
 /*
  * Quadrant IV (step > 0).
  */
-static int
+STATIC_OVL int
 _q4_path(srow,scol,y2,x2)
     int scol, srow, y2, x2;
 {
@@ -1380,7 +1404,7 @@ _q4_path(srow,scol,y2,x2)
 /*
  * Quadrant II (step < 0).
  */
-static int
+STATIC_OVL int
 _q2_path(srow,scol,y2,x2)
     int scol, srow, y2, x2;
 {
@@ -1424,7 +1448,7 @@ _q2_path(srow,scol,y2,x2)
 /*
  * Quadrant III (step > 0).
  */
-static int
+STATIC_OVL int
 _q3_path(srow,scol,y2,x2)
     int scol, srow, y2, x2;
 {
@@ -1523,16 +1547,16 @@ cleardone:
 static close2d *close_dy[CLOSE_MAX_BC_DY];
 static far2d   *far_dy[FAR_MAX_BC_DY];
 
-static void FDECL(right_side, (int,int,int,int,int,int,int,char*));
-static void FDECL(left_side, (int,int,int,int,int,int,int,char*));
-static int FDECL(close_shadow, (int,int,int,int));
-static int FDECL(far_shadow, (int,int,int,int));
+STATIC_DCL void FDECL(right_side, (int,int,int,int,int,int,int,char*));
+STATIC_DCL void FDECL(left_side, (int,int,int,int,int,int,int,char*));
+STATIC_DCL int FDECL(close_shadow, (int,int,int,int));
+STATIC_DCL int FDECL(far_shadow, (int,int,int,int));
 
 /*
  * Initialize algorithm D's table pointers.  If we don't have these,
  * then we do 3D table lookups.  Verrrry slow.
  */
-static void
+STATIC_OVL void
 view_init()
 {
     int i;
@@ -1552,7 +1576,7 @@ view_init()
  */
 #define OFF_TABLE 0xff
 
-static int
+STATIC_OVL int
 close_shadow(side,this_row,block_row,block_col)
     int side,this_row,block_row,block_col;
 {
@@ -1581,7 +1605,7 @@ close_shadow(side,this_row,block_row,block_col)
 }
 
 
-static int
+STATIC_OVL int
 far_shadow(side,this_row,block_row,block_col)
     int side,this_row,block_row,block_col;
 {
@@ -1621,7 +1645,7 @@ far_shadow(side,this_row,block_row,block_col)
  *
  * Figure out what could be seen on the right side of the source.
  */
-static void
+STATIC_OVL void
 right_side(row, cb_row, cb_col, fb_row, fb_col, left, right_mark, limits)
     int row;		/* current row */
     int	cb_row, cb_col;	/* close block row and col */
@@ -1873,7 +1897,7 @@ right_side(row, cb_row, cb_col, fb_row, fb_col, left, right_mark, limits)
  * This routine is the mirror image of right_side().  Please see right_side()
  * for blow by blow comments.
  */
-static void
+STATIC_OVL void
 left_side(row, cb_row, cb_col, fb_row, fb_col, left_mark, right, limits)
     int row;		/* the current row */
     int	cb_row, cb_col;	/* close block row and col */
@@ -2051,7 +2075,7 @@ left_side(row, cb_row, cb_col, fb_row, fb_col, left_mark, right, limits)
  * seen from the source location.  Initialize and fill the left most
  * and right most boundaries of what could be seen.
  */
-static void
+STATIC_OVL void
 view_from(srow,scol,loc_cs_rows,left_most,right_most, range, func, arg)
     int  srow, scol;			/* source row and column */
     char **loc_cs_rows;			/* could_see array (row pointers) */
@@ -2142,11 +2166,11 @@ view_from(srow,scol,loc_cs_rows,left_most,right_most, range, func, arg)
 /*
  * Defines local to Algorithm C.
  */
-static void FDECL(right_side, (int,int,int,char*));
-static void FDECL(left_side, (int,int,int,char*));
+STATIC_DCL void FDECL(right_side, (int,int,int,char*));
+STATIC_DCL void FDECL(left_side, (int,int,int,char*));
 
 /* Initialize algorithm C (nothing). */
-static void
+STATIC_OVL void
 view_init()
 {
 }
@@ -2155,7 +2179,7 @@ view_init()
  * Mark positions as visible on one quadrant of the right side.  The
  * quadrant is determined by the value of the global variable step.
  */
-static void
+STATIC_OVL void
 right_side(row, left, right_mark, limits)
     int row;		/* current row */
     int left;		/* first (left side) visible spot on prev row */
@@ -2337,7 +2361,7 @@ rside2:					/* used if q?_path() is a macro */
  * This routine is the mirror image of right_side().  See right_side() for
  * extensive comments.
  */
-static void
+STATIC_OVL void
 left_side(row, left_mark, right, limits)
     int row, left_mark, right;
     char *limits;
@@ -2459,7 +2483,7 @@ lside2:					/* used if q?_path() is a macro */
  * (srow,scol).  NOTE this is (y,x)!  Mark the visible locations in the
  * array provided.
  */
-static void
+STATIC_OVL void
 view_from(srow, scol, loc_cs_rows, left_most, right_most, range, func, arg)
     int  srow, scol;	/* starting row and column */
     char **loc_cs_rows;	/* pointers to the rows of the could_see array */
