@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)weapon.c	3.3	1999/11/29	*/
+/*	SCCS Id: @(#)weapon.c	3.4	2002/02/07	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -26,6 +26,8 @@
 #define PN_ESCAPE_SPELL			(-13)
 #define PN_MATTER_SPELL			(-14)
 
+STATIC_DCL void FDECL(give_may_advance_msg, (int));
+
 #ifndef OVLB
 
 STATIC_DCL NEARDATA const short skill_names_indices[];
@@ -80,7 +82,7 @@ STATIC_OVL
 void give_may_advance_msg(skill)
 int skill;
 {
-	You("feel more confident in your %sskills.",
+	You_feel("more confident in your %sskills.",
 		skill == P_NONE ?
 			"" :
 		skill <= P_LAST_WEAPON ?
@@ -93,6 +95,8 @@ int skill;
 #endif	/* OVLB */
 
 STATIC_DCL boolean FDECL(can_advance, (int, BOOLEAN_P));
+STATIC_DCL boolean FDECL(could_advance, (int));
+STATIC_DCL boolean FDECL(peaked_skill, (int));
 STATIC_DCL int FDECL(slots_required, (int));
 
 #ifdef OVL1
@@ -140,7 +144,7 @@ struct monst *mon;
 
 	if (is_spear(otmp) &&
 	   index(kebabable, ptr->mlet)) tmp += 2;
-  
+
 	/* trident is highly effective against swimmers */
 	if (otmp->otyp == TRIDENT && is_swimmer(ptr)) {
 	   if (is_pool(mon->mx, mon->my)) tmp += 4;
@@ -181,6 +185,7 @@ struct monst *mon;
  * Second edition AD&D came out a few years later; luckily it used the same
  * table.  As of this writing (1999), third edition is in progress but not
  * released.  Let's see if the weapon table stays the same.  --KAA
+ * October 2000: It didn't.  Oh, well.
  */
 
 /*
@@ -341,16 +346,17 @@ int x;
 }
 
 static NEARDATA const int rwep[] =
-{	DWARVISH_SPEAR, ELVEN_SPEAR, SPEAR, ORCISH_SPEAR, JAVELIN,
-	  SHURIKEN, YA, SILVER_ARROW, ELVEN_ARROW, ARROW, ORCISH_ARROW,
-	  CROSSBOW_BOLT, ELVEN_DAGGER, DAGGER, ORCISH_DAGGER, KNIFE, FLINT,
-	  ROCK, LOADSTONE, LUCKSTONE, DART, /* BOOMERANG, */ CREAM_PIE
-	  /* note: CREAM_PIE should NOT be #ifdef KOPS */
+{	DWARVISH_SPEAR, SILVER_SPEAR, ELVEN_SPEAR, SPEAR, ORCISH_SPEAR,
+	JAVELIN, SHURIKEN, YA, SILVER_ARROW, ELVEN_ARROW, ARROW,
+	ORCISH_ARROW, CROSSBOW_BOLT, SILVER_DAGGER, ELVEN_DAGGER, DAGGER,
+	ORCISH_DAGGER, KNIFE, FLINT, ROCK, LOADSTONE, LUCKSTONE, DART,
+	/* BOOMERANG, */ CREAM_PIE
+	/* note: CREAM_PIE should NOT be #ifdef KOPS */
 };
 
 static NEARDATA const int pwep[] =
-{	HALBERD, PARTISAN, LANCE, FAUCHARD, BILL_GUISARME, BEC_DE_CORBIN,
-	GUISARME, RANSEUR, SPETUM, VOULGE, BARDICHE, GLAIVE
+{	HALBERD, BARDICHE, SPETUM, BILL_GUISARME, VOULGE, RANSEUR, GUISARME,
+	GLAIVE, LUCERN_HAMMER, BEC_DE_CORBIN, FAUCHARD, PARTISAN, LANCE
 };
 
 static struct obj *propellor;
@@ -376,7 +382,13 @@ register struct monst *mtmp;
 	    Oselect(BOULDER);
 
 	/* Select polearms first; they do more damage and aren't expendable */
-	for (i = 0; i < SIZE(pwep); i++) {
+	/* The limit of 13 here is based on the monster polearm range limit
+	 * (defined as 5 in mthrowu.c).  5 corresponds to a distance of 2 in
+	 * one direction and 1 in another; one space beyond that would be 3 in
+	 * one direction and 2 in another; 3^2+2^2=13.
+	 */
+	if (dist2(mtmp->mx, mtmp->my, mtmp->mux, mtmp->muy) <= 13 && couldsee(mtmp->mx, mtmp->my)) {
+	    for (i = 0; i < SIZE(pwep); i++) {
 		/* Only strong monsters can wield big (esp. long) weapons.
 		 * Big weapon is basically the same as bimanual.
 		 * All monsters can wield the remaining weapons.
@@ -384,8 +396,13 @@ register struct monst *mtmp;
 		if (((strongmonst(mtmp->data) && (mtmp->misc_worn_check & W_ARMS) == 0)
 			|| !objects[pwep[i]].oc_bimanual) &&
 		    (objects[pwep[i]].oc_material != SILVER
-			|| !hates_silver(mtmp->data)))
-			Oselect(pwep[i]);
+			|| !hates_silver(mtmp->data))) {
+		    if ((otmp = oselect(mtmp, pwep[i])) != 0) {
+			propellor = otmp; /* force the monster to wield it */
+			return otmp;
+		    }
+		}
+	    }
 	}
 
 	/*
@@ -411,8 +428,8 @@ register struct monst *mtmp;
 	    propellor = &zeroobj;
 
 	    prop = (objects[rwep[i]]).oc_skill;
-	    if (prop > 0) {
-		switch (prop) {
+	    if (prop < 0) {
+		switch (-prop) {
 		case P_BOW:
 		  propellor = (oselect(mtmp, YUMI));
 		  if (!propellor) propellor = (oselect(mtmp, ELVEN_BOW));
@@ -459,16 +476,16 @@ static NEARDATA short hwep[] = {
 	  CORPSE,  /* cockatrice corpse */
 	  TSURUGI, RUNESWORD, DWARVISH_MATTOCK, TWO_HANDED_SWORD, BATTLE_AXE,
 	  KATANA, UNICORN_HORN, CRYSKNIFE, TRIDENT, LONG_SWORD,
-	  ELVEN_BROADSWORD, BROADSWORD, LUCERN_HAMMER, SCIMITAR, SILVER_SABER,
+	  ELVEN_BROADSWORD, BROADSWORD, SCIMITAR, SILVER_SABER,
 	  MORNING_STAR, ELVEN_SHORT_SWORD, DWARVISH_SHORT_SWORD, SHORT_SWORD,
-	  ORCISH_SHORT_SWORD, MACE, AXE, DWARVISH_SPEAR, ELVEN_SPEAR, SPEAR,
-	  ORCISH_SPEAR, FLAIL, BULLWHIP, QUARTERSTAFF, JAVELIN, AKLYS, CLUB,
-	  PICK_AXE,
+	  ORCISH_SHORT_SWORD, MACE, AXE, DWARVISH_SPEAR, SILVER_SPEAR,
+	  ELVEN_SPEAR, SPEAR, ORCISH_SPEAR, FLAIL, BULLWHIP, QUARTERSTAFF,
+	  JAVELIN, AKLYS, CLUB, PICK_AXE,
 #ifdef KOPS
 	  RUBBER_HOSE,
 #endif /* KOPS */
-	  WAR_HAMMER, ELVEN_DAGGER, DAGGER, ORCISH_DAGGER, ATHAME, SCALPEL,
-	  KNIFE, WORM_TOOTH
+	  WAR_HAMMER, SILVER_DAGGER, ELVEN_DAGGER, DAGGER, ORCISH_DAGGER,
+	  ATHAME, SCALPEL, KNIFE, WORM_TOOTH
 	};
 
 struct obj *
@@ -495,12 +512,15 @@ register struct monst *mtmp;
 	/* only strong monsters can wield big (esp. long) weapons */
 	/* big weapon is basically the same as bimanual */
 	/* all monsters can wield the remaining weapons */
-	for (i = 0; i < SIZE(hwep); i++)
+	for (i = 0; i < SIZE(hwep); i++) {
+	    if (hwep[i] == CORPSE && !(mtmp->misc_worn_check & W_ARMG))
+		continue;
 	    if (((strong && !wearing_shield)
 			|| !objects[hwep[i]].oc_bimanual) &&
 		    (objects[hwep[i]].oc_material != SILVER
 			|| !hates_silver(mtmp->data)))
 		Oselect(hwep[i]);
+	}
 
 	/* failure */
 	return (struct obj *)0;
@@ -526,7 +546,7 @@ register struct monst *mon;
 		return;
 	}
 	if (!attacktype(mon->data, AT_WEAP)) {
-		mw_tmp->owornmask &= ~W_WEP;
+		setmnotwielded(mon, mw_tmp);
 		MON_NOWEP(mon);
 		mon->weapon_check = NO_WEAPON_WANTED;
 		obj_extract_self(obj);
@@ -545,14 +565,16 @@ register struct monst *mon;
 	 * choice of weapons.  This has no parallel for players.  It can
 	 * be handled by waiting until mon_wield_item is actually called.
 	 * Though the monster still wields the wrong weapon until then,
-	 * this is OK since the player can't see it.
+	 * this is OK since the player can't see it.  (FIXME: Not okay since
+	 * probing can reveal it.)
 	 * Note that if there is no change, setting the check to NEED_WEAPON
 	 * is harmless.
 	 * Possible problem: big monster with big cursed weapon gets
 	 * polymorphed into little monster.  But it's not quite clear how to
 	 * handle this anyway....
 	 */
-	mon->weapon_check = NEED_WEAPON;
+	if (!(mw_tmp->cursed && mon->weapon_check == NO_WEAPON_WANTED))
+	    mon->weapon_check = NEED_WEAPON;
 }
 
 /* Let a monster try to wield a weapon, based on mon->weapon_check.
@@ -566,7 +588,6 @@ register struct monst *mon;
 
 	/* This case actually should never happen */
 	if (mon->weapon_check == NO_WEAPON_WANTED) return 0;
-
 	switch(mon->weapon_check) {
 		case NEED_HTH_WEAPON:
 			obj = select_hwep(mon);
@@ -603,8 +624,8 @@ register struct monst *mon;
 
 			if (bimanual(mw_tmp)) mon_hand = makeplural(mon_hand);
 			Sprintf(welded_buf, "%s welded to %s %s",
-				(mw_tmp->quan == 1L) ? "is" : "are",
-				his[pronoun_gender(mon)], mon_hand);
+				otense(mw_tmp, "are"),
+				mhis(mon), mon_hand);
 
 			if (obj->otyp == PICK_AXE) {
 			    pline("Since %s weapon%s %s,",
@@ -625,18 +646,24 @@ register struct monst *mon;
 		    return 1;
 		}
 		mon->mw = obj;		/* wield obj */
-		if (mw_tmp) mw_tmp->owornmask &= ~W_WEP;
+		setmnotwielded(mon, mw_tmp);
 		mon->weapon_check = NEED_WEAPON;
 		if (canseemon(mon)) {
-			pline("%s wields %s!", Monnam(mon), doname(obj));
-			if (obj->cursed && obj->otyp != CORPSE) {
-				pline("%s %s to %s hand!",
-					The(xname(obj)),
-					(obj->quan == 1L) ? "welds itself"
-					    : "weld themselves",
-					s_suffix(mon_nam(mon)));
-				obj->bknown = 1;
-			}
+		    pline("%s wields %s!", Monnam(mon), doname(obj));
+		    if (obj->cursed && obj->otyp != CORPSE) {
+			pline("%s %s to %s %s!",
+			    Tobjnam(obj, "weld"),
+			    is_plural(obj) ? "themselves" : "itself",
+			    s_suffix(mon_nam(mon)), mbodypart(mon,HAND));
+			obj->bknown = 1;
+		    }
+		}
+		if (artifact_light(obj) && !obj->lamplit) {
+		    begin_burn(obj, FALSE);
+		    if (canseemon(mon))
+			pline("%s brilliantly in %s %s!",
+			    Tobjnam(obj, "glow"), 
+			    s_suffix(mon_nam(mon)), mbodypart(mon,HAND));
 		}
 		obj->owornmask = W_WEP;
 		return 1;
@@ -738,6 +765,7 @@ int skill;
 }
 
 /* return true if this skill can be advanced */
+/*ARGSUSED*/
 STATIC_OVL boolean
 can_advance(skill, speedy)
 int skill;
@@ -754,6 +782,30 @@ boolean speedy;
 	    && u.weapon_slots >= slots_required(skill)));
 }
 
+/* return true if this skill could be advanced if more slots were available */
+STATIC_OVL boolean
+could_advance(skill)
+int skill;
+{
+    return !P_RESTRICTED(skill)
+	    && P_SKILL(skill) < P_MAX_SKILL(skill) && (
+	    (P_ADVANCE(skill) >=
+		(unsigned) practice_needed_to_advance(P_SKILL(skill))
+	    && u.skills_advanced < P_SKILL_LIMIT));
+}
+
+/* return true if this skill has reached its maximum and there's been enough
+   practice to become eligible for the next step if that had been possible */
+STATIC_OVL boolean
+peaked_skill(skill)
+int skill;
+{
+    return !P_RESTRICTED(skill)
+	    && P_SKILL(skill) >= P_MAX_SKILL(skill) && (
+	    (P_ADVANCE(skill) >=
+		(unsigned) practice_needed_to_advance(P_SKILL(skill))));
+}
+
 STATIC_OVL void
 skill_advance(skill)
 int skill;
@@ -761,10 +813,10 @@ int skill;
     u.weapon_slots -= slots_required(skill);
     P_SKILL(skill)++;
     u.skill_record[u.skills_advanced++] = skill;
-    /* subtly change the adavnce message to indicate no more advancement */
+    /* subtly change the advance message to indicate no more advancement */
     You("are now %s skilled in %s.",
-    	P_SKILL(skill) >= P_MAX_SKILL(skill) ? "most" : "more",
-    	P_NAME(skill));
+	P_SKILL(skill) >= P_MAX_SKILL(skill) ? "most" : "more",
+	P_NAME(skill));
 }
 
 static struct skill_range {
@@ -787,14 +839,15 @@ static struct skill_range {
 int
 enhance_weapon_skill()
 {
-    int pass, i, n, len, longest, to_advance;
-    char buf[BUFSZ], buf2[BUFSZ];
+    int pass, i, n, len, longest,
+	to_advance, eventually_advance, maxxed_cnt;
+    char buf[BUFSZ], sklnambuf[BUFSZ];
+    const char *prefix;
     menu_item *selected;
     anything any;
     winid win;
     boolean speedy = FALSE;
 
-
 #ifdef WIZARD
 	if (wizard && yn("Advance skills without practice?") == 'y')
 	    speedy = TRUE;
@@ -802,15 +855,44 @@ enhance_weapon_skill()
 
 	do {
 	    /* find longest available skill name, count those that can advance */
-	    for (longest = 0, to_advance = 0, i = 0; i < P_NUM_SKILLS; i++) {
-		if (!P_RESTRICTED(i) && (len = strlen(P_NAME(i))) > longest)
+	    to_advance = eventually_advance = maxxed_cnt = 0;
+	    for (longest = 0, i = 0; i < P_NUM_SKILLS; i++) {
+		if (P_RESTRICTED(i)) continue;
+		if ((len = strlen(P_NAME(i))) > longest)
 		    longest = len;
 		if (can_advance(i, speedy)) to_advance++;
+		else if (could_advance(i)) eventually_advance++;
+		else if (peaked_skill(i)) maxxed_cnt++;
 	    }
 
 	    win = create_nhwindow(NHW_MENU);
 	    start_menu(win);
 
+	    /* start with a legend if any entries will be annotated
+	       with "*" or "#" below */
+	    if (eventually_advance > 0 || maxxed_cnt > 0) {
+		any.a_int = 0;
+		if (eventually_advance > 0) {
+		    Sprintf(buf,
+			    "(Skill%s flagged by \"*\" may be enhanced %s.)",
+			    plur(eventually_advance),
+			    (u.ulevel < MAXULEV) ?
+				"when you're more experienced" :
+				"if skill slots become available");
+		    add_menu(win, NO_GLYPH, &any, 0, 0, ATR_NONE,
+			     buf, MENU_UNSELECTED);
+		}
+		if (maxxed_cnt > 0) {
+		    Sprintf(buf,
+		  "(Skill%s flagged by \"#\" cannot be enhanced any further.)",
+			    plur(maxxed_cnt));
+		    add_menu(win, NO_GLYPH, &any, 0, 0, ATR_NONE,
+			     buf, MENU_UNSELECTED);
+		}
+		add_menu(win, NO_GLYPH, &any, 0, 0, ATR_NONE,
+			     "", MENU_UNSELECTED);
+	    }
+
 	    /* List the skills, making ones that could be advanced
 	       selectable.  List the miscellaneous skills first.
 	       Possible future enhancement:  list spell skills before
@@ -826,32 +908,55 @@ enhance_weapon_skill()
 
 		if (P_RESTRICTED(i)) continue;
 		/*
-		 * Sigh, this assumes a monospaced font.
+		 * Sigh, this assumes a monospaced font unless
+		 * iflags.menu_tab_sep is set in which case it puts
+		 * tabs between columns.
 		 * The 12 is the longest skill level name.
 		 * The "    " is room for a selection letter and dash, "a - ".
 		 */
-#ifdef WIZARD
-		if (wizard)
-		    Sprintf(buf2, " %s%-*s %-12s %4d(%4d)",
-			    to_advance == 0 || can_advance(i, speedy) ? "" : "    " ,
-			    longest, P_NAME(i),
-			    skill_level_name(i, buf),
-			    P_ADVANCE(i), practice_needed_to_advance(P_SKILL(i)));
+		if (can_advance(i, speedy))
+		    prefix = "";	/* will be preceded by menu choice */
+		else if (could_advance(i))
+		    prefix = "  * ";
+		else if (peaked_skill(i))
+		    prefix = "  # ";
 		else
+		    prefix = (to_advance + eventually_advance +
+				maxxed_cnt > 0) ? "    " : "";
+		(void) skill_level_name(i, sklnambuf);
+#ifdef WIZARD
+		if (wizard) {
+		    if (!iflags.menu_tab_sep)
+			Sprintf(buf, " %s%-*s %-12s %5d(%4d)",
+			    prefix, longest, P_NAME(i), sklnambuf,
+			    P_ADVANCE(i),
+			    practice_needed_to_advance(P_SKILL(i)));
+		    else
+			Sprintf(buf, " %s%s\t%s\t%5d(%4d)",
+			    prefix, P_NAME(i), sklnambuf,
+			    P_ADVANCE(i),
+			    practice_needed_to_advance(P_SKILL(i)));
+		 } else
 #endif
-		    Sprintf(buf2, " %s %-*s [%s]",
-			    to_advance == 0 || can_advance(i, speedy) ? "" : "    ",
-			    longest, P_NAME(i),
-			    skill_level_name(i, buf));
-
+		{
+		    if (!iflags.menu_tab_sep)
+			Sprintf(buf, " %s %-*s [%s]",
+			    prefix, longest, P_NAME(i), sklnambuf);
+		    else
+			Sprintf(buf, " %s%s\t[%s]",
+			    prefix, P_NAME(i), sklnambuf);
+		}
 		any.a_int = can_advance(i, speedy) ? i+1 : 0;
-		add_menu(win, NO_GLYPH, &any, 0, 0, ATR_NONE, buf2, MENU_UNSELECTED);
+		add_menu(win, NO_GLYPH, &any, 0, 0, ATR_NONE,
+			 buf, MENU_UNSELECTED);
 	    }
 
-	    Strcpy(buf, to_advance ? "Pick a skill to advance:" : "Current skills:");
+	    Strcpy(buf, (to_advance > 0) ? "Pick a skill to advance:" :
+					   "Current skills:");
 #ifdef WIZARD
-	    if (wizard && !speedy) Sprintf(eos(buf), "  (%d slot%s available)",
-				u.weapon_slots, plur(u.weapon_slots));
+	    if (wizard && !speedy)
+		Sprintf(eos(buf), "  (%d slot%s available)",
+			u.weapon_slots, plur(u.weapon_slots));
 #endif
 	    end_menu(win, buf);
 	    n = select_menu(win, to_advance ? PICK_ONE : PICK_NONE, &selected);
@@ -940,7 +1045,7 @@ int n;	/* number of slots to lose; normally one */
 	    u.weapon_slots = slots_required(skill) - 1;
 	    /* It might now be possible to advance some other pending
 	       skill by using the refunded slots, but giving a message
-	       to that affect would seem pretty confusing.... */
+	       to that effect would seem pretty confusing.... */
 	}
     }
 }
@@ -950,14 +1055,11 @@ weapon_type(obj)
 struct obj *obj;
 {
 	/* KMH -- now uses the object table */
-    int type;
-
+	int type;
 
 	if (!obj)
 		/* Not using a weapon */
 		return (P_BARE_HANDED_COMBAT);
-	if (u.twoweap)
-		return (P_TWO_WEAPON_COMBAT);
 	if (obj->oclass != WEAPON_CLASS && obj->oclass != TOOL_CLASS &&
 	    obj->oclass != GEM_CLASS)
 		/* Not a weapon, weapon-tool, or ammo */
@@ -966,6 +1068,14 @@ struct obj *obj;
 	return ((type < 0) ? -type : type);
 }
 
+int
+uwep_skill_type()
+{
+	if (u.twoweap)
+		return P_TWO_WEAPON_COMBAT;
+	return weapon_type(uwep);
+}
+
 /*
  * Return hit bonus/penalty based on skill of weapon.
  * Treat restricted weapons as unskilled.
@@ -974,10 +1084,13 @@ int
 weapon_hit_bonus(weapon)
 struct obj *weapon;
 {
-    int type, bonus = 0;
+    int type, wep_type, skill, bonus = 0;
     static const char bad_skill[] = "weapon_hit_bonus: bad skill %d";
 
-    type = weapon_type(weapon);
+    wep_type = weapon_type(weapon);
+    /* use two weapon skill only if attacking with one of the wielded weapons */
+    type = (u.twoweap && (weapon == uwep || weapon == uswapwep)) ?
+	    P_TWO_WEAPON_COMBAT : wep_type;
     if (type == P_NONE) {
 	bonus = 0;
     } else if (type <= P_LAST_WEAPON) {
@@ -990,8 +1103,10 @@ struct obj *weapon;
 	    case P_EXPERT:      bonus =  3; break;
 	}
     } else if (type == P_TWO_WEAPON_COMBAT) {
-	switch (P_SKILL(type)) {
-	    default: impossible(bad_skill, P_SKILL(type)); /* fall through */
+	skill = P_SKILL(P_TWO_WEAPON_COMBAT);
+	if (P_SKILL(wep_type) < skill) skill = P_SKILL(wep_type);
+	switch (skill) {
+	    default: impossible(bad_skill, skill); /* fall through */
 	    case P_ISRESTRICTED:
 	    case P_UNSKILLED:   bonus = -9; break;
 	    case P_BASIC:	bonus = -7; break;
@@ -999,13 +1114,23 @@ struct obj *weapon;
 	    case P_EXPERT:	bonus = -3; break;
 	}
     } else if (type == P_BARE_HANDED_COMBAT) {
-	/* restricted == 0 */
-	bonus = ((P_SKILL(type) + 1) * (martial_bonus() ? 2 : 1)) / 2;
+	/*
+	 *	       b.h.  m.a.
+	 *	unskl:	+1   n/a
+	 *	basic:	+1    +3
+	 *	skild:	+2    +4
+	 *	exprt:	+2    +5
+	 *	mastr:	+3    +6
+	 *	grand:	+3    +7
+	 */
+	bonus = P_SKILL(type);
+	bonus = max(bonus,P_UNSKILLED) - 1;	/* unskilled => 0 */
+	bonus = ((bonus + 2) * (martial_bonus() ? 2 : 1)) / 2;
     }
 
 #ifdef STEED
 	/* KMH -- It's harder to hit while you are riding */
-	if (u.usteed)
+	if (u.usteed) {
 		switch (P_SKILL(P_RIDING)) {
 		    case P_ISRESTRICTED:
 		    case P_UNSKILLED:   bonus -= 2; break;
@@ -1013,6 +1138,8 @@ struct obj *weapon;
 		    case P_SKILLED:     break;
 		    case P_EXPERT:      break;
 		}
+		if (u.twoweap) bonus -= 2;
+	}
 #endif
 
     return bonus;
@@ -1026,12 +1153,15 @@ int
 weapon_dam_bonus(weapon)
 struct obj *weapon;
 {
-    int type, bonus = 0;
+    int type, wep_type, skill, bonus = 0;
 
-    type = weapon_type(weapon);
+    wep_type = weapon_type(weapon);
+    /* use two weapon skill only if attacking with one of the wielded weapons */
+    type = (u.twoweap && (weapon == uwep || weapon == uswapwep)) ?
+	    P_TWO_WEAPON_COMBAT : wep_type;
     if (type == P_NONE) {
 	bonus = 0;
-    } else if (P_RESTRICTED(type) || type <= P_LAST_WEAPON) {
+    } else if (type <= P_LAST_WEAPON) {
 	switch (P_SKILL(type)) {
 	    default: impossible("weapon_dam_bonus: bad skill %d",P_SKILL(type));
 		     /* fall through */
@@ -1041,13 +1171,35 @@ struct obj *weapon;
 	    case P_SKILLED:	bonus =  1; break;
 	    case P_EXPERT:	bonus =  2; break;
 	}
-    } else if (type == P_BARE_HANDED_COMBAT && P_SKILL(type)) {
-	bonus = (P_SKILL(type) * (martial_bonus() ? 2 : 1)) / 2;
+    } else if (type == P_TWO_WEAPON_COMBAT) {
+	skill = P_SKILL(P_TWO_WEAPON_COMBAT);
+	if (P_SKILL(wep_type) < skill) skill = P_SKILL(wep_type);
+	switch (skill) {
+	    default:
+	    case P_ISRESTRICTED:
+	    case P_UNSKILLED:	bonus = -3; break;
+	    case P_BASIC:	bonus = -1; break;
+	    case P_SKILLED:	bonus = 0; break;
+	    case P_EXPERT:	bonus = 1; break;
+	}
+    } else if (type == P_BARE_HANDED_COMBAT) {
+	/*
+	 *	       b.h.  m.a.
+	 *	unskl:	 0   n/a
+	 *	basic:	+1    +3
+	 *	skild:	+1    +4
+	 *	exprt:	+2    +6
+	 *	mastr:	+2    +7
+	 *	grand:	+3    +9
+	 */
+	bonus = P_SKILL(type);
+	bonus = max(bonus,P_UNSKILLED) - 1;	/* unskilled => 0 */
+	bonus = ((bonus + 1) * (martial_bonus() ? 3 : 1)) / 2;
     }
 
 #ifdef STEED
 	/* KMH -- Riding gives some thrusting damage */
-	if (u.usteed) {
+	if (u.usteed && type != P_TWO_WEAPON_COMBAT) {
 		switch (P_SKILL(P_RIDING)) {
 		    case P_ISRESTRICTED:
 		    case P_UNSKILLED:   break;
@@ -1133,6 +1285,22 @@ struct def_skill *class_skill;
 	}
 }
 
+void
+setmnotwielded(mon,obj)
+register struct monst *mon;
+register struct obj *obj;
+{
+    if (!obj) return;
+    if (artifact_light(obj) && obj->lamplit) {
+	end_burn(obj, FALSE);
+	if (canseemon(mon))
+	    pline("%s in %s %s %s glowing.", The(xname(obj)),
+		  s_suffix(mon_nam(mon)), mbodypart(mon,HAND),
+		  otense(obj, "stop"));
+    }
+    obj->owornmask &= ~W_WEP;
+}
+
 #endif /* OVLB */
 
 /*weapon.c*/
