@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)winami.c	3.2	96/02/02	*/
+/*	SCCS Id: @(#)winami.c	3.2	2000/01/12	*/
 /* Copyright (c) Gregg Wonderly, Naperville, Illinois,  1991,1992,1993,1996. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -29,6 +29,7 @@ long amii_scrnmode;
 struct window_procs amii_procs =
 {
     "amii",
+    WC_COLOR|WC_HILITE_PET|WC_INVERSE,
     amii_init_nhwindows,
     amii_player_selection,
     amii_askname,
@@ -76,7 +77,8 @@ struct window_procs amii_procs =
     /* other defs that really should go away (they're tty specific) */
     amii_delay_output,
     amii_delay_output,
-    amii_outrip
+    amii_outrip,
+    genl_preference_update
 };
 
 /* The view window layout uses the same function names so we can use
@@ -85,6 +87,7 @@ struct window_procs amii_procs =
 struct window_procs amiv_procs =
 {
     "amitile",
+    WC_COLOR|WC_HILITE_PET|WC_INVERSE,
     amii_init_nhwindows,
     amii_player_selection,
     amii_askname,
@@ -132,11 +135,12 @@ struct window_procs amiv_procs =
     /* other defs that really should go away (they're tty specific) */
     amii_delay_output,
     amii_delay_output,
-    amii_outrip
+    amii_outrip,
+    genl_preference_update
 };
 
 unsigned short amii_initmap[ AMII_MAXCOLORS ];
-/* Default pens used unless use overides in nethack.cnf. */
+/* Default pens used unless user overides in nethack.cnf. */
 unsigned short amii_init_map[ AMII_MAXCOLORS ] =
 {
     0x0000, /* color #0  C_BLACK    */
@@ -153,20 +157,20 @@ unsigned short amiv_init_map[ AMII_MAXCOLORS ] =
 {
     0x0000, /* color #0  C_BLACK    */
     0x0fff, /* color #1  C_WHITE    */
-    0x06f0, /* color #2  C_LTGREEN  */
-    0x0ff0, /* color #3  C_YELLOW   */
+    0x00bf, /* color #2  C_CYAN     */
+    0x0f60, /* color #3  C_ORANGE   */
     0x000f, /* color #4  C_BLUE     */
-    0x0f0f, /* color #5  C_MAGENTA  */
-    0x00bf, /* color #6  C_CYAN     */
-    0x0b40, /* color #7  C_LTBROWN  */
-    0x0466, /* color #8  C_GREYBLUE */
-    0x0f60, /* color #9  C_ORANGE   */
-    0x0090, /* color #10 C_GREEN    */
+    0x0090, /* color #5  C_GREEN    */
+    0x069b, /* color #6  C_GREY     */
+    0x0f00, /* color #7  C_RED      */
+    0x06f0, /* color #8  C_LTGREEN  */
+    0x0ff0, /* color #9  C_YELLOW   */
+    0x0f0f, /* color #10 C_MAGENTA  */
     0x0940, /* color #11 C_BROWN    */
-    0x069b, /* color #12 C_GREY     */
-    0x0fb9, /* color #13 C_PEACH    */
+    0x0466, /* color #12 C_GREYBLUE */
+    0x0c40, /* color #13 C_LTBROWN  */
     0x0ddb, /* color #14 C_LTGREY   */
-    0x0c00, /* color #15 C_RED      */
+    0x0fb9, /* color #15 C_PEACH    */
 
     /* Pens for dripens etc under AA or better */
     0x0222, /* color #16 */
@@ -235,8 +239,10 @@ int wincnt=0;   /* # of nh windows opened */
  */
 
 #ifdef  INTUI_NEW_LOOK
+extern struct Hook fillhook;
 struct TagItem tags[] =
 {
+    { WA_BackFill, (ULONG)&fillhook },
     { WA_PubScreenName, (ULONG)"NetHack" },
     { TAG_DONE, 0 },
 };
@@ -262,9 +268,11 @@ struct win_setup new_wins[] =
     |WFLG_NW_EXTENDED
 #endif
     ,
-    NULL,NULL,(UBYTE*)"Messages",NULL,NULL,320,40,0xffff,0xffff,CUSTOMSCREEN,
+    NULL,NULL,(UBYTE*)"Messages",NULL,NULL,320,40,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,1,1,80,80},
@@ -273,14 +281,16 @@ struct win_setup new_wins[] =
     {{0,181,640,24,
     0xff,0xff,
     RAWKEY|MENUPICK|DISKINSERTED,
-    BORDERLESS|ACTIVATE|SMART_REFRESH
+    BORDERLESS|ACTIVATE|SMART_REFRESH|BACKDROP
 #ifdef  INTUI_NEW_LOOK
     |WFLG_NW_EXTENDED
 #endif
     ,
-    NULL,NULL,(UBYTE*)"Game Status",NULL,NULL,-1,-1,0xffff,0xffff,CUSTOMSCREEN,
+    NULL,NULL,(UBYTE*)"Game Status",NULL,NULL,0,0,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,2,2,78,78},
@@ -294,9 +304,11 @@ struct win_setup new_wins[] =
     |WFLG_NW_EXTENDED
 #endif
     ,
-    NULL,NULL,(UBYTE*)"Dungeon Map",NULL,NULL,-1,-1,0xffff,0xffff,CUSTOMSCREEN,
+    NULL,NULL,(UBYTE*)"Dungeon Map",NULL,NULL,64,64,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,22,22,80,80},
@@ -311,10 +323,11 @@ struct win_setup new_wins[] =
     |WFLG_NW_EXTENDED
 #endif
     ,
-    &MenuScroll,NULL,NULL,
-    NULL,NULL,-1,-1,0xffff,0xffff,CUSTOMSCREEN,
+    &MenuScroll,NULL,NULL,NULL,NULL,64,32,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,1,1,22,78},
@@ -329,9 +342,11 @@ struct win_setup new_wins[] =
     |WFLG_NW_EXTENDED
 #endif
     ,
-    &MenuScroll,NULL,(UBYTE*)NULL,NULL,NULL,-1,-1,0xffff,0xffff,CUSTOMSCREEN,
+    &MenuScroll,NULL,(UBYTE*)NULL,NULL,NULL,100,32,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,1,1,22,78},
@@ -345,9 +360,11 @@ struct win_setup new_wins[] =
     |WFLG_NW_EXTENDED
 #endif
     ,
-    NULL,NULL,(UBYTE*)NULL,NULL,NULL,-1,-1,0xffff,0xffff,CUSTOMSCREEN,
+    NULL,NULL,(UBYTE*)NULL,NULL,NULL,-1,-1,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,22,22,80,80},
@@ -361,9 +378,11 @@ struct win_setup new_wins[] =
     |WFLG_NW_EXTENDED
 #endif
     ,
-    NULL,NULL,(UBYTE*)NULL,NULL,NULL,-1,-1,0xffff,0xffff,CUSTOMSCREEN,
+    NULL,NULL,(UBYTE*)NULL,NULL,NULL,64,32,0xffff,0xffff,
 #ifdef  INTUI_NEW_LOOK
-    tags
+    PUBLICSCREEN,tags
+#else
+    CUSTOMSCREEN
 #endif
     },
     0,0,22,22,80,80},
@@ -424,10 +443,14 @@ struct NewScreen NewHackScreen =
 void
 amii_askname()
 {
-    *plname = 0;
+    char plnametmp[300]; /* From winreq.c: sizeof(StrStringSIBuff) */
+    *plnametmp = 0;
     do {
-	amii_getlin( "Who are you?", plname );
-    } while( strlen( plname ) == 0 );
+	amii_getlin( "Who are you?", plnametmp );
+    } while( strlen( plnametmp ) == 0 );
+
+    strncpy(plname, plnametmp, PL_NSIZ-1); /* Avoid overflowing plname[] */
+    plname[PL_NSIZ-1] = 0;
 
     if( *plname == '\33' )
     {
@@ -437,10 +460,13 @@ amii_askname()
     }
 }
 
+/* Discarded ... -jhsa
 #include "NH:sys/amiga/char.c"
+*/
 
 /* Get the player selection character */
 
+#if 0 /* New function at the bottom */
 void
 amii_player_selection()
 {
@@ -453,7 +479,13 @@ amii_player_selection()
 
     amii_clear_nhwindow( WIN_BASE );
     if (validrole(flags.initrole))
-    	return;
+	return;
+    else {
+	flags.initrole=randrole();
+	return;
+    }
+#if 0 /* Don't query the user ... instead give random character -jhsa */
+
 #if 0	/* OBSOLETE */
     if( *pl_character ){
 	pl_character[ 0 ] = toupper( pl_character[ 0 ] );
@@ -576,7 +608,11 @@ amii_player_selection()
     }
     amii_clear_nhwindow( WIN_BASE );
     CloseShWindow( cwin );
+#endif /* Do not query user ... -jhsa */
 }
+#endif /* Function elsewhere */
+
+#if 0 /* Unused ... -jhsa */
 
 #include "NH:sys/amiga/randwin.c"
 
@@ -593,13 +629,13 @@ RandomWindow( name )
     int ticks = 0, aredone = 0, timerdone = 0;
     long mask, got;
 
-    tport = CreatePort( 0, 0 );
-    trq = (struct timerequest *)CreateExtIO( tport, sizeof( *trq ) );
+    tport = CreateMsgPort();
+    trq = (struct timerequest *)CreateIORequest( tport, sizeof( *trq ) );
     if( tport == NULL || trq == NULL )
     {
 allocerr:
-	if( tport ) DeletePort( tport );
-	if( trq ) DeleteExtIO( (struct IORequest *)trq );
+	if( tport ) DeleteMsgPort( tport );
+	if( trq ) DeleteIORequest( (struct IORequest *)trq );
 	Delay( 8 * 50 );
 	return;
     }
@@ -670,8 +706,8 @@ allocerr:
 	AbortIO( (struct IORequest *)trq );
 	WaitIO( (struct IORequest *)trq );
 	CloseDevice( (struct IORequest *)trq );
-	DeleteExtIO( (struct IORequest *) trq );
-	DeletePort( tport );
+	DeleteIORequest( (struct IORequest *) trq );
+	DeleteMsgPort( tport );
 	Delay( 50 * 8 );
 	return;
     }
@@ -690,7 +726,7 @@ allocerr:
         }
         while( w && ( imsg = (struct IntuiMessage *) GetMsg( w->UserPort ) ) )
         {
-	    switch( imsg->Class )
+	    switch( (long)imsg->Class )
 	    {
 		/* Must be up for a little while... */
 	    case INACTIVEWINDOW:
@@ -722,10 +758,11 @@ allocerr:
     }
 
     CloseDevice( (struct IORequest *)trq );
-    DeleteExtIO( (struct IORequest *) trq );
-    DeletePort( tport );
+    DeleteIORequest( (struct IORequest *) trq );
+    DeleteMsgPort( tport );
     if(w) CloseShWindow( w );
 }
+#endif /* Discarded randwin ... -jhsa */
 
 /* this should probably not be needed (or be renamed)
 void
@@ -740,6 +777,11 @@ amii_get_ext_cmd( void )
     menu_item *mip;
     anything id;
     struct amii_WinDesc *cw;
+#ifdef EXTMENU
+    winid win;
+    int i;
+    char buf[256];
+#endif
     int colx;
     int bottom = 0;
     struct Window *w;
@@ -753,11 +795,11 @@ amii_get_ext_cmd( void )
     if( WIN_MESSAGE == WIN_ERR || ( cw = amii_wins[ WIN_MESSAGE ] ) == NULL )
 	panic(winpanicstr, WIN_MESSAGE, "get_ext_cmd");
     w = cw->win;
-    if( bigscreen )
-	bottom = amii_msgborder( w );
+    bottom = amii_msgborder( w );
     colx = 3;
 
 #ifdef	EXTMENU
+    if (iflags.extmenu) {
     win = amii_create_nhwindow( NHW_MENU );
     amii_start_menu( win );
     pline("#");
@@ -797,9 +839,10 @@ amii_get_ext_cmd( void )
     }
 
     return( -1 );
+    } else {
 #else
 
-    amii_clear_nhwindow( NHW_MESSAGE );
+    amii_clear_nhwindow( WIN_MESSAGE ); /* Was NHW_MESSAGE */
     if( scrollmsg )
     {
 	pline("#");
@@ -971,12 +1014,12 @@ put_ext_cmd( obufp, colx, cw, bottom )
 	{
 	    sprintf( t, "xxx%s", obufp );
 	    t[0] = 1;
-	    t[1] = -1;
-	    t[2] = ' ';
+	    t[1] = 1;
+	    t[2] = '#';
 	    amii_curs( WIN_MESSAGE, 0, bottom);
 	    SetAPen( w->RPort, C_WHITE );
 	    Text(w->RPort, "># ", 3 );
-	    SetAPen( w->RPort, C_BLACK );
+	    /* SetAPen( w->RPort, C_BLACK ); */ /* Black text on black screen doesn't look too well ... -jhsa */
 	    Text(w->RPort, t+3, strlen( t ) - 3 );
 	}
 	else
@@ -997,7 +1040,7 @@ put_ext_cmd( obufp, colx, cw, bottom )
 	amii_curs( WIN_MESSAGE, 0, bottom);
 	SetAPen( w->RPort, C_WHITE );
 	Text(w->RPort, "# ", 2 );
-	SetAPen( w->RPort, C_BLACK );
+	/* SetAPen( w->RPort, C_BLACK ); */ /* Black on black ... -jhsa */
 	Text(w->RPort, obufp, strlen( obufp ) );
 	SetAPen( w->RPort, C_WHITE );
     }
@@ -1157,7 +1200,7 @@ boolean complain;
     {
 	if (complain) {
 	    sprintf( buf, "Can't display %s: %s", fn,
-#ifdef _DCC
+#if defined(_DCC) || defined(__GNUC__)
 			strerror(errno)
 #else
 # ifdef  __SASC_60
@@ -1358,4 +1401,323 @@ SetBorder( gd )
     }
 }
 
+/* Following function copied from wintty.c */
+/* Modified slightly to fit amiga needs */
+
+void
+amii_player_selection()
+{
+	int i, k, n;
+	char pick4u = 'n', thisch, lastch = 0;
+	char pbuf[QBUFSZ], plbuf[QBUFSZ], rolenamebuf[QBUFSZ];
+	winid win;
+	anything any;
+	menu_item *selected = 0;
+
+	rigid_role_checks();
+
+	/* Should we randomly pick for the player? */
+	if (flags.initrole == ROLE_NONE || flags.initrace == ROLE_NONE ||
+		flags.initgend == ROLE_NONE || flags.initalign == ROLE_NONE) {
+	    char *prompt = build_plselection_prompt(pbuf, QBUFSZ, flags.initrole,
+				flags.initrace, flags.initgend, flags.initalign);
+	    pline("%s", prompt);
+	    do {	/* loop until we get valid input */
+		cursor_on(WIN_MESSAGE);
+		pick4u = lowc(WindowGetchar());
+		cursor_off(WIN_MESSAGE);
+		if (index(quitchars, pick4u)) pick4u = 'y';
+	    } while(!index(ynqchars, pick4u));
+	    pbuf[0] = pick4u;
+	    pbuf[1] = 0;
+	    amii_addtopl(pbuf);
+
+	    if (pick4u != 'y' && pick4u != 'n') {
+give_up:	/* Quit */
+		if (selected) free((genericptr_t) selected);
+		clearlocks();
+		exit_nhwindows(NULL);
+		terminate(0);
+		/*NOTREACHED*/
+		return;
+	    }
+	}
+
+	(void) root_plselection_prompt(plbuf, QBUFSZ - 1,
+			flags.initrole, flags.initrace, flags.initgend, flags.initalign);
+
+	/* Select a role, if necessary */
+	/* we'll try to be compatible with pre-selected race/gender/alignment,
+	 * but may not succeed */
+	if (flags.initrole < 0) {
+	    /* Process the choice */
+	    if (pick4u == 'y' || flags.initrole == ROLE_RANDOM || flags.randomall) {
+		/* Pick a random role */
+		flags.initrole = pick_role(flags.initrace, flags.initgend,
+						flags.initalign, PICK_RANDOM);
+		if (flags.initrole < 0) {
+		    amii_putstr(WIN_MESSAGE, 0, "Incompatible role!");
+		    flags.initrole = randrole();
+		}
+	    } else {
+		/* Prompt for a role */
+		win = create_nhwindow(NHW_MENU);
+		start_menu(win);
+		any.a_void = 0;         /* zero out all bits */
+		for (i = 0; roles[i].name.m; i++) {
+		    if (ok_role(i, flags.initrace, flags.initgend,
+							flags.initalign)) {
+			any.a_int = i+1;	/* must be non-zero */
+			thisch = lowc(roles[i].name.m[0]);
+			if (thisch == lastch) thisch = highc(thisch);
+			if (flags.initgend != ROLE_NONE && flags.initgend != ROLE_RANDOM) {
+				if (flags.initgend == 1  && roles[i].name.f)
+					Strcpy(rolenamebuf, roles[i].name.f);
+				else
+					Strcpy(rolenamebuf, roles[i].name.m);
+			} else {
+				if (roles[i].name.f) {
+					Strcpy(rolenamebuf, roles[i].name.m);
+					Strcat(rolenamebuf, "/");
+					Strcat(rolenamebuf, roles[i].name.f);
+				} else 
+					Strcpy(rolenamebuf, roles[i].name.m);
+			}	
+			add_menu(win, NO_GLYPH, &any, thisch,
+			    0, ATR_NONE, an(rolenamebuf), MENU_UNSELECTED);
+			lastch = thisch;
+		    }
+		}
+		any.a_int = pick_role(flags.initrace, flags.initgend,
+				    flags.initalign, PICK_RANDOM)+1;
+		if (any.a_int == 0)	/* must be non-zero */
+		    any.a_int = randrole()+1;
+		add_menu(win, NO_GLYPH, &any , '*', 0, ATR_NONE,
+				"Random", MENU_UNSELECTED);
+		any.a_int = i+1;	/* must be non-zero */
+		add_menu(win, NO_GLYPH, &any , 'q', 0, ATR_NONE,
+				"Quit", MENU_UNSELECTED);
+		Sprintf(pbuf, "Pick a role for your %s", plbuf);
+		end_menu(win, pbuf);
+		n = select_menu(win, PICK_ONE, &selected);
+		destroy_nhwindow(win);
+
+		/* Process the choice */
+		if (n != 1 || selected[0].item.a_int == any.a_int)
+		    goto give_up;		/* Selected quit */
+
+		flags.initrole = selected[0].item.a_int - 1;
+		free((genericptr_t) selected),	selected = 0;
+	    }
+	    (void) root_plselection_prompt(plbuf, QBUFSZ - 1,
+			flags.initrole, flags.initrace, flags.initgend, flags.initalign);
+	}
+
+	/* Select a race, if necessary */
+	/* force compatibility with role, try for compatibility with
+	 * pre-selected gender/alignment */
+	if (flags.initrace < 0 || !validrace(flags.initrole, flags.initrace)) {
+	    /* pre-selected race not valid */
+	    if (pick4u == 'y' || flags.initrace == ROLE_RANDOM || flags.randomall) {
+		flags.initrace = pick_race(flags.initrole, flags.initgend,
+							flags.initalign, PICK_RANDOM);
+		if (flags.initrace < 0) {
+		    amii_putstr(WIN_MESSAGE, 0, "Incompatible race!");
+		    flags.initrace = randrace(flags.initrole);
+		}
+	    } else {	/* pick4u == 'n' */
+		/* Count the number of valid races */
+		n = 0;	/* number valid */
+		k = 0;	/* valid race */
+		for (i = 0; races[i].noun; i++) {
+		    if (ok_race(flags.initrole, i, flags.initgend,
+							flags.initalign)) {
+			n++;
+			k = i;
+		    }
+		}
+		if (n == 0) {
+		    for (i = 0; races[i].noun; i++) {
+			if (validrace(flags.initrole, i)) {
+			    n++;
+			    k = i;
+			}
+		    }
+		}
+
+		/* Permit the user to pick, if there is more than one */
+		if (n > 1) {
+		    win = create_nhwindow(NHW_MENU);
+		    start_menu(win);
+		    any.a_void = 0;         /* zero out all bits */
+		    for (i = 0; races[i].noun; i++)
+			if (ok_race(flags.initrole, i, flags.initgend,
+							flags.initalign)) {
+			    any.a_int = i+1;	/* must be non-zero */
+			    add_menu(win, NO_GLYPH, &any, races[i].noun[0],
+				0, ATR_NONE, races[i].noun, MENU_UNSELECTED);
+			}
+		    any.a_int = pick_race(flags.initrole, flags.initgend,
+					flags.initalign, PICK_RANDOM)+1;
+		    if (any.a_int == 0)	/* must be non-zero */
+			any.a_int = randrace(flags.initrole)+1;
+		    add_menu(win, NO_GLYPH, &any , '*', 0, ATR_NONE,
+				    "Random", MENU_UNSELECTED);
+		    any.a_int = i+1;	/* must be non-zero */
+		    add_menu(win, NO_GLYPH, &any , 'q', 0, ATR_NONE,
+				    "Quit", MENU_UNSELECTED);
+		    Sprintf(pbuf, "Pick the race of your %s", plbuf);
+		    end_menu(win, pbuf);
+		    n = select_menu(win, PICK_ONE, &selected);
+		    destroy_nhwindow(win);
+		    if (n != 1 || selected[0].item.a_int == any.a_int)
+			goto give_up;		/* Selected quit */
+
+		    k = selected[0].item.a_int - 1;
+		    free((genericptr_t) selected),	selected = 0;
+		}
+		flags.initrace = k;
+	    }
+	    (void) root_plselection_prompt(plbuf, QBUFSZ - 1,
+			flags.initrole, flags.initrace, flags.initgend, flags.initalign);
+	}
+
+	/* Select a gender, if necessary */
+	/* force compatibility with role/race, try for compatibility with
+	 * pre-selected alignment */
+	if (flags.initgend < 0 || !validgend(flags.initrole, flags.initrace,
+						flags.initgend)) {
+	    /* pre-selected gender not valid */
+	    if (pick4u == 'y' || flags.initgend == ROLE_RANDOM || flags.randomall) {
+		flags.initgend = pick_gend(flags.initrole, flags.initrace,
+						flags.initalign, PICK_RANDOM);
+		if (flags.initgend < 0) {
+		    amii_putstr(WIN_MESSAGE, 0, "Incompatible gender!");
+		    flags.initgend = randgend(flags.initrole, flags.initrace);
+		}
+	    } else {	/* pick4u == 'n' */
+		/* Count the number of valid genders */
+		n = 0;	/* number valid */
+		k = 0;	/* valid gender */
+		for (i = 0; i < ROLE_GENDERS; i++) {
+		    if (ok_gend(flags.initrole, flags.initrace, i,
+							flags.initalign)) {
+			n++;
+			k = i;
+		    }
+		}
+		if (n == 0) {
+		    for (i = 0; i < ROLE_GENDERS; i++) {
+			if (validgend(flags.initrole, flags.initrace, i)) {
+			    n++;
+			    k = i;
+			}
+		    }
+		}
+
+		/* Permit the user to pick, if there is more than one */
+		if (n > 1) {
+		    win = create_nhwindow(NHW_MENU);
+		    start_menu(win);
+		    any.a_void = 0;         /* zero out all bits */
+		    for (i = 0; i < ROLE_GENDERS; i++)
+			if (ok_gend(flags.initrole, flags.initrace, i,
+							    flags.initalign)) {
+			    any.a_int = i+1;
+			    add_menu(win, NO_GLYPH, &any, genders[i].adj[0],
+				0, ATR_NONE, genders[i].adj, MENU_UNSELECTED);
+			}
+		    any.a_int = pick_gend(flags.initrole, flags.initrace,
+					    flags.initalign, PICK_RANDOM)+1;
+		    if (any.a_int == 0)	/* must be non-zero */
+			any.a_int = randgend(flags.initrole, flags.initrace)+1;
+		    add_menu(win, NO_GLYPH, &any , '*', 0, ATR_NONE,
+				    "Random", MENU_UNSELECTED);
+		    any.a_int = i+1;	/* must be non-zero */
+		    add_menu(win, NO_GLYPH, &any , 'q', 0, ATR_NONE,
+				    "Quit", MENU_UNSELECTED);
+		    Sprintf(pbuf, "Pick the gender of your %s", plbuf);
+		    end_menu(win, pbuf);
+		    n = select_menu(win, PICK_ONE, &selected);
+		    destroy_nhwindow(win);
+		    if (n != 1 || selected[0].item.a_int == any.a_int)
+			goto give_up;		/* Selected quit */
+
+		    k = selected[0].item.a_int - 1;
+		    free((genericptr_t) selected),	selected = 0;
+		}
+		flags.initgend = k;
+	    }
+	    (void)  root_plselection_prompt(plbuf, QBUFSZ - 1,
+			flags.initrole, flags.initrace, flags.initgend, flags.initalign);
+	}
+
+	/* Select an alignment, if necessary */
+	/* force compatibility with role/race/gender */
+	if (flags.initalign < 0 || !validalign(flags.initrole, flags.initrace,
+							flags.initalign)) {
+	    /* pre-selected alignment not valid */
+	    if (pick4u == 'y' || flags.initalign == ROLE_RANDOM || flags.randomall) {
+		flags.initalign = pick_align(flags.initrole, flags.initrace,
+							flags.initgend, PICK_RANDOM);
+		if (flags.initalign < 0) {
+		    amii_putstr(WIN_MESSAGE, 0, "Incompatible alignment!");
+		    flags.initalign = randalign(flags.initrole, flags.initrace);
+		}
+	    } else {	/* pick4u == 'n' */
+		/* Count the number of valid alignments */
+		n = 0;	/* number valid */
+		k = 0;	/* valid alignment */
+		for (i = 0; i < ROLE_ALIGNS; i++) {
+		    if (ok_align(flags.initrole, flags.initrace, flags.initgend,
+							i)) {
+			n++;
+			k = i;
+		    }
+		}
+		if (n == 0) {
+		    for (i = 0; i < ROLE_ALIGNS; i++) {
+			if (validalign(flags.initrole, flags.initrace, i)) {
+			    n++;
+			    k = i;
+			}
+		    }
+		}
+
+		/* Permit the user to pick, if there is more than one */
+		if (n > 1) {
+		    win = create_nhwindow(NHW_MENU);
+		    start_menu(win);
+		    any.a_void = 0;         /* zero out all bits */
+		    for (i = 0; i < ROLE_ALIGNS; i++)
+			if (ok_align(flags.initrole, flags.initrace,
+							flags.initgend, i)) {
+			    any.a_int = i+1;
+			    add_menu(win, NO_GLYPH, &any, aligns[i].adj[0],
+				 0, ATR_NONE, aligns[i].adj, MENU_UNSELECTED);
+			}
+		    any.a_int = pick_align(flags.initrole, flags.initrace,
+					    flags.initgend, PICK_RANDOM)+1;
+		    if (any.a_int == 0)	/* must be non-zero */
+			any.a_int = randalign(flags.initrole, flags.initrace)+1;
+		    add_menu(win, NO_GLYPH, &any , '*', 0, ATR_NONE,
+				    "Random", MENU_UNSELECTED);
+		    any.a_int = i+1;	/* must be non-zero */
+		    add_menu(win, NO_GLYPH, &any , 'q', 0, ATR_NONE,
+				    "Quit", MENU_UNSELECTED);
+		    Sprintf(pbuf, "Pick the alignment of your %s", plbuf);
+		    end_menu(win, pbuf);
+		    n = select_menu(win, PICK_ONE, &selected);
+		    destroy_nhwindow(win);
+		    if (n != 1 || selected[0].item.a_int == any.a_int)
+			goto give_up;		/* Selected quit */
+
+		    k = selected[0].item.a_int - 1;
+		    free((genericptr_t) selected),	selected = 0;
+		}
+		flags.initalign = k;
+	    }
+	}
+	/* Success! */
+}
 #endif /* AMIGA_INTUITION */
