@@ -115,18 +115,29 @@ EditColor( )
 		((struct PropInfo *)Col_GreenPen.SpecialInfo)->Flags |= PROPNEWLOOK;
     }
 #endif
-	if( WINVERS_AMIV )
+	if( WINVERS_AMIV || WINVERS_AMII )
 	{
 #ifdef	INTUI_NEW_LOOK
 		Col_NewWindowStructure1.Extension = wintags;
 		Col_NewWindowStructure1.Flags |= WFLG_NW_EXTENDED;
+# ifdef __GNUC__
+		fillhook.h_Entry = (void *)&LayerFillHook;
+# else
 		fillhook.h_Entry = (ULONG(*)())LayerFillHook;
+# endif
 		fillhook.h_Data = (void *)-2;
 		fillhook.h_SubEntry = 0;
 #endif
 	}
 
     nw = OpenWindow( (void *)&Col_NewWindowStructure1 );
+
+    if( nw == NULL )
+    {
+	DisplayBeep( NULL );
+	return;
+    }
+
     PrintIText( nw->RPort, &Col_IntuiTextList1, 0, 0 );
 
     ClearCol( nw );
@@ -407,12 +418,16 @@ EditClipping( void )
 	((struct PropInfo *)ClipYCLIP.SpecialInfo)->Flags |= PROPNEWLOOK;
     }
 #endif
-    if( WINVERS_AMIV )
+    if( WINVERS_AMIV || WINVERS_AMII )
     {
 # ifdef	INTUI_NEW_LOOK
 	ClipNewWindowStructure1.Extension = wintags;
 	ClipNewWindowStructure1.Flags |= WFLG_NW_EXTENDED;
+#  ifdef __GNUC__
+	fillhook.h_Entry = (void *)&LayerFillHook;
+#  else
 	fillhook.h_Entry = (ULONG(*)())LayerFillHook;
+#  endif
 	fillhook.h_Data = (void *)-2;
 	fillhook.h_SubEntry = 0;
 # endif
@@ -675,20 +690,20 @@ struct COLDEF amiv_colnames[ AMII_MAXCOLORS ] =
 {
     "Black","(000)",
     "White","(fff)",
+    "Cyan","(0bf)",
+    "Orange","(f60)",
+    "Blue","(00f)",
+    "Green","(090)",
+    "Grey","(69b)",
+    "Red","(f00)",
     "Light Green","(6f0)",
     "Yellow","(ff0)",
-    "Blue","(13a)",
     "Magenta","(f0f)",
-    "Cyan","(bef)",
-    "Light Brown","(b40)",
-    "Grey Blue","(466)",
-    "Orange","(f60)",
-    "Green","(090)",
     "Brown","(940)",
-    "Grey","(58c)",
-    "Peach","(fb9)",
+    "Grey Blue","(466)",
+    "Light Brown","(c40)",
     "Light Grey","(ddb)",
-    "Red","(c00)",
+    "Peach","(fb9)",
     "Col 16","(222)",
     "Col 17","(eee)",
     "Col 18","(000)",
@@ -957,19 +972,29 @@ void getlind(prompt,bufp, dflt)
 
     if( !once )
     {
-	if( bigscreen )
-	    StrWindow.TopEdge = (HackScreen->Height/2) - (StrWindow.Height/2);
+	if( bigscreen ) {
+	    StrWindow.LeftEdge = (HackScreen->Width/2) - (StrWindow.Width/2);
+	    if (amii_wins[WIN_MAP]) {
+		StrWindow.TopEdge = amii_wins[WIN_MAP]->win->TopEdge;
+	    } else {
+		StrWindow.TopEdge = (HackScreen->Height/2) - (StrWindow.Height/2);
+	    }
+	}
 	SetBorder( &String );
 	SetBorder( &Gadget2 );
 	once = 1;
     }
 
-    if( WINVERS_AMIV )
+    if( WINVERS_AMIV || WINVERS_AMII )
     {
 #ifdef	INTUI_NEW_LOOK
 	StrWindow.Extension = wintags;
 	StrWindow.Flags |= WFLG_NW_EXTENDED;
+# ifdef __GNUC__
+	fillhook.h_Entry = (void *)&LayerFillHook;
+# else
 	fillhook.h_Entry = (ULONG(*)())LayerFillHook;
+# endif
 	fillhook.h_Data = (void *)-2;
 	fillhook.h_SubEntry = 0;
 #endif
@@ -1139,10 +1164,10 @@ amii_get_color_string( )
 {
     int i;
     char s[ 10 ];
-    static char buf[ 100 ];
+    static char buf[ BUFSZ ];
 
     *buf = 0;
-    for( i = 0; i < amii_numcolors; ++i )
+    for( i = 0; i < min(32,amii_numcolors); ++i )
     {
     	sprintf( s, "%s%03lx", i ? "/" : "", (long)flags.amii_curmap[ i ] );
    	strcat( buf, s );
