@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)getline.c	3.1	90/22/02
+/*	SCCS Id: @(#)getline.c	3.1	90/22/02 */
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -8,6 +8,8 @@
 #include "macpopup.h"
 #include "func_tab.h"
 
+extern int NDECL(extcmd_via_menu);	/* cmd.c */
+
 typedef Boolean FDECL ((* key_func), (unsigned char));
 
 int
@@ -27,14 +29,15 @@ get_line_from_key_queue (char * bufp) {
 
 
 static void
-topl_getlin(const char *query, char *bufp, key_func key) {
+topl_getlin(const char *query, char *bufp, Boolean ext) {
 	int q_len = strlen(query);
 
 	if (get_line_from_key_queue (bufp))
 		return;
 
 	enter_topl_mode((char *) query);
-	while ((*key)(nhgetch()));
+	while (topl_key(nhgetch(), ext))
+		;
 	leave_topl_mode(bufp);
 }
 
@@ -47,13 +50,7 @@ topl_getlin(const char *query, char *bufp, key_func key) {
  */
 void
 mac_getlin(const char *query, char *bufp) {
-
-#if ENABLE_MAC_POPUP
-	if (iflags.popup_dialog)
-		popup_getlin (query, bufp);
-	else
-#endif
-		topl_getlin (query, bufp, &topl_key);
+	topl_getlin (query, bufp, false);
 }
 
 
@@ -66,7 +63,8 @@ mac_get_ext_cmd() {
 	char bufp[BUFSZ];
 	int i;
 
-	topl_getlin("# ", bufp, &topl_ext_key);
+	if (iflags.extmenu) return extcmd_via_menu();
+	topl_getlin("# ", bufp, true);
 	for (i = 0; extcmdlist[i].ef_txt != (char *)0; i++)
 		if (!strcmp(bufp, extcmdlist[i].ef_txt)) break;
 	if (extcmdlist[i].ef_txt == (char *)0) i = -1;    /* not found */
