@@ -3,12 +3,11 @@
 /* NetHack may be freely redistributed.  See license for details.	*/
 
 #include "hack.h"
+#include "macwin.h"
 #include "mttypriv.h"
 #include "mactty.h"
-#include "macwin.h"
 #include "wintty.h"
 
-#include <stdarg.h>
 #include <Palettes.h>
 
 #define MT_WINDOW 135
@@ -23,9 +22,6 @@
  * Mac-tty becomes mt_
  */
 
-
-static void _mt_set_colors (long *colors);
-
 static long _mt_attrs [5] [2] = {
 	{ 0x000000, 0xffffff }, /* Normal */
 	{ 0xff8080, 0xffffff }, /* Underline */
@@ -214,9 +210,7 @@ short hor, vert;
 	 * offscreen GWorld
 	 */
 	if (!Gestalt (gestaltQuickdrawVersion, &resp) && resp > 0x1ff) {
-	GDHandle gdh;
-
-		gdh = GetDeviceList ();
+		GDHandle gdh = GetDeviceList ();
 		while (gdh) {
 			if (TestDeviceAttribute (gdh, screenDevice)) {
 				if (HasDepth (gdh, 4, 1, 1) ||
@@ -231,17 +225,21 @@ short hor, vert;
 		}
 	}
 
-	mustwork (create_tty (&_mt_window, WIN_BASE_KIND + NHW_MAP, _mt_in_color));
+	if (create_tty (&_mt_window, WIN_BASE_KIND + NHW_MAP, _mt_in_color) != noErr)
+		error("_mt_init_stuff: Couldn't create tty.");
 	((WindowPeek) _mt_window)->windowKind = (WIN_BASE_KIND + NHW_MAP);
 	SelectWindow (_mt_window);
 	SetPort (_mt_window);
 	SetOrigin (-1, -1);
 	
-	font_size = (iflags.large_font && !small_screen) ? 12 : 9;
-	mustwork (init_tty_number (_mt_window, win_fonts [NHW_MAP], font_size, CO, LI));
+	font_size = iflags.wc_fontsiz_map ? iflags.wc_fontsiz_map :
+		(iflags.large_font && !small_screen) ? 12 : 9;
+	if (init_tty_number (_mt_window, win_fonts [NHW_MAP], font_size, CO, LI) != noErr)
+		error("_mt_init_stuff: Couldn't init tty.");
 
-	mustwork (get_tty_metrics (_mt_window, &num_cols, &num_rows, &win_width ,
-		&win_height, &font_num, &font_size, &char_width, &row_height));
+	if (get_tty_metrics (_mt_window, &num_cols, &num_rows, &win_width ,
+		&win_height, &font_num, &font_size, &char_width, &row_height))
+		error("_mt_init_stuff: Couldn't get tty metrics.");
 
 	SizeWindow (_mt_window, win_width + 2, win_height + 2, 1);
 	if (RetrievePosition (kMapWindow, &vert, &hor)) {
@@ -251,20 +249,17 @@ short hor, vert;
 	ShowWindow (_mt_window);
 
 	/* Start in raw, always flushing mode */
-	mustwork (get_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, &flag));
+	get_tty_attrib(_mt_window, TTY_ATTRIB_FLAGS, &flag);
 	flag |= TA_ALWAYS_REFRESH | TA_WRAP_AROUND;
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, flag));
+	set_tty_attrib(_mt_window, TTY_ATTRIB_FLAGS, flag);
 
-	mustwork (get_tty_attrib (_mt_window, TTY_ATTRIB_CURSOR, &flag));
-	flag |= TA_BLINKING_CURSOR;
-#ifdef applec
-	flag &= ~ TA_CR_ADD_NL;
-#else
-	flag &= ~ TA_NL_ADD_CR;
-#endif
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_CURSOR, flag));
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_FOREGROUND, _mt_colors [NO_COLOR] [0]));
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_BACKGROUND, _mt_colors [NO_COLOR] [1]));
+	get_tty_attrib(_mt_window, TTY_ATTRIB_CURSOR, &flag);
+	flag |= (TA_BLINKING_CURSOR | TA_NL_ADD_CR);
+	set_tty_attrib(_mt_window, TTY_ATTRIB_CURSOR, flag);
+
+	set_tty_attrib(_mt_window, TTY_ATTRIB_FOREGROUND, _mt_colors[NO_COLOR][0]);
+	set_tty_attrib(_mt_window, TTY_ATTRIB_BACKGROUND, _mt_colors[NO_COLOR][1]);
+	clear_tty (_mt_window);
 
 	InitMenuRes ();
 }
@@ -275,7 +270,6 @@ tgetch (void) {
 EventRecord event;
 long sleepTime = 0;
 int ret = 0;
-int key;
 
 	for (;!ret;) {
 		WaitNextEvent (-1, &event, sleepTime, 0);
@@ -303,7 +297,7 @@ getreturn (char *str) {
 
 int
 has_color (int color) {
-#if defined(applec)
+#if defined(__SC__) || defined(__MRC__)
 # pragma unused(color)
 #endif
 Rect r;
@@ -316,7 +310,7 @@ GDHandle gh;
 
 	r = _mt_window->portRect;
 	SetPort (_mt_window);
-	GlobalToLocal (&p);
+	LocalToGlobal (&p);
 	OffsetRect (&r, p.h, p.v);
 
 	gh = GetMaxDevice (&r);
@@ -371,7 +365,7 @@ short err;
 
 void
 term_end_attr (int attr) {
-#if defined(applec)
+#if defined(__SC__) || defined(__MRC__)
 # pragma unused (attr)
 #endif
 	_mt_set_colors (_mt_attrs [0]);
@@ -421,7 +415,7 @@ term_end_color (void) {
 void
 cl_end (void) {
 	_mt_set_colors (_mt_attrs [0]);
-	clear_tty_window (_mt_window, ttyDisplay->curx, ttyDisplay->cury ,
+	clear_tty_window (_mt_window, ttyDisplay->curx, ttyDisplay->cury,
 		CO - 1, ttyDisplay->cury);
 }
 
@@ -436,7 +430,7 @@ clear_screen (void) {
 void
 cl_eos (void) {
 	_mt_set_colors (_mt_attrs [0]);
-	clear_tty_window (_mt_window, ttyDisplay->curx, ttyDisplay->cury, CO - 1 ,
+	clear_tty_window (_mt_window, ttyDisplay->curx, ttyDisplay->cury, CO - 1,
 		LI - 1);
 }
 
@@ -491,25 +485,15 @@ term_start_color (int color) {
 
 
 void
-setftty (void) {
-long flag;
+setftty (void)
+{
+	long flag;
 
-	mustwork (get_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, &flag));
-/* Buffered output in the game */
+	/* Buffered output for the game */
+	get_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, &flag);
 	flag &= ~ TA_ALWAYS_REFRESH;
 	flag |= TA_INHIBIT_VERT_SCROLL; /* don't scroll */
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, flag));
-
-	mustwork (get_tty_attrib (_mt_window, TTY_ATTRIB_CURSOR, &flag));
-
-#ifdef applec
-	flag &= ~ TA_CR_ADD_NL;
-#else
-	flag &= ~ TA_NL_ADD_CR;
-#endif
-
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_CURSOR, flag));
-
+	set_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, flag);
 	iflags.cbreak = 1;
 }
 
@@ -528,26 +512,17 @@ gettty (void) {
 
 
 void
-settty (const char *str) {
-long flag;
+settty (const char *str)
+{
+	long flag;
 
 	update_tty (_mt_window);
 
-	mustwork (get_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, &flag));
-/* Buffered output in the game, raw in "raw" mode */
+	/* Buffered output for the game, raw in "raw" mode */
+	get_tty_attrib(_mt_window, TTY_ATTRIB_FLAGS, &flag);
 	flag &= ~ TA_INHIBIT_VERT_SCROLL; /* scroll */
 	flag |= TA_ALWAYS_REFRESH;
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_FLAGS, flag));
-
-	mustwork (get_tty_attrib (_mt_window, TTY_ATTRIB_CURSOR, &flag));
-
-#ifdef applec
-	flag |= TA_CR_ADD_NL;
-#else
-	flag |= TA_NL_ADD_CR;
-#endif
-
-	mustwork (set_tty_attrib (_mt_window, TTY_ATTRIB_CURSOR, flag));
+	set_tty_attrib(_mt_window, TTY_ATTRIB_FLAGS, flag);
 
 	tty_raw_print ("\n");
 	if (str) {
@@ -558,7 +533,7 @@ long flag;
 
 void
 tty_number_pad (int arg) {
-#if defined(applec)
+#if defined(__SC__) || defined(__MRC__)
 # pragma unused(arg)
 #endif
 }
