@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)vmsmain.c	3.3	1997/01/22	*/
+/*	SCCS Id: @(#)vmsmain.c	3.4	2001/07/27	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 /* main.c - VMS NetHack */
@@ -45,7 +45,6 @@ char *argv[];
 
 	atexit(byebye);
 	hname = argv[0];
-	gethdate(hname);		/* find executable's creation date */
 	hname = vms_basename(hname);	/* name used in 'usage' type messages */
 	hackpid = getpid();
 	(void) umask(0);
@@ -60,8 +59,8 @@ char *argv[];
 	 * The logical name HACKDIR is overridden by a
 	 *  -d command line option (must be the first option given)
 	 */
-	dir = getenv("NETHACKDIR");
-	if (!dir) dir = getenv("HACKDIR");
+	dir = nh_getenv("NETHACKDIR");
+	if (!dir) dir = nh_getenv("HACKDIR");
 #endif
 	if(argc > 1) {
 #ifdef CHDIR
@@ -184,7 +183,9 @@ char *argv[];
 		 */
 		boolean remember_wiz_mode = wizard;
 #endif
-		(void) chmod(SAVEF,0);	/* disallow parallel restores */
+		const char *fq_save = fqname(SAVEF, SAVEPREFIX, 0);
+
+		(void) chmod(fq_save,0);	/* disallow parallel restores */
 		(void) signal(SIGINT, (SIG_RET_TYPE) done1);
 #ifdef NEWS
 		if(iflags.news) {
@@ -206,7 +207,7 @@ char *argv[];
 			if (yn("Do you want to keep the save file?") == 'n')
 			    (void) delete_savefile();
 			else
-			    (void) chmod(SAVEF,FCMASK); /* back to readable */
+			    (void) chmod(fq_save,FCMASK); /* back to readable */
 		}
 
 		flags.move = 0;
@@ -218,7 +219,7 @@ not_recovered:
 
 		flags.move = 0;
 		set_wear();
-		pickup(1);
+		(void) pickup(1);
 	}
 
 	moveloop();
@@ -244,7 +245,7 @@ char *argv[];
 		switch(argv[0][1]){
 		case 'D':
 #ifdef WIZARD
-			if(!strcmpi(getenv("USER"), WIZARD_NAME)) {
+			if(!strcmpi(nh_getenv("USER"), WIZARD_NAME)) {
 				wizard = TRUE;
 				break;
 			}
@@ -302,6 +303,9 @@ char *argv[];
 			    	flags.initrace = i;
 			}
 			break;
+		case '@':
+			flags.randomall = 1;
+			break;
 		default:
 			if ((i = str2role(&argv[0][1])) >= 0) {
 			    flags.initrole = i;
@@ -364,7 +368,7 @@ whoami()
 	 */
 	register char *s;
 
-	if (!*plname && (s = getenv("USER")))
+	if (!*plname && (s = nh_getenv("USER")))
 		(void) lcase(strncpy(plname, s, sizeof(plname)-1));
 }
 
