@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)lev_main.c	3.3	96/06/22	*/
+/*	SCCS Id: @(#)lev_main.c	3.4	2000/08/01	*/
 /*	Copyright (c) 1989 by Jean-Christophe Collet */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -6,6 +6,8 @@
  * This file contains the main function for the parser
  * and some useful functions needed by yacc
  */
+#define SPEC_LEV	/* for MPW */
+/* although, why don't we move those special defines here.. and in dgn_main? */
 
 #include "hack.h"
 #include "date.h"
@@ -15,7 +17,7 @@
 #endif
 
 #ifdef MAC
-# ifdef applec
+# if defined(__SC__) || defined(__MRC__)
 #  define MPWTOOL
 #include <CursorCtl.h>
 # else
@@ -23,6 +25,10 @@
 # endif
 #endif
 
+#ifdef WIN_CE
+#define PREFIX "\\nethack\\dat\\"
+#endif
+
 #ifndef MPWTOOL
 # define SpinCursor(x)
 #endif
@@ -47,15 +53,15 @@
 # define OMASK 0644
 #endif
 
-#define NEWLINE	10	/* under Mac MPW C '\n' is 13 so don't use it. */
+#define NEWLINE	'\n'	/* changes to 13 for MPW */
 
 #define ERR		(-1)
 
 #define NewTab(type, size)	(type **) alloc(sizeof(type *) * size)
 #define Free(ptr)		if(ptr) free((genericptr_t) (ptr))
-#define Write(fd, item, size)	(void) write(fd, (genericptr_t)(item), size)
+#define Write(fd, item, size)	if (write(fd, (genericptr_t)(item), size) != size) return FALSE;
 
-#ifdef __BORLANDC__
+#if defined(__BORLANDC__) && !defined(_WIN32)
 extern unsigned _stklen = STKSIZ;
 #endif
 #define MAX_ERRORS	25
@@ -72,7 +78,7 @@ int FDECL(get_floor_type, (CHAR_P));
 int FDECL(get_room_type, (char *));
 int FDECL(get_trap_type, (char *));
 int FDECL(get_monster_id, (char *,CHAR_P));
-int FDECL(get_object_id, (char *));
+int FDECL(get_object_id, (char *,CHAR_P));
 boolean FDECL(check_monster_char, (CHAR_P));
 boolean FDECL(check_object_char, (CHAR_P));
 char FDECL(what_map_char, (CHAR_P));
@@ -89,12 +95,13 @@ extern void NDECL(monst_init);
 extern void NDECL(objects_init);
 extern void NDECL(decl_init);
 
-static void FDECL(write_common_data, (int,int,lev_init *,long));
-static void FDECL(write_monsters, (int,char *,monster ***));
-static void FDECL(write_objects, (int,char *,object ***));
-static void FDECL(write_engravings, (int,char *,engraving ***));
-static void FDECL(write_maze, (int,specialmaze *));
-static void FDECL(write_rooms, (int,splev *));
+static boolean FDECL(write_common_data, (int,int,lev_init *,long));
+static boolean FDECL(write_monsters, (int,char *,monster ***));
+static boolean FDECL(write_objects, (int,char *,object ***));
+static boolean FDECL(write_engravings, (int,char *,engraving ***));
+static boolean FDECL(write_maze, (int,specialmaze *));
+static boolean FDECL(write_rooms, (int,splev *));
+static void NDECL(init_obj_classes);
 
 static struct {
 	const char *name;
@@ -113,7 +120,7 @@ static struct {
 	{ "pit",	PIT },
 	{ "spiked pit",	SPIKED_PIT },
 	{ "hole",	HOLE },
-	{ "trapdoor",	TRAPDOOR },
+	{ "trap door",	TRAPDOOR },
 	{ "teleport",	TELEP_TRAP },
 	{ "level teleport", LEVEL_TELEP },
 	{ "magic portal",   MAGIC_PORTAL },
@@ -141,6 +148,9 @@ static struct {
 	{ "zoo",	 ZOO },
 	{ "delphi",	 DELPHI },
 	{ "temple",	 TEMPLE },
+	{ "anthole",	 ANTHOLE },
+	{ "cocknest",	 COCKNEST },
+	{ "leprehall",	 LEPREHALL },
 	{ "shop",	 SHOPBASE },
 	{ "armor shop",	 ARMORSHOP },
 	{ "scroll shop", SCROLLSHOP },
@@ -244,6 +254,8 @@ char **argv;
 	monst_init();
 	objects_init();
 	decl_init();
+	/* this one does something... */
+	init_obj_classes();
 
 	init_yyout(stdout);
 	if (argc == 1) {		/* Read standard input */
@@ -395,14 +407,19 @@ char c;
  * Find the index of an object in the table, knowing its name.
  */
 int
-get_object_id(s)
+get_object_id(s, c)
 char *s;
+char c;		/* class */
 {
-	register int i;
-	register const char *objname;
+	int i, class;
+	const char *objname;
 
 	SpinCursor(3);
-	for (i=0; i<NUM_OBJECTS; i++) {
+	class = (c > 0) ? def_char_to_objclass(c) : 0;
+	if (class == MAXOCLASSES) return ERR;
+
+	for (i = class ? bases[class] : 0; i < NUM_OBJECTS; i++) {
+	    if (class && objects[i].oc_class != class) break;
 	    objname = obj_descr[i].oc_name;
 	    if (objname && !strcmp(s, objname))
 		return i;
@@ -410,6 +427,21 @@ char *s;
 	return ERR;
 }
 
+static void
+init_obj_classes()
+{
+	int i, class, prev_class;
+
+	prev_class = -1;
+	for (i = 0; i < NUM_OBJECTS; i++) {
+	    class = objects[i].oc_class;
+	    if (class != prev_class) {
+		bases[class] = i;
+		prev_class = class;
+	    }
+	}
+}
+
 /*
  * Is the character 'c' a valid monster class ?
  */
@@ -484,8 +516,13 @@ char *map;
 	int max_hig = 0;
 	char msg[256];
 
-	/* First : find the max width of the map */
+	/* First, strip out digits 0-9 (line numbering) */
+	for (s1 = s2 = map; *s1; s1++)
+	    if (*s1 < '0' || *s1 > '9')
+		*s2++ = *s1;
+	*s2 = '\0';
 
+	/* Second, find the max width of the map */
 	s1 = map;
 	while (s1 && *s1) {
 		s2 = index(s1, NEWLINE);
@@ -500,7 +537,6 @@ char *map;
 	}
 
 	/* Then parse it now */
-
 	while (map && *map) {
 		tmpmap[max_hig] = (char *) alloc(max_len);
 		s1 = index(map, NEWLINE);
@@ -930,7 +966,7 @@ store_room()
 /*
  * Output some info common to all special levels.
  */
-static void
+static boolean
 write_common_data(fd, typ, init, flgs)
 int fd, typ;
 lev_init *init;
@@ -953,12 +989,13 @@ long flgs;
 	Write(fd, &len, sizeof len);
 	if (len) Write(fd, tmpmessage, (int) len);
 	tmpmessage[0] = '\0';
+	return TRUE;
 }
 
 /*
  * Output monster info, which needs string fixups, then release memory.
  */
-static void
+static boolean
 write_monsters(fd, nmonster_p, monsters_p)
 int fd;
 char *nmonster_p;
@@ -992,12 +1029,13 @@ monster ***monsters_p;
 	    *monsters_p = 0;
 	}
 	*nmonster_p = 0;
+	return TRUE;
 }
 
 /*
  * Output object info, which needs string fixup, then release memory.
  */
-static void
+static boolean
 write_objects(fd, nobject_p, objects_p)
 int fd;
 char *nobject_p;
@@ -1025,12 +1063,13 @@ object ***objects_p;
 	    *objects_p = 0;
 	}
 	*nobject_p = 0;
+	return TRUE;
 }
 
 /*
  * Output engraving info, which needs string fixup, then release memory.
  */
-static void
+static boolean
 write_engravings(fd, nengraving_p, engravings_p)
 int fd;
 char *nengraving_p;
@@ -1056,6 +1095,7 @@ engraving ***engravings_p;
 	    *engravings_p = 0;
 	}
 	*nengraving_p = 0;
+	return TRUE;
 }
 
 /*
@@ -1078,14 +1118,20 @@ specialmaze *maze_level;
 	Strcat(lbuf, filename);
 	Strcat(lbuf, LEV_EXT);
 
+#if defined(MAC) && (defined(__SC__) || defined(__MRC__))
+	fout = open(lbuf, O_WRONLY|O_CREAT|O_BINARY);
+#else
 	fout = open(lbuf, O_WRONLY|O_CREAT|O_BINARY, OMASK);
+#endif
 	if (fout < 0) return FALSE;
 
-	if (room_level)
-	    write_rooms(fout, room_level);
-	else if (maze_level)
-	    write_maze(fout, maze_level);
-	else
+	if (room_level) {
+	    if (!write_rooms(fout, room_level))
+		return FALSE;
+	} else if (maze_level) {
+	    if (!write_maze(fout, maze_level))
+		return FALSE;
+	} else
 	    panic("write_level_file");
 
 	(void) close(fout);
@@ -1096,7 +1142,7 @@ specialmaze *maze_level;
  * Here we write the structure of the maze in the specified file (fd).
  * Also, we have to free the memory allocated via alloc().
  */
-static void
+static boolean
 write_maze(fd, maze)
 int fd;
 specialmaze *maze;
@@ -1104,7 +1150,8 @@ specialmaze *maze;
 	short i,j;
 	mazepart *pt;
 
-	write_common_data(fd, SP_LEV_MAZE, &(maze->init_lev), maze->flags);
+	if (!write_common_data(fd, SP_LEV_MAZE, &(maze->init_lev), maze->flags))
+	    return FALSE;
 
 	Write(fd, &(maze->filling), sizeof(maze->filling));
 	Write(fd, &(maze->numpart), sizeof(maze->numpart));
@@ -1120,8 +1167,19 @@ specialmaze *maze;
 	    Write(fd, &(pt->ysize), sizeof(pt->ysize));
 	    for(j=0;j<pt->ysize;j++) {
 		if(!maze->init_lev.init_present ||
-		   pt->xsize > 1 || pt->ysize > 1)
-		    Write(fd, pt->map[j], pt->xsize * sizeof *pt->map[j]);
+		   pt->xsize > 1 || pt->ysize > 1) {
+#if !defined(_MSC_VER) && !defined(__BORLANDC__)
+			Write(fd, pt->map[j], pt->xsize * sizeof *pt->map[j]);
+#else
+			/*
+			 * On MSVC and Borland C compilers the Write macro above caused:
+			 * warning '!=' : signed/unsigned mismatch
+			 */
+			unsigned reslt, sz = pt->xsize * sizeof *pt->map[j];
+			reslt = write(fd, (genericptr_t)(pt->map[j]), sz);
+			if (reslt != sz) return FALSE;
+#endif
+		}
 		Free(pt->map[j]);
 	    }
 	    Free(pt->map);
@@ -1262,13 +1320,15 @@ specialmaze *maze;
 		    Free(pt->traps);
 
 	    /* The monsters */
-	    write_monsters(fd, &pt->nmonster, &pt->monsters);
+	    if (!write_monsters(fd, &pt->nmonster, &pt->monsters))
+		    return FALSE;
 
 	    /* The objects */
-	    write_objects(fd, &pt->nobject, &pt->objects);
+	    if (!write_objects(fd, &pt->nobject, &pt->objects))
+		    return FALSE;
 
 	    /* The gold piles */
-	    Write(fd, &(pt->ngold), sizeof(pt->naltar));
+	    Write(fd, &(pt->ngold), sizeof(pt->ngold));
 	    for(j=0;j<pt->ngold;j++) {
 		    Write(fd, pt->golds[j], sizeof(gold));
 		    Free(pt->golds[j]);
@@ -1277,7 +1337,8 @@ specialmaze *maze;
 		    Free(pt->golds);
 
 	    /* The engravings */
-	    write_engravings(fd, &pt->nengraving, &pt->engravings);
+	    if (!write_engravings(fd, &pt->nengraving, &pt->engravings))
+		    return FALSE;
 
 	    Free(pt);
 	}
@@ -1285,12 +1346,13 @@ specialmaze *maze;
 	Free(maze->parts);
 	maze->parts = (mazepart **)0;
 	maze->numpart = 0;
+	return TRUE;
 }
 
 /*
  * Here we write the structure of the room level in the specified file (fd).
  */
-static void
+static boolean
 write_rooms(fd, lev)
 int fd;
 splev *lev;
@@ -1298,7 +1360,8 @@ splev *lev;
 	short i,j, size;
 	room *pt;
 
-	write_common_data(fd, SP_LEV_ROOMS, &(lev->init_lev), lev->flags);
+	if (!write_common_data(fd, SP_LEV_ROOMS, &(lev->init_lev), lev->flags))
+		return FALSE;
 
 	/* Random registers */
 
@@ -1373,10 +1436,12 @@ splev *lev;
 			Write(fd, pt->traps[j], sizeof(trap));
 
 		/* The monsters */
-		write_monsters(fd, &pt->nmonster, &pt->monsters);
+		if (!write_monsters(fd, &pt->nmonster, &pt->monsters))
+			return FALSE;
 
 		/* The objects */
-		write_objects(fd, &pt->nobject, &pt->objects);
+		if (!write_objects(fd, &pt->nobject, &pt->objects))
+			return FALSE;
 
 		/* The gold piles */
 		Write(fd, &(pt->ngold), sizeof(pt->ngold));
@@ -1384,7 +1449,8 @@ splev *lev;
 			Write(fd, pt->golds[j], sizeof(gold));
 
 		/* The engravings */
-		write_engravings(fd, &pt->nengraving, &pt->engravings);
+		if (!write_engravings(fd, &pt->nengraving, &pt->engravings))
+			return FALSE;
 
 	}
 
@@ -1392,6 +1458,7 @@ splev *lev;
 	Write(fd, &lev->ncorr, sizeof(lev->ncorr));
 	for (i=0; i < lev->ncorr; i++)
 		Write(fd, lev->corrs[i], sizeof(corridor));
+	return TRUE;
 }
 
 /*
