@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)tilemap.c	3.3	1999/03/28	*/
+/*	SCCS Id: @(#)tilemap.c	3.4	2000/06/04	*/
 /* NetHack may be freely redistributed.  See license for details. */
 
 /*
@@ -11,6 +11,7 @@
 
 const char * FDECL(tilename, (int, int));
 void NDECL(init_tilemap);
+void FDECL(process_substitutions, (FILE *));
 
 #ifdef MICRO
 #undef exit
@@ -86,7 +87,7 @@ struct conditionals {
 	 * don't know what a slime mold should look like when renamed anyway
 	 */
 #ifndef MAIL
-	{ OBJ_GLYPH, SCR_CHARGING+6, "stamped / mail" },
+	{ OBJ_GLYPH, SCR_STINKING_CLOUD+4, "stamped / mail" },
 #endif
 	{ 0, 0, 0}
 };
@@ -102,8 +103,8 @@ struct conditionals {
  */
 struct substitute {
 	int first_glyph, last_glyph;
-	char *sub_name;		/* for explanations */
-	char *level_test;
+	const char *sub_name;		/* for explanations */
+	const char *level_test;
 } substitutes[] = {
 	{ GLYPH_CMAP_OFF + S_vwall, GLYPH_CMAP_OFF + S_trwall,
 					"mine walls", "In_mines(plev)" },
@@ -179,8 +180,8 @@ int set, entry;
 	}
 
 	tilenum = 0;	/* set-relative number */
-	for (i = 0; i < MAXPCHARS; i++) {
-		if (set == OTH_GLYPH && tilenum == entry)
+	for (i = 0; i < (MAXPCHARS - MAXEXPCHARS); i++) {
+		if (set == OTH_GLYPH && tilenum == entry) {
 			if (*defsyms[i].explanation)
 				return defsyms[i].explanation;
 			else {
@@ -200,6 +201,7 @@ int set, entry;
 				}
 				return buf;
 			}
+		}
 		tilenum++;
 		while (conditionals[condnum].sequence == OTH_GLYPH &&
 			conditionals[condnum].predecessor == i) {
@@ -209,6 +211,21 @@ int set, entry;
 			tilenum++;
 		}
 	}
+	/* explosions */
+	tilenum = MAXPCHARS - MAXEXPCHARS;
+	i = entry - tilenum;
+	if (i < (MAXEXPCHARS * EXPL_MAX)) {
+	    if (set == OTH_GLYPH) {
+		static char *explosion_types[] = { /* hack.h */
+			"dark", "noxious", "muddy", "wet",
+			"magical", "fiery", "frosty"
+		};
+		Sprintf(buf, "explosion %s %d",
+			explosion_types[i / MAXEXPCHARS], i % MAXEXPCHARS);
+		return buf;
+	    }
+	}
+	tilenum += (MAXEXPCHARS * EXPL_MAX);
 
 	i = entry - tilenum;
 	if (i < (NUM_ZAP << 2)) {
@@ -219,6 +236,15 @@ int set, entry;
 	}
 	tilenum += (NUM_ZAP << 2);
 
+	i = entry - tilenum;
+	if (i < WARNCOUNT) {
+		if (set == OTH_GLYPH) {
+			Sprintf(buf, "warning %d", i);
+			return buf;
+	        }
+	}
+	tilenum += WARNCOUNT;
+
 	for (i = 0; i < SIZE(substitutes); i++) {
 	    j = entry - tilenum;
 	    if (j <= substitutes[i].last_glyph - substitutes[i].first_glyph) {
@@ -262,7 +288,8 @@ int lastmontile, lastobjtile, lastothtile;
  * with entries for all supported compilation options
  *
  * "other" contains cmap and zaps (the swallow sets are a repeated portion
- * of cmap)
+ * of cmap), as well as the "flash" glyphs for the new warning system
+ * introduced in 3.3.1.
  */
 void
 init_tilemap()
@@ -333,7 +360,7 @@ init_tilemap()
 	}
 	lastobjtile = tilenum - 1;
 
-	for (i = 0; i < MAXPCHARS; i++) {
+	for (i = 0; i < (MAXPCHARS - MAXEXPCHARS); i++) {
 		tilemap[GLYPH_CMAP_OFF+i] = tilenum;
 		tilenum++;
 		while (conditionals[condnum].sequence == OTH_GLYPH &&
@@ -343,14 +370,35 @@ init_tilemap()
 		}
 	}
 
+	for (i = 0; i < (MAXEXPCHARS * EXPL_MAX); i++) {
+		tilemap[GLYPH_EXPLODE_OFF+i] = tilenum;
+		tilenum++;
+		while (conditionals[condnum].sequence == OTH_GLYPH &&
+			conditionals[condnum].predecessor == (i + MAXPCHARS)) {
+			condnum++;
+			tilenum++;
+		}
+	}
+
 	for (i = 0; i < NUM_ZAP << 2; i++) {
 		tilemap[GLYPH_ZAP_OFF+i] = tilenum;
 		tilenum++;
+		while (conditionals[condnum].sequence == OTH_GLYPH &&
+			conditionals[condnum].predecessor == (i + MAXEXPCHARS)) {
+			condnum++;
+			tilenum++;
+		}
 	}
+
+	for (i = 0; i < WARNCOUNT; i++) {
+		tilemap[GLYPH_WARNING_OFF+i] = tilenum;
+		tilenum++;
+	}
+
 	lastothtile = tilenum - 1;
 }
 
-char *prolog[] = {
+const char *prolog[] = {
 	"",
 	"",
 	"void",
@@ -361,7 +409,7 @@ char *prolog[] = {
 	""
 };
 
-char *epilog[] = {
+const char *epilog[] = {
 	"}"
 };
 
