@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)getline.c	3.3	96/01/27	*/
+/*	SCCS Id: @(#)getline.c	3.4	1996/01/27	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -72,12 +72,25 @@ getlin_hook_proc hook;
 		    *bufp = 0;
 		}
 		if(c == '\020') { /* ctrl-P */
-		    if(!doprev)
-			(void) tty_doprev_message(); /* need two initially */
-		    (void) tty_doprev_message();
-		    doprev = 1;
-		    continue;
-		} else if(doprev) {
+		    if (iflags.prevmsg_window) {
+			int sav = ttyDisplay->inread;
+			ttyDisplay->inread = 0;
+			(void) tty_doprev_message();
+			ttyDisplay->inread = sav;
+			tty_clear_nhwindow(WIN_MESSAGE);
+			cw->maxcol = cw->maxrow;
+			addtopl(query);
+			addtopl(" ");
+			*bufp = 0;
+			addtopl(obufp);
+		    } else {
+			if (!doprev)
+			    (void) tty_doprev_message();/* need two initially */
+			(void) tty_doprev_message();
+			doprev = 1;
+			continue;
+		    }
+		} else if (doprev && !iflags.prevmsg_window) {
 		    tty_clear_nhwindow(WIN_MESSAGE);
 		    cw->maxcol = cw->maxrow;
 		    doprev = 0;
@@ -98,7 +111,7 @@ getlin_hook_proc hook;
 #endif
 			*bufp = 0;
 			break;
-		} else if(' ' <= c && c < '\177' &&
+		} else if(' ' <= (unsigned char) c && c != '\177' &&
 			    (bufp-obufp < BUFSZ-1 && bufp-obufp < COLNO)) {
 				/* avoid isprint() - some people don't have it
 				   ' ' is not always a printing char */
@@ -167,11 +180,12 @@ ext_cmd_getlin_hook(base)
 
 	com_index = -1;
 	for (oindex = 0; extcmdlist[oindex].ef_txt != (char *)0; oindex++) {
-		if (!strncmpi(base, extcmdlist[oindex].ef_txt, strlen(base)))
+		if (!strncmpi(base, extcmdlist[oindex].ef_txt, strlen(base))) {
 			if (com_index == -1)	/* no matches yet */
 			    com_index = oindex;
 			else			/* more than 1 match */
 			    return FALSE;
+		}
 	}
 	if (com_index >= 0) {
 		Strcpy(base, extcmdlist[com_index].ef_txt);
